#!/usr/bin/perl -w

## =====================================================================
## Name:        area2digitetrap.pl
##              (Adapted from area2digit-multi-fullyAuto.pl in April 2009)
##                  
## Author:      Jianbin Yang/PSGS   04/06/2009
## Language:    Perl
## Purpose:     Convert the 6 hourly and 24 hour total eTRaP products from
##              Mcidas AREA format to text format (digital eTRaP) and
##              send out an notification message to a specific customer
##              list according to the area the storm fell in.
##                  
##
## Usage:       area2digitetrap.pl <basin1> <basin2> [<product List Prefix>] [<product Area List>] [<latest Position File>]
##
## Note:        For 24 hour total TRaP the AREA file number is 647 or 637
##              In this program it is expect the input is the 24 hour toatl TRaP
##              This program is expect to run after the program "restoreAREA.pl"
##              is called, so that the positon file can be archived
##              "GIF file" -- The GIF file name of the 24 total TRaP GIF sent to SAB homepage   
##
## Please note that: 
##              The position track from the pctrap.pl is used Mcidas convention
##              That means east longitude is a negative number.  
##              But when the digital trap should use the international convention
##	        as the east longitude is a positive number.  So it is coded to 
##		convert back to international convention.
##		And noted that the second input argument "Actual time of Rain rate data"
##		is the time orbit passed over the storm.  It is input by SAB analyst currently.
##		It will be used in the file name of the digital TRaP.  If there is no input, 
##		The composite time of the image will be uesed as default.
##              
## Modified at:   08/10/2003       To switch the formal email list as operational 
##
## Modified: 08/18/2003
##
## NOTED: The orbit time input format maybe different. Will confirm this.   07/10/2003 
##
## Modified: 11/20/2003
##
## NOTED: Not input the orbit time, since it is got from the POS text file. 
##
## Modified: 01/16/2004
##
## NOTED: Produce four 6 hourly digital TRaPs as well as 24 total digital TRaP.
##
## Modified: 02/03/2004
##
## NOTED: Change the TPC/CPHC WMO header to WT**2*
##
## Modified: 03/22/2004
##        Remove the longitude restriction for the Australia bulletins
##        So that there is no missing case if the longitude not fall in the range
##        That was a missing case for "FAY" (center at 127 East, bulletin WTAU03 ) last week
##
## Modified: 07/28/2004
##
## Modified: 08/18/2004
##        DNS changes for FTP server
##        (from "gp16.wwb.noaa.gov" to "gp16.ssd.nesdis.noaa.gov")
##
## Modified: 03/14/2005
##        Add the ICAO bulletin FKAU to the "getBasin" subroutine          
##
## Modified: 09/19/2006
##        Modify for Fully Automate TRaP, the program is called within the Main TRaP program.
##        The customer email notification is turn off to reduce the frequency of the notification.
##        Write the TRaP list and email list to other file for other program to read.         
## 
##   Notes: will need to modify the name POS2.TXT and take care of the line to get the TRaP set name
##          also the rawa GIF file --- on 09/19/2006
##          will send the GIF TRaP files to SAB Web page, and call createTRaPDir.pl inside ftp -- on 09/20/06 
##
## Modified: 10/10/2006
##        Will write the basin name in a text file for every storm 
##
## Modified: 03/05/2007
##	  Break down the boundaries for the ICAO FKAU bulletin
##
## Modified: 03/19/2007
## 	  Set the basin and basin abbreviation as hash table
##
## Modified: 04/10/2007
##        Remove the latitude restriction on the Southeast and Southwest Pacific,
##	  Remove the Usage of "OtherList", etc.
##
## Modified: 06/14/2007
##	  Write the digital TRaP to SABNA4
##
## Modified: 06/26/2007
## 	  Add New Delhi and New Zealand RSMC bulletins
##        Add FKPS01 for Fiji and FKIO20 for Reunion  	07/05/2007
##
## Modified: 10/23/2007
##        Distribute the TRaP GIF files to SATEPS Server for GEONETCast
##
## Modified: 01/11/2008
##        Change the Southern hemisphere boundary from 130 to 125
##
## Modified: 02/04/2008
##        Additionally send the bulletin, rain rate data out to users
##        along with the digital TRaP
##
## Modified: 02/25/2008
##        Create a single ftp directory where customers can find the 
##        latest TRaPs for all storms using the storm name as the name of the file
##
## Modified: 04/14/2008
##         Added functionality for AXAU bulletins
##
## Modified: 05/01/2008
##         Update the active storm list file with new storm when the first TRaP from the storm is created
##         (notes: remove of the storm entry will be done in the cleanTRaP program)
##
## Modified: 05/29/2008
##         Add the NSOF GP5 Web server
##
## Modified: 02/06/09
##         Changed to use $HOME without specifying the oper account
##
## Modified: 12/26/2008
##         Add Jakarta bulletin
##
## ADAPTED: 04/10 - 05/05/2009
##          For eTRaP
## Modified : 05/28/09 
##          Add error checking for monitoring
##          Need to check the FTP connection in some way ( in driver script )
##
## Modified : 08/19/09
##          Add to send GIf and ASCII Text to Satepsdist1 for GeonetCast
## 
## Modified : 09/15/09
##          Convert the log file to html to include the TRaP GIF to ETRaP Web page
##
## Modified : 11/10/09
##          Disseminate the TRaP member log files to the FTP and SABNa4 server
##
## Last Modified : 10/20/11
##          Assure overwrite compression of non GIFs, needed
##          because of hourly eTRaPs
##
## =====================================================================

use Class::Struct;

####---------------------- Define  and delcare ------------------------####

##------------------------   Define structures   ------------------------##

# structure to store the created AREA file list
struct ProdsInfo =>
{
   fcstTime   => '$',
   areaNum    => '$',
   numOfArea  => '$',
   pdNameList => '@',
};

@prodsList = ();  # array to store the "ProdsInfo" structure
@fcstTimeList = ();

#--------- Define the directories and files ---------#


$HOME = $ENV{"HOME"};

$srcDir = "$HOME/mcidas/etrap/scripts";
$dataDir = "$HOME/mcidas/data";
$eTRaPDataDir = '/data/Petrap/data/ETRAP';
$digitTRAP_sent_Dir = "$eTRaPDataDir/digitETRAP/";  	# it holds the digit TRaP sent to FTP server

$etrapSetLog = "$dataDir/etrapSet.log";

# File to keep the position of the track of the storm and information to generate the digital TRaP.
# PLEASE NOTE: This file name must be CONSISTENT as the one in etrap.f90/run_etrap.sh
# generated from pctrap.pl as the TRaP generated.
 
$digitFile1 = 'digite1.txt';             # File to hold the first part of the digital TRaP
$digitFile2 = 'digite2.txt';		# File to hold the text format TRaP(part 2), not the final file 	
$digitFile = 'digite.txt';             # the final digital file

$sftpfile1 = '/tmp/etrapFtpRmtPut1';   # FTP server
$sftpfile2 = '/tmp/etrapFtpRmtPut2';   # Web - GP5
$sftpfile3 = '/tmp/etrapFtpRmtPut3';   # Web - GP50
$sftpfile4 = '/tmp/etrapFtpRmtPut4';   # SAB Data Server
$sftpfile5 = '/tmp/etrapFtpRmtPut5';   # Satepsdist1 for GeonetCast
$ftpfile6 = '/tmp/etrapFtpRmtPut6';   # DDS

## The log file for this program
#$stdoutLog = 'DIGITE.OUT';
#$stderrLog = 'DIGITE.ERR';
#$logFile = "$eTRaPDataDir/logs/digitETRAP.log";


##-------------------- Define some constants --------------------##

# Some eTRaP constants

$averagePattern = "avgqpf";     # product type pattern in the file name
$trapNumPattern = "TRAPNUM";    # product type pattern for TRaP member number
$logPattern = "ETRAP";          # product type pattern for log file
$logSearchPattern = "independent TRaPs used";
$trapListPattern = "TRAPGIFS";  # product type pattern for TRAP GIFs List file, and also the sub directory in web

# Javascript line for TRaP member number
$tempTrapNumJSline = "function getTRAPNUM_HH() {"; 
$tempHour = "HH"; # HH is a part of above line, and will be replaced

#--------- basins and basin abbreviation ---------#

# Define the hash table for the basin (dir here) and the corresponding basin abbreviation (basin here)
%basinTbl = qw(
   Atlantic AL 
   EastPacific EP 
   CentralPacific CP
   NWPacific WP
   SWPacific SW
   SEPacific SE
   NorthIndian NI
   SWIndian WI
   SEIndian EI
   NTerritory NT 
);


#--------- SAB Tropical Cyclone Homepage URL ---------#
$sabURL = 'http://www.ssd.noaa.gov/PS/TROP/etrap.html';


#--------- anonymous ftp server and directories ---------#

# the domain name is not working since 11/25/08 to 11/26/08 
# gp16.ssd.nesdis.noaa.gov
# $ftpServer1 = 'oper\@140.90.195.198';
# satepsanon.nesdis.noaa.gov
$ftpServer1 = 'oper\@10.144.3.43';
$ftpHomeDir ='pub/TRAP/ETRAP';
$ftpHomeDir2 ='TRAP/ETRAP';
$ftpURL = 'ftp://satepsanone.nesdis.noaa.gov/' . $ftpHomeDir2;

$readmeDir = 'README';

#--------- SAB Data Server ---------#

#$sabDataServer = 'sab\@sabna4.ssd.nesdis.noaa.gov';
$sabDataServer = 'sab\@140.90.195.121';
$sabDataHomeDir = "/home/sab/TRAP/ETRAP";
#$sabDataTrapDir = "/data/TRAP/ETRAP/digitETRAP";
$sabDataTrapDir = "/data/ext/SABTrap1/ETRAP/digitETRAP";
$sabDataRemoteCM = "$sabDataHomeDir/createDigEtrapDir.pl";


#--------- SAB Web server and directories ---------#

$webServer1 = 'sabweb\@10.144.3.87';
#$webServer2 = 'WEBMGR\@gp50.ssd.nesdis.noaa.gov';
$webServer2 = 'sabweb\@gp50.ssd.nesdis.noaa.gov';
$remoteETrapDir = "/data/web/mirror/www.ssd.noaa.gov/htdocs/PS/TROP/DATA/ETRAP/images/";
$remoteTrapGifDir = "/data/web/mirror/www.ssd.noaa.gov/htdocs/PS/TROP/DATA/ETRAP/$trapListPattern/";
#$remoteHomeDir = "/home/WEBMGR/TRAP/ETRAP";
$remoteHomeDir = "/home/sabweb/TRAP/ETRAP";
$remoteCM = "$remoteHomeDir/createETRaPDir.pl";


#--------- SATEPS Product Server server and directories ---------#
# scp $nn*GIF spider@satepsdist1e.nesdis.noaa.gov:/data/GEONET/

$satepServer = "spider\@satepsdist1.nesdis.noaa.gov";
$remoteGeonetDir = "/data/GEONET/data";
$geonetFilePrefix = "eTRaP.";

#--------- DDS Product Server server and directories ---------#
$DDSServer = "dds.nesdis.noaa.gov";
$NCDCDir="push";

#--------- the email notification list of different agent ---------#

# New define email list names are as following, must match the names in "TRaPEmailList.txt".  Added on 09/11/2006
# TPCList1, TPCList, AtlanticList, CPHCList, 
# PerthList, DarwinList, BrisbaneList, ReunionList, NewDelhiList
# NadiList, NWPacificList, and BCCList.

$TPCList 	= "TPCList";
$AtlanticList 	= "AtlanticList";
$CPHCList 	= "CPHCList";
$PerthList 	= "PerthList";
$DarwinList 	= "DarwinList";
$BrisbaneList 	= "BrisbaneList";
$ReunionList 	= "ReunionList";
$NewDelhiList 	= "NewDelhiList";
$NadiList 	= "NadiList";
$NWPacificList 	= "NWPacificList";
$ccList 	= "BCCList";
# $OtherList      = ""; 

# $MWOGuamList   = "";
# $TokyoList     = "";


##-------------------- initialize some variables --------------------##

# the following can be get from the "getBasin" function
$basin = "";
# @emailList = ();		# emailList maybe empty even after the "getbasin" is called for JTWC, for SWPacifc? at 09/11/2006
$subDir = "";
$remoteTRAPDir = "";		# destination directory for the digital file in the FTP server
$basinArea = "";		# similar to "subdirectories", for TRaP List file 

# Some default values for digital eTRaP
$defaultStNum = 99;
$defaultLat   = 99.99;
$defaultLon   = 999.99;


####----------------------------------------------------------------------------####

####----------------------------  Start the program ----------------------------####

$path = $ENV{"PATH"};
$ENV{"PATH"} = $path;

$ENV{"MCPATH"} = "$HOME/mcidas/data:/home/mcidas/data";

$startdate=`date`;
print "Start  $0 at $startdate \n";

chdir("$dataDir") || die "ERROR: Cannot change to directory $dataDir";
print  "Data directory is:";
system("pwd");

# remove the path from the command name

@comandFields = split(/\//, $0);
$commandName = $comandFields[-1];

# if no argument input, exit the program

if (@ARGV < 2)
{
   print "Usage: $commandName <basin1> <basin2> [<product List Prefix>] [<product Area List>] [<latest Position File>]\n";
   die "ERROR: No basins input - exit script";
}
else 
{
   $basin1 = $ARGV[0];
   $basin2 = $ARGV[1];
   # Set the default values
   $pdListPre   = "$dataDir/prodList";
   $pdAreaList  = "$dataDir/prodAreaList.txt";
   $lastPosFile = "$dataDir/LATEST_POS.TXT";
 
   if (@ARGV == 3)
   {
      $pdListPre = $ARGV[2];
   }
   elsif (@ARGV == 4)
   {  
      $pdListPre = $ARGV[2]; 
      $pdAreaList = $ARGV[3];
   }
   elsif (@ARGV >= 5)
   {
      $pdListPre = $ARGV[2];
      $pdAreaList = $ARGV[3];
      $lastPosFile = $ARGV[4];
   }
}

if (!(-s $pdAreaList))
{
   die "ERROR: $pdAreaList is blank or not exist! - exit script\n";
}

$pdListExist = `ls $pdListPre.??.txt |grep $pdListPre`;
if ($pdListExist eq "")
{
  die "ERROR: $pdListPre files not exist! - exit script\n";
}


print "\n##--------------------- Read the created AREA list file and the product file name list ---------------------##\n\n";

open(AREASLIST, "< $pdAreaList") || die "ERROR: Can not open $pdAreaList for reading: $!";
@areasList = <AREASLIST>;
close(AREASLIST);

print "number of areasList lines is $#areasList\n";

#for ($i = 0; $i < $#areasList+1; $i++)
#for ($i = 0; $i <= $#areasList; $i++)
for ($areaIndex = 0; $areaIndex <= $#areasList; $areaIndex++)
{
   chomp($areasList[$areaIndex]);
   print "areasList - $areaIndex is $areasList[$areaIndex]\n";
   if ($areasList[$areaIndex] !~ /^\d{2}\s+\d{3,4}\s+\d+/)
   {
      die "ERROR: Unexpected format for the created AREA file line: $areasList[$i] - EXIT Script.\n";
   }
   @splitList = split(/\s+/, $areasList[$areaIndex]);
   $fcstTime = $splitList[0];
   $areaNum  = $splitList[1];
   $numOfArea = $splitList[2];

   print "fcstTime, areaNum, numOfArea is $fcstTime $areaNum $numOfArea\n";

   my $prodInfo = ProdsInfo->new();
   $prodInfo->fcstTime($fcstTime);
   $prodInfo->areaNum($areaNum);
   $prodInfo->numOfArea($numOfArea);
   
   # get the product name list for that forecast time
   $pdFile = $pdListPre . '.' . $fcstTime . ".txt";
   open(PRODLIST, "< $pdFile") || die "ERROR: Can not open $pdFile for reading: $!";
   @pdList = <PRODLIST>;
   close(PRODLIST);
   
   if ($numOfArea != $#pdList + 1)
   {
      print "numOfArea is $numOfArea, pdList is @pdList";
      die "ERROR: Unmatched number of the products for time period $fcstTime. - EXIT Script.\n"; 
   }

   # Chomp the product name list
   for ($i = 0; $i <= $#pdList; $i++)
   {
      chomp($pdList[$i]);
      #print "pdList-$i is $pdList[$i].\n";
   }

   @{$prodInfo->pdNameList} = @pdList;
   push (@prodsList, $prodInfo);
   push (@fcstTimeList, $fcstTime);
}

####-----------------------------------------------------------------####

if (@prodsList == 0)
{
   print "ERROR: No eTRaP product generated for this storm, exit program!\n";
   exit; 
}

# Total Product file name list for all the time period
@totalProdList = ();
@totalTRAPNUMList = ();
@totalLogList = ();
@totalLogList_Ori = ();
$timeIndex = 0;

# The file name of first product in the set, will pass the the remote command
$firstFileName = "";

$curDir = `pwd`;
chomp($curDir);

foreach $prodInfo (@prodsList)
{
   $fcstTime  = $prodInfo->fcstTime;
   $areaNum1  = $prodInfo->areaNum;
   $numOfArea = $prodInfo->numOfArea;
   @pdList    = @{$prodInfo->pdNameList};

   # Skip the first element:the avg QPF 
   @pdListTmp = @pdList;
   $firstProd = shift(@pdListTmp);
   print "firstProd is $firstProd.\n";
   if ( $firstProd !~ /$averagePattern/i)
   {
      print "WARNING! The first product for time $fcstTime is not the average product, unshift it.\n";
      @pdListTmp = @pdList;
      $stIndex = 0; 
   }
   else
   {
      $stIndex = 1;
      system("compress -f $firstProd");
   }

   push (@totalProdList, @pdListTmp);

   for ($pdIndex = $stIndex; $pdIndex < $numOfArea; $pdIndex++)
   {
      $areaNum = $areaNum1 + $pdIndex;
      $fileName = $pdList[$pdIndex];
      $digitfileName = $fileName . ".TXT";

      # QPF scale 
      if ($pdIndex < 2 )
      {
         if ($fcstTime < 24)
         {
            $scale = 20;
         }
         else
         {
            $scale = 40;
         }
      }
      # Probability scale
      else
      {
         $scale = 255;
      }

      print "The current Area file number is $areaNum, scale is $scale. \n";

      ####------ Convert the input AREA eTRaP to text format eTRaP ------####
      ####------ This is part 2 / data  of the final digital eTRaP ------####

      #print "Start to conver the AREA eTRaP file to digital eTRaP.\n";
 
      # convert data part of the AREA eTRaP to digital TRaP pasr e, 
      # For the first file only, use it to get the part 1

      print "Run this:digitaletrap.k $areaNum $digitFile2 $scale DEV=T temptext\n";
      system("digitaletrap.k $areaNum $digitFile2 $scale DEV=T temptext");

      # Only get the part 1 for the first file, and it will be used for the others
      

      ####------       This is part 1 of the final digital TRaP     ------####

      ####------ Get the forecast bulletin and position information ------####

      ## The coverage area are all the same between the 24 hour total and 6 hourly   ##
      ## so the part 1 is common for all the 6 hourly and 24 hour total digital TRaP ##

      ## ONLY get it for the first area file
      
      if (($timeIndex == 0) && ($pdIndex == $stIndex))
      {
         ##----------------------------------------------------------------------##
         ##------- Get the first product name which including time stamp --------##

         $firstFileName1 = $pdList[$pdIndex];
         # remove the path
         $firstFileName = `basename $firstFileName1`;
         chomp($firstFileName);
         $firstGIF = $firstFileName . ".GIF";
         $firstGIF1 = $firstFileName1 . ".GIF";
         
       
         @splitMsg = split(/\./, $firstFileName);
         $splitMsgLen = @splitMsg;

         printf ("the number of splitMsg is %d.\n",($#splitMsg + 1));

         if ($splitMsgLen < 4)
         {
            print "Format of <Name of the eTRaP file> is:
                   \rYYYYnameNNB.ProductType.MMDDhhmm.hh\n\n";
            die "ERROR: The first file name in the list is not correct - exit";
         }
         $yyyy = substr($splitMsg[0], 0, 4);
         $nameNnb = substr($splitMsg[0],4);  
         $dateTime = $splitMsg[2];
         $dateHour = substr($dateTime,0, -2);
         $timePeriod = $splitMsg[3];


         # Get the storm directory path
         @splitMsg1 = split(/\//, $firstFileName1);
         $splitMsgLen1 = @splitMsg1;

         #$path1="/";
         $stPath="";
         
         # The first split element is empty, the path will have "/" in the end
         for ($j=0; $j < $splitMsgLen1 -1 ;$j++)
         {
           #print "i-$i, splitMsg1 is $splitMsg1[$j]\n";
           $stPath = $stPath . $splitMsg1[$j] . '/';
         }
         print "The storm directory path is $stPath\n";

         # Get the Last position file
         open (POSTXT, $lastPosFile) || die "ERROR: can't open $lastPosFile: $!";
         @posLines = <POSTXT>;
         close (POSTXT) || die "can't close $lastPosFile: $!";

## redirect the print out to some files, to avoid the mcidas wondow error

#open STDOUT, "> $stdoutLog" || die "Can't create $stdoutLog:$!";
#open STDERR, "> $stderrLog" || die "Can't create $stderrLog:$!";


         ##------ Get the first line(header?) of the first part of digital TRaP ------##

         @tempMsg = split(/\s+/, $posLines[0]);
         $lat_0 = $tempMsg[-2];
         $lon_0 = $tempMsg[-1];

         @tempMsg = split(/\s+/, $posLines[5]);

         # NOTES: PAY Attention! When we apply the reconcile technique, 
         # we need to make sure the storm name match the name in the eTRaP file 

         #($stormName, $stNumber, $bulYjd, $wmoHd, $station, $bulTime, $curTime, $forecastHour, $basin) = @tempMsg;
         ($stormName, $stNumber, $bulYjd, $wmoHd, $station, $bulTime, $curTime, $forecastHour) = @tempMsg;
         
         if ($stormName ne $nameNnb)
         {
            print "ERROR:The storm name in the file name is not match to the one in the Last POS file:$nameNnb to $stormName.\n";
            $stormName = $nameNnb;
         } 

         # Change the year on 06/14/2007, use the rain rate year other than the forecast year, this consistent with the file name 
         # $year = substr($bulYjd, 0, 4);
         $year = substr($posLines[6], 0, 4);
         ## chomp($forecastHour);
         if ($year ne $yyyy)
         {
            print "ERROR:The year in the file name is not match to the one in the Last POS file:$yyyy to $year.\n";
            $year = $yyyy;
         }


         print "the stormname is $stormName, bulYjd is $bulYjd, year is $year, wmoHd is $wmoHd, 
                bulTime is $bulTime, curtime is $curTime, stNumber is $stNumber, forecastHour is $forecastHour\n"; 


         ##----------------------------------------------------------------------------##
         ##------------- Get the basin, subdirecotry and email list to send -----------##

         # Please Noted that the "$lon_0" is Micdas convention
         # which is reverse to regular convention, so reverse it here

         $basin = $basinTbl{$basin2};
         $subDir = $basin2;
         $basinArea = $basin2;
         print "Basin just got, the basin is $basin\n";

         # Will give a if/else block for the basins vs email list
         # @emailList = $AtlanticList;
         #getBasin($wmoHd, $station, $lat_0, (-1) * $lon_0);

         ## Check if the basin is got succefully, emailList can be empty if it is from JTWC
         #if (($basin eq "") || ($subDir eq ""))
         #{
         #   print "ERROR!! Could not get the basin or subdirectory for the digital TRaP file in 
         #   \rprogram $commandName. Please contact the programmer.\n";
         #   die "-- exit script";
         #}

         $remoteTRAPDir = "$ftpHomeDir/$subDir";
         $finalURL = "$ftpURL/$subDir";

         # Fixed directory for GRACS users
         $remoteTRAPDir2 = "$ftpHomeDir/$year";
         $finalURL2 = "$ftpURL/$year";
 
         # Overwrite the storm number with 99 if it is not for AL, EP or CP basin
         if (($basin ne "AL") || ($basin ne "EP") || ($basin ne "CP"))
         {
            $stNumber = $defaultStNum;
         }
   
         # write the AREA file information into the digit1 file part 1
         open (DIGIT1, ">$digitFile1") || die "ERROR: can't create $digitFile1: $!";
         #printf DIGIT1 "%s%s%s %s%s %s\n", $basin, $stNumber, $year, $curTime, $forecastHour, $stormName;

         # Get the date and time/forecastHour from the file name instead from the POS file
         printf DIGIT1 "%s%s%s %s %s\n", $basin, $stNumber, $year, $dateHour, $stormName;
         #printf "%s%s%s %s%s %s\n", $basin, $stNumber, $year, $curTime, $forecastHour, $stormName;


         ##-------------------------------------------------------------------##
         ##------ Get the second line of the first part of digital TRaP ------##

         #--- get the start and end lat and lon ---#

         $line = `head -1 $digitFile2`;
         @pos = split(/\s+/, $line);
         print "head pos1 is $pos[0], 2 is $pos[1], 3 is $pos[2]\n";
         $startLon = $pos[1];
         $startLat = $pos[2];

         $line = `tail -1 $digitFile2`;
         @pos = split(/\s+/, $line);
         print "tail pos1 is $pos[0], 2 is $pos[1], 3 is $pos[2]\n";
         $endLon = $pos[1];
         $endLat = $pos[2];

         #--- get the line and element number and resolution degree ---#

         # @lines = `imglist.k LA.$areaNumber FOR=EXP`;    # this is not working under Mcidas window

         system("imglist.k LA.$areaNum FOR=EXP DEV=T tmp.txt");
         open(TMP, "tmp.txt") || die "ERROR: Can not open 'tmp.txt': $!";
         @lines = <TMP>;
         close(TMP) || die "can't close 'tmp.txt': $!";
         unlink "tmp.txt";

         foreach $line (@lines)
         {
            #-- get the line and element number --#
            # $line = $lines[5];
            if ($line =~ /^\s+Band:.*\d$/)
            {
               @tempMsg = split(/\s+/, $line);
               $lsize = $tempMsg[-3];
               $esize = $tempMsg[-1];
            }
            #else
            #{
            #   die "The format is unexpected, cann't get the line and element number--exit.";
            #}

            #-- get the line and element resolution in km --#

            # $line = $lines[12];
            if ($line =~ /^\s+Resolution Factors.*\d$/)
            {
               @tempMsg = split(/\s+/, $line);
               $lres = $tempMsg[-3];
               $eres = $tempMsg[-1];
            }
            #else
            #{
            #   die "The format is unexpected, cann't get the line and element resolution--exit.";
            #}

         } # End of the foreach to read the imglist lines

         #-- convert the km resolution to degree resolution --#

         # $lresDegree = `cu.k $lres KM DEGL`;    # this is not working under Micdas window
         system("cu.k $lres KM DEGL DEV=T cuout");
         system("cu.k $eres KM DEGL DEV=T cuout A");     	# append the output to the cuout file
         open(CUOUT, "cuout") || die "ERROR: Can not open 'cuout': $!";
         $lresDegree = <CUOUT>;
         $eresDegree = <CUOUT>;
         close(CUOUT) || die "can't close 'cuout': $!";;
         unlink "cuout";

         @tempMsg = split(/\s+/, $lresDegree);
         $lresDegree = $tempMsg[3]; 

         # $eresDegree = `cu.k $eres KM DEGL`;
         @tempMsg = split(/\s+/, $eresDegree);
         $eresDegree = $tempMsg[3];

         # print "the lines size is $lsize and the element size is $esize, line-res is $lres, element-res is $eres
         #       grid spacing degree are $lresDegree and $eresDegree\n";


         printf DIGIT1 "%6d%9.3f%10.3f%10.3f%6d%9.3f%10.3f%10.3f\n", $lsize, $lresDegree, 
                        $startLon, $endLon, $esize, $eresDegree, $startLat, $endLat;
         #printf "%6d%9.3f%10.3f%10.3f%6d%9.3f%10.3f%10.3f\n", $lsize, $lresDegree, 
         #               $startLon, $endLon, $esize, $eresDegree, $startLat, $endLat;


         ##----------------------------------------------------------------------##
         ##------ Get the position track in the first part of digital TRaP ------##

         ## PLEASE NOTED THAT:
         ## The longitude has to multiple by -1 to convert back from Mcidas convention

         # get the 0, 12, and 24 hour forecast positions
         # get the position track for every 12 hour

         for ($i = 0; $i < 6; $i += 2)
         {
            @tempMsg = split(/\s+/, $posLines[$i]);
            # ($hh, $lat, $lon) = @tempMsg;  not work well, since there is space before 0 or 6 hour
            $hh = $tempMsg[-3];
            
            # assign the default values, 04/16/09
            #$lat = $tempMsg[-2];
            #$lon = $tempMsg[-1];
            #$lon *= -1;
            $lat = $defaultLat;
            $lon = $defaultLon;
            #  print "hh is $hh, lat is $lat, lon is $lon\n";
            printf DIGIT1 "%9.2f%9.2f%7.1f\n", $lon, $lat, $hh;
            #printf "%9.2f%9.2f%7.1f\n", $lon, $lat, $hh;
         }

         $lat = 0;
         $lon = 0;
         for ($i = 36; $i <= 72; $i += 12)
         {
            printf DIGIT1 "%9.2f%9.2f%7.1f\n", $lon, $lat, $i;
            #printf "%9.2f%9.2f%7.1f\n", $lon, $lat, $i;
         }

         close (DIGIT1) || die "ERROR: can't close $digitFile1: $!";

         ##----------------------------------------------------------------------##
         ##------------  Get the digital file name for GRACS here    ------------##

         $digit24File = $stormName . '-' . $basin . '.ETRAP.TXT';

      } # End of the if block for "if (($timeIndex == 0) && ($pdIndex == $stIndex))"  

      ####-------  End of geting the PART 1 for the digital eTRaP inside "if" -------#### 


      ####-------------------   Get the final digital eTRaP -------------------####

      system("cat $digitFile1  $digitFile2 > $digitFile"); 

      $success=$?;
      if ($success != 0)
      {
         print "ERROR: Digital eTRaP conversion not success.\n";
         die "eTRaP Files not found, please input the correct eTRaP Area number - exit script.";
      }

      #-----  create an extract copy of 24 hrs digital eTRaP for GRACS users ----#
      # Will check if they need the probability files   
      if (($fcstTime == 24) && ($pdIndex == $stIndex))
      {
         # file name got in the first product right after getting the part1 of the digital file
         ##$digit24File = $stormName . '-' . $basin . '.TXT';
         #$digit24File = $stormName . '-' . $basin . '.ETRAP.TXT';
         system("cp $digitFile $digit24File");
         system("compress -f $digit24File");
         $digit24File = $digit24File . '.Z';
      }
      
      system("mv $digitFile $digitfileName"); 

      #print "The final filename is $digitfileName\n"; 
      #system("pwd");


      # can not use -f here as the GIF will compress as well
      #system("compress -f $fileNamePre*");
      system("compress -f $fileName");
      system("compress -f $digitfileName");

      unlink("$digitFile2") || die "cannot unlink $digitFile2: $!";
   
   } # End of the foreach for the product list
   ##---- End of the FOR loop "for ($pdIndex = $stIndex; $pdIndex < $numOfArea; $pdIndex++)" ----##

   ##---- Write the HTML version of the log file for web server ----##

   $logFile2 = $yyyy . $nameNnb . '.' . $logPattern . '.' . $dateTime . '.' . $fcstTime . ".log";
   $logFile2 = $stPath . $logFile2;
   $htmlLogFile = $yyyy . $nameNnb . '.' . $logPattern . '.' . $dateTime . '.' . $fcstTime . ".html";
   $trapGifsList = $yyyy . $nameNnb . '.' . $trapListPattern . '.' . $dateTime . ".txt";

   if ($fcstTime < 24)
   {
      # call the convert program
      system("$srcDir/log2html.pl $logFile2 $curDir/$htmlLogFile $curDir/$trapGifsList");
   }
   else
   {
       # keep the same log file in web for 24 hours eTRaP
       #system ("cp $logFile2 $htmlLogFile");
       $htmlLogFile = $logFile2; 
   }

   push (@totalLogList_Ori, $logFile2);
   push (@totalLogList, $htmlLogFile);   

   ##---- Write the TRAP Member number Javascript file for web server ----##
   
   #$trapNumfile = $yyyy . $nameNnb . '.' . $trapNumPattern . '.' . $dateTime . '.' . $timePeriod . ".txt";
   $trapNumfile = $yyyy . $nameNnb . '.' . $trapNumPattern . '.' . $dateTime . '.' . $fcstTime . ".js";

   # in case there is binary character, use -a
   $trapNum = `grep -a "$logSearchPattern" $logFile2 |cut -d":" -f2`;
   chomp($trapNum);
   $trapNum = $trapNum + 0;        # to remove the space
   print "TRaP memeber number for $trapNumfile is:$trapNum.\n";

   # Write the file
   #system("echo $trapNum > $trapNumfile");
   open (TRAPNUMJS, ">$trapNumfile") || die "ERROR: can't create $trapNumfile: $!";
   #printf TRAPNUMJS "%s\n", $trapNumJSline;
   $trapNumJSline=`echo "$tempTrapNumJSline" |sed 's/$tempHour/$fcstTime/'`;   
   chomp($trapNumJSline);
   print TRAPNUMJS "$trapNumJSline\n";
   print TRAPNUMJS "   return \"$trapNum\";\n";
   print TRAPNUMJS "}\n";
   close ( TRAPNUMJS) || die "Can't close $trapNumfile: $!";

   push (@totalTRAPNUMList, $trapNumfile);
   #push (@totalLogList, $logFile2);   

   $timeIndex++;

}  ####---- End of the outer FOREACH loop "foreach $prodInfo (@prodsList)" ----#### 

# The part 1 file can only be deleted in the end, as it is used for each products 
unlink("$digitFile1") || die "cannot unlink $digitFile1: $!";



##------------------------------------------------------------------------------##
##------------------ Update the eTRaP set/record log file     ------------------##

print "fcstTimeList:@fcstTimeList\n";

open (LOGFILE, ">>$etrapSetLog") || die " Can't seem to open $etrapSetLog:$!";
#$etrapSetName=$yyyy . $nameNnb . '.' . $dateTime . '.' . $fcstTimeList[0] . '-' . $fcstTimeList[-1];
$etrapSetName=$yyyy . $nameNnb . '.' . $dateTime . '.';

foreach $fcstHh (@fcstTimeList)
{
   $etrapSetName = $etrapSetName . $fcstHh . '-';
}

# Remove the last '-'
if ($etrapSetName =~ /\-$/)
{
    chop($etrapSetName);
}

$createdTime = `date +%Y-%m-%d-%H%M'Z'`;
chomp ($createdTime);
#print LOGFILE "$etrapSetName\n";
printf LOGFILE "%-50s %s\n", $etrapSetName, $createdTime;

close (LOGFILE);

print "!!totalTRAPNUMList is:@totalTRAPNUMList\n";

##------------------------------------------------------------------------------##

##------ Send (SFTP) out the digital and AREA TRaP to FTP server  - Satepsanon ------##

## Will send the file to FTP server(gp16). if the emailList is not empty, then 
## send email to the corresponding people in the following

print " Create sftp script to send the TRaP files to $ftpServer1\n";

# the absolute remote ftp direcotry
$abs_remoteTRAPDir = "/data/ftp/" . $remoteTRAPDir;
$abs_remoteTRAPDir2 = "/data/ftp/" . $remoteTRAPDir2;

open  ( FTPOUT1,  ">$sftpfile1" )  ||  die "ERROR: Can't open $sftpfile1 to write: $!";

   print  FTPOUT1   "# SFTP command file:  Put digit files to FTP server\n";
   print  FTPOUT1   " sftp $ftpServer1 << EOF\n";
   print  FTPOUT1   "cd  $abs_remoteTRAPDir\n";
   foreach  $fileName  ( @totalProdList)
   {
      print  FTPOUT1   "   mput  $fileName*\n";
   }   
   foreach  $fileName2 (@totalLogList_Ori)
   {
      print  FTPOUT1   "   put  $fileName2\n";
   }
   if ( -s $digit24File )
   {
      print  FTPOUT1   "   cd  $abs_remoteTRAPDir2\n";
      print  FTPOUT1   "   put  $digit24File\n";
   }
   print  FTPOUT1   "   bye\n";
   print  FTPOUT1   "EOF\n";

close ( FTPOUT1 ) || die "Can't close $sftpfile: $!";
chmod  (0777,  $sftpfile1) ;

# Run the sftp file to get the files from NASA
$enddate=`date`;
print " Start  sftp digit files to $ftpServer1 at $enddate \n";
open ( SFTPOUT,  " $sftpfile1  | " ) || die "ERROR: Can't open $sftpfile1 to run: $!";

@sftpOutput = <SFTPOUT>;

close (SFTPOUT ) || die "Can't close $sftpfile1: $!";


##--------------------------------------------------------------------------------------------##

##------------------------ Send the eTRaP GIF files to SAB Web server  ------------------------##

# Get the TraP member GIF files list
#open(TRAPLIST, "< $curDir/$trapGifsList") || die "Can not open $curDir/$trapGifsList for reading: $!";
open(TRAPLIST, "< $curDir/$trapGifsList") || print "ERROR:Can not open $curDir/$trapGifsList for reading: $!";
@trapList = <TRAPLIST>;
close(TRAPLIST);

unlink("$curDir/$trapGifsList");

# Send to GP5
open  ( FTPOUT2,  ">$sftpfile2" )  ||  die "ERROR: Can't open $sftpfile2 to write: $!";

   print  FTPOUT2   "# SFTP command file:  Put GIF files to Web server\n";
   print  FTPOUT2   " sftp $webServer1 << EOF\n";
   print  FTPOUT2   "   cd  $remoteETrapDir\n";
   foreach  $fileName  ( @totalProdList)
   {
      print  FTPOUT2   "   put  $fileName.GIF\n";
   }
   foreach  $fileName1 (@totalTRAPNUMList)
   {
      print  FTPOUT2   "   put  $fileName1\n";
   }
   foreach  $fileName2 (@totalLogList)
   {
      print  FTPOUT2   "   put  $fileName2\n";
   }

   # send the TraP member GIF files 
   print  FTPOUT2   "   cd  $remoteTrapGifDir\n";
   foreach  $trapName  ( @trapList)
   {
      chomp($trapName);
      print  FTPOUT2   "   put  $trapName\n";
   }

   print  FTPOUT2   "   bye\n";
   print  FTPOUT2   "EOF\n";

close ( FTPOUT2 ) || die "Can't close $sftpfile2: $!";
chmod  (0777,  $sftpfile2) ;

# Run the sftp file to get the files from NASA
$enddate=`date`;
print " Start  sftp files to $webServer1 at $enddate \n";
open ( SFTPOUT,  " $sftpfile2 | " ) || die "ERROR: Can't open $sftpfile2 to run: $!";

@sftpOutput = <SFTPOUT>;

close (SFTPOUT ) || die "Can't close $sftpfile2: $!";


# Send to GP50

open  ( FTPOUT3,  ">$sftpfile3" )  ||  die "ERROR: Can't open $sftpfile3 to write: $!";

   print  FTPOUT3   "# SFTP command file:  Put GIF files to Web server\n";
   print  FTPOUT3   " sftp $webServer2 << EOF\n";
   print  FTPOUT3   "   cd  $remoteETrapDir\n";
   foreach  $fileName  ( @totalProdList)
   {
      print  FTPOUT3   "   put  $fileName.GIF\n";
   }
   foreach  $fileName1 (@totalTRAPNUMList)
   {
      print  FTPOUT3   "   put  $fileName1\n";
   }
   foreach  $fileName2 (@totalLogList)
   {
      print  FTPOUT3   "   put  $fileName2\n";
   }

   # send the TraP member GIF files
   print  FTPOUT3   "   cd  $remoteTrapGifDir\n";
   foreach  $trapName  ( @trapList)
   {
      chomp($trapName);
      print  FTPOUT3   "   put  $trapName\n";
   }

   print  FTPOUT3   "   bye\n";
   print  FTPOUT3   "EOF\n";

close ( FTPOUT3 ) || die "Can't close $sftpfile3: $!";
chmod  (0777,  $sftpfile3) ;

# Run the sftp file to get the files from NASA
$enddate=`date`;

# No connection to GP50 now, so turn off the following as of 06/17/09
#print " Start  sftp files to $webServer2 at $enddate \n";
#open ( SFTPOUT,  " $sftpfile3 | " ) || die "ERROR: Can't open $sftpfile3 to run: $!";

#@sftpOutput = <SFTPOUT>;

#close (SFTPOUT ) || die "Can't close $sftpfile3: $!";


##--------------------------------------------------------------------------------------------##
##------------------------ Call the remote command in the Web server  ------------------------##

if (!( -s $firstGIF1))
{
   print "ERROR: $firstGIF1 is not exist.\n";
   print "The remote command $remoteCM will not be called.\n";
}
  
# if ($firstFileName ne "")
# etrapSetName Format: 2009BIJLI.04170600.00-06-12
elsif ($etrapSetName ne "")
{
   print "\nRun the following remote command in the Web server: ";
   #print "ssh $webServer2 $remoteCM  $firstFileName $basin1 $basin2\n";
   ##print "ssh $webServer1 $remoteCM  $etrapSetName $firstGIF $basin1 $basin2\n";
   ##system ("ssh $webServer1 $remoteCM  $etrapSetName $firstGIF $basin1 $basin2");

   print "\nRun the following remote command in the $webServer2.\n.";
   print "ssh $webServer2 $remoteCM  $etrapSetName $firstGIF $basin1 $basin2\n";
   system ("ssh $webServer2 $remoteCM  $etrapSetName $firstGIF $basin1 $basin2");
}
else
{
   print "ERROR:Fail to get the eTRaP set name.\n";
}


##-----------------------------------------------------------------------------------##

##------ Send (SFTP) out the digital and AREA TRaP to SAB Data server - Sabna4 ------##

#print " Create sftp script to send the eTRaP files to $sabDataServer\n";

#system ("ssh $sabDataServer $sabDataRemoteCM  $year $stormName");

# the absolute remote ftp direcotry
#$abs_remoteTRAPDir = "$sabDataTrapDir/$year/$stormName";

#open  ( FTPOUT4,  ">$sftpfile4" )  ||  die "ERROR: Can't open $sftpfile4 to write: $!";

#   print  FTPOUT4   "# SFTP command file:  Put digit files to FTP server\n";
#   print  FTPOUT4   " sftp $sabDataServer << EOF\n";
#   print  FTPOUT4   "   cd  $abs_remoteTRAPDir\n";
#   foreach  $fileName  ( @totalProdList)
#   {
#      print  FTPOUT4   "   mput  $fileName*\n";
#   }
#   foreach  $fileName2 (@totalLogList_Ori)
#   {
#      print  FTPOUT4   "   put  $fileName2\n";
#   }

#   print  FTPOUT4   "   bye\n";
#   print  FTPOUT4   "EOF\n";

#close ( FTPOUT4 ) || die "Can't close $sftpfile4: $!";
#chmod  (0777,  $sftpfile4) ;

# Run the sftp file to get the files from NASA
$enddate=`date`;
##print " Start  sftp digit files to $sabDataServer at $enddate \n";
##open ( SFTPOUT,  " $sftpfile4  | " ) || die "ERROR: Can't open $sftpfile4 to run: $!";

##@sftpOutput = <SFTPOUT>;
##close (SFTPOUT ) || die "Can't close $sftpfile4: $!";

##--------------------------------------------------------------------------------------------##

##------------- Send the eTRaP GIF/ASCII files to SATEPS Server for GEONETCast ---------------##

#system ("scp $fileNamePre*GIF $satepServer:$remoteGeonetDir > /tmp/scp.out");

print "Send the eTRaP GIF and Text files to SATEPSDist1 Server for GEONETCast.\n";

# Send to Satepsdist1 
open  ( FTPOUT5,  ">$sftpfile5" )  ||  die "ERROR: Can't open $sftpfile5 to write: $!";

   print  FTPOUT5   "# SFTP command file:  Put GIF files to satepsdist1 server\n";
   print  FTPOUT5   " sftp $satepServer << EOF\n";
   print  FTPOUT5   "cd  $remoteGeonetDir\n";
   foreach  $fileName  ( @totalProdList)
   {
      $baseFileName = `basename $fileName`;
      chomp($baseFileName);
      $geonetFileName1 = $geonetFilePrefix . $baseFileName . ".GIF";
      $geonetFileName2 = $geonetFilePrefix . $baseFileName . ".TXT.Z";
      print  FTPOUT5   "   put  $fileName.GIF $geonetFileName1\n";
      print  FTPOUT5   "   put  $fileName.TXT.Z $geonetFileName2\n";
   }

   print  FTPOUT5   "   bye\n";
   print  FTPOUT5   "EOF\n";

close ( FTPOUT5 ) || die "Can't close $sftpfile5: $!";
chmod  (0777,  $sftpfile5) ;

# Run the sftp file to get the files from NASA
$enddate=`date`;
##print " Start  sftp files to $satepServer at $enddate \n";
##open ( SFTPOUT,  " $sftpfile5 | " ) || die "ERROR: Can't open $sftpfile5 to run: $!";

##@sftpOutput = <SFTPOUT>;

##close (SFTPOUT ) || die "Can't close $sftpfile5: $!";

##--------------------------------------------------------------------------------------------##

##------------- Send the eTRaP TXT files to the DDS ---------------##

#system ("scp $fileNamePre*GIF $satepServer:$remoteGeonetDir > /tmp/scp.out");

print "Send the eTRaP TXT files to the DDS.\n";

# Send to DDS
open  ( FTPOUT6,  ">$ftpfile6" )  ||  die "ERROR: Can't open $ftpfile6 to write: $!";

   print  FTPOUT6   "# FTP command file:  Put TXT files to DDS server\n";
   print  FTPOUT6   " ftp $DDSServer << END\n";
   print  FTPOUT6   "cd  $NCDCDir\n";
   foreach  $fileName  ( @totalProdList)
   {
       @fileFields = split(/\//,$fileName);
       $fileName_only = $fileFields[-1];
       $tFile = $fileName_only . ".TXT.Z";
       print FTPOUT6 "TXTFile=$tFile\n";
       print FTPOUT6 "put $fileName.TXT.Z  xfr_$tFile\n";
       print FTPOUT6 "rename xfr_$tFile $tFile\n";

   }

   print  FTPOUT6   "bye \n";
   print  FTPOUT6   "END \n";

close ( FTPOUT6 ) || die "Can't close $ftpfile6: $!";
chmod  (0777,  $ftpfile6) ;

# Run the ftp file to get the TXT files to the DDS
$enddate=`date`;
print " Start  ftp files to $DDSServer at $enddate \n";
open ( SFTPOUT,  " $sftpfile6 | " ) || die "ERROR: Can't open $ftpfile6 to run: $!";

@sftpOutput = <SFTPOUT>;

close (SFTPOUT ) || die "Can't close $ftpfile6: $!";

##--------------------------------------------------------------------------------------------##

##------ Write the TRaP set name and the email List to the TRaPList file for the future ------##


## check if the email is empty, 
  
#$emailStr = "";
#for ($j = 0; $j < $#emailList + 1; $j++)
#{
#   $emailStr .= $emailList[$j];
#}

#if ($emailStr ne "")
#{
#   system("$srcDir/writeList.pl $fileNamePre $basinArea @emailList");
#}
#else
#{
#   print "WARNING!!! The email list is EMPTY, don't write to text file.\n";
#   print "The email list is EMPTY, don't write to text file.\n";
#}



# move the TRaP files sent to other directory, will store them for 24 hour

foreach  $fileName  ( @totalProdList)
{
   system("mv $fileName.TXT* $digitTRAP_sent_Dir");
}
foreach  $fileName1  ( @totalTRAPNUMList)
{
   # Or remove it
   system("mv $fileName1 $digitTRAP_sent_Dir");
}

foreach  $fileName2  ( @totalLogList)
{
   if ($fileName2 =~ /\.html$/)
   {
      system("mv $fileName2 $digitTRAP_sent_Dir");
   }
}

# Delete the file for GRACE after it is sent out to FTP server
if ( -s $digit24File )
{
   unlink("$digit24File");
}


$enddate = `date`;
print "End of the program $0 for $stormName at $enddate.\n\n";

# record to the log file
#`echo "start $0 at $startdate" | cat >> $logFile`;
#`echo "+++++++++++++++ STDERR Log +++++++++++++++" |cat >> $logFile`;
#`cat $stderrLog >> $logFile`;
#`echo "+++++++++++++++ STDOUT Log +++++++++++++++" |cat >> $logFile`;
#`cat $stdoutLog >> $logFile`;



####------------------------------------------------------------------------------------------------####

