#!/usr/bin/perl -w

## =====================================================================
## Name:        cleanETRAP.pl  
##                  
## Author:      Jianbin Yang/PSGS   02/09/2009
## Language:    Perl
## Purpose:     This program clean the ETRaP data for the storm 
##		8 days after it was over .  It also send user alert 
##		for saving the data one and 6 day after the storm is over. 
##              The program clean the eTRaP log files as well.
##                         
## Adapted from program "cleanTRAP.pl"
##                 
## Modified: 02/09/2009
##               Clean the inacitve storm entry for the active storm list
##
## Adapted : 06/10/2009
## Last Modified: 04/21/2010
##               Clean the NCDC tar file
##
## =====================================================================

use Class::Struct;
use File::Path;           # for "rmtree"

####---------------------- Define  and delcare ------------------------####

##------------------------   Define structures   ------------------------##

# structure to store the information to call the program on web server
struct StormInfo =>
{
   stormName => '$',        
   year      => '$',        
   basinArea => '@',        
};


# Define the directories and files

$HOME = $ENV{"HOME"};

$etrapDataDir = '/data/TRAP/ETRAP';          # the actual TRaP directory is '/data/TRAP/ETRAP/$year/$stormName'
$savedListPrefix = "etrapSaved";
$tempOut = '/tmp/etraptarout';

$digitEtrapDataDir = "$etrapDataDir/digitETRAP";                     # the digital eTRaP directory in local Server
$NCDCDataDir = "$etrapDataDir/NCDC_Arch";                            # the directory for NCDC data 


## initialize the arrays which alert the user to save 7 days or 2 days 
#  before they are deleted.  storms in alertList7 will be deleted 7 days
#  after the alert, and storms in alertList2 will be deleted 2 days after the alert.


#@deleteStormList = ();  # array to store the struct StormInfo for the web server

$deleteDay = 3.0; 	# delete the entry (for digital)which is older than this day

#@stormCountList = ();   # array to store the struct StormCount

@alertList7 = ();
@alertList2 = ();

####------------------------------------------------------------------------------####

####--------------------------------- Start --------------------------------------####


$startdate=`date`;
print "Start  $0 at $startdate \n";

chdir("$etrapDataDir") || die "Cannot change to directory $etrapDataDir: $!";
print  "ETRAP Data directory is:";
system("pwd");

#------------------- Clean up the analog eTRaP data -------------------#

$in7days = `date --date '7 days' +%D`;
chop($in7days);
$in2days = `date --date '2 days' +%D`;
chop($in2days);

$year = `date +%Y`;
chop($year);
$oldYear = $year - 1;

if (-d $oldYear)
{
   purgeData($oldYear);
}

if (-d $year)
{
   purgeData($year);
}

print "the alert list are @alertList7\n @alertList2\n";


#------------------- Clean up the digital eTRaP data -------------------#

chdir("$digitEtrapDataDir") || die "Cannot change to directory $digitEtrapDataDir: $!";
print  "ETRAP digital Data directory is:";
system("pwd");

@fileList = `ls`;

foreach $file (@fileList)
{
   chop($file);

   if (-f $file)
   {
      if (-M $file > $deleteDay)
      {
         unlink("$file") || die "cannot unlink $file: $!";
         print "$file is deleted.\n";
      }
   }
}  # End of the Foreach loop

#------------------- Clean up the NCDC eTRaP data -------------------#

chdir("$NCDCDataDir") || die "Cannot change to directory $NCDCDataDir: $!";
print  "ETRAP NCDC Data directory is:";
system("pwd");

@fileList = `ls`;

foreach $file (@fileList)
{
   chop($file);

   if (-f $file)
   {
      if (-M $file > $deleteDay)
      {
         unlink("$file") || die "cannot unlink $file: $!";
         print "$file is deleted.\n";
      }
   }
}  # End of the Foreach loop

print "End of clean old eTRaP data\n";

############  Subroutine "purgeData" ###############
########################################################

sub purgeData
{
   my($yyyy) = @_;

   chdir($yyyy) || die "Cannot change to directory $yyyy: $!";
   print  "Current eTRAP Data directory is:";
   system("pwd");

   my(@stormList) = `ls`;
   my($wholeName, $tarName, $count);
   my($savedList) = $savedListPrefix . $yyyy;
   my(@lsInfoList) = ();

   foreach $storm (@stormList)
   {
      chop($storm);
      
      # if ($storm =~ /\.tar$/)
     # {
     #    next;
     # }
   
      # will skip the non-directory files under the year      
      if (-d $storm)
      {
          chdir($storm) || die "Cannot change to directory $storm: $!";
          system("pwd");
      }
      else 
      {
          next;
      }
      

      # get the last eTRAP created in this directory, and check its modification time
      $lastETRAP = `ls -tr|tail -1`;

      chop($lastETRAP);
      
      ## check whether the directory is for a storm or not
      # if not a storm, skip it
        
      $count = 0;
      while (($lastETRAP !~ /$storm\./) && ($count < 2))
      {
         $lastETRAP = `ls -tr |grep '$storm.'|tail -1`;
         print "count 2: lastETRAP is $lastETRAP\n";
         chop($lastETRAP);
         $count++;
      }

      if ($count > 1)
      {
         print "the current directory $storm is not for a storm\n";
         chdir("..") || die "Cannot change to upper directory: $!";
         next;
      }

      print "the last ETRAP for the storm is **$lastETRAP**\n";
 
      $tarName = $storm . ".ETRAP." . "tar";
      $tarGzName = $tarName . '.' . "gz";
      $wholeName = $yyyy . '/' . $storm;

      # delete the whole storm data 9 days after the storm was over
      if (-M $lastETRAP > 14)
      {
         print "delete the whole storm directory $storm\n";
         rmtree("../$storm") || die "cannot rmtree $storm: $!";
         if ( -s "../$tarGzName" )
         {
            print "delete the tar file $tarGzName\n";
            unlink("../$tarGzName") || die "cannot unlink $tarGzName: $!";
         }
      }
      elsif ((-M $lastETRAP > 2.0) && (-M $lastETRAP <= 3.0))
      {
         print "$storm is over, will be purged in 7 days, please save it before it is removed\n";
         print "tar file: $tarName is created for it.\n";
         chdir("..") || die "Cannot change to upper directory: $!";         
         #system("tar cvfP '$tarName' '$storm' > $tempOut");
         system("tar cvf '$tarName' '$storm' > $tempOut");
         system("gzip -f '$tarName' ");
         chdir("$storm") || die "Cannot change to $storm directory: $!";
         push(@alertList7, $wholeName );   

         # get the ls information for every storm to be deleted
         # and later will append all into the saved list file

         $lsInfo = `ls -l ../$tarName'.gz'`;
         @lsInfos = split(/\s+/, $lsInfo);

         $lsInfo = $lsInfos[4] . " \t" . $lsInfos[5] . " " . $lsInfos[6] . " " .
                   $lsInfos[7] . "\t" . $tarName . ".gz";

         push(@lsInfoList, $lsInfo);

      }
      elsif ((-M $lastETRAP > 7.0) && (-M $lastETRAP <= 8.0))
      {
         print "$storm is over, will be purged in 2 days, please save it before it is removed\n";

         # if the tar.gz file is not there, assumed it had been saved by analyst. 
         # So won't issue 2nd warning 
         if ( -s "../$tarGzName" )
         {
            print "$tarGzName is still there, put $storm into the 2nd warning list.\n"; 
            push(@alertList2 , $wholeName );
         }
      }

      # go back to the year directory for next storm
      chdir("..") || die "Cannot change to upper directory: $!";
   }

   # append the ls information for the saved storms to the saved list
   if (@lsInfoList != 0)
   {
      # print the header of the saved list file if the file is being created
      if (-s $savedList)
      {
         $listHeader = "";
      }
      else
      {
         $listHeader = "size\t\t time\t\tname\n";
      }

      open (LISTFILE, ">> $savedList") || die " Can't seem to open $savedList:$!";
      print LISTFILE $listHeader;

      foreach $lsInfo (@lsInfoList)
      {
         print LISTFILE "$lsInfo\n";
      }

      close (LISTFILE);
   }

   # get the current exist storms in that year to see if it is empty in the year
   @stormList = `ls`;
   
   # go back to the "TRAP" directory
   chdir("..") || die "Cannot change to upper directory: $!";  # change to the upper year directory

   if ((@stormList == 0) && ($yyyy == $oldYear))
   {
      print "There is no storm in last year $yyyy, delete the directory $yyyy\n\n";
      rmdir($yyyy)|| die "Cannot rmdir $yyyy: $!";
   }
}

############  Subroutine "isElement" ###############
########################################################
## if it is one of the element of the array the index is return,
## otherwise return -1

sub isElement
{
   # my($element, @array) = @_;
   my($array, $element) = @_;
   my($foundIndex) = -1;

   # must specify $i as local variable, otherwise will conflict with $i in
   # the calling phase (main function)
   my($i);

   # can also write as @{$array}
   ### print " the array is @array\n";

   for ($i= 0; $i < @$array; $i++)
   {
      ### print "inside the isElement, the element is @$array[$i], i is $i\n";
      if (@$array[$i] eq $element)
      {
         $foundIndex = $i;
         last;
      }
   }

   return $foundIndex;
}

