#!/usr/bin/perl -w

## =====================================================================
## Name:        createDigEtrapDir.pl
##              (Adapted from createDigDir.pl)
##
## Author:      Jianbin Yang/Perot System   04/03/2007
## Language:    Perl
## Purpose:     Create the directories structure on the SAB data server to save digital eTRaP
##              This program runs on Sabna4
##
## Usage:       createDigEtrapDir.pl <year> <storm name>
##
## Modified:   06/10/2009
##
## Last Modified: 01/26/2010
##                Add error checking to detect the problem of external drive 
##                for the monitoring tool
##
## =====================================================================

# Define the directories and files

# $webHomeDir = "/home/sabweb/TRAP";
$etrapDataDir = '/data/ext/SABTrap1/ETRAP/digitETRAP';          # the actual TRaP directory is '/data/TRAP/$year/$stormNameNNB'

##--------------------------------- Start --------------------------------------##

$startdate=`date`;
print "Start  $0 at $startdate \n";

chdir("$etrapDataDir") || die "ERROR:Cannot change to directory $etrapDataDir";
print  "eTRaP Data directory is:";
system("pwd");

# remove the path from the command name

@comandFields = split(/\//, $0);
$commandName = $comandFields[-1];

if (@ARGV < 2)
{
   print "Usage: $commandName <year> <storm name>\n";
   die "ERROR:Not enough arguments - exit script";
}
else
{
   $year = $ARGV[0];
   $stormName = $ARGV[1];
}


##------------ Create the yyyy/stormName directories if not exist -------------##

if (!(-d $year))
{
   print "Create the directory '$year' for the current year.\n";
   mkdir($year,0755) || die "cannot mkdir $year: $!";
}

chdir("$year") || die "Cannot change to directory $year";

if (!(-d $stormName))
{
   print "Create the directory '$stormName' for the new storm.\n";
   mkdir($stormName,0755) || die "cannot mkdir $stormName: $!";
}

