#!/usr/bin/perl -w

## =====================================================================
## Name:        createETRaPDir.pl
##              (Adapted from createTRaPDir.pl in April 2009)
##                
## Author:      Jianbin Yang/QSS   08/28/2006
## Language:    Perl
## Purpose:     Create the directories structure on the SAB Web server for TRaP display           
##                  
## Usage:       createETRaPDir.pl <Name of the eTRaP set name> <first GIF file > <basin1> <basin2>
##
##              Format of <Name of the eTRaP set> is: 
##                     $YYYY$StormName.$MMDDhhmm.$hh1-$hh2...  (2009BIJLI.04170600.00-06-12)
##              First GIF file: for storm archive to display
##   
## NOTE:        This program will run on the Web server remotely
##
## Modified:   09/21/2006
##              Add the code to send the TRaP GIF file to the destination 
##		direcotry
##
## Modified:   10/05/2006
##              Change the agency directory name to be consistent with the ones on FTP server
##
## Modified:   10/17/2006
##              Call the program "updateBasinFile.pl" when it is the first storm for a basin
##
## Modified: 10/18/2006
##              Call the bash program to generate some TRaP web page
##
## Modified: 10/26/2006
##              Create the current html to include the agency html
##
## Modified: 03/01/2007
##		Handle the case when the name is LOW for Australian area, as it may exist before
##
## Modified: 03/07/2007
##              Force the FKAU TRaP to Appropriate agent, other than determine base on the station (Darwin)
##
## Modified: 04/04/2007
##		Month is added to the TRaP name for the rain rate time, so some modification need to be made here
##
## Modified: 06/25/2007
##		Handle the case when the storm becomes active again.  For example, 03B in NorthIndian 
##		after inactive on 06/21/07, get back on 06/25/07
##
## Modified: 07/03/2007
##		Add the New Delhi and New Zealand bulletin
##		Remove some code which copied unused text files for web page
##
## Modified: 10/29/2007
##		Add to send the AMSR-E TRaP to web page
##
## Last Modified: 12/26/2008
##              Add the Jakarta agent
##
## ADAPTED for eTRaP: 04/15/09 - 06/02/09
##
## Last Modified: 09/25/2009
##              Add the distribution of TRaP member GIF files
##              Changed the Unavailable term to "No Recent TRaP or Not formed..."
##
##                01/26/2010
##              Remove the year directory for basin file
##
## =====================================================================

# Define the directories and files

$HOME = $ENV{"HOME"};
$webHomeDir = "$HOME/TRAP/ETRAP";

$etrapDataDir = "/data/web/mirror/www.ssd.noaa.gov/htdocs/PS/TROP/DATA/ETRAP";

# Directory which temporarily hold the eTRaP image files, mush match with the area2digitetrap.pl
$etrapImgDir = "$etrapDataDir/images";

# Directory which temporarily hold the TRaP member image files
# pattern for TRaP list file and also the sub-directory under the storm
# must match area2digitetrap.pl and log2html.pl
$trapListPattern = "TRAPGIFS";                 
$trapImgDir = "$etrapDataDir/$trapListPattern";

#$newTrapList = "$trapDataDir/newTRaPList.log";

$basinCM = "$webHomeDir/updateBasinFile_etrap.pl";
$textDir = "$webHomeDir/txt";

# the script to generate eTRaP archive pages
$genETRaPPageCM = "$webHomeDir/scripts/gen-etrap-archive-pages.bash";

# the script to update the year archive pages when crossing basins
$moveBasinCM = "$webHomeDir/scripts/moveBasin.bash";

# basinBlankFile will be $basin + $basinBlankFileSuffix, located in the $textDir
$basinBlankFileSuffix = ".txt.blank";

# mode for the updateBasinFile.pl command line input
$createMode = 1;
$updateMode = 2;
$deleteMode = 3;

# for the Java script template files
$stNameTemplateFile = "$textDir/stName-template.js";
$dateTimeTemplateFile = "$textDir/dateTime-template.js";

# For html template files
# For storm archive
$stListTemplateFile = "$textDir/stListBlank-template.txt";
$stLineTemplateFile = "$textDir/stLine-template.txt";

## For one line of the basin file in the left of the page
#$basinLnTemplateFile = "$textDir/basinLine-template.txt";


# define some constants list in the template file
$tempStName = "STORMNAME";
$tempDate = "DATE";                   # Actually in format of YYYY-MM-DD
$tempDateTime = "YYYYMMDDHH";         # Actually in format of YYYY-MM-DD-HH   
$tempHour = "HH";                     # For the forecast hour  
$tempImgName = "IMAGENAME";

$stNoformPattern = "Not Formed or Renamed";
#$emptyPattern = "UNAVAILABLE";
$emptyPattern = "No Recent TRaP (<2)";

$tempDtTmFunc = "getDateTime";        # Must match the one in $dateTimeTemplateFile
$tempStDtTmFunc = "getStartDateTime";     

## For basin file, moved to the updateBasin program
#$tempNoSt = "noStorms";
#$tempHaveSt= "stormPresent";

# Files need to be copied to other directories
$nameFile = "getCurrent.js";                            # will be in  the ETRAP directory
##$dash = " -";
$dateTimeFile = "getDateTime.js";
#$curDateTimeFile = "getCurDateTime.js";
$curDateTimeFile = "getCurrentDateTime.js";             # will be in  the ETRAP directory
# $curStDateTimeFile = "getCurrentStartDateTime.js";      # will be in  the ETRAP directory
$stDateTimeFile = "getStDateTime.js";    # Only create when it is the first etrap for the storm.

$stormListFile = "storm-list.txt";



##----------------------------- Start the Program -----------------------------##

$startdate=`date`;
chomp($startdate);

print "\n-------------------- Start  $0 at $startdate --------------------\n";

chdir("$etrapDataDir") || die "Cannot change to directory $etrapDataDir";
print  "eTRaP Data directory is:";
system("pwd");

# remove the path from the command name

@comandFields = split(/\//, $0);
$commandName = $comandFields[-1];

# if no argument input, exit the program

# if ($ARGV[0] eq "")
if (@ARGV < 4)
{
   print "Usage: $commandName <Name of the eTRaP set name> <first GIF file > <basin1> <basin2>\n";
   print "Format of <Name of the eTRaP set> is:$YYYY$StormName.$MMDDhhmm.$hh1-$hh2...\n";
   die "ERROR: Not enough arguments - exit script";
}

$etrapSetName = $ARGV[0];
$firstGIF = $ARGV[1];
$oldBasin = $ARGV[2];
$basin = $ARGV[3];

print "etrapSetName is $etrapSetName, first GIF is $firstGIF, old basin is $oldBasin, basin is $basin.\n";

# get the directories name of the specific eTRaP set

#@splitMsg = split(/\./, $etrapSetName);
@splitMsg = split(/\./, $firstGIF);
$splitMsgLen = @splitMsg;

# printf ("the number of splitMsg is %d.\n",($#splitMsg + 1));

if ($splitMsgLen < 5)
{
   print "Format of <first GIF file> is: 
          \rYYYYnameNNB.ProdType.MMDDHHMM.hh.GIF)\n\n";
   die "ERROR: Input eTRaP GIF is not long enough or not correct - exit";
}

# system("echo $trapSetName >> $newTrapList");

$dirMsg = $splitMsg[0];
$year = substr($dirMsg, 0, 4);
$nameNnb = substr($dirMsg,4);
# $outputDir = "$trapDataDir/$year/$nameNnb";
$stormName = $nameNnb;
$dateTime = $splitMsg[2];
$mon = substr($dateTime,0, 2);
$day = substr($dateTime,2, 2);
$hour = substr($dateTime,4, 2);
#$dateHour = substr($dateTime,0, -2);
#$timePeriod = $splitMsg[3];

$dateTimeStr = $year . '-' . $mon . '-' . $day . '-' . $hour;
$dateStr = $year . '-' . $mon . '-' . $day;
 
$basinFile = $basin . ".txt";

#$basinFile = "$etrapDataDir/$year/$basinFile";
$basinFile = "$etrapDataDir/$basinFile";
$blankFile = "$textDir/$basin" . $basinBlankFileSuffix;


print "StormName is $stormName, year is $year, dateTime is $dateTime: $mon-$day-$hour, basin is $basin  \n";

print "dirMsg is $dirMsg, year is $year, nameNNB is $nameNnb.\n";


chdir("$etrapDataDir") || die "Cannot change to directory $trapDataDir";
print  "eTRaP Data directory is:";
system("pwd");


##------------ Create the yyyy/basin/stormName/agency/sensor directories if not exist -------------##

if (!(-d $year))
{
   print "Create the directory '$year' for the current year.\n";
   mkdir($year,0755) || die "cannot mkdir $year: $!";
}

chdir("$year") || die "Cannot change to directory $year";

if (!(-d $basin))
{
   $isBasinExist = 0;
   print "Create the directory '$basin'.\n";
   mkdir($basin,0755) || die "cannot mkdir $basin: $!";
   print "Call the program $basinCM to create the HTML text for basin $basin.\n";

   # Will check if need to add the condition of $isCrossbasin != 1
   system ("$basinCM $year $stormName $basin $createMode");

   $success=$?;
   if ($success != 0)
   {
      print "ERROR: Failed to run $basinCM.\n";
   }

   print "End of Calling the program $basinCM to create the basin HTML text.\n";
}
else
{
  $isBasinExist = 1;
}

# check for the basin 1 & 2
if ($oldBasin ne $basin)
{
   if (-d "$oldBasin/$stormName")
   {
      $isCrossBasin = 1;
      print "The storm cross basins! Move the storm $stormName directory from $oldBasin to $basin.\n";
      system("mv $oldBasin/$stormName $basin/");
      $success=$?;
      if ($success != 0)
      {
         print "ERROR: Failed to Move the storm $stormName directory from $oldBasin to $basin.\n";
      }

      print "Run this: $basinCM $year $stormName $oldBasin $deleteMode\n";
      system ("$basinCM $year $stormName $oldBasin $deleteMode");
      $success=$?;
      if ($success != 0)
      {
         print "ERROR: Failed to run $basinCM.\n";
      }

      # move the entry from oldbasin to newBasin in year archive, Call some program in the following
      print "Run this: $moveBasinCM $year $stormName $oldBasin $basin\n";
      system ("$moveBasinCM $year $stormName $oldBasin $basin");
      $success=$?;
      if ($success != 0)
      {
         print "ERROR: Failed to run $moveBasinCM.\n";
      }
   }
   else
   {
      print "ERROR: No storm $stormName under the previous basin $oldBasin, please check if it does cross basins!\n";
      $isCrossBasin = 0;
   }
}
else
{
   $isCrossBasin = 0;
}

chdir("$basin") || die "Cannot change to directory $basin";


if (!(-d $stormName))
{
   if ($isCrossBasin == 0)
   {
      $isFirstTRaP = 1;
   }
   else
   {
      $isFirstTRaP = 0;
   }
   print "Create the directory '$stormName' for the new storm.\n";
   mkdir($stormName,0755) || die "cannot mkdir $stormName: $!";
   if ($isBasinExist == 1)
   {
      print "Call the program $basinCM to update the basin HTML text for new storm $stormName.\n";

      # Noted that in the program "basinCM", it may change the mode to create mode 
      # if there is no storm entry in the basin file

      system ("$basinCM $year $stormName $basin $updateMode");
      $success=$?;
      if ($success != 0)
      {
         print "ERROR: Failed to run $basinCM.\n";
      }
      print "End of Calling the program $basinCM to update the basin HTML text.\n";
   }
}
else
{
  #PAT attention that if isCrossBasin=1, the year archive is updated to include that entry, 04/24
  $isFirstTRaP = 0;

  # check if there is anything in the storm directory
  @tempList = `ls $stormName/$year$stormName*GIF`;
  $tempFileNum = @tempList;

  # 03B became active again after 3 days' quiet in NorthIndian on /06/25/07
     
  $sizeBasin = -s $basinFile;
  $sizeBlank = -s $blankFile;
  print "sizeBasin is $sizeBasin, sizeBlank is $sizeBlank\n";
  # if ($sizeBasin <= $sizeBlank )
  if ( ((-s $basinFile) == (-s $blankFile)) || ($sizeBasin eq "") )
  {
     print "\nStorm $stormName exist before, but the $basinFile is blank.\n";
     print "There are total $tempFileNum GIF files under the storm directory.\n\n";
     print "Call the program $basinCM to create the HTML text for basin $basin.\n";
     system ("$basinCM $year $stormName $basin $createMode");
     $success=$?;
     if ($success != 0)
     {
        print "ERROR: Failed to run $basinCM.\n";
     }
     print "End of Calling the program $basinCM to create the basin HTML text.\n";

     # check if there is anything in the storm directory
     # May need to check if "$stDateTimeFile" is not exist, 04/27
     if ($tempFileNum <= 1)
     {
        $isFirstTRaP = 1;
     }
     elsif (!(-s "$stormName/$stDateTimeFile"))
     {
        $isFirstTRaP = 1;
     }

  }
  else
  {
     # print "Storm $stormName exist before, the $basinFile is not blank.\n";
     $isNew = `grep -i $stormName $basinFile`; 
        
     if ($isNew eq "")
     {
        print "\nStorm $stormName exist before, but the storm is not in the $basinFile, so storm is new.\n";
        print "There are total $tempFileNum GIF files under the storm directory.\n\n";
        print "Call the program $basinCM to update the basin HTML text for new storm $stormName.\n";
        system ("$basinCM $year $stormName $basin $updateMode");
        $success=$?;
        if ($success != 0)
        {
           print "ERROR: Failed to run $basinCM.\n";
        }
        print "End of Calling the program $basinCM to update the basin HTML text.\n"; 
  
        # check if there is anything in the storm directory
        if ($tempFileNum <= 1)
        {
           $isFirstTRaP = 1;
        }
        elsif (!(-s "$stormName/$stDateTimeFile"))
        {
           $isFirstTRaP = 1;
        }
             
     }
  }
}

chdir("$stormName") || die "Cannot change to directory $stormName";

print  "Current Data directory is:";
system("pwd");

if (!(-d $trapListPattern))
{
   print "Create the sub directory '$trapListPattern' for the TRaP member GIFs.\n";
   mkdir($trapListPattern,0755) || die "cannot mkdir $trapListPattern: $!";
}


##-------------- Send the eTRaP data to destination directory --------------##

$destDir = `pwd`;
chomp($destDir);

print "destDir is $destDir\n";

chdir() || die "Cannot change to home directory";
print  "Current directory is:";
system("pwd");

print "Move the eTRaP GIF file to $destDir\n";
system("mv $etrapImgDir/$dirMsg*$dateTime* $destDir");
$success=$?;
if ($success != 0)
{
   print "WARNING: Failed to move the eTRaP GIF file to storm directory.\n";
}

print "Move the TRaP member GIF files to $destDir/$trapListPattern\n";
system("mv $trapImgDir/*$stormName*.GIF $destDir/$trapListPattern");
$success=$?;
if ($success != 0)
{
   print "WARNING: Failed to move the TRaP member GIF file to storm directory.\n";
}


#print "Update the log file $newTrapList\n";
#system("echo $year/$basin/$stormName/$agency/$dataSource/$trapSetName >> $newTrapList");

##------------------- Create the basin Html Text file if there is none -------------------##

$basinFile = $basin . ".txt";
#$basinFile = "$etrapDataDir/$year/$basinFile";
$basinFile = "$etrapDataDir/$basinFile";

if (!(-s $basinFile))
{
   print "PAY attention! $basinFile is still not exist.  Create one.\n";
   system ("$basinCM $year $stormName $basin $createMode");
   $success=$?;
   if ($success != 0)
   {
      print "ERROR: Failed to run $moveBasinCM.\n";
   }
}


##-----------------------------------------------------------------------------------------------------##

##------------------- Create the Java script files for the storm name and date/time -------------------##


chdir("$etrapDataDir") || die "Cannot change to directory $etrapDataDir";
print  "Current Data directory is:";
system("pwd");

### will delete below-------------------

# will create amsu.txt here, 10/13/2006
# use tr/A-Z/a-z/;     
#$tmSource =~ tr/A-Z/a-z/;


## Have to double "\" as one for Perl, another for "sed' in Shell
## $pathSetName = "$basin/$stormName/$agency/$dataSource/$trapSetName";
## $pathSetName = "$basin\\/$stormName\\/$agency\\/$dataSource\\/$trapSetName";
#$pathSetName = "$year\\/$basin\\/$stormName\\/$agency\\/$dataSource\\/$trapSetName";

#system("sed -e 's/$tempSensor/$dataSource1/' -e 's/$tempSetName/$pathSetName/g' -e 's/$tempTime/$rrTime1/g' $sensorTemplateFile > sensor_tm.txt");

#### will delete above----------------


#------------------ Create the date/time files ------------------#

system("sed 's/$tempDateTime/$dateTimeStr/' $dateTimeTemplateFile > dateTime_tm.txt");
$success=$?;
if ($success != 0)
{
   print "ERROR: Failed to run sed command to get the current date/time Javascript file .\n";
}

if (-s "dateTime_tm.txt")
{
   print "Created the date/time file of the latest eTRaP file.\n";
   rename("dateTime_tm.txt", $curDateTimeFile);
   system("cp $curDateTimeFile $destDir/$dateTimeFile");
   
   if ($isFirstTRaP == 1)
   {
      print "Call the script $genETRaPPageCM to generate eTRaP (archive) web pages.\n";
      ## bash gen-trap-pages.bash 2006/Atlantic/HELENE/TPC/AMSU/2006HELENE.WTNT23.KNHC.240835.AMSU.240843
      system("bash $genETRaPPageCM $year $basin $stormName");

      # create the startTime/date file
      print "This is the first eTRaP for the storm $stormName, create the start date/time file.\n";
      #system("cp $curDateTimeFile $destDir/$stDateTimeFile");  
      system("sed 's/$tempDtTmFunc/$tempStDtTmFunc/' $curDateTimeFile > $destDir/$stDateTimeFile");
      $success=$?;
      if ($success != 0)
      {
         print "ERROR: Failed to run sed command to get the start date/time Javascript file .\n";
      }

   }
   
   # Copy the startDateTime file from the storm directory to the ETRAP directory
   #print "Copy $destDir/$stDateTimeFile to $curStDateTimeFile in ETRAP directory.\n";
   #system("cp $destDir/$stDateTimeFile $curStDateTimeFile");
}
else
{
   print "ERROR!! Failed to get the date/time Javascript file, dateTime_tm.txt is empty.  Exit the program $0!.\n";
   exit;
}


#------------------ Create the storm name file in the etrap directory ------------------#

system("sed 's/$tempStName/$stormName/' $stNameTemplateFile > stName_tm.txt");

if (-s "stName_tm.txt")
{
   print "Create the storm name text file $nameFile and update the $stormListFile.\n";
   rename("stName_tm.txt", $nameFile);
   chmod(0644, $nameFile);
}
else
{
   print "ERROR!! Failed to get the storm name text file, stName_tm.txt is empty.  Exit the program $0!.\n";
   exit;
}

   
#------------------------ update the list file for storm archive  ------------------------#

chdir("$etrapDataDir/$year/$basin/$stormName") || die "Cannot change to directory $etrapDataDir/$year/$basin/$stormName";

# Get the storm line
# <a href=?storm="STORMNAME"&time="HH"&date="DATE"><img src='"IMAGENAME"' width='100px' height='50px'/></a>

system("sed -e 's/$tempStName/$stormName/' -e 's/$tempHour/$hour/' -e 's/$tempDate/$dateStr/' -e 's/$tempImgName/$firstGIF/' $stLineTemplateFile > stLine_tm.txt");
$success=$?;
if ($success != 0)
{
   print "ERROR: Failed to run sed command to get the storm line for Archive .\n";
}

if (-s "stLine_tm.txt")
{
   print "Get the Storm line for archive page.\n";
   $stLine = `cat stLine_tm.txt`;
   chomp ($stLine);
}
else
{
   print "ERROR!! Failed to get the Storm line for archive page for storm $stormName, stLine_tm.txt is empty.  Exit the program $0!.\n";
   exit;
}

# Get the line number to update, 00 hour will be the line #3
$lineIncrement = $hour / 6;
#$lineNum = 3 + $lineIncrement;    # line number start from "tr"
$lineNum1 = 1 + $lineIncrement;    # line number after "DATE", so the real line number is + $DATELineNum

# get the date line number
# grep -n -o DATE stListBlank-template.txt
# check for the total line match, if more than 1, there is something wrong
# grep -n -o DATE stListBlank-template.txt|wc -l

if (!(-s $stormListFile))
{
   system("cp $stListTemplateFile $stormListFile");
   $DATELineNum = 2;
   $lineNum = $lineNum1 + $DATELineNum;

   system("sed -e 's/$tempDate/$dateStr/' -e '$lineNum s/$emptyPattern/$stLine/' $stormListFile > stList_tm.txt");
   if (!(-s "stList_tm.txt"))
   {
      print "ERROR!! Failed to replace the $emptyPattern with $stLine in $stormListFile, stLine_tm.txt is empty.  Exit the program $0!.\n";
      exit;
   }

   $sedStr = "";
   for ($i = $DATELineNum + 1; $i < $lineNum; $i++)
   {
      $sedStr = $sedStr . " -e '$i s/$emptyPattern/$stNoformPattern/' ";
   }
   if ($sedStr ne "")
   {
      system("sed $sedStr stList_tm.txt > stList_tm2.txt");
      rename("stList_tm2.txt", "stList_tm.txt");
   }
   rename("stList_tm.txt", $stormListFile);
}
else
{
   $isDateExist = `grep $dateStr $stormListFile`;

   # It is the first file for that date   
   if ( $isDateExist eq "" )
   {
      system("cp $stListTemplateFile stList_tm.txt");
      system("sed  's/$tempDate/$dateStr/' stList_tm.txt  > stList_tm2.txt");
      if (!(-s "stList_tm2.txt"))
      {
         print "ERROR!! Failed to replace the $tempDate with $dateStr in stList_tm.txt, stLine_tm2.txt is empty.  Exit the program $0!.\n";
         exit;
      }

      system("cat $stormListFile stList_tm2.txt > stList_tm.txt");
      rename("stList_tm.txt", $stormListFile);
   }

   # replace the storm line for that hour
   # Get the line number first, 
   # Notes: there maybe more than one line return if the hour is not 00, 
   #        but the first line is always for the DATE line, not storm lines
   $lineNumStr = `grep -n -o $dateStr $stormListFile`;    # output is something like "2:DATE"
   @splitMsg1 = split(/\:/, $lineNumStr);
   $DATELineNum = $splitMsg1[0];
   print "lineNumStr is $lineNumStr, DATELineNum is $DATELineNum\n";
   $lineNum = $lineNum1 + $DATELineNum;
   $isLineExist = `grep $firstGIF $stormListFile`;
   if ($isLineExist eq "")
   {
      system("sed '$lineNum s/$emptyPattern/$stLine/' $stormListFile > stList_tm.txt");
      if (!(-s "stList_tm.txt"))
      {
         print "ERROR!! Failed to replace the $emptyPattern with $stLine in $stormListFile, stLine_tm.txt is empty.  Exit the program $0!.\n";
         exit;
      }

      rename("stList_tm.txt", $stormListFile);
   }
   else
   {
      print "$firstGIF exist in $stormListFile, NO change!\n";
   }
}
chmod(0644, $stormListFile);


#Added by Liqun Ma on 11/28/2011
$BIGYEAR=`date -u +%Y`;
$MON=`date -u +%m`;
$DAY=`date -u +%d`;
$TIME=`date -u +%H`;

chomp($BIGYEAR);
chomp($MON);
chomp($DAY);
chomp($TIME);

open (JS, ">$etrapDataDir/lastUpdate.js") || die "ERROR: can't create lastUpdate.js: $!";
printf JS "function lastUpdate(){\n";
printf JS "return \"%s-%s-%s-%s\";\n", $BIGYEAR, $MON, $DAY, $TIME;
printf JS "}\n";

close (JS) || die "ERROR: can't close JS file. $!";



print "\n-------------------- End of the program $0 --------------------\n\n";
