#!/bin/bash
###############################################################################
# File :   createTar_NCDC_test.sh
#
# Author: Jianbin Yang/PSGS   March 2010
#
# Purpose: This shell script tar up the eTRaP product for each active storm for yesterday
#          and create the distribution to DDS for NCDC for testing purpose
#
# Usage: createTar_NCDC.sh
#
# Last Modified: 04/06/2010
#
################################################################################

# Set up path

PATH=$PATH:$HOME/mcidas/bin:/home/mcidas/bin:$HOME/mcidas/etrap:.
MCPATH=$HOME/mcidas/data:/home/mcidas/data
export PATH MCPATH

# Include the startup script for the current date/time, directories and log files
etrapSrcDir=$HOME/mcidas/etrap/scripts
. $etrapSrcDir/startup.sh

successStup=$?
if (($successStup != 0 ))
then
   # Main Log files
   # The following should be got from the startup scripts

   Mdhm=`date +%m%d%H%M`
   logDir="/data/TRAP/ETRAP/logs"
   logFile="$logDir/etrap.$Mdhm.log.NCDC"
# 08/21/2009, will need to modify the NCDC log ile name is startup script

   #
   # Inform the status monitor we are staring.
   #
   monLogDir="$logDir/monitorLogs"
   status_file_distNCDC="$monLogDir/etrap_distributionNCDC.status"

   logfile_distNCDC="$monLogDir/etrap_distributionNCDC.$Mdhm.log"

   echo "ERROR: Failed to run the startup.sh to include the time and directories!" >> $logFile
   cp $logFile $logfile_distNCDC
else
   logFile=$logFile.NCDC
fi

# Site for DDS server for NCDC archive
NCDCSite="dds.nesdis.noaa.gov"
NCDCDir="push"
ftpNCDCCmd="$etrapSrcDir/push2NCDC.sh"

#local NCDC tar file directory
localNCDCDir="$etrapDataDir/NCDC_Arch"
[ -d ${localNCDCDir} ] || mkdir ${localNCDCDir}

# Site for monitoring log
# GP5
remoteSite="sabweb@10.144.3.87"
#remoteSite="sabweb@gp50.ssd.nesdis.noaa.gov"
rmDir="/data/web/mirror/www.ssd.noaa.gov/htdocs/PS/TROP/DATA/ETRAP/monitor/logs"

# Sandia
remoteSite2="etrapmonitor@10.144.3.35"
rmDir2="/prod/intra/OSDPD/docs/ETRAP/monitor/logs"

ftpLogCmd="$etrapSrcDir/pushLogs_NCDC.sh"

stormFile=$trapSrcDir/activeStorms.txt
#stormFile=$trapSrcDir/activeStorms.txt.test

stormSortFile=$etrapSrcDir/activeStormsSorted.txt
sortStCmd=$etrapSrcDir/sortStList.sh

exec >> $logFile   2>&1


##---------------------- Start the program ----------------------##

echo "start $0 at `date`"
echo

statStTime=`date +'%Y%m%d%H%M%S'`
#curYyyy=`date +'%Y'`
#yyyyJjj=`date +'%Y%j'`
#mmDd=`date +'%m%d'`

curYyyy=`date -d yesterday +'%Y'`
yyyyJjj=`date -d yesterday +'%Y%j'`
mmDd=`date -d yesterday +'%m%d'`

cd $etrapSrcDir

#-------------------- Set up the FTP script for monitoring logs --------------------#
echo "sftp $remoteSite <<'END' " > $ftpLogCmd
echo "cd $rmDir" >> $ftpLogCmd
echo "put $logfile_distNCDC  ">> $ftpLogCmd
echo "put $status_file_distNCDC ">> $ftpLogCmd
echo "bye " >> $ftpLogCmd
echo "END" >> $ftpLogCmd
echo " " >> $ftpLogCmd

echo "sftp $remoteSite2 <<'END1' " >> $ftpLogCmd
echo "cd $rmDir2" >> $ftpLogCmd
echo "put $logfile_distNCDC  ">> $ftpLogCmd
echo "put $status_file_distNCDC ">> $ftpLogCmd
echo "bye " >> $ftpLogCmd
echo "END1 " >> $ftpLogCmd

chmod u+x $ftpLogCmd

#-------------------- Get the storm list --------------------#

# call the script to sort the active storm list based on the basin.  Atlantic has higher priority

# check the file size first
if [ ! -s $stormFile ]
then
   echo "No active storm. EXIT the program!"

   # The following are for the monitoring page
   message="\"No active storm\""
   cp $logFile $logfile_distNCDC
   statEndTime=`date +'%Y%m%d%H%M%S'`
   echo "$statStTime $statEndTime N $(basename $logfile_distNCDC) $message" >> $status_file_distNCDC  

   #  SFTP to send the log and status files to Web serve for monitoring page
   echo "Run $ftpLogCmd"
   tempLog='/tmp/pushNCDCLogs.log'
#   $ftpLogCmd > $tempLog 2>&1

   successFTP=$?
   cat $tempLog

   if (($successFTP != 0 ))
   then
     echo "ERROR: Failed to send the log files to Web server for monitoring!"
     echo "SFTP status is $successFTP, SFTP to Web server failed."
     ftpError=`grep -a "Connection closed" $tempLog`
     echo "The FTP error is $ftpError"
   fi

   exit
fi

$sortStCmd $stormFile $stormSortFile

found_error=0

# declare the array
declare -a tarList

#index=0

for stormLine in `cat $stormSortFile `
do
   # will handle the case when there are two entries for the same storm
   # especially for the beginning of the year
   echo "stormLine = --$stormLine--"

   #stormName=`echo $stormLine |cut -d" " -f1`
   stormName=`echo $stormLine |cut -d"+" -f1`
   year=`echo $stormLine |cut -d"+" -f2`
   basin=`echo $stormLine |cut -d"+" -f3`

   echo "Storm Name=$stormName, year=$year and basin=$basin" 

   curYear=`date +%Y`

   # avoid to call the same storm twice when it is in the beginning of the year
   # skip the entry if that is the case
   if  [[ $year != $curYear ]]
   then 
      echo "year is $year, curYear is $curYear, they are different, skip this entry"
      continue
   fi

   #-------------------- Run eTRaP for each storm --------------------#

   echo "Create the tar file for storm $stormName at `date`"
   cd $etrapDataDir/$curYyyy/$stormName 
   success=$?

   if (($success != 0 ))
   then
     echo "ERROR: Failed to change directory $etrapDataDir/$curYyyy/$stormName!"
     continue 
   fi 

   #tarFile='eTRaP-'$stormName'-'$curYyyy$mmDd'.tar'
   tarFile='eTRaP_'$stormName'_'$curYyyy$mmDd'.tar'
   echo "tarFile:$tarFile"
   ls *$stormName.p*.$mmDd* > /dev/null 2>&1 
# will work redirect the ls output to tmp/, 12/10
   success=$?

   if (($success != 0 ))
   then
     echo "WARNING: there is no eTRaP for storm $stormName for date $curYyyy$mmDd, skip this one!"
     continue
   fi

# seem to have problem here, see the log "etrap.12102113.log.NCDC", 12/10
   echo "Current diretory is :`pwd`"
   pwd
   echo "Run this: tar cvf $localNCDCDir/$tarFile *$stormName.p*.$mmDd*"
   tar cvf $localNCDCDir/$tarFile *$stormName.p*.$mmDd* > /dev/null 2>&1
  
   cd $etrapDigDir
   ls *$stormName.p*.$mmDd*.TXT* > /dev/null 2>&1
   success=$?

   if (($success != 0 ))
   then
     echo "WARNING: there is no digital eTRaP for storm $stormName for date $curYyyy$mmDd!"
   else
     echo "Add the digital eTRaP files to the same tar file."
     tar rvf $localNCDCDir/$tarFile *$stormName.p*.$mmDd*.TXT* > /dev/null 2>&1
   fi
   
   # add the tar file to the array
   tarList=(${tarList[@]} $tarFile)
   echo "Made the tar file $tarFile for $stormName"
   #tarList[$index]=$tarFile
   #echo "TARLIST:${tarList[@]}"
   #index=`expr $index + 1`
done


#-------------------- Set up the FTP script --------------------#
echo "ftp $NCDCSite <<'END' " > $ftpNCDCCmd
echo "passive" >> $ftpNCDCCmd 
echo "cd $NCDCDir" >> $ftpNCDCCmd
for tFile in ${tarList[*]}
do
   echo "tFile=$tFile"
   echo "put $tFile  xfr_$tFile" >> $ftpNCDCCmd
   echo "rename xfr_$tFile $tFile" >> $ftpNCDCCmd
done
echo "bye " >> $ftpNCDCCmd
echo "END " >> $ftpNCDCCmd

chmod u+x $ftpNCDCCmd

cd $localNCDCDir

#  SFTP to send the tar file for NCDC archive
echo "Run $ftpNCDCCmd"
tempLog='/tmp/pushNCDC.log'
#$ftpNCDCCmd > $tempLog 2>&1

successFTP=$?
cat $tempLog

if (($successFTP != 0 ))
then
   echo "ERROR: Failed to send the tar files to DDS for NCDC archive!"
   echo "SFTP status is $successFTP, SFTP to DDS server failed."
   ftpError=`grep -a "Connection closed" $tempLog`
   echo "The FTP error is $ftpError"
fi

#--------------------------------------------------------------------#

#-------------------- Write the status log files --------------------#

   
echo "Write the status log files for monitoring"

statEndTime=`date +'%Y%m%d%H%M%S'`


#------------- For the eTRaP product NCDC distribution status -------------#

errStr1="Connection closed "
errStr2="Connection reset "
errStr3="Connection refused"
# The following is for "Couldn't write to remote file" error
errStr4="Failure" 
errExist=`grep -e "ERROR" -e "$errStr1" -e "$errStr2" -e "$errStr3" -e "$errStr4" $logFile`

# To elminate the WARNING lines for the FTP entry "**WARNING**WARNING**WARNING*"
wngExist=`grep WARNING $logFile |grep -v "\*\*WARNING\*\*"`

cp $logFile $logfile_distNCDC

if [[ $errExist != "" ]]
then
   echo " "
   echo "There are Errors as following during the eTRaP product NCDC distribution process:"
   echo "$errExist"
   echo " "
   message="ERROR: Problems encountered with eTRaP product NCDC distribution process"
   echo "$statStTime $statEndTime E $(basename $logfile_distNCDC) $message" >> $status_file_distNCDC

   echo " " >> $logfile_distNCDC
   echo "There are Errors as following during the eTRaP product NCDC distribution process:" >> $logfile_distNCDC
   echo "$errExist" >> $logfile_distNCDC
elif [[ $wngExist != "" ]]
then
   echo " "
   echo "There are Warnings as following during the eTRaP product NCDC distribution process:"
   echo "$wngExist"
   echo " "
   message="WARNING: Problems encountered with eTRaP product NCDC distribution process"
   echo "$statStTime $statEndTime W $(basename $logfile_distNCDC) $message" >> $status_file_distNCDC

   echo " " >> $logfile_distNCDC
   echo "There are Warnings as following during the eTRaP product NCDC distribution process:" >> $logfile_distNCDC
   echo "$wngExist" >> $logfile_distNCDC
else
   message="Successful with eTRaP product distribution process"
   echo "$statStTime $statEndTime S $(basename $logfile_distNCDC) $message" >> $status_file_distNCDC
fi

echo " " >> $logfile_distNCDC
echo "$message" >> $logfile_distNCDC

#  SFTP to send the log and status files to Web serve for monitoring page

echo "Run $ftpLogCmd"

tempLog='/tmp/pushNCDCLogs.log'
#$ftpLogCmd > $tempLog 2>&1
successFTP=$?

cat $tempLog

if (($successFTP != 0 ))
then
  echo "ERROR: Failed to send the log files to Web server for monitoring!"
  echo "SFTP status is $successFTP, SFTP to Web server failed."
  ftpError=`grep -a "Connection closed" $tempLog`
  echo "The FTP error is $ftpError"
fi

echo
echo "End of Script $0 at `date`"

exit

