#!/bin/bash
###############################################################################
# File :  etrap_main_Past.sh (will need to change the date and time
#
# Author: Jianbin Yang/PSGS   May 2008
#
# Purpose: This shell script is the driver script to run Fortran etrap 
#
# Usage: etrap_main.sh 
#
# Modified : 11/18/2008
#
# Modified: 04/08/2009
#              Handle the case cross basins
#                
# Last Modified: 05/21/2009 - 05/28
#              Orginaized the log files and do error checking for the monitoring
#
################################################################################

# Set up path

PATH=$PATH:$HOME/mcidas/bin:/home/mcidas/bin:$HOME/mcidas/etrap:.
MCPATH=$HOME/mcidas/data:/home/mcidas/data
export PATH MCPATH

# Include the startup script for the current date/time, directories and log files
etrapSrcDir=$HOME/mcidas/etrap/scripts
. $etrapSrcDir/startup_Past.sh

successStup=$?
if (($successStup != 0 ))
then
   # Main Log files
   # The following should be got from the startup scripts

   Mdhm=`date +%m%d%H%M`
   logDir="/data/TRAP/ETRAP/logs"
   logFile="$logDir/etrap.$Mdhm.log"

   #
   # Inform the status monitor we are staring.
   #
   monLogDir="$logDir/monitorLogs"
   status_file_stList="$monLogDir/get_storm_list.status"
   status_file_gen="$monLogDir/etrap_creation.status"
   status_file_img="$monLogDir/image_generation.status"
   status_file_dist="$monLogDir/etrap_distribution.status"

   logfile_stList="$monLogDir/get_storm_list.$Mdhm.log"
   logfile_gen="$monLogDir/etrap_creation.$Mdhm.log"
   logfile_img="$monLogDir/image_generation.$Mdhm.log"
   logfile_dist="$monLogDir/etrap_distribution.$Mdhm.log"

   echo "ERROR: Failed to run the startup.sh to include the time and directories!" >> $logFile
   cp $logFile $logfile_stList
   cp $logFile $logfile_gen
   cp $logFile $logfile_img
   cp $logFile $logfile_dist
fi


logfile_genTmp="$monLogDir/etrap_creation.tmp.log"

remoteSite="sabweb@10.144.3.87"
#remoteSite="sabweb@gp50.ssd.nesdis.noaa.gov"
rmDir="/data/web/mirror/www.ssd.noaa.gov/htdocs/PS/TROP/DATA/ETRAP/monitor/logs"
ftpLogCmd="$etrapSrcDir/pushLogs.sh"

#
# input text and sub commands
#
#trapSrcDir=$HOME/mcidas/pctrap-V3
#etrapSrcDir=$HOME/mcidas/etrap/scripts

stormFile=$trapSrcDir/activeStorms_Past.txt
tempFile=$trapSrcDir/tempStList.txt
bkFile=$stormFile.cross

stormSortFile=$etrapSrcDir/activeStormsSorted.txt
sortStCmd=$etrapSrcDir/sortStList.sh
etrapCmd=$etrapSrcDir/run_etrap_Past.sh

exec >> $logFile   2>&1


##---------------------- Start the program ----------------------##

echo "start $0 at `date`"
echo

statStTime=`date +'%Y%m%d%H%M%S'`

cd $etrapSrcDir

#-------------------- Set up the FTP script --------------------#
echo "sftp $remoteSite <<'END' " > $ftpLogCmd
echo "cd $rmDir" >> $ftpLogCmd
echo "put $logfile_stList  ">> $ftpLogCmd
echo "put $logfile_gen  ">> $ftpLogCmd
echo "put $logfile_img  ">> $ftpLogCmd
echo "put $logfile_dist  ">> $ftpLogCmd
echo "put $status_file_stList ">> $ftpLogCmd
echo "put $status_file_gen ">> $ftpLogCmd
echo "put $status_file_img ">> $ftpLogCmd
echo "put $status_file_dist ">> $ftpLogCmd
echo "bye " >> $ftpLogCmd
echo "END " >> $ftpLogCmd

chmod u+x $ftpLogCmd

#-------------------- Get the storm list --------------------#

# call the script to sort the active storm list based on the basin.  Atlantic has higher priority

# check the file size first
if [ ! -s $stormFile ]
then
   echo "No active storm. EXIT the program!"

   # The following are for the monitoring page
   message="\"No active storm\""
   cp $logFile $logfile_stList
   cp $logFile $logfile_gen
   cp $logFile $logfile_img
   cp $logFile $logfile_dist
   statEndTime=`date +'%Y%m%d%H%M%S'`
   echo "$statStTime $statEndTime N $(basename $logfile_stList) $message" >> $status_file_stList  
   echo "$statStTime $statEndTime N $(basename $logfile_gen) $message" >> $status_file_gen  
   echo "$statStTime $statEndTime N $(basename $logfile_img) $message" >> $status_file_img  
   echo "$statStTime $statEndTime N $(basename $logfile_dist) $message" >> $status_file_dist  

   #  SFTP to send the log and status files to Web serve for monitoring page
   echo "Run $ftpLogCmd"
   tempLog='/tmp/pushLogs.log'
   #$ftpLogCmd > $tempLog 2>&1

   successFTP=$?
   cat $tempLog

   if (($successFTP != 0 ))
   then
     echo "ERROR: Failed to send the log files to Web server for monitoring!"
     echo "SFTP status is $successFTP, SFTP to Web server failed."
     ftpError=`grep -a "Connection closed" $tempLog`
     echo "The FTP error is $ftpError"
   fi

   exit
fi

$sortStCmd $stormFile $stormSortFile
cp $logFile $logfile_stList

found_error_stList=0

for stormLine in `cat $stormSortFile `
do
   # will handle the case when there are two entries for the same storm
   # especially for the beginning of the year
   echo "stormLine = --$stormLine--"
   echo "stormLine = --$stormLine--" >> $logfile_stList

   #stormName=`echo $stormLine |cut -d" " -f1`
   stormName=`echo $stormLine |cut -d"+" -f1`
   year=`echo $stormLine |cut -d"+" -f2`
   basin=`echo $stormLine |cut -d"+" -f3`

   echo "Storm Name=$stormName, year=$year and basin=$basin" >> $logfile_stList

   # check if the basin field has hypen "-" to connect two basins, 
   # if so it means the storm just cross the basin1 to basin2
   # get them and update the stormFile to remove the basin1
   isCrsBasins=0
   if [[ $basin = *-* ]]
   then 
      isCrsBasins=1
      basin1=`echo $stormLine |cut -d"+" -f3|cut -d"-" -f1`
      basin2=`echo $stormLine |cut -d"+" -f3|cut -d"-" -f2`
      if [[ $basin2 = "" ]]
      then
         echo "ERROR: basin2 is empty"
         echo "ERROR: basin2 is empty" >> $logfile_stList
         found_error_stList=1
         continue
      fi

      echo "The storm $stormName cross the basins:$basin1 to $basin2"
      echo "The storm $stormName cross the basins:$basin1 to $basin2" >> $logfile_stList

      stormLineN=`echo $stormLine | sed s/$basin/$basin2/`
      sed s/$stormLine/$stormLineN/ $stormFile > $tempFile
      if [ $? -ne 0 ]
      then
         echo "ERROR: unable to replace the basin for entry of $stormLine"
         echo "ERROR: unable to replace the basin for entry of $stormLine" >> $logfile_stList
         found_error_stList=1
         continue
      fi

      cp -p $stormFile $bkFile
      mv $tempFile $stormFile  
   fi

   curYear=`date +%Y`

   # avoid to call the same storm twice when it is in the beginning of the year
   # skip the entry if that is the case
   if  [[ $year != $curYear ]]
   then 
      echo "year is $year, curYear is $curYear, they are different, skip this entry"
      echo "year is $year, curYear is $curYear, they are different, skip this entry" >> $logfile_stList
      continue
   fi

   #-------------------- Run eTRaP for each storm --------------------#

   echo "Run the following etrap command $etrapCmd at `date`"
   
   if [ $isCrsBasins -eq 0 ]
   then
      echo "$etrapCmd $stormName $basin"  > $logfile_genTmp
      $etrapCmd $stormName $basin >> $logfile_genTmp 2>&1
      cat $logfile_genTmp >> $logfile_gen
      cat $logfile_genTmp
   else
      echo "$etrapCmd $stormName $basin1 $basin2" > $logfile_genTmp
      $etrapCmd $stormName $basin1 $basin2 >> $logfile_genTmp 2>&1
      cat $logfile_genTmp >> $logfile_gen
      cat $logfile_genTmp
   fi

   echo

done


#--------------------------------------------------------------------#

#-------------------- Write the status log files --------------------#

echo "Check if the other log files not exist"

# If there is something wrong with the get storm list for the active storm(s), 
# then no logfile_gen, logfile_img or logfile_dist created
# copy that from the logfile_stList
if [ ! -s $logfile_gen ] && [ -s $logfile_stList ]
then
   message="Something wrong with the first step to get the active storm list, sequential steps stop."
   echo $message >> $logfile_gen
   cat $logfile_stList >> $logfile_gen
   cp $logfile_gen $logfile_img
   cp $logfile_gen $logfile_dist
fi
   
echo "Write the status log files for monitoring"
# test run the program with CHAN-HOM for non exist time for the log files stuff

statEndTime=`date +'%Y%m%d%H%M%S'`

#------------- For the get storm list status -------------#

if [ $found_error_stList -eq 1 ]
then
   message="ERROR: Problems encountered with get active storms list process"
   echo "$statStTime $statEndTime E $(basename $logfile_stList) $message" >> $status_file_stList
else
   message="Successful with get active storms list process"
   echo "$statStTime $statEndTime S $(basename $logfile_stList) $message" >> $status_file_stList
fi

#------------- For the eTRaP product creation status -------------#

errExist=`grep ERROR $logfile_gen` 
wngExist=`grep WARNING $logfile_gen` 
if [[ $errExist != "" ]]
then
   echo " "
   echo "There are Errors as following during the eTRaP product creation process:"
   echo "$errExist"
   echo " "
   message="ERROR: Problems encountered with eTRaP product creation process"
   echo "$statStTime $statEndTime E $(basename $logfile_gen) $message" >> $status_file_gen

   echo " " >> $logfile_gen
   echo "There are Errors as following during the eTRaP product creation process:" >> $logfile_gen
   echo "$errExist" >> $logfile_gen
elif [[ $wngExist != "" ]]
then 
   echo " "
   echo "There are Warnings as following during the eTRaP product creation process:"
   echo "$wngExist"
   echo " "
   message="WARNING: Problems encountered with eTRaP product creation process"
   echo "$statStTime $statEndTime W $(basename $logfile_gen) $message" >> $status_file_gen

   echo " " >> $logfile_gen
   echo "There are Warnings as following during the eTRaP product creation process:" >> $logfile_gen
   echo "$wngExist" >> $logfile_gen
else
   message="Successful with eTRaP product creation process"
   echo "$statStTime $statEndTime S $(basename $logfile_gen) $message" >> $status_file_gen
fi

echo " " >> $logfile_gen
echo "$message" >> $logfile_gen

#------------- For the eTRaP image/GIF generation status -------------#  
# Notes: will detect more error in the MKPROB.sh, 05/27

errStr1="No images satisfy the selection criteria"
errStr2="NO IMAGE OR NO NAVIGATION FOR FRAME"
errExist=`grep -i -e "ERROR" -e "$errStr1" -e "$errStr2" $logfile_img`

#May add more wngStr here
wngExist=`grep WARNING $logfile_img`

if [[ $errExist != "" ]]
then
   echo " "
   echo "There are Errors as following during the eTRaP GIF generation process:"
   echo "$errExist"
   echo " "
   message="ERROR: Problems encountered with eTRaP GIF generation process"
   echo "$statStTime $statEndTime E $(basename $logfile_img) $message" >> $status_file_img

   echo " " >> $logfile_img
   echo "There are Errors as following during the eTRaP GIF generation process:" >> $logfile_img
   echo "$errExist" >> $logfile_img
elif [[ $wngExist != "" ]]
then
   echo " "
   echo "There are Warnings as following during the eTRaP GIF generation process:"
   echo "$wngExist"
   echo " "
   message="WARNING: Problems encountered with eTRaP GIF generation process"
   echo "$statStTime $statEndTime W $(basename $logfile_img) $message" >> $status_file_img

   echo " " >> $logfile_img
   echo "There are Warnings as following during the eTRaP GIF generation process:" >> $logfile_img
   echo "$wngExist" >> $logfile_img
else
   message="Successful with eTRaP GIF generation process"
   echo "$statStTime $statEndTime S $(basename $logfile_img) $message" >> $status_file_img
fi

echo " " >> $logfile_img
echo "$message" >> $logfile_img

#------------- For the eTRaP product distribution status -------------#

errStr1="Connection closed "
errStr2="Connection reset "
errStr3="Connection refused"
errExist=`grep -e "ERROR" -e "$errStr1" -e "$errStr2" -e "$errStr3" $logfile_dist`

# To elminate the WARNING lines for the FTP entry "**WARNING**WARNING**WARNING*"
wngExist=`grep WARNING $logfile_dist |grep -v "\*\*WARNING\*\*"`
if [[ $errExist != "" ]]
then
   echo " "
   echo "There are Errors as following during the eTRaP product distribution process:"
   echo "$errExist"
   echo " "
   message="ERROR: Problems encountered with eTRaP product distribution process"
   echo "$statStTime $statEndTime E $(basename $logfile_dist) $message" >> $status_file_dist

   echo " " >> $logfile_dist
   echo "There are Errors as following during the eTRaP product distribution process:" >> $logfile_dist
   echo "$errExist" >> $logfile_dist
elif [[ $wngExist != "" ]]
then
   echo " "
   echo "There are Warnings as following during the eTRaP product distribution process:"
   echo "$wngExist"
   echo " "
   message="WARNING: Problems encountered with eTRaP product distribution process"
   echo "$statStTime $statEndTime W $(basename $logfile_dist) $message" >> $status_file_dist

   echo " " >> $logfile_dist
   echo "There are Warnings as following during the eTRaP product distribution process:" >> $logfile_dist
   echo "$wngExist" >> $logfile_dist
else
   message="Successful with eTRaP product distribution process"
   echo "$statStTime $statEndTime S $(basename $logfile_dist) $message" >> $status_file_dist
fi

echo " " >> $logfile_dist
echo "$message" >> $logfile_dist

#  SFTP to send the log and status files to Web serve for monitoring page

echo "Run $ftpLogCmd"

tempLog='/tmp/pushLogs.log'
#$ftpLogCmd > $tempLog 2>&1
successFTP=$?

cat $tempLog

if (($successFTP != 0 ))
then
  echo "ERROR: Failed to send the log files to Web server for monitoring!"
  echo "SFTP status is $successFTP, SFTP to Web server failed."
  ftpError=`grep -a "Connection closed" $tempLog`
  echo "The FTP error is $ftpError"
fi

echo
echo "End of Script $0 at `date`"

exit

