#!/usr/bin/perl -w
## =====================================================================
## Name:        log2html.pl
##                  
## Author:      Jianbin Yang/PSGS   09/15/2009
## Language:    Perl
## Purpose:     The program convert the eTRaP log file to HTML to include the 
##              thumbnail of the TRaP member.  It also updates the TRaP GIF list.
##
## Usage:       log2html.pl <input log file> <output html file> <TRaP GIF list file>
##              (notes: file including the path)
##              "log2html.pl $logFile2 $curDir/$htmlLogFile $curDir/$trapGifsList"
## 
## PLEASE NOTE: This script must be called inside "area2etrapdigit.pl"             
##                  
## Modified:   09/16/2009
##
## =====================================================================

# Some constants
$logSearchPattern = "Ensemble TRaP members";      # must match the eTRaP log file
$colNum = 3;                                      # the number of the column in the table for TRaP GIFs

$trapListPattern = "TRAPGIFS";	                  # pattern for TRaP list file and also the sub-directory
                                                  # must match area2digitetrap.pl and createETRAPDir.pl

# Define the directories and files

$HOME = $ENV{"HOME"};

$dataDir = "$HOME/mcidas/data";
# $listFile = "$dataDir/TRaPGifsList.txt";

$startdate=`date`;
print " Start  $0 at $startdate \n";

chdir("$dataDir") || die "Cannot change to directory $dataDir";
print  "Data directory is:";
system("pwd");

# remove the path from the command name

@comandFields = split(/\//, $0);
$commandName = $comandFields[-1];

# if no argument input, exit the program

if (@ARGV < 3)
{
   print "Usage: $commandName <input log file> <output html file> <TRaP GIF list file>\n";
   die "No name input - exit script";
}
else
{
   $inLogFile = $ARGV[0];
   $htmlFile = $ARGV[1];
   $TRAPListFile = $ARGV[2];
}

print "inLogFile is $inLogFile, htmlFile is $htmlFile, TRAPListFile is $TRAPListFile\n";

## Keep the log file for the first part of the html file
#system("cp $inLogFile $htmlFile");

$logName = `basename $inLogFile`;
chomp($logName);
@tempMsg = split(/\./, $logName);
$yyyySt = $tempMsg[0];
$etrapName = substr($logName, 0, -4);

print "The year-storm name is $yyyySt, etrapName is $etrapName\n";

#----- Write the first part of the HTML file -----#

open(HTMLFILE, "> $htmlFile") || die "Can not open $htmlFile for writing: $!";
print HTMLFILE "<HTML>\n";
print HTMLFILE "<BODY>\n";
print HTMLFILE "<H3><CENTER> TRaP Members list for $etrapName </CENTER></H3>\n";
print HTMLFILE "<PRE>\n";
close(HTMLFILE);

system("cat $inLogFile >> $htmlFile");

## open(TRAPLIST, "+>> $listFile") || die "Can not open $listFile for reading and writing: $!";
#open(INLOG, "< $inLogFile") || die "Can not open $inLogFile for reading: $!";
#@dataList = <INLOG>;
#close(INLOG);


$fileList = `grep $yyyySt $inLogFile`;
if ($fileList ne "")
{
   @trapList = split(/\n/, $fileList);
}
else
{  
   system("cp $inLogFile $htmlFile");
   print "ERROR: No TRaP file list in the eTRaP log file, exit program!!\n";
   exit 1;
}

#seek(TRAPLIST, 0, 0) or die "Couldn't seek to the beginning:$!\n";


open(HTMLFILE, ">> $htmlFile") || die "Can not open $htmlFile for appending: $!";
flock(HTMLFILE, 2);         # and lock it

open(TRAPLIST, ">> $TRAPListFile") || die "Can not open $TRAPListFile for appending: $!";
flock(TRAPLIST, 2);

$trapNum = `grep -a "$logSearchPattern" $inLogFile |cut -d":" -f2`;
chomp($trapNum);
$trapNum = $trapNum + 0;        # to remove the space

for ($i = 0; $i < $#trapList+1; $i++)
{
   $fileNum = $i + 1;
   chomp($trapList[$i]);
  
   #----- write the TRaP GIF list file -----#
   # remove the ".Z"
   if ($trapList[$i] =~ /\.Z$/)
   {
      $trapName = substr($trapList[$i], 0, -2);
      $gifName = $trapName . ".GIF";
   }
   else
   {
      print "ERROR: the TRaP file name format change!  It is expected  '.Z' in the end.\n";
      $gifName = "";
   }
  
   print TRAPLIST "$gifName\n";
   
   #----- include the TRaP GIFs to the HTML file -----#
   
   # write the table header
   if ($i == 0)
   {
      print HTMLFILE "</PRE>\n";
      print HTMLFILE "<P>\n";
      print HTMLFILE "<CENTER>\n";
      print HTMLFILE "<table border=0 cellpadding=\"10\">\n";
   }

   if ($fileNum % $colNum == 1)
   {
      print HTMLFILE "<tr align=\"center\" valign=\"middle\">\n";
   }
 
   # get the label of the TRaP file, such as "AMSU.09112241.18"
   @tempMsg = split(/\./, $trapName);
   #$trapLabel = $tempMsg[-3] . '.' . $tempMsg[-2] . '.' . $tempMsg[-1];
   $trapLabel = $tempMsg[-6] . '.' . $tempMsg[-5] . '.' . $tempMsg[-4] . '.' . $tempMsg[-3] . '.' . $tempMsg[-2] . '.' . $tempMsg[-1];
   #print "TRaP label is $trapLabel\n";
   $gifName2 = `basename $gifName`;
   chomp($gifName2);   
   $trapName2 = `basename $trapName`;
   chomp($trapName2);   
   #print HTMLFILE "<td><a href=\"$trapListPattern/$gifName2\"><font size=2>$trapLabel</font><br>";
   print HTMLFILE "<td><a href=\"$trapListPattern/$gifName2\"><font size=1>$trapName2</font><br>";
   print HTMLFILE "<img src=\"$trapListPattern/$gifName2\" width=240 height=180 alt=\"$trapName2\"></a></td>\n";

   if ($fileNum % $colNum == 0)
   {
      print HTMLFILE "</tr>\n";
   }

}

# write the remaining table columns in the HTML file

if ($trapNum % $colNum != 0)
{
   for ($i = 0; $i < $colNum - $trapNum % $colNum; $i++)
   {
      print HTMLFILE "<td></td>\n";
   }
   print HTMLFILE "</tr>\n";
}

print HTMLFILE "</table>\n";
print HTMLFILE "</CENTER>\n";
print HTMLFILE "</BODY>\n";
print HTMLFILE "</HTML>\n";

flock(TRAPLIST, 8);         # and unlock it
close(TRAPLIST) || die "Can not close $TRAPListFile: $!";

flock(HTMLFILE, 8);         # and unlock it
close(HTMLFILE) || die "Can not close $htmlFile: $!";

$enddate=`date`;
print " End of the program $0 at $enddate \n";

