#!/bin/bash
###############################################################################
# File :  regenGIF_main.sh
#
# Author: Jianbin Yang/PSGS   Oct 2009
#
# Purpose: This shell script is the driver script to regenerate the QPF GIf for some storms 
#
# Usage: regenGIF_main.sh
#
# Modified: 10/29/2009
#
################################################################################

# Set up path

PATH=$PATH:$HOME/mcidas/bin:/home/mcidas/bin:$HOME/mcidas/etrap:.
MCPATH=$HOME/mcidas/data:/home/mcidas/data
export PATH MCPATH

# Include the startup script for the current date/time, directories and log files
etrapSrcDir=$HOME/mcidas/etrap/scripts
regenCmd=$etrapSrcDir/regenGIF.pl

trapDataDir=/data/TRAP
etrapDataDir="$trapDataDir/ETRAP"
yyyy=2009
etrapRegenDir="$etrapDataDir/$yyyy/ETRAP18"

stormFile=$etrapDataDir/$yyyy/sortEtrap18.txt
#stormFile=$etrapDataDir/$yyyy/sortEtrap18-test.txt

# GP5
remoteSite1="sabweb@10.144.3.87"
rmDirPre1="/data/web/mirror/www.ssd.noaa.gov/htdocs/PS/TROP/DATA/ETRAP"
ftpLogCmdPre1=$etrapDataDir/$yyyy/ftps/ftpCmd1

# SABNa4
remoteSite2="sab\@140.90.195.121"
rmDirPre2="/data/ext/SABTrap1/ETRAP/digitETRAP"
ftpLogCmdPre2=$etrapDataDir/$yyyy/ftps/ftpCmd2


##---------------------- Start the program ----------------------##

echo "start $0 at `date`"
echo

#-------------------- Get the storm list --------------------#
preBasin=""

for stormLine in `cat $stormFile `
do
   # will handle the case when there are two entries for the same storm
   # especially for the beginning of the year
   echo "stormLine = --$stormLine--"

   #stormName=`echo $stormLine |cut -d" " -f1`
   stormName=`echo $stormLine |cut -d"+" -f1`
   year=`echo $stormLine |cut -d"+" -f2`
   basin=`echo $stormLine |cut -d"+" -f3`

   echo "Storm Name=$stormName, year=$year and basin=$basin" 

   curYear=`date +%Y`

   # avoid to call the same storm twice when it is in the beginning of the year
   # skip the entry if that is the case
   if  [[ $year != $curYear ]]
   then 
      echo "year is $year, curYear is $curYear, they are different!"
   fi

   #-------------------- Regenerate eTRaP GIF for each storm --------------------#

   if [ ! -e $etrapRegenDir/$stormName ]
   then
      echo "WARNING!! $stormName is in the storm list but not in ETRAP18 directory, it is in basin $basin. Skip it!"
      continue
   fi

   echo "Run the following etrap command $etrapCmd at `date`"
   
   #echo "$regenCmd $etrapRegenDir $stormName $basin"  > $logfile_genTmp
   echo "$regenCmd $etrapRegenDir $stormName" 
   $regenCmd $etrapRegenDir $stormName 

   echo

   #-------------------- Set up the FTP script --------------------#

   ftpLogCmd1=$ftpLogCmdPre1.$basin.txt 
   rmDir1=$rmDirPre1/$year/$basin/$stormName/

   ftpLogCmd2=$ftpLogCmdPre2.$basin.txt 
   rmDir2=$rmDirPre2/$year/$stormName/

   if [[ $basin != $preBasin ]]
   then
      echo "sftp $remoteSite1 <<'END' " > $ftpLogCmd1
      #echo "cd $rmDir1" >> $ftpLogCmd1

      echo "sftp $remoteSite2 <<'END' " > $ftpLogCmd2
      #echo "cd $rmDir2" >> $ftpLogCmd2
   
      if [[ $preBasin != "" ]]
      then 
         preftpLogCmd1=$ftpLogCmdPre1.$preBasin.txt
         echo "bye " >> $preftpLogCmd1
         echo "END" >> $preftpLogCmd1
         echo " " >> $preftpLogCmd1
         chmod u+x $preftpLogCmd1

         preftpLogCmd2=$ftpLogCmdPre2.$preBasin.txt
         echo "bye " >> $preftpLogCmd2
         echo "END" >> $preftpLogCmd2
         echo " " >> $preftpLogCmd2
         chmod u+x $preftpLogCmd2
      fi
   fi
   echo "cd $rmDir1" >> $ftpLogCmd1
   echo "put $etrapRegenDir/$stormName/$year$stormName.pmqpf.*.18.GIF ">> $ftpLogCmd1
   echo "cd $rmDir2" >> $ftpLogCmd2
   echo "put $etrapRegenDir/$stormName/$year$stormName.pmqpf.*.18.GIF ">> $ftpLogCmd2

   preBasin=$basin      
done

if [[ $preBasin != "" ]]
then
   echo "Outside the foreach loop, complete the ftp script for the last basin"

   preftpLogCmd1=$ftpLogCmdPre1.$preBasin.txt
   echo "bye " >> $preftpLogCmd1
   echo "END" >> $preftpLogCmd1
   echo " " >> $preftpLogCmd1
   chmod u+x $preftpLogCmd1

   preftpLogCmd2=$ftpLogCmdPre2.$preBasin.txt
   echo "bye " >> $preftpLogCmd2
   echo "END" >> $preftpLogCmd2
   echo " " >> $preftpLogCmd2
   chmod u+x $preftpLogCmd2
fi

#--------------------------------------------------------------------#

echo
echo "End of Script $0 at `date`"

exit

