#!/bin/bash

## =====================================================================
## Name:        sortStList.sh
##                  
## Author:      Jianbin Yang/PSGS   03/31/2009
## Language:    Shell script
## Purpose:     Sort the active storm list based on the basine area
##              Atlantic has highest priority and will be placed in the first line, then EastPacific.
##                  
## Usage:       sortStList.sh <input $stormFile> <output $stormSortFile>
##
## Modified: 03/31/2009
##              
## Modified: 04/08/2009
##              Handle the case cross basins
## 
## Last Modified: 03/17/2010
##              Remove the duplicated line without sorting, especially for crossing basins case
##
## =====================================================================

basin1=Atlantic
basin2=EastPacific
basin3=CentralPacific
basinsOrder=(Atlantic CentralPacific EastPacific NWPacific NTerritory SWPacific \ 
             NorthIndian SEIndian SEPacific SWIndian)
echo "start $0 at `date`"

# Define the directories and files

if [ $# -lt 2 ]
then
   echo "Usage: sortStList.sh <input $stormFile> <output $stormSortFile>"
   echo " "
   exit
fi

inStFile=$1
outStFile=$2

i=0
cmdString="grep "
for basin in ${basinsOrder[*]}
do
   #echo "The No. $i basin is $basin"
   if [ $i -eq 0 ]
   then
      grep -i $basin $inStFile > $outStFile
   else
      grep -i $basin $inStFile >> $outStFile
   fi
   let i=$i+1
   cmdString=$cmdString"-vie "$basin" "
done

cmdString=$cmdString$inStFile
numInStLines=`grep "+" $inStFile|wc -l`
numOutStLines=`grep "+" $outStFile|wc -l`
#numOutUniqLns=`grep "+" $outStFile|uniq|wc -l`
numOutUniqLns=`grep "+" $outStFile|sort -u|wc -l`
echo "cmdString=$cmdString;numInStLines=$numInStLines;numOutStLines=$numOutStLines;numOutUniqLns=$numOutUniqLns"

# This case happens when the storm cross basins, basins will be $basin1-$basin2
# but the output after this if block will not sort base on the basin orders defined above 
if [ $numOutUniqLns -lt $numOutStLines ]
then
   echo "There is duplicated storm entry. Uniq it."
   ## the output from the following two lines will not uniq without sorting
   ##cat $outStFile | uniq > stListUniq.txt
   ##sort -u $outStFile  > stListUniq.txt
   awk ' !line[$0]++' $outStFile  > stListUniq.txt
   echo "The extract line is:"
   sort $outStFile |uniq -d 
   mv stListUniq.txt $outStFile
fi

# Will think about the following, as if there is duplicated it fits the condition but no error
if [ $numInStLines -gt $numOutStLines ]
#if [ $numInStLines -gt $numOutUniqLns ]
then
   echo " "
   echo "ERROR:Some storm lines are missing in the output sorted storm file."
   #echo "Run the following command:$cmdString"
   echo "Here are the missing lines, please verify the basin."
   $cmdString
fi


usStorms=`grep -e $basin1 -e $basin2 -e $basin3 $inStFile`
#echo "$usStorms"
if [ "$usStorms" = "" ]
then 
   echo "No storm falls in the US area"
   #echo "So no need to sort the storm file, just create a copy"
   #cp -p $inStFile $outStFile
   #exit
fi

##echo "$usStorms" > $outStFile
#grep -i $basin1 $inStFile > $outStFile
#grep -i $basin2 $inStFile >> $outStFile
#grep -i $basin3 $inStFile >> $outStFile
#grep -vie $basin1 -vie $basin2 -vie $basin3 $inStFile >> $outStFile

# 
echo "Done with sorting the storm file"

exit
