#!/usr/bin/perl -w

## =====================================================================
## Name:        updateArchTbl_etrap.pl
##              (Adapted from createETRaPDir.pl)
##                
## Author:      Jianbin Yang/PSGS   09/29/2009
## Language:    Perl
## Purpose:     Another program to update the storm archive table beside the createETRaPDir.pl.
##              Such as "Storm End..."           
##                  
## Usage:       updateArchTbl_etrap.pl <Year> <Storm Name> <basin Area> [<Update Mode>]
##
##              "Update Mode" is optional for now:
##               1 - Storm end
##               2...
##
## NOTE:        This program will run on the Web server remotely
##
## Last Modified: 09/29/2009
##
## =====================================================================

# Define the directories and files

$HOME = $ENV{"HOME"};
$webHomeDir = "$HOME/TRAP/ETRAP";

$etrapDataDir = "/data/web/mirror/www.ssd.noaa.gov/htdocs/PS/TROP/DATA/ETRAP";

$textDir = "$webHomeDir/txt";
$textDir = $etrapDataDir;


# mode for the updateBasinFile.pl command line input
#$stEndMode = 1;
#$updateMode = 2;
#$deleteMode = 3;

# for the Java script template files
#$stNameTemplateFile = "$textDir/stName-template.js";
#$dateTimeTemplateFile = "$textDir/dateTime-template.js";

# For html template files
# For storm archive
#$stListTemplateFile = "$textDir/stListBlank-template.txt";
#$stLineTemplateFile = "$textDir/stLine-template.txt";

# define some constants list in the template file
#$tempStName = "STORMNAME";
#$tempDate = "DATE";                   # Actually in format of YYYY-MM-DD
#$tempDateTime = "YYYYMMDDHH";         # Actually in format of YYYY-MM-DD-HH   
#$tempHour = "HH";                     # For the forecast hour  
#$tempImgName = "IMAGENAME";

$tempGetDatePattern = "return ";

#$stNoformPattern = "Not Formed or Renamed";
$stEndPattern = "Renamed or Ended";
#$emptyPattern = "UNAVAILABLE";
$emptyPattern = "No Recent TRaP (<2)";

# Files must match in createETRaPDir.pl
$dateTimeFile = "getDateTime.js";
$stormListFile = "storm-list.txt";
#$stormListFile = "storm-list-test.txt";


##----------------------------- Start the Program -----------------------------##

$startdate=`date`;
chomp($startdate);

print "\n-------------------- Start  $0 at $startdate --------------------\n";

chdir("$etrapDataDir") || die "Cannot change to directory $etrapDataDir";
print  "eTRaP Data directory is:";
system("pwd");

# remove the path from the command name

@comandFields = split(/\//, $0);
$commandName = $comandFields[-1];

# if no argument input, exit the program

# if ($ARGV[0] eq "")
if (@ARGV < 3)
{
   print "Usage: $commandName <Year> <Storm Name> <basin Area> [<Update Mode>]\n";
   die "ERROR: Not enough arguments - exit script";
}

$year = $ARGV[0];
$stormName = $ARGV[1];
$basin = $ARGV[2];

print "StormName is $stormName, year is $year, basin is $basin  \n";


chdir("$etrapDataDir") || die "Cannot change to directory $etrapDataDir";
print  "eTRaP Data directory is:";
system("pwd");

chdir("$year/$basin/$stormName") || die "ERROR:Cannot change to storm directory $year/$basin/$stormName\n";
print  "Storm directory is:";
system("pwd");

##-----------------------------------------------------------------------------------------------------##
   
#------------------------ update the list file for storm archive  ------------------------#


# The storm line format is as following
# <a href=?storm="STORMNAME"&time="HH"&date="DATE"><img src='"IMAGENAME"' width='100px' height='50px'/></a>

# Get the Date and time for the last eTRaP received from the Javascript file
$dateTimeLine = `grep $tempGetDatePattern $dateTimeFile | grep "\;"`;
chomp($dateTimeLine);

# remove the double quote "
$dateTimeLine =~ s/[\"\;]//g;
print "dateTimeLine = $dateTimeLine\n";
@tempMsg = split(/\s+/, $dateTimeLine);
$dateTimeLine = $tempMsg[-1];
print "dateTimeLine = $dateTimeLine\n";

$dateLine = substr($dateTimeLine,0,-3);
$hour = substr($dateTimeLine,-2);
print "dateLine=$dateLine,hour=$hour.\n";


# Get the line number to update, 00 hour will be the line #3
$lineIncrement = $hour / 6;
#$lineNum = 3 + $lineIncrement;    # line number start from "tr"
$lineNum1 = 1 + $lineIncrement;    # line number after "DATE", so the real line number is + $DATELineNum

# get the date line number
# grep -n -o DATE stListBlank-template.txt
# check for the total line match, if more than 1, there is something wrong
# grep -n -o DATE stListBlank-template.txt|wc -l

if (!(-s $stormListFile))
{
   print "ERROR!! $stormListFile is not exist.  Exit $commandName.\n";
}

#$lineNumStr = `grep -n $dateLine $stormListFile | grep "time=$hour"`;    
$lineNumStr = `grep -n $dateLine $stormListFile`;    
@splitMsg1 = split(/\:/, $lineNumStr);
$DATELineNum = $splitMsg1[0];
print "lineNumStr is $lineNumStr, DATELineNum is $DATELineNum\n";
$latestStLineNum = $lineNum1 + $DATELineNum;
$stLine18Num = 4 + $DATELineNum;
print "latestStLineNum=$latestStLineNum, stLine18Num=$stLine18Num.\n";

if ($latestStLineNum == $stLine18Num)
{
   print "The last eTRaP was for $hour Z, no need to update for the storm end status.\n";
}
else
{ 
   $sedStr = "";
   for ($i = $latestStLineNum + 1; $i <= $stLine18Num; $i++)
   {
      $sedStr = $sedStr . " -e '$i s/$emptyPattern/$stEndPattern/' ";
   }

   print "sedStr is $sedStr\n";
   if ($sedStr ne "")
   {
      print "sed $sedStr $stormListFile > stList_tm.txt\n";
      system("sed $sedStr $stormListFile > stList_tm.txt");
      if (!(-s "stList_tm.txt"))
      {
         print "ERROR!! Failed to replace the $emptyPattern with $stEndPattern in $stormListFile, stLine_tm.txt is empty.  Exit the program $0!.\n";
         exit;
      }

      rename("stList_tm.txt", $stormListFile);
   }
   else
   {
      print "WARNING!! sedStr is empty.\n";
   }
}
chmod(0644, $stormListFile);



print "\n-------------------- End of the program $0 --------------------\n\n";
