#!/usr/bin/perl -w
## =====================================================================
## Name:        updateBasinFile_etrap.pl
##              (Adapted from updateBasinFile.pl in April 2009)
##        
## Author:      Jianbin Yang/PSGS   10/11/2006
## Language:    Perl
## Purpose:     The program creates the basin text file in the Web server, 
##              or update or delete the entry for a storm within the basin text file
## 
## Usage:       updateBasinFile.pl <Year> <Storm Name> <basin Area> <Update Mode>
##             
##              "Update Mode" values: 
##               1 - Create New basin Text file
##               2 - Add the entry for the storm in the basin file
##               3 - Delete the entry for an inactive storm
##                   or delete the file if there is no entry 
##
## PLEASE NOTE: This script will run on the web server remotely             
##                  
## Modified:   10/16/2006
##                  Check the basin file and switch Mode if neccessary
##
## Modified: 03/19/2007
##		Change the storm page name to include the basin abbreviation
##
## ADAPTED: 04/27/09
## 
## Modified: 04/29/09
##               Add the getBasin.js
##
## Last Modified: 01/06/2010
##               Change the basin file from year directory to upper 'ETRAP' directory
##
## =====================================================================

# Define the directories and files

$HOME = $ENV{"HOME"};
$webHomeDir = "$HOME/TRAP/ETRAP";
#$webHomeDir = "$HOME/mcidas/etrap/scripts";
$textDir = "$webHomeDir/txt";

$etrapDataDir = "/data/web/mirror/www.ssd.noaa.gov/htdocs/PS/TROP/DATA/ETRAP";
#$etrapDataDir = "$webHomeDir";

# basinBlankFile will be $basin + $basinBlankFileSuffix, located in the $textDir
$basinBlankFileSuffix = ".txt.blank";

# For one line of the basin file in the left of the page
$basinLnTemplateFile = "$textDir/basinLine-template.txt";

# For locating basin directory for the GIF files
$getBasinTemplateFile = "$textDir/getBasin-template.js";
$getBasinFile = "getBasin.js";

#$templateFile = "$webHomeDir/basin-template.txt";
#$blankFile = "$webHomeDir/blank-template.txt";
$newFile = "$etrapDataDir/newBasin.txt";

# define some constants list in the basin template file
#$tempYear = "YEAR";
$tempStorm = "STORMNAME";
$tempBasin = "BASINAREA";
# Must use double \\ to escape the '('
$tempSwitchLine = "switch\\(stormName\\)";
#$tempStormLow = "STORMNAME_LOW";

# For basin file
$tempNoSt = "noStorms";
$tempHaveSt = "stormPresent";

# modes must match the ones in createETRAP.pl and clean up script
$createMode = 1;
$updateMode = 2;
$deleteMode = 3;

#--------- basins and basin abbreviation ---------#

# Define the hash table for the basin (dir here) and the corresponding basin abbreviation (basin here)
# This hash table should be the same as the one in area2digit.pl

%basinTbl = qw(
   Atlantic AL
   EastPacific EP
   CentralPacific CP
   NWPacific WP
   SWPacific SW
   SEPacific SE
   NorthIndian NI
   SWIndian WI
   SEIndian EI
   NTerritory NT
);


##----------------------------- Start the Program -----------------------------##

$startdate=`date`;
print "\nStart  $0 at $startdate \n\n";

# remove the path from the command name

@comandFields = split(/\//, $0);
$commandName = $comandFields[-1];

# if no argument input, exit the program

if (@ARGV < 4)
{
   print "Usage: $commandName <Year> <Storm Name> <basin Area> <Mode>\n";
   die "Not enough input arguments - exit script";
}
else
{
   $year = $ARGV[0];
   $stormName = $ARGV[1];
   $basinArea = $ARGV[2];
   $mode =  $ARGV[3];
}

$basinFile = $basinArea . ".txt";
$oldFile = $basinArea . ".txt.old";

#$basinFile = "$etrapDataDir/$year/$basinFile";
$basinFile = "$etrapDataDir/$basinFile";
$blankFile = "$textDir/$basinArea" . $basinBlankFileSuffix;

$getBasinFile = "$etrapDataDir/$year/$getBasinFile";
$getBasinOldFile = "$getBasinFile" . ".old";

# Will delete the following
$stormNameLow = $stormName;
$stormNameLow =~ tr/A-Z/a-z/;
$stormNameLow = substr($stormName, 0,1) . substr($stormNameLow, 1);

# Add the sasin abbreviation to the storm name
# $stormNameLow = $stormNameLow . '_' . $basinTbl{$basinArea};
$stormNameLow = $stormNameLow . '-' . $basinTbl{$basinArea};
# will delete the above

print "Year is $year, StormName is $stormName, basinArea is $basinArea, update mode is $mode, basinFile is $basinFile.\n";

if (($mode > $deleteMode) || ($mode < $createMode))
{
   print "ERROR: Invalid update mode - $mode, Exit program!\n";
   exit 1;
}

chdir("$etrapDataDir/$year") || die "Cannot change to directory $etrapDataDir/$year";
print  "Work directory is:";
system("pwd");

# check the basin file for update mode - 2, if line number is 3, means blank, then switch to create mode -1

if ($mode == $updateMode)
{
   if (!(-s $basinFile))
   {
      print "WARNING:The $basinFile is not exist, but update mode is used. Please check! Use create mode instead.\n";
      $mode = $createMode;
   }
   else
   {
      $lineNum = `wc -l $basinFile | awk '{print \$1}'`;
      chomp($lineNum);
      print "Total line Number is $lineNum\n";
      $line = `cat $basinFile`;
      chomp($line);
      if (($lineNum <= 3) && ($line =~ /$tempNoSt/i ))
      {
         print "Mode was $mode, switch to create mode - 1\n";
         $mode = $createMode;
      }
   }
}


##-------------------------------------------------------------------------------------------------##

##------------------------ Start to Create/Update the basin HTML text file ------------------------##


if ($mode == $createMode)
{
   if (-s $basinFile)
   {
      # Use Warning other than WARNING to avoid the catch of the monitoring
      print "Warning!! $basinFile will be overwritten!\n";
      rename($basinFile, $oldFile)          or die "can't rename $basinFile to $oldFile: $!";
   }
   
   if (!(-s $blankFile))
   {
      print "ERROR:$blankFile is not exist.  Exit program!\n";
      exit 1;
   }

   system("sed 's/$tempNoSt/$tempHaveSt/' $blankFile > $basinFile");  
}

$stormLine = `cat $basinLnTemplateFile`;
chomp($stormLine);
$stormLine =~ s/$tempStorm/$stormName/g;

# May need to change to if mode =1 or 2, then...
# open the file to insert for two mode. before the <div>
if (($mode == $createMode) || ($mode == $updateMode))
{
   #---------------- Update the basin file to include the storm entry in that basin ----------------#

   print "The added stormLine is $stormLine\n";
 
   open(BASINFILE, "< $basinFile") || die "Can not open $basinFile for reading: $!";
   @dataList = <BASINFILE>;
   close(BASINFILE);

   open(NEWFILE, "> $newFile") || die "Can not open $newFile for writing: $!";
   flock(NEWFILE, 2);         # and lock it
 
   $i = 0;
    
   # The last line is "</div>" 
   #while (($dataList[$i] !~ /^\<\/div\>$/) && ($i < $#dataList+1))
   while (($dataList[$i] !~ /^\<\/div\>/) && ($i < $#dataList+1))
   {
      chomp($dataList[$i]);
      #print "$dataList[$i]\n";
      # Avoid the duplicated entry
      if ($dataList[$i] eq $stormLine)
      {
         print "WARNING:Pay attention! The entry for storm $stormName exists, please check!\n";
      }
      else
      {
         print NEWFILE "$dataList[$i]\n";
      }
      $i++;
   }
  
   print NEWFILE "$stormLine\n";
   chomp($dataList[$i]);  
   print NEWFILE "$dataList[$i]\n";

   flock(NEWFILE, 8);         # and unlock it
   close(NEWFILE) || die "Can not close $newFile: $!";

   rename($basinFile, $oldFile)          or die "can't rename $basinFile to $oldFile: $!";
   rename($newFile, $basinFile)          or die "can't rename $newFile to $basinFile: $!";

   #---------------- Update the getBasin Javascript file to include the storm entry ----------------#

   # Notes that the entry always add in the top, so that if the storm cross basin, the latest basin will be returned.
   
   if (!(-s $getBasinFile))
   {
      print "$getBasinFile is not exist, created now to include the entry for $stormName!\n";
      system("sed -e 's/$tempStorm/$stormName/' -e 's/$tempBasin/$basinArea/' $getBasinTemplateFile > $getBasinFile");
   }
   else
   {
      print "Update $getBasinFile to include the entry for $stormName.\n";
      $stBasinLine = `grep $tempBasin $getBasinTemplateFile`;
      chomp ($stBasinLine);
      $stBasinLine =~ s/$tempStorm/$stormName/;
      $stBasinLine =~ s/$tempBasin/$basinArea/;
      print "The storm-basin line to add is $stBasinLine\n";
      $isStBasinExist = `grep '$stBasinLine' $getBasinFile`;

      if ($isStBasinExist ne "")
      {
         print "WARNING: the storm-basin line exists in the file $getBasinFile.  Skip update...\n";
      }
      else
      {
         open(GETBASINFILE, "< $getBasinFile") || die "Can not open $getBasinFile for reading: $!";
         @dataList = <GETBASINFILE>;
         close(GETBASINFILE);

         open(NEWFILE, "> $newFile") || die "Can not open $newFile for writing: $!";
         flock(NEWFILE, 2);         # and lock it

         foreach $dataLine (@dataList)
         #while (($dataList[$i] !~ /$stBasinLine/) && ($i < $#dataList+1))
         {
            print NEWFILE "$dataLine";
            #print "dataLine is $dataLine";
            # Will check why the following line not work 04/29
            if ($dataLine =~ /$tempSwitchLine/)
            #if ($dataLine =~ /switch\(stormName\)/)
            {
               print "dataLine is $dataLine, add the $stBasinLine after it.\n";
               print NEWFILE "$stBasinLine\n";
            }
         }

         flock(NEWFILE, 8);         # and unlock it
         close(NEWFILE) || die "Can not close $newFile: $!";

         # Check if the storm entry is added
         # Not working with double quote for those contain double quote in the string
         #$isStLineAdded = `grep "$stBasinLine" $newFile`;
         $isStLineAdded = `grep '$stBasinLine' $newFile`;
         if ($isStLineAdded eq "")
         {
            print "ERROR:Failed to add the $stBasinLine to the getBasin file. Exit program!\n";
            exit 1;
         }

         rename($getBasinFile, $getBasinOldFile)    or die "ERROR: can't rename $getBasinFile to $getBasinOldFile: $!";
         rename($newFile, $getBasinFile)            or die "ERROR: can't rename $newFile to $getBasinFile: $!";
      }

   } 
}
##------------------------ Delete mode ------------------------##

elsif ($mode == $deleteMode)
{
   print "The deleted stormLine is $stormLine\n";

   $stormList = `grep -i "\<a href=" $basinFile`;
   chomp ($stormList);

   if ($stormList eq "")
   {
      die "ERROR: There is no any storm entry in the $basinFile, exit program $commandName for Mode 3\n";
   }
   # The following line not work as it contains "?" without escape
   # need to pay attention on the next "elsif" if storm line format is changed!!
   #elsif ($stormList !~ /$stormLine/)
   elsif ($stormList !~ /storm=$stormName\"/)
   {
      die "ERROR: There is no entry for storm $stormName in the $basinFile, exit program $commandName for Mode 3\n";
   }

   @splitMsg = split(/\n/, $stormList);
   $lineNum = @splitMsg;
   print "there is total $lineNum storm entries under $basinArea\n";
   if ($lineNum == 1)
   {
      print "Only 1 entry, so copy the blank file\n";

      if (!(-s $blankFile))
      {
         print "ERROR:$blankFile is not exist.  Exit program!\n";
         exit 1;
      }

      system("cp $blankFile $basinFile");
   }
   elsif ($lineNum > 1)
   {
      print "More than 1 entry, delete the storm entry\n";
      @newDataList = ();

      open(BASINFILE, "< $basinFile") || die "Can not open $basinFile for reading: $!";
      @dataList = <BASINFILE>;
      close(BASINFILE);

      for ($i = 0; $i < $#dataList+1; $i++)
      {
         chomp($dataList[$i]);
         # The following line not work as it contains "?" without escape, second line works
         #if ($dataList[$i] !~ /$stormLine/)
         #if ($dataList[$i] !~ /storm=$stormName\"/)
         if ($dataList[$i] ne "$stormLine")
         { 
            print "$dataList[$i]\n";
            push(@newDataList, $dataList[$i]);
         }
   
      }
    
      open(NEWFILE, "> $newFile") || die "Can not open $newFile for writing: $!";
      flock(NEWFILE, 2);  # and lock it

      for ($i = 0; $i < $#newDataList+1; $i++)
      {
         print NEWFILE "$newDataList[$i]\n";
      }
      close(NEWFILE);

      rename($basinFile, $oldFile)          or die "can't rename $basinFile to $oldFile: $!";
      rename($newFile, $basinFile)          or die "can't rename $newFile to $basinFile: $!";

   }  	# End of the else if for line numbe > 1
   else
   {
      print "line Number is $lineNum.\n";
      die "ERROR: There is no any storm entry in the $basinFile, exit program $commandName for Mode 3\n";
   }
}

#system("chmod +x $basinFile");
chmod(0755, $basinFile);

$enddate=`date`;
print "\nEnd of the program $0 at $enddate \n\n";

