!-------------------------------------------------------------------------------
!
! NAME:
!   Type_Kinds
!
! FUNCTION:
!   Module to hold specification kinds, with parameter attributes, for
!   variable declarations.
!
! DESCRIPTION:
!   Define type kinds for these specific variable types:
!    Byte   - Specification kind for byte integer variable
!    Short  - Specification kind for short integer variable
!    Long   - Specification kind for long integer variable
!    DLong  - Specification kind for double long integer variable
!    Single - Specification kind for single precision real variable
!    Double - Specification kind for double precision real variable
!
! REFERENCE:
!
! CALLING SEQUENCE:
!   USE type_kinds
!
! INPUTS:
!
! OUTPUTS:
!
! DEPENDENCIES:
!
! RESTRICTIONS:
!
! HISTORY:
!   1998/06/17 19:54:55  walterw - Initial revision
!
!   1997/12/01 20:21:50  paulv   - Initial revision
!
!-------------------------------------------------------------------------------

MODULE Type_Kinds
 
  IMPLICIT NONE 
 
  INTEGER, PARAMETER :: BYTE   = SELECTED_INT_KIND(1)  ! Byte integer
  INTEGER, PARAMETER :: SHORT  = SELECTED_INT_KIND(4)  ! Short integer
  INTEGER, PARAMETER :: LONG   = SELECTED_INT_KIND(8)  ! Long integer
  INTEGER, PARAMETER :: DLONG  = SELECTED_INT_KIND(10) ! Double Long integer
  INTEGER, PARAMETER :: SINGLE = SELECTED_REAL_KIND(6) ! Single precision
  INTEGER, PARAMETER :: DOUBLE = SELECTED_REAL_KIND(12)! Double precision 

END MODULE Type_Kinds

