#!/usr/bin/perl -w

## =====================================================================
## Name:        area2digit-multi-fullyAuto.pl
##                  
## Author:      Jianbin Yang/RSIS   04/03/2003
## Language:    Perl
## Purpose:     Convert the 6 hourly and 24 hour total TRaP product from
##              Mcidas AREA format to text format (digital TRaP) and
##              send out an notification message to a specific customer
##              list according to the area the storm fell in.
##                  
### Usage:       area2digit.pl <TRaP AREA file number> [<GIF file>] [<Actual time of Rain rate data-- hhmm>] [<date>] 
### Usage:       area2digit-multi.pl [<TRaP AREA file number>] [<GIF file>][<Actual time of Rain rate data-- hhmm>] [<date>] 
##
## Usage:       area2digit-multi-fullyAuto.pl [<TRaP AREA file number>] [<Position file>] [<Bulletin file>]
##                                            [<GIF file>][<Actual time of Rain rate data-- hhmm>] [<date>] 
##
## Note:        For 24 hour total TRaP the AREA file number is 647 or 637
##              In this program it is expect the input is the 24 hour toatl TRaP
##              This program is expect to run after the program "restoreAREA.pl"
##              is called, so that the positon file can be archived
##              "GIF file" -- The GIF file name of the 24 total TRaP GIF sent to SAB homepage   
##
## Please note that: 
##              The position track from the pctrap.pl is used Mcidas convention
##              That means east longitude is a negative number.  
##              But when the digital trap should use the international convention
##	        as the east longitude is a positive number.  So it is coded to 
##		convert back to international convention.
##		And noted that the second input argument "Actual time of Rain rate data"
##		is the time orbit passed over the storm.  It is input by SAB analyst currently.
##		It will be used in the file name of the digital TRaP.  If there is no input, 
##		The composite time of the image will be uesed as default.
##              
## Modified at:   08/10/2003       To switch the formal email list as operational 
##
## Modified: 08/18/2003
##
## NOTED: The orbit time input format maybe different. Will confirm this.   07/10/2003 
##
## Modified: 11/20/2003
##
## NOTED: Not input the orbit time, since it is got from the POS text file. 
##
## Modified: 01/16/2004
##
## NOTED: Produce four 6 hourly digital TRaPs as well as 24 total digital TRaP.
##
## Modified: 02/03/2004
##
## NOTED: Change the TPC/CPHC WMO header to WT**2*
##
## Modified: 03/22/2004
##        Remove the longitude restriction for the Australia bulletins
##        So that there is no missing case if the longitude not fall in the range
##        That was a missing case for "FAY" (center at 127 East, bulletin WTAU03 ) last week
##
## Modified: 07/28/2004
##
## Modified: 08/18/2004
##        DNS changes for FTP server
##        (from "gp16.wwb.noaa.gov" to "gp16.ssd.nesdis.noaa.gov")
##
## Modified: 03/14/2005
##        Add the ICAO bulletin FKAU to the "getBasin" subroutine          
##
## Modified: 09/19/2006
##        Modify for Fully Automate TRaP, the program is called within the Main TRaP program.
##        The customer email notification is turn off to reduce the frequency of the notification.
##        Write the TRaP list and email list to other file for other program to read.         
## 
##   Notes: will need to modify the name POS2.TXT and take care of the line to get the TRaP set name
##          also the rawa GIF file --- on 09/19/2006
##          will send the GIF TRaP files to SAB Web page, and call createTRaPDir.pl inside ftp -- on 09/20/06 
##
## Modified: 10/10/2006
##        Will write the basin name in a text file for every storm 
##
## Modified: 03/05/2007
##	  Break down the boundaries for the ICAO FKAU bulletin
##
## Modified: 03/19/2007
## 	  Set the basin and basin abbreviation as hash table
##
## Modified: 04/10/2007
##        Remove the latitude restriction on the Southeast and Southwest Pacific,
##	  Remove the Usage of "OtherList", etc.
##
## Modified: 06/14/2007
##	  Write the digital TRaP to SABNA4
##
## Modified: 06/26/2007
## 	  Add New Delhi and New Zealand RSMC bulletins
##        Add FKPS01 for Fiji and FKIO20 for Reunion  	07/05/2007
##
## Modified: 10/23/2007
##        Distribute the TRaP GIF files to SATEPS Server for GEONETCast
##
## Modified: 01/11/2008
##        Change the Southern hemisphere boundary from 130 to 125
##
## Modified: 02/04/2008
##        Additionally send the bulletin, rain rate data out to users
##        along with the digital TRaP
##
## Modified: 02/25/2008
##        Create a single ftp directory where customers can find the 
##        latest TRaPs for all storms using the storm name as the name of the file
##
## Modified 04/14/2008
##         Added functionality for AXAU bulletins
##
## Modified 05/01/2008
##         Update the active storm list file with new storm when the first TRaP from the storm is created
##         (notes: remove of the storm entry will be done in the cleanTRaP program)
##
## Modified 05/29/2008
##         Add the NSOF GP5 Web server
##
## Modified: 02/06/09
##         Changed to use $HOME without specifying the oper account
##
## Modified 12/26/2008
##         Add Jakarta bulletin
##
## Last Modified: 04/08/09
##         Updated the active storm list if the storm cross basins (for etrap)
##
## =====================================================================

#--------- Define the directories and files ---------#

$HOME = $ENV{"HOME"};

$srcDir = "$HOME/mcidas/pctrap-V3";
$dataDir = "$HOME/mcidas/data";
$TRaPDataDir = '/data/TRAP';
$digitTRAP_sent_Dir = "$TRaPDataDir/digitTRAP";  	# it holds the digit TRaP sent to FTP server

# File to keep the position of the track of the storm and information to generate the digital TRaP.
# PLEASE NOTE: This file name must be CONSISTENT as the one in restoreAREA.pl, which was actually
# generated from pctrap.pl as the TRaP generated.
$positionFile = 'POS1.TXT';  	     	
$bulFile = 'POS.DOC';          # This file hold the bulletin content for specific storm
 
# $totalGifFile = 'TRAP24hh.GIF';         # File got from reastoreAREA.pl, as the default of the GIF file
# $totalGifFile = 'SSMI24A.GIF';            # will use the previous line when new SENDTRAP(3) is used 
$gifPrefix = 'TRAP';                      # MUST BE CONSISTENT with the one in restoreAREA.pl

$digitFile1 = 'digit1.txt';             # File to hold the first part of the digital TRaP
$digitFile2 = 'digit2.txt';		# File to hold the text format TRaP(part 2), not the final file 	
$digitFile = 'digit.txt';             # the final digital file

$sftpfile1 = '/tmp/ftpSeverRmtPut1';
$sftpfile2 = '/tmp/ftpSeverRmtPut2';
$sftpfile3 = '/tmp/ftpSeverRmtPut3';
$sftpfile4 = '/tmp/ftpSeverRmtPut4';
$sftpfile5 = '/tmp/ftpSeverRmtPut5';

# The log file for this program
$stdoutLog = 'DIGIT.OUT';
$stderrLog = 'DIGIT.ERR';
$logFile = '/data/TRAP/log/digitTRAP.log';

# file to keep the new TRaP set name for customer email notification
$listFile = "$dataDir/TRaPList.txt";

# file list the basin name actually is same as the subdirectory name 
$basinFile = "basinName.txt";

# the active storm list file
$stormFile = "$srcDir/activeStorms.txt"; 



##-------------------- Define some constants --------------------##

#--------- basins and basin abbreviation ---------#

# Define the hash table for the basin (dir here) and the corresponding basin abbreviation (basin here)
%basinTbl = qw(
   Atlantic AL 
   EastPacific EP 
   CentralPacific CP
   NWPacific WP
   SWPacific SW
   SEPacific SE
   NorthIndian NI
   SWIndian WI
   SEIndian EI
   NTerritory NT 
);

#--------- basins ---------#
$Atlantic_basin 		= 'AL';
$EastPacific_basin 		= 'EP';
$CentralPacific_basin 		= 'CP';
$NorthwestPacific_basin 	= 'WP';
$SouthwestPacific_basin         = 'SW';
$SoutheastPacific_basin         = 'SE';
$NorthIndian_basin 		= 'NI';
$SouthwestIndian_basin		= 'WI';
$SoutheastIndian_basin		= 'EI';
$NorthernTerritory_basin	= 'NT';
# The 'OT' is for Southease Pacific(south of 25S)
# $Other_basin			= 'OT';		


#--------- SAB Tropical Cyclone Homepage URL ---------#
$sabURL = 'http://www.ssd.noaa.gov/PS/TROP/trap-img.html';


#--------- anonymous ftp server and directories ---------#

# the domain name is not working since 11/25/08 to 11/26/08 
#$ftpServer1 = 'oper\@gp16.ssd.nesdis.noaa.gov';
$ftpServer1 = 'oper\@140.90.195.198';
$ftpServer2 = 'oper\@satepsanon.nesdis.noaa.gov';
#$ftpServer2 = 'oper\@gp32';
# $ftpHomeDir ='/data/ftp/pub/TRAP';
$ftpHomeDir ='pub/TRAP';
# $ftpHomeDir ='pub/testTRAP';       # for testing purpose, will use the previous when become operational
# $ftpURL = 'ftp://gp16.wwb.noaa.gov/pub/TRAP/';
$ftpURL = 'ftp://gp16.ssd.nesdis.noaa.gov/' . $ftpHomeDir;

$readmeDir = 'README';

#--------- SAB Data Server ---------#

#$sabDataServer = 'sab\@sabna4.ssd.nesdis.noaa.gov';
$sabDataServer = 'sab\@140.90.195.121';
$sabDataHomeDir = "/home/sab/TRAP";
$sabDataTrapDir = "/data/TRAP/digitTRAP";
$sabDataRemoteCM = "$sabDataHomeDir/createDigDir.pl";


#--------- SAB Web server and directories ---------#

# $webUserName = "sabweb";
# $webServer1 = 'sabweb\@140.90.46.29';
$webServer1 = 'sabweb\@gp5.ssd.nesdis.noaa.gov';
$webServer2 = 'sabweb\@10.144.3.87';
# $remoteTrapDir = "/data/web/mirror/www.ssd.noaa.gov/htdocs/PS/TROP/DATA/TRAP/";
$remoteHomeDir = "/home/sabweb/TRAP";
# $remoteCM = "./createTRaPDir.pl";
$remoteCM = "$remoteHomeDir/createTRaPDir.pl";


#--------- SATEPS Product Server server and directories ---------#
# scp $nn*GIF spider@satepsdist1e.nesdis.noaa.gov:/data/GEONET/

# changed this at 17:30Z, 11/24/08
#$satepServer = "spider\@satepsdist1e.nesdis.noaa.gov";
$satepServer = "spider\@satepsdist1.nesdis.noaa.gov";
$remoteGeonetDir = "/data/GEONET/data";


#--------- Basins subdirectories ---------#

$Atlantic_dir 			= 'Atlantic';
$EastPacific_dir 		= 'EastPacific';
$CentralPacific_dir 		= 'CentralPacific';
$NorthwestPacific_dir 		= 'NWPacific';
$SouthwestPacific_dir 		= 'SWPacific';
$SoutheastPacific_dir 		= 'SEPacific';
$NorthIndian_dir 		= 'NorthIndian';
$SouthwestIndian_dir 		= 'SWIndian';
$SoutheastIndian_dir 		= 'SEIndian';
$NorthernTerritory_dir 		= 'NTerritory';
# $Other_dir			= 'Other';

#--------- the email notification list of different agent ---------#

# New define email list names are as following, must match the names in "TRaPEmailList.txt".  Added on 09/11/2006
# TPCList1, TPCList, AtlanticList, CPHCList, 
# PerthList, DarwinList, BrisbaneList, ReunionList, NewDelhiList
# NadiList, NWPacificList, and BCCList.

$TPCList 	= "TPCList";
$AtlanticList 	= "AtlanticList";
$CPHCList 	= "CPHCList";
$PerthList 	= "PerthList";
$DarwinList 	= "DarwinList";
$BrisbaneList 	= "BrisbaneList";
$ReunionList 	= "ReunionList";
$NewDelhiList 	= "NewDelhiList";
$NadiList 	= "NadiList";
$NWPacificList 	= "NWPacificList";
$ccList 	= "BCCList";
# $OtherList      = ""; 

# $MWOGuamList   = "";
# $TokyoList     = "";

# $testingList = "jianbin.yang\@noaa.gov";

# $TPCList       = $testingList;
# $AtlanticList  = $testingList;
# $CPHCList      = $testingList;
# $NWPacificList = $testingList;
## $MWOGuamList	  = $testingList;
# $ReunionList   = $testingList;
# $NewDelhiList  = $testingList;
# $PerthList     = $testingList;
# $DarwinList    = $testingList;
# $BrisbaneList  = $testingList;
# $NadiList      = $testingList;		# For Fiji
## $TokyoList     = $testingList;
# $OtherList     = "";				# Empty for now, will fill in if there is any

##-------------------- initialize some variables --------------------##

# the following can be get from the "getBasin" function
$basin = "";
# $emailList = "";		# emailList maybe empty even after the "getbasin" is called for JTWC
@emailList = ();		# emailList maybe empty even after the "getbasin" is called for JTWC, for SWPacifc? at 09/11/2006
$subDir = "";
$remoteTRAPDir = "";		# destination directory for the digital file in the FTP server
$basinArea = "";		# similar to "subdirectories", for TRaP List file 


####----------------------------------------------------------------------------####

####----------------------------  Start the program ----------------------------####

$startdate=`date`;
print "Start  $0 at $startdate \n";

chdir("$dataDir") || die "Cannot change to directory $dataDir";
print  "Data directory is:";
system("pwd");

# remove the path from the command name

@comandFields = split(/\//, $0);
$commandName = $comandFields[-1];

# if no argument input, exit the program

if (@ARGV < 1)
{
   print "Usage: $commandName [<TRaP AREA file number>] [<Position file>] [<Bulletin file>]";
   print " [<GIF file>][<Actual time of Rain rate data-- hhmm>] [<date>]\n";
 #  print "Usage: $commandName [<TRaP AREA file number>] [<GIF file>] [<Actual time of Rain rate -- hhmm>] [<date>]\n";
   die "No name input - exit script";
   print "Use default AREA number 0642 -0647.\n";
   # $areaNumber = "647";
   $areaNumber = "637";
   $orbitTime = "";
}
elsif (@ARGV == 1)
{
   $areaNumber = $ARGV[0];
   $orbitTime = "";         # if it is empty, will use the time in the end of the TRaP set name as default
}
elsif (@ARGV == 2)
{
   $areaNumber = $ARGV[0];
   $positionFile = $ARGV[1];
   $orbitTime = "";
}
elsif (@ARGV == 3)
{
   $areaNumber = $ARGV[0];
   $positionFile = $ARGV[1];
   $bulFile = $ARGV[2];
   $orbitTime = "";
}

elsif (@ARGV == 4)
{
   $areaNumber = $ARGV[0];
   $positionFile = $ARGV[1];
   $bulFile = $ARGV[2];
   $totalGifFile = $ARGV[3];
   $orbitTime = ""; 
}
elsif (@ARGV == 5)
{
   $areaNumber = $ARGV[0];
   $positionFile = $ARGV[1];
   $bulFile = $ARGV[2];
   $totalGifFile = $ARGV[3];
   $orbitTime = $ARGV[4];  
}
elsif (@ARGV >= 6)
{
   $areaNumber = $ARGV[0];
   $positionFile = $ARGV[1];
   $bulFile = $ARGV[2];
   $totalGifFile = $ARGV[3];
   $orbitHhmm = $ARGV[4];
   $orbitDd = $ARGV[5];
   if ($orbitDd < 10)
   {
      $orbitDd = '0' . $orbitDd;
   }
   # get the orbit time ddhhmm
   $orbitTime = $orbitDd . $orbitHhmm;
}

# get the other AREA files number, noted that $areaNumber is for 24 hpout total

# for 0-6 hour, the number is 643 or 633
$areaNumber0 = substr($areaNumber, 0, -1) . '3'; 
# for 6-12 hour, the number is 644 or 634
$areaNumber6 = substr($areaNumber, 0, -1) . '4'; 
# for 12-18 hour, the number is 645 or 635
$areaNumber12 = substr($areaNumber, 0, -1) . '5'; 
# for 18-24 hour, the number is 646 or 636
$areaNumber18 = substr($areaNumber, 0, -1) . '6'; 

# for rain rate raw data the number is 630 or 640
$areaNumberRR = substr($areaNumber, 0, -1) . '0';

if (($orbitTime ne "") && ($orbitTime !~ /^\d{6}$/))
{
   print "The input orbit time is $orbitTime\n";
   die "Wrong input orbit time, should be in 'DDHHMM' or 'HHMM DD'format - exit script";
} 


####-----------------------------------------------------------------####

####------ Convert the input AREA TRaP to text format TRaP ------####
####       This is part 2 of the final digital TRaP              ####

print "Start to conver the AREA TRaP file to digital TRaP.\n";
 
# convert the 24 hour total AREA TRaP to digital TRaP, and use it to get the part 1
# use 40 inches scale

#system("digitaltrap3.k $areaNumber $digitFile2  DEV=T temptext");
system("digitaltrap3.k $areaNumber $digitFile2 40 DEV=T temptext");


####------ Get the forecast bulletin and position information ------####
####       This is part 1 of the final digital TRaP                 ####

## The coverage area are all the same between the 24 hour total and 6 hourly   ##
## so the part 1 is common for all the 6 hourly and 24 hour total digital TRaP ##

open (POSTXT, $positionFile) || die "can't open $positionFile: $!";
@posLines = <POSTXT>;
close (POSTXT) || die "can't close $positionFile: $!";

# redirect the print out to some files, to avoid the mcidas wondow error

open STDOUT, "> $stdoutLog" || die "Can't create $stdoutLog:$!";
open STDERR, "> $stderrLog" || die "Can't create $stderrLog:$!";


##------ Get the first line(header?) of the first part of digital TRaP ------##

@tempMsg = split(/\s+/, $posLines[0]);
$lat_0 = $tempMsg[-2];
$lon_0 = $tempMsg[-1];


# !!!!!!!!!!!!!!!
# !!!!!Noted on 08/13/2002, May need to get the basin from WTPA22 or WTPZ22 in getBulInfo.pl
# and write into POS.TXT, since if storm moved from E.Pac to C.pac, such as storm GUILLERMO,
# then the basin is still "EP", it is better to get the basin from bulletin WTPA22 other than determined by
# the WMO header itself. So plan to write the basin in the end of position file for WTPA /WTPZ bulletin
# in getbulInfo.pl, and give a specific name to the following tempMsg, so that in the getBasin function
# can get the basin in the end of @tempMsg
#!!!!!!!!!!!!!!!!

## Noted at 08/29
# will check if there is error msg when there is no basin in the POS file (the following line)

@tempMsg = split(/\s+/, $posLines[5]);
($stormName, $stNumber, $bulYjd, $wmoHd, $station, $bulTime, $curTime, $forecastHour, $basin) = @tempMsg;

# Change the year on 06/14/2007, use the rain rate year other than the forecast year, this consistent with the file name 
# $year = substr($bulYjd, 0, 4);
$year = substr($posLines[6], 0, 4);
## chomp($forecastHour);


print "the stormname is $stormName, bulYjd is $bulYjd, year is $year, wmoHd is $wmoHd, 
       bulTime is $bulTime, curtime is $curTime, stNumber is $stNumber, forecastHour is $forecastHour\n"; 


##----------------------------------------------------------------------------##

##------------- Get the basin, subdirecotry and email list to send -----------##

# Please Noted that the "$lon_0" is Micdas convention
# which is reverse to regular convention, so reverse it here
getBasin($wmoHd, $station, $lat_0, (-1) * $lon_0);

# Check if the basin is got succefully, emailList can be empty if it is from JTWC
if (($basin eq "") || ($subDir eq ""))
{
   print "ERROR!! Could not get the basin or subdirectory for the digital TRaP file in 
          \rprogram $commandName. Please contact the programmer.\n";
   die "-- exit script";
}

$remoteTRAPDir = "$ftpHomeDir/$subDir";
$finalURL = "$ftpURL/$subDir";

# Fixed directory for GRACS users
$remoteTRAPDir2 = "$ftpHomeDir/$year";
$finalURL2 = "$ftpURL/$year";
 
print "Basin just got, the basin is $basin\n";

# write the AREA file information into the digit1 file part 1
open (DIGIT1, ">$digitFile1") || die "can't create $digitFile1: $!";
printf DIGIT1 "%s%s%s %s%s %s\n", $basin, $stNumber, $year, $curTime, $forecastHour, $stormName;


##-------------------------------------------------------------------##

##------ Get the second line of the first part of digital TRaP ------##

# get the start and end lat and lon

$line = `head -1 $digitFile2`;
@pos = split(/\s+/, $line);
print "head pos1 is $pos[0], 2 is $pos[1], 3 is $pos[2]\n";
$startLon = $pos[1];
$startLat = $pos[2];

$line = `tail -1 $digitFile2`;
@pos = split(/\s+/, $line);
print "tail pos1 is $pos[0], 2 is $pos[1], 3 is $pos[2]\n";
$endLon = $pos[1];
$endLat = $pos[2];

# get the lines and elements number in the AREA file

# @lines = `imglist.k LA.$areaNumber FOR=MEMO`;
# $line = $lines[3];
# print "the imglist line is \n$line\n";
# @tempMsg = split(/\s+/, $line);
# $lsize = $tempMsg[10];
# $esize = $tempMsg[11];
# print "the lines size is $lsize and the element size is $esize\n";

## get the line and element number and resolution degree

# @lines = `imglist.k LA.$areaNumber FOR=EXP`;    # this is not working under Mcidas window
system("imglist.k LA.$areaNumber FOR=EXP DEV=T tmp.txt");
open(TMP, "tmp.txt") || die "Can not open 'tmp.txt': $!";
@lines = <TMP>;
close(TMP) || die "can't close 'tmp.txt': $!";
unlink "tmp.txt";

# get the line and element number

foreach $line (@lines)
{
   # $line = $lines[5];
   if ($line =~ /^\s+Band:.*\d$/)
   {
      @tempMsg = split(/\s+/, $line);
      $lsize = $tempMsg[-3];
      $esize = $tempMsg[-1];
   }
#else
#{
#   die "The format is unexpected, cann't get the line and element number--exit.";
#}

   # get the line and element resolution in km

   # $line = $lines[12];
   if ($line =~ /^\s+Resolution Factors.*\d$/)
   {
      @tempMsg = split(/\s+/, $line);
      $lres = $tempMsg[-3];
      $eres = $tempMsg[-1];
   }
#else
#{
#   die "The format is unexpected, cann't get the line and element resolution--exit.";
#}

}

# convert the km resolution to degree resolution

# $lresDegree = `cu.k $lres KM DEGL`;    # this is not working under Micdas window
system("cu.k $lres KM DEGL DEV=T cuout");
system("cu.k $eres KM DEGL DEV=T cuout A");     	# append the output to the cuout file
open(CUOUT, "cuout") || die "Can not open 'cuout': $!";
$lresDegree = <CUOUT>;
$eresDegree = <CUOUT>;
close(CUOUT) || die "can't close 'cuout': $!";;
unlink "cuout";

@tempMsg = split(/\s+/, $lresDegree);
$lresDegree = $tempMsg[3]; 

# $eresDegree = `cu.k $eres KM DEGL`;
@tempMsg = split(/\s+/, $eresDegree);
$eresDegree = $tempMsg[3];

# print "the lines size is $lsize and the element size is $esize, line-res is $lres, element-res is $eres
#       grid spacing degree are $lresDegree and $eresDegree\n";


printf DIGIT1 "%6d%9.3f%10.3f%10.3f%6d%9.3f%10.3f%10.3f\n", $lsize, $lresDegree, 
               $startLon, $endLon, $esize, $eresDegree, $startLat, $endLat;


##----------------------------------------------------------------------##

##------ Get the position track in the first part of digital TRaP ------##

## PLEASE NOTED THAT:
## The longitude has to multiple by -1 to convert back from Mcidas convention

# get the 0, 12, and 24 hour forecast positions
# get the position track for every 12 hour

for ($i = 0; $i < 6; $i += 2)
{
   @tempMsg = split(/\s+/, $posLines[$i]);
   # ($hh, $lat, $lon) = @tempMsg;  not work well, since there is space before 0 or 6 hour
   $hh = $tempMsg[-3];
   $lat = $tempMsg[-2];
   $lon = $tempMsg[-1];
   $lon *= -1;
 #  print "hh is $hh, lat is $lat, lon is $lon\n";
   printf DIGIT1 "%9.2f%9.2f%7.1f\n", $lon, $lat, $hh;
}

$lat = 0;
$lon = 0;
for ($i = 36; $i <= 72; $i += 12)
{
   printf DIGIT1 "%9.2f%9.2f%7.1f\n", $lon, $lat, $i;
}

close (DIGIT1) || die "can't close $digitFile1: $!";

# the 24 hours total digital file is got here.
system("cat $digitFile1  $digitFile2 > $digitFile"); 

$success=$?;
if ($success != 0)
{
   print "!!Warning: Digital TRaP conversion not success.\n";
   die "TRaP Files not found, please input the correct TRaP Area number - exit script.";
}

# unlink("$digitFile1") || die "cannot unlink $digitFile1: $!";
# unlink("$digitFile2") || die "cannot unlink $digitFile2: $!";


##-------------------------------------------------------------##

##------ Get the full file name of the digital TRaP file ------##

$fileNamePre = $posLines[6];
chomp($fileNamePre);

# get the satellite name
$dataSource = substr($fileNamePre, -11, 4);

# some TRaP is with "-2" in the end of the regular file name,
# when there is another set of TRaP with the same name
if ( $fileNamePre =~ /-2$/)
{
   $dataSource = substr($fileNamePre, -13, 4);
}

# replace the data source time (last 6 characters) with the input actual orbit time
if ($orbitTime ne "")
{
   substr($fileNamePre, -6) = $orbitTime;
}
else
{
   $orbitTime = substr($fileNamePre, -6);
   # some TRaP is with "-2" in the end of the regular file name,
   # when there is another set of TRaP with the same name
   if ( $fileNamePre =~ /-2$/)
   {
      $orbitTime = substr($fileNamePre, -8, 6);
   }
}

# print "the orbit time is $orbitTime, the filename is $fileNamePre\n";

## get the 24 hour total digital TRaP ##

$fileName = $fileNamePre . ".24";
# $fileName = $fileNamePre . ".24.hi"; 	# change at Thu Oct 19 15:23:16 UTC 2006

# print "The final filename is $fileName\n"; 
system("pwd");

#-----------  create an extract copy of the digital TRaP for GRACS users -----------#
$digit24File = $stormName . '-' . $basin . '.txt';

print "the extract copy digit24File is $digit24File\n";
system("cp $digitFile $digit24File");
system("compress -f $digit24File");
$digit24File = $digit24File . '.Z';

system("mv $digitFile '$fileName.txt'");
system("cp AREA0$areaNumber $fileName");

system("cp $gifPrefix'24'.GIF '$fileName.GIF'");

## Save the Raw data and GIF file ##

$fileName = $fileNamePre . ".raw";
system("cp AREA0$areaNumberRR $fileName");

$gifRR = $gifPrefix . "rr";

system("cp $gifRR.GIF '$fileName.GIF'");

## save the bulletin and position file ##
$fileName = $fileNamePre . ".txt";
system("cp $positionFile $fileName");

$fileName = $fileNamePre . ".bul";
system("cp $bulFile $fileName");

##----------------------------------------------------------------------------##

##----------------------  Get the 6 hourly digital TRaPs ---------------------##

# for 0-6 hour TRaP, use 20 inches scale
system("digitaltrap3.k $areaNumber0 $digitFile2 20 DEV=T temptext");
system("cat $digitFile1  $digitFile2 > $digitFile");
$fileName = $fileNamePre . ".00";
system("mv $digitFile '$fileName.txt'");
system("cp AREA0$areaNumber0 $fileName");
system("cp $gifPrefix'00'.GIF '$fileName.GIF'");

## work up to here 01/16/2004, will get the other gif files from reastoreAREA.pl or 
## other place , will modify the image time by lwu.k POKE in pctrap-new.pl
 
## system("cp $totalGifFile '$fileName.GIF'");

# for 6-12 hour TRaP
system("digitaltrap3.k $areaNumber6 $digitFile2 20 DEV=T temptext");
system("cat $digitFile1  $digitFile2 > $digitFile");
$fileName = $fileNamePre . ".06";
system("mv $digitFile '$fileName.txt'");
system("cp AREA0$areaNumber6 $fileName");
system("cp $gifPrefix'06'.GIF '$fileName.GIF'");

# for 12-18 hour TRaP
system("digitaltrap3.k $areaNumber12 $digitFile2 20 DEV=T temptext");
system("cat $digitFile1  $digitFile2 > $digitFile");
$fileName = $fileNamePre . ".12";
system("mv $digitFile '$fileName.txt'");
system("cp AREA0$areaNumber12 $fileName");
system("cp $gifPrefix'12'.GIF '$fileName.GIF'");

# for 18-24 hour TRaP
system("digitaltrap3.k $areaNumber18 $digitFile2 20 DEV=T temptext");
system("cat $digitFile1  $digitFile2 > $digitFile");
$fileName = $fileNamePre . ".18";
system("mv $digitFile '$fileName.txt'");
system("cp AREA0$areaNumber18 $fileName");
system("cp $gifPrefix'18'.GIF '$fileName.GIF'");

# can not use -f here as the GIF will compress as well
#system("compress -f $fileNamePre*");
system("compress $fileNamePre*");
# system("cp $totalGifFile '$fileName.GIF'");

### 04/01/04 check up to here, will check the following. and check if the compress work well.
### 02/26/04, work up to here, will check with the GIF file, and change th mail content to include the 6 hourly TRAPs.


unlink("$digitFile1") || die "cannot unlink $digitFile1: $!";
unlink("$digitFile2") || die "cannot unlink $digitFile2: $!";


##------------------------------------------------------------------------------##

##------ Send (SFTP) out the digital and AREA TRaP to FTP server 1 - GP16 ------##

# Will send the file to FTP server(gp16). if the emailList is not empty, then 
# send email to the corresponding people in the following

print " Create sftp script to send the TRaP files to $ftpServer1\n";

# the absolute remote ftp direcotry
$abs_remoteTRAPDir = "/data/ftp/" . $remoteTRAPDir;
$abs_remoteTRAPDir2 = "/data/ftp/" . $remoteTRAPDir2;

open  ( FTPOUT1,  ">$sftpfile1" )  ||  die " Can't open $sftpfile1 to write: $!";

   print  FTPOUT1   "# SFTP command file:  Put digit files to FTP server\n";
   print  FTPOUT1   " sftp $ftpServer1 << EOF\n";
   print  FTPOUT1   "   cd  $abs_remoteTRAPDir\n";
   print  FTPOUT1   "   mput  $fileNamePre*\n";
   print  FTPOUT1   "   cd  $abs_remoteTRAPDir2\n";
   print  FTPOUT1   "   put  $digit24File\n";
   print  FTPOUT1   "   bye\n";
   print  FTPOUT1   "EOF\n";

close ( FTPOUT1 ) || die "Can't close $sftpfile: $!";
chmod  (0777,  $sftpfile1) ;

# Run the sftp file to get the files from NASA
$enddate=`date`;
print " Start  sftp digit files to $ftpServer1 at $enddate \n";
open ( SFTPOUT,  " $sftpfile1  | " ) || die " Can't open $sftpfile1 to run: $!";

@sftpOutput = <SFTPOUT>;

close (SFTPOUT ) || die "Can't close $sftpfile1: $!";

##------ Send (SFTP) out the digital and AREA TRaP to FTP server 2 - GP32 ------##

# Will send the file to FTP server(gp32). if the emailList is not empty, then
# send email to the corresponding people in the following

print " Create sftp script to send the TRaP files to $ftpServer2\n";

# the absolute remote ftp direcotry
$abs_remoteTRAPDir = "/data/ftp/" . $remoteTRAPDir;

open  ( FTPOUT2,  ">$sftpfile2" )  ||  die " Can't open $sftpfile2 to write: $!";

   print  FTPOUT2   "# SFTP command file:  Put digit files to FTP server\n";
   print  FTPOUT2   " sftp $ftpServer2 << EOF\n";
   print  FTPOUT2   "   cd  $abs_remoteTRAPDir\n";
   print  FTPOUT2   "   mput  $fileNamePre*\n";
   print  FTPOUT2   "   cd  $abs_remoteTRAPDir2\n";
   print  FTPOUT2   "   put  $digit24File\n";
   print  FTPOUT2   "   bye\n";
   print  FTPOUT2   "EOF\n";

close ( FTPOUT2 ) || die "Can't close $sftpfile2: $!";
chmod  (0777,  $sftpfile2) ;

# Run the sftp file to get the files from NASA
$enddate=`date`;
print " Start  sftp digit files tp $ftpServer2 at $enddate \n";
open ( SFTPOUT,  " $sftpfile2 | " ) || die " Can't open $sftpfile2 to run: $!";

@sftpOutput = <SFTPOUT>;

close (SFTPOUT ) || die "Can't close $sftpfile2: $!";


##--------------------------------------------------------------------------------------------##

##------------------------ Send the TRaP GIF files to SAB Web server  ------------------------##

## code here on 09/20/2006

open  ( FTPOUT3,  ">$sftpfile3" )  ||  die " Can't open $sftpfile3 to write: $!";

   print  FTPOUT3   "# SFTP command file:  Put GIF files to Web server\n";
   print  FTPOUT3   " sftp $webServer2 << EOF\n";
   print  FTPOUT3   "   mput  $fileNamePre*.GIF\n";
   print  FTPOUT3   "   bye\n";
   print  FTPOUT3   "EOF\n";

close ( FTPOUT3 ) || die "Can't close $sftpfile3: $!";
chmod  (0777,  $sftpfile3) ;

# Run the sftp file to get the files from NASA
$enddate=`date`;
print " Start  sftp files to $webServer2 at $enddate \n";
open ( SFTPOUT,  " $sftpfile3 | " ) || die " Can't open $sftpfile3 to run: $!";

@sftpOutput = <SFTPOUT>;

close (SFTPOUT ) || die "Can't close $sftpfile3: $!";

system ("ssh $webServer2 $remoteCM  $fileNamePre $basinArea");

#---------- Send to another Web server NSOF GP5 ----------#

#print "\nSend the TRaP data to another Web server: NSOF GP5!\n";

#open  ( FTPOUT5,  ">$sftpfile5" )  ||  die " Can't open $sftpfile5 to write: $!";

#   print  FTPOUT5   "# SFTP command file:  Put GIF files to Web server\n";
#   print  FTPOUT5   " sftp $webServer2 << EOF\n";
#   print  FTPOUT5   "   mput  $fileNamePre*.GIF\n";
#   print  FTPOUT5   "   bye\n";
#   print  FTPOUT5   "EOF\n";

#close ( FTPOUT5 ) || die "Can't close $sftpfile5: $!";
#chmod  (0777,  $sftpfile5) ;

## Run the sftp file to get the files from NASA
#$enddate=`date`;
#print " Start  sftp digit files to $webServer2 at $enddate \n";
#open ( SFTPOUT,  " $sftpfile5 | " ) || die " Can't open $sftpfile5 to run: $!";

#@sftpOutput = <SFTPOUT>;

#close (SFTPOUT ) || die "Can't close $sftpfile5: $!";

#system ("ssh $webServer2 $remoteCM  $fileNamePre $basinArea");


##-----------------------------------------------------------------------------------##

##------ Send (SFTP) out the digital and AREA TRaP to SAB Data server - Sabna4 ------##

print " Create sftp script to send the TRaP files to $ftpServer1\n";

system ("ssh $sabDataServer $sabDataRemoteCM  $year $stormName");

# the absolute remote ftp direcotry
$abs_remoteTRAPDir = "$sabDataTrapDir/$year/$stormName";

open  ( FTPOUT4,  ">$sftpfile4" )  ||  die " Can't open $sftpfile4 to write: $!";

   print  FTPOUT4   "# SFTP command file:  Put digit files to FTP server\n";
   print  FTPOUT4   " sftp $sabDataServer << EOF\n";
   print  FTPOUT4   "   cd  $abs_remoteTRAPDir\n";
   print  FTPOUT4   "   mput  $fileNamePre*\n";
   print  FTPOUT4   "   bye\n";
   print  FTPOUT4   "EOF\n";

close ( FTPOUT4 ) || die "Can't close $sftpfile4: $!";
chmod  (0777,  $sftpfile4) ;

# Run the sftp file to get the files from NASA
$enddate=`date`;
print " Start  sftp digit files to $sabDataServer at $enddate \n";
open ( SFTPOUT,  " $sftpfile4  | " ) || die " Can't open $sftpfile4 to run: $!";

@sftpOutput = <SFTPOUT>;

##--------------------------------------------------------------------------------------------##

##----------------- Send the TRaP GIF files to SATEPS Server for GEONETCast ------------------##

system ("scp $fileNamePre*GIF $satepServer:$remoteGeonetDir > /tmp/scp.out");


##--------------------------------------------------------------------------------------------##

##------ Write the TRaP set name and the email List to the TRaPList file for the future ------##


# check if the email is empty, 
# change had been made on 9/12/2006 on the SEPacific, so should not have empty email list.
  
$emailStr = "";
for ($j = 0; $j < $#emailList + 1; $j++)
{
   $emailStr .= $emailList[$j];
}

if ($emailStr ne "")
{
   system("$srcDir/writeList.pl $fileNamePre $basinArea @emailList");
}
else
{
   print "WARNING!!! The email list is EMPTY, don't write to text file.\n";
}



# move the TRaP files sent to other directory, will store them for 24 hour

system("mv $fileNamePre* $digitTRAP_sent_Dir");

unlink("digit24File");

##--------------------------------------------------------------------------------------------##

##------          Write the basin AREA name for the storm for the future use            ------##

# $basinFile

@splitMsg = split(/\./, $fileNamePre);
$dirMsg = $splitMsg[0];
$year = substr($dirMsg, 0, 4);
$nameNnb = substr($dirMsg,4);
$outputDir = "$TRaPDataDir/$year/$nameNnb";
$stormName = $nameNnb;

if (!(-s "$outputDir/$basinFile"))
{
print "Create the Basin file $outputDir/$basinFile\n"; 

   # `touch $HEPassDataLog`;
   system("echo $basinArea > $outputDir/$basinFile");

   # It is a new storm as basin file created here, 
   # so update the active storm list file here

   #$stormStTime = `date +%Y%m%d/%H%M'Z'`;
   $stormStTime = `date +%Y%m%d%H%M'Z'`;
   chomp($stormStTime);
   
   #$stormEntry = $stormName . " " . $year . " " . $basinArea . "   " . $stormStTime;
   $stormEntry = $stormName . "+" . $year . "+" . $basinArea . "+" . $stormStTime;
   print "stormEntry will be added to the active storm list file: $stormFile\n";

   if (!(-s "$stormFile"))
   {
      system("echo $stormEntry > $stormFile");
   }
   else
   {
      system("echo $stormEntry >> $stormFile");   
   }

   # push the storm list file to GP36 for etrap to run, will turn off when Etrap operational
   # added on 05/01/2008
   print "Call pushStList.sh to push the storm list file to GP36 for etrap to run.\n\n";
   system ("$srcDir/pushStList.sh");
}
else
{
   $exist=`grep $basinArea $outputDir/$basinFile`;


   # Append to the Basin file and the active storm list 
   if ($exist eq "")
   {
      print "Storm $stormName cross the basin to $basinArea\n";
      print "Append to the Basin file $outputDir/$basinFile\n";
      system("echo $basinArea >> $outputDir/$basinFile");

      # Update the active storm list with new basin
      $curEntryPre = $stormName . "+" . $year;
      $curEntry=`grep $curEntryPre $stormFile`;
      chomp($curEntry);
      print "Existing Entry for $stormName is $curEntry\n";
      @splitMsg = split(/\+/, $curEntry);
      $basin1 = $splitMsg[2];
      print "Storm $stormName cross the basin from $basin1 to $basinArea.\n";
      $curEntryPre = $stormName . "+" . $year . "+" . $basin1;

      # The following new entry is for eTRaP to know it just cross basins, will be replaced with $basinArea in etrap process
      $newEntryPre = $stormName . "+" . $year . "+" . $basin1 . '-' . $basinArea;
      system("sed 's/$curEntryPre/$newEntryPre/' $stormFile > basinCrTmp.txt");
      rename("basinCrTmp.txt", $stormFile)  or die "ERROR:can't rename basinCrTmp.txt to $stormFile: $!";
   }
}


$enddate = `date`;
print "End of the program $0 at $enddate.\n\n";

# record to the log file
`echo "start $0 at $startdate" | cat >> $logFile`;
`echo "+++++++++++++++ STDERR Log +++++++++++++++" |cat >> $logFile`;
`cat $stderrLog >> $logFile`;
`echo "+++++++++++++++ STDOUT Log +++++++++++++++" |cat >> $logFile`;
`cat $stdoutLog >> $logFile`;



####------------------------------------------------------------------------------------------------####


##%%%%%%%%%%%%%%%%%%%%%%%% Subroutines %%%%%%%%%%%%%%%%%%%%%%%%##


###############  Subroutine "getBasin" #################
########################################################

sub getBasin
{
   my($wmoHd, $station, $lat, $lon) = @_;
   my($i);
 
   print "Get the basin of the bulletin $wmoHd $station in $lat/$lon (regular longitude)\n"; 
  
   #----------- Atlantic -----------#

   if ($wmoHd =~ /^WTNT2/)
   {
      print "This will send to TPC\n";
      if ($basin eq "")
      {
         # $basin = $Atlantic_basin;
         $basin = $basinTbl{$Atlantic_dir};
      }

      # $emailList = $AtlanticList;
      # push(@emailList, $AtlanticList);
      @emailList = $AtlanticList;
      $subDir = $Atlantic_dir;
      $basinArea = $Atlantic_dir;
   }

   #----------- East Pacific -----------#

### Noted at 08/29/2003, will work more on the overlapped case Storm fall in around Central Pacific and East pacific
## for basin and email notification, if it is neccessary
# will check if there is error msg when there is no basin in the POS file 

   elsif($wmoHd =~ /^WTPZ2/)
   {
      print "This will send to TPC\n";
      if ($basin eq "")
      {
         # $basin = $EastPacific_basin;
         $basin = $basinTbl{$EastPacific_dir};
      }

      # $emailList = $TPCList;
      # push(@emailList, $TPCList);
      @emailList = $TPCList;
      $subDir = $EastPacific_dir;
      $basinArea = $EastPacific_dir;
   }

   #----------- Central Pacific -----------#

   elsif($wmoHd =~ /^WTPA2/)
   {
      print "This will send to CPHC\n";
      if ($basin eq "")
      {
         # $basin = $CentralPacific_basin;
         $basin = $basinTbl{$CentralPacific_dir};
      }
                                
      @emailList = $CPHCList;
      $subDir = $CentralPacific_dir;
      $basinArea = $CentralPacific_dir;
   }

   #----------- NW Pacific -----------#

   elsif($wmoHd =~ /^WTPN3/)
   {
      print "This will send to MWO Guam\n";
      # $basin = $NorthwestPacific_basin;
      $basin = $basinTbl{$NorthwestPacific_dir};
      # $emailList = $MWOGuamList;
      @emailList = $NWPacificList;
      $subDir = "$NorthwestPacific_dir/JTWC";
      $basinArea = $NorthwestPacific_dir;
   }

   # station ID of FKPQ30 is also RJTD.  06/28/2007 
   elsif(($wmoHd =~ /^WTJP2/) || ($wmoHd =~ /^FKPQ3/))		
   {
      print "This will send to Tokyo\n";
      # $basin = $NorthwestPacific_basin;
      $basin = $basinTbl{$NorthwestPacific_dir};
      # $emailList = $TokyoList;
      @emailList = $NWPacificList;
      $subDir = "$NorthwestPacific_dir/Tokyo";
      $basinArea = $NorthwestPacific_dir;
   }


   #----------- North Indian -----------#

#   elsif( (($wmoHd =~ /^WTIO3/) && ($station eq "PGTW"))
#          || ($wmoHd =~ /^WTIN/)
#          || ($wmoHd =~ /^FKIN/) )  
   elsif(($wmoHd =~ /^WTIO3/) && ($station eq "PGTW"))
   {
    #  if ($lat >= 0)
      print "This will send to RSMC New Delhi\n";
      # $basin = $NorthIndian_basin;
      $basin = $basinTbl{$NorthIndian_dir};
      @emailList = $NewDelhiList;

      # will let nancy know 06/28/07
      $subDir = "$NorthIndian_dir/JTWC";
      $basinArea = $NorthIndian_dir;
   }

   # FKIN20 VIDP, WTIN20 DEMS, stations are different, will check, and may need to modify the createTRaPDir for agency name
   elsif(($wmoHd =~ /^WTIN/) || ($wmoHd =~ /^FKIN/))
   {
    #  if ($lat >= 0)
      print "This will send to RSMC New Delhi\n";
      # $basin = $NorthIndian_basin;
      $basin = $basinTbl{$NorthIndian_dir};
      @emailList = $NewDelhiList;
      # ??
      $subDir = "$NorthIndian_dir/NewDelhi";
      $basinArea = $NorthIndian_dir;
   }



   # will send the notification to the neighborhood if the storm fall within 
   # 5/10 degree of the boundary

   ##----------- JTWC bulletins - WTXS -----------##

   # The TRaP from JTWC bulletin will alert no people, so the emailList is empty
   # except the WTIO31 and WTPN31, Maybe change in the future
   
   elsif(($wmoHd =~ /^WTXS3/) && ($station eq "PGTW"))
   {
      if ($lon < 90)
      {
         print "This will send to RSMC Reunion\n";
         # $basin = $SouthwestIndian_basin;
         $basin = $basinTbl{$SouthwestIndian_dir};
         $subDir = "$SouthwestIndian_dir/JTWC";
         $basinArea = $SouthwestIndian_dir;
         push(@emailList, $ReunionList);
         if ($lon >= 85)
         {
            push(@emailList, $PerthList);
         }                 
      }
      #elsif (($lon >= 90) && ($lon < 130))
      elsif (($lon >= 90) && ($lon < 125))
      {
         print "This will send to RSMC Perth\n";
         # $basin = $SoutheastIndian_basin;
         $basin = $basinTbl{$SoutheastIndian_dir};
         $subDir = "$SoutheastIndian_dir/JTWC";
         $basinArea = $SoutheastIndian_dir;
         push(@emailList, $PerthList);
         if ($lon < 95) 
         {
            push(@emailList, $ReunionList);
         }
         elsif ($lon >= 120)
         {
            push(@emailList, $DarwinList);
         }
      }
      #elsif (($lon >= 130) && ($lon < 142))
      elsif (($lon >= 125) && ($lon < 142))
      {
         print "This will send to RSMC Darwin\n";
         # $basin = $NorthernTerritory_basin;     
         $basin = $basinTbl{$NorthernTerritory_dir};     
         $subDir = "$NorthernTerritory_dir/JTWC";   
         $basinArea = $NorthernTerritory_dir;   
         @emailList = ($DarwinList, $PerthList, $BrisbaneList);
 #        if ($lon < 140)
  #       {
   #         $emailList = $emailList . " " . $PerthList;
    #     }
      }
   }   	# End of bulletin WTXS31-35 (PGTW)


   ##----------- Reunion bulletins -----------##

   # elsif(($wmoHd =~ /^WTIO2/) && ($station eq "FMEE") && ($lon < 90))   # WTIO30 ?
   # elsif(($wmoHd =~ /^WTIO/) && ($station eq "FMEE") && ($lon < 90))   # WTIO30 ?
   elsif( (($wmoHd =~ /^WTIO/) || ($wmoHd =~ /^FKIO/)) && ($station eq "FMEE") && ($lon < 90))   # WTIO30 ?
   {
      print "This will send to RSMC Reunion\n";
      # $basin = $SouthwestIndian_basin;
      $basin = $basinTbl{$SouthwestIndian_dir};
      @emailList = $ReunionList;
      $subDir = "$SouthwestIndian_dir/Reunion";
      $basinArea = $SouthwestIndian_dir;
      if  ($lon >= 85)
      {
         push(@emailList, $PerthList);
      }
   }

   ##----------- Australian bulletins -----------##   

   # elsif($wmoHd =~ /^WTAU0/) 
   elsif($wmoHd =~ /^AXAU/)
   {
      $tempWmo = $wmoHd;
      $lastDigit = chop($tempWmo);
  
      # For "WTAU05", 06, 07
    ##  if (($lastDigit >= 5) && ($station eq "APRF") && ($lon >= 80) && ($lon <= 140))
    ##  if (($lastDigit >= 5) && ($station eq "APRF") && ($lon >= 90) && ($lon < 130))
    #  if (($lastDigit >= 5) && ($station eq "APRF"))
      if ($station eq "APRF")
      {
         print "This will send to RSMC Perth\n";
         # $basin = $SoutheastIndian_basin;         # SE Indian Ocean
         $basin = $basinTbl{$SoutheastIndian_dir};         # SE Indian Ocean
         @emailList = $PerthList;
         $subDir = "$SoutheastIndian_dir/Perth";
         $basinArea = $SoutheastIndian_dir;
         if ($lon < 95)
         {
            push(@emailList, $ReunionList);
         }
         elsif ($lon >= 120)
         {
            push(@emailList, $DarwinList);
         }
      }
      # For "WTAU03", 04, 
    ##  elsif (($lastDigit <= 4) && ($station eq "ADRM") && ($lon >= 120) && ($lon <= 152))
     ## elsif (($lastDigit <= 4) && ($station eq "ADRM") && ($lon >= 130) && ($lon < 142))
      ## change on 03/17/04
     ## elsif (($lastDigit <= 4) && ($station eq "ADRM") && ($lon >= 125) && ($lon < 142))
     # elsif (($lastDigit <= 4) && ($station eq "ADRM"))
      elsif ($station eq "ADRM")
      {
         print "This will send to RSMC Darwin\n";
         # $basin = $NorthernTerritory_basin;         # Northern Territory
         $basin = $basinTbl{$NorthernTerritory_dir};         # Northern Territory

         # The TRaP in Northern Territory will send to other two agents:Perth and Brisbane
         @emailList = ($DarwinList, $PerthList, $BrisbaneList);
         $subDir = "$NorthernTerritory_dir/Darwin";
         $basinArea = $NorthernTerritory_dir;
      }
   }	 # End of the bulletin "AXAU"

   # elsif ((($wmoHd eq "WOAU01") || ($wmoHd eq "WOAU02")) && ($station eq "ABRF") 
   #       && ($lon > 132) && ($lon <= 170)) 
   #          && ($lon >= 142) && ($lon < 160))
   #elsif (($wmoHd =~ /^WOAU0/) && ($station eq "ABRF"))
   elsif ($station eq "ABRF")
   {  
      print "This will send to RSMC Brisbane\n";
      # $basin = $SouthwestPacific_basin;         # SW Pacific
      $basin = $basinTbl{$SouthwestPacific_dir};         # SW Pacific
      @emailList = $BrisbaneList;
      $subDir = "$SouthwestPacific_dir/Brisbane";
      $basinArea = $SouthwestPacific_dir;
      if ($lon < 152)
      {
         push(@emailList, $DarwinList);
      }
      elsif ($lon >= 155)
      {
        # if ($lat >= -15)
        # {

            push(@emailList, $NadiList);

        # }
        # elsif (($lat < -15) && ($lat >= -35))
        # {
        #    push(@emailList, $NadiList, $OtherList);
        # }
        # else
        # {
        #    push(@emailList, $OtherList);
        # }

      }
   }	 # End of the bulletin "AXAU"


   ##----------- ICAO Australian bulletin -----------##

   elsif($wmoHd =~ /^FKAU0/)
   {
      if (($lon >= 90) && ($lon < 125))
      {
         print "This will send to ICAO Perth\n";
         # $basin = $SoutheastIndian_basin;
         $basin = $basinTbl{$SoutheastIndian_dir};
         $subDir = "$SoutheastIndian_dir/Perth";
         $basinArea = $SoutheastIndian_dir;

         @emailList = $PerthList;
         if ($lon < 95)
         {
            push(@emailList, $ReunionList);
         }
         elsif ($lon >= 120)
         {
            push(@emailList, $DarwinList);
         }
      }
      elsif (($lon >= 125) && ($lon < 142))
      {
         print "This will send to ICAO Darwin\n";
         # $basin = $NorthernTerritory_basin;         # Northern Territory
         $basin = $basinTbl{$NorthernTerritory_dir};         # Northern Territory
         $subDir = "$NorthernTerritory_dir/Darwin";
         $basinArea = $NorthernTerritory_dir;

         # The TRaP in Northern Territory will send to other two agents:Perth and Brisbane
         @emailList = ($DarwinList, $PerthList, $BrisbaneList);
      }
      elsif (($lon >= 142) && ($lon <= 160))
      {
         print "This will send to ICAO Brisbane\n";
         # $basin = $SouthwestPacific_basin;         # SW Pacific
         $basin = $basinTbl{$SouthwestPacific_dir};         # SW Pacific
         $subDir = "$SouthwestPacific_dir/Brisbane";
         $basinArea = $SouthwestPacific_dir;

         @emailList = $BrisbaneList;
         if ($lon < 152)
         {
            push(@emailList, $DarwinList);
         }
         elsif ($lon >= 155)
         {
           # if ($lat >= -15)
           # {

               push(@emailList, $NadiList);

           # }
           # elsif (($lat < -15) && ($lat >= -35))
           # {
           #    push(@emailList, $NadiList, $OtherList);
           # }
           # else
           # {
           #    push(@emailList, $OtherList);
           # }
         }
      }	# End of elsif for Brisbane

   }	# End of if for FKAU


   ##----------- Jakarta bulletin -----------##

   elsif($wmoHd =~ /^WVID/)
   {
      #if (($lon >= 90) && ($lon < 125))
      #{
         print "This will send to Jakarta\n";
         $basin = $basinTbl{$SoutheastIndian_dir};
         #$subDir = "$SoutheastIndian_dir/Perth";
         $subDir = "$SoutheastIndian_dir/Jakarta";
         $basinArea = $SoutheastIndian_dir;

         @emailList = $PerthList;
         if ($lon < 95)
         {
            push(@emailList, $ReunionList);
         }
         elsif ($lon >= 120)
         {
            push(@emailList, $DarwinList);
         }
     # }
   } # End of elsif for Jakarta


   ##----------- Fiji bulletin -----------##

   # elsif(($wmoHd =~ /^WTPS0/) && ($station eq "NFFN") &&
   #elsif(($wmoHd =~ /^W.PS0/) && ($station eq "NFFN") &&
   #      (($lon >= 160) || ($lon < 0)) && ($lat >= -25))

   # modified on 04/06/07, for CLIFF, "WTPS01 NFFN in -26.00/-171.5"
   #elsif(($wmoHd =~ /^W.PS0/) && ($station eq "NFFN") &&
   elsif( (($wmoHd =~ /^W.PS0/) || ($wmoHd =~ /^FKPS/)) && ($station eq "NFFN") &&
         (($lon >= 160) || ($lon < 0)) )
   {
      print "This will send to RSMC Nadi\n";
      # $basin = $SoutheastPacific_basin;
      $basin = $basinTbl{$SoutheastPacific_dir};
      @emailList = $NadiList;
      # $subDir = "$SoutheastPacific_dir/Nadi";
      $subDir = "$SoutheastPacific_dir/Fiji";
      $basinArea = $SoutheastPacific_dir;
      if ($lon < 165)
      {
         push(@emailList, $BrisbaneList);
      }
   }


   ##----------- JTWC bulletins - WTPS3X -----------##

   elsif(($wmoHd =~ /^WTPS3/) && ($station eq "PGTW"))
   {
      #if (($lon >= 130) && ($lon < 142))
      if (($lon >= 125) && ($lon < 142))
      {
         print "This will send to RSMC Darwin\n";
         # $basin = $NorthernTerritory_basin;         # Northern Territory
         $basin = $basinTbl{$NorthernTerritory_dir};         # Northern Territory
         $subDir = "$NorthernTerritory_dir/JTWC";
         $basinArea = $NorthernTerritory_dir;
         @emailList = ($DarwinList, $PerthList, $BrisbaneList);
 #        if ($lon >= 132)
  #       {
   #         $emailList = $emailList . " " . $BrisbaneList;
    #     }
      }
      elsif (($lon >= 142) && ($lon < 160))
      {
         print "This may send to RSMC Brisbane\n";
         # $basin = $SouthwestPacific_basin;         # SW Pacific
         $basin = $basinTbl{$SouthwestPacific_dir};         # SW Pacific
         $subDir = "$SouthwestPacific_dir/JTWC";
         $basinArea = $SouthwestPacific_dir;
         @emailList = $BrisbaneList;
         if ($lon < 152)
         {
            push(@emailList, $DarwinList);
         }
         elsif ($lon >= 155)
         {
           # if ($lat >= -15)
           # {

               push(@emailList, $NadiList);

           # }
           # elsif (($lat < -15) && ($lat >= -35))
           # {
           #    push(@emailList, $NadiList, $OtherList);
           # }
           # else
           # {
           #    push(@emailList, $OtherList);
           # }
         }
      }
      #elsif ((($lon >= 160) || ($lon < 0)) && ($lat >= -25))
      elsif (($lon >= 160) || ($lon < 0))
      {
         print "This may send to RSMC Nadi\n";
         # $basin = $SoutheastPacific_basin;         # SE Pacific
         $basin = $basinTbl{$SoutheastPacific_dir};         # SE Pacific
         $subDir = "$SoutheastPacific_dir/JTWC";
         $basinArea = $SoutheastPacific_dir;
         @emailList = $NadiList;
         if ($lon < 165)
         {
            push(@emailList, $BrisbaneList);
         }
         ## if ($lat < -15)
         #if ($lat < -20)
         #{
         #   push(@emailList, $OtherList);
         #   print "\nPAY ATTENTION!! The storm is in south of -20 in Southeast Pacific (Near New Zealand Area).\n";
         #}

      }

#      elsif ((($lon >= 160) || ($lon < 0)) && ($lat < -25))
#      {
#         print "This may send to Other\n";
#         $basin = $Other_basin;         # SE Pacific
#         $subDir = "$SoutheastPacific_dir/JTWC";
#         $basinArea = $SoutheastPacific_dir;
#         @emailList = $OtherList;
#         if ($lon < 165)
#         {
#            push(@emailList, $BrisbaneList);
#         }
#         if ($lat > -35)
#         {
#            push(@emailList, $NadiList);
#         }
#      }

   }    # End of the bulletin "WTPS31-35 (PGTW)


   ##----------- New Zealand bulletins - WTNZ -----------##

   # WTNZ or WHNZ
   elsif(($wmoHd =~ /^W[T,H]NZ/) )
   {
      if ($lon < 160)
      {
         print "This may send to RSMC Brisbane/Wellington\n";
         # $basin = $SouthwestPacific_basin;         # SW Pacific
         $basin = $basinTbl{$SouthwestPacific_dir};         # SW Pacific
         $subDir = "$SouthwestPacific_dir/Wellington";
         $basinArea = $SouthwestPacific_dir;
         @emailList = $BrisbaneList;
         if ($lon < 152)
         {
            push(@emailList, $DarwinList);
         }
         elsif ($lon >= 155)
         {
               push(@emailList, $NadiList);

         }
      }
      elsif (($lon >= 160) || ($lon < 0))
      {
         print "This may send to RSMC Nadi/Wellington\n";
         # $basin = $SoutheastPacific_basin;         # SE Pacific
         $basin = $basinTbl{$SoutheastPacific_dir};         # SE Pacific
         $subDir = "$SoutheastPacific_dir/Wellington";
         $basinArea = $SoutheastPacific_dir;
         @emailList = $NadiList;
         if ($lon < 165)
         {
            push(@emailList, $BrisbaneList);
         }

      }
   }    # End of the bulletin "WTNZ/WHNZ"

}	# End of the subroutine "getBasin"
