#!/usr/bin/perl -w

## =====================================================================
## Name:       checkList.pl
##                  
## Author:      Jianbin Yang/RSIS   11/21/2003
## Language:    Perl
## Purpose:     Check if the TraP receive AMSU/TRMM/SSMI geolocation data
##              and rain rate data from Polar server normally and the 
##              TRaP product send out to FTP server successfully. It also
##              check if the single orbit Rain rate source is generated 
##              as well as the corresponding geolocation file.  
##                  
## Modified:   01/14/2004
##                  change PLB to PLR
##
## Modified: 01/23/2007
##             Add to check N-18 pass, remove the N-16 pass
##
## Modified: 10/04/2007
##             Add to check M2, N-16 and AMSR-E passes
##
## Modified: 09/08/2008
##
## Modified: 3/4/2011
##             email change from jianbin to renata
##
## =====================================================================

$path = $ENV{"PATH"};
$ENV{"PATH"} = $path;

$HOME = $ENV{"HOME"};
$ENV{"MCPATH"} = "$HOME/mcidas/data:/home/mcidas/data";

####---------------------- Define  and delcare ------------------------####

##------------------ Define the directories and files -------------------##

$dataDir = "$HOME/mcidas/data";
$passDataDir = "$dataDir/polerPassData";
$logDir = '/data/TRAP/log';

# The log file for this program
$stdoutLog = "$dataDir/check.out";
$stderrLog = "$dataDir/check.err";
$logFile = "$dataDir/checkError.log";
$bulLogFile = "$logDir/getBul.log";

## Define the AMSUB/TRMM/SSMI geolocation data file names ##

$amsubNKId = "amsub\.D*NK";
$amsubNLId = "amsub\.D*NL";
$amsubNMId = "amsub\.D*NM";
$amsubNNId = "amsub\.D*NN";
$amsubM2Id = "amsub\.D*M2";
$amsubNPId = "amsub\.D*NP";
$trmmId = "trmm\.*";
$ssmiS7Id = "SSGEO\.NPR\.EDRR*S7*";
$ssmiS8Id = "SSGEO\.NPR\.EDRR*S8*";
$ssmiS9Id = "SSGEO\.NPR\.EDRR*S9*";

$amsreId = "amsre\.Rain*";

$mins = 180;
$amsumins = 480;
$errorExist = 0;

$userAddress = "'renata.barski\@noaa.gov' ";

##---------------- initialize the array and hash table ----------------##

# key is the data source id name, and the value is the long list of the last geo file for it
%lastGeoFileList = ();

# key is the data set name, and the value is the array of latest two geo file for it
%latest2GeoFileList = ();

#----------------------------------------------------------------------------------#
open STDOUT, "> $stdoutLog" || die "Can't create $stdoutLog:$!";
open STDERR, "> $stderrLog" || die "Can't create $stderrLog:$!";

####----------------------------  Start the program ----------------------------####

$startdate=`date`;
print "Start  $0 at $startdate \n";

##-------------------------- Check the geolocation file --------------------------##

chdir("$passDataDir") || die "Cannot change to directory $passDataDir";

# check if there is no file within 3 hour new, there is something wrong

$lastAMSUNK = `find . -name "$amsubNKId" -mmin -$amsumins -exec ls -l {} ';'`;
$lastAMSUNL = `find . -name "$amsubNLId" -mmin -$amsumins -exec ls -l {} ';'`;
$lastAMSUNM = `find . -name "$amsubNMId" -mmin -$amsumins -exec ls -l {} ';'`;
$lastAMSUNN = `find . -name "$amsubNNId" -mmin -$amsumins -exec ls -l {} ';'`;
$lastAMSUM2 = `find . -name "$amsubM2Id" -mmin -$amsumins -exec ls -l {} ';'`;
$lastAMSUNP = `find . -name "$amsubNPId" -mmin -$amsumins -exec ls -l {} ';'`;
$lastSSMIS7 = `find . -name "$ssmiS7Id" -mmin -$mins -exec ls -l {} ';'`;

# S8 always delay, so use amsumins
# $lastSSMIS8 = `find . -name "$ssmiS8Id" -mmin -$mins -exec ls -l {} ';'`;
$lastSSMIS8 = `find . -name "$ssmiS8Id" -mmin -$amsumins -exec ls -l {} ';'`;
$lastSSMIS9 = `find . -name "$ssmiS9Id" -mmin -$mins -exec ls -l {} ';'`;

$lastTRMM = `find . -name "$trmmId" -mmin -$mins -exec ls -l {} ';'`;
$lastAMSRE = `find . -name "$amsreId" -mmin -$amsumins -exec ls -l {} ';'`;

# assign them into the hash table
$lastGeoFileList{"$amsubNKId"} = $lastAMSUNK;
$lastGeoFileList{"$amsubNLId"} = $lastAMSUNL;
$lastGeoFileList{"$amsubNMId"} = $lastAMSUNM;
$lastGeoFileList{"$amsubNNId"} = $lastAMSUNN;
$lastGeoFileList{"$amsubM2Id"} = $lastAMSUM2;
$lastGeoFileList{"$amsubNPId"} = $lastAMSUNP;
$lastGeoFileList{"$ssmiS7Id"} = $lastSSMIS7;
$lastGeoFileList{"$ssmiS8Id"} = $lastSSMIS8;
$lastGeoFileList{"$ssmiS9Id"} = $lastSSMIS9;
$lastGeoFileList{"$trmmId"} = $lastTRMM;
$lastGeoFileList{"$amsreId"} = $lastAMSRE;

print "------ Check the latest geolocation file for every data source ------\n";

while (($dataSourceId, $lastFile) = each(%lastGeoFileList))
{
   if ($lastFile eq "")
   {
      $lastFile = `ls -l $dataSourceId |tail -1`;
      print "There is no updated for $dataSourceId within certain time. ";
      print "($mins for TRMM/SSMI, or $amsumins minutes for AMSU/AMSR-E data)\n";
      print "The latest file is $lastFile\n";
      $errorExist = 1;
   }
   # else
   # {
   #    print "The files within the certain time ($mins for TRMM/SSMI, $amsumins minutes for ASMU) are \n$lastFile\n"; 
   # }
}


##--------------- Check the imglist compared to geolocation file -----------------##

print "------ Check the imglist for corresponding rain rate data ------\n";

# get the latest two geo file for every dataset
@fileList = `ls -tr $trmmId|tail -2`;
$latest2GeoFileList{"TRMM"} = [ @fileList ];

@fileList = `ls -tr $amsubNKId|tail -2`;
$latest2GeoFileList{"AMSUNK"} = [@fileList];

@fileList = `ls -tr $amsubNLId|tail -2`;
$latest2GeoFileList{"AMSUNL"} = [@fileList];

@fileList = `ls -tr $amsubNMId|tail -2`;
$latest2GeoFileList{"AMSUNM"} = [@fileList];

@fileList = `ls -tr $amsubNNId|tail -2`;
$latest2GeoFileList{"AMSUNN"} = [@fileList];

@fileList = `ls -tr $amsubM2Id|tail -2`;
$latest2GeoFileList{"AMSUM2"} = [@fileList];

@fileList = `ls -tr $amsubNPId|tail -2`;
$latest2GeoFileList{"AMSUNP"} = [@fileList];

@fileList = `ls -tr $ssmiS7Id|tail -2`;
$latest2GeoFileList{"SSMIS7"} = [@fileList];

@fileList = `ls -tr $ssmiS8Id|tail -2`;
$latest2GeoFileList{"SSMIS8"} = [@fileList];

# @fileList = `ls -tr $ssmiS9Id|tail -2`;
# $latest2GeoFileList{"SSMIS9"} = [@fileList];

@fileList = `ls -tr $amsreId|tail -2`;
$latest2GeoFileList{"AMSRESINRR"} = [@fileList];

# while (($dataSet, $fileList1 ) = each(%latest2GeoFileList))
for $dataSet(keys %latest2GeoFileList)
{
  @fileList = @{ $latest2GeoFileList{$dataSet} };
  # print "The data set name is $dataSet, the fileList is **@fileList**.\n";
  $group = "PLR";
  if ($dataSet eq "TRMM")
  {
      $startHhmm1 = substr($fileList[0],16,2) . ':' . substr($fileList[0],19,2);
      $startHhmm2 = substr($fileList[1],16,2) . ':' . substr($fileList[1],19,2);
  }
  elsif ($dataSet =~ /AMSU/)
  {
      $startHhmm1 = substr($fileList[0],14,2) . ':' . substr($fileList[0],16,2);
      $startHhmm2 = substr($fileList[1],14,2) . ':' . substr($fileList[1],16,2);
  }    
  elsif ($dataSet =~ /SSMIS/)
  {
      # $startHhmm1 = substr($fileList[0],22,2) . ':' . substr($fileList[0],24,2);
      # $startHhmm2 = substr($fileList[1],22,2) . ':' . substr($fileList[1],24,2);
      $startHhmm1 = substr($fileList[0],26,2) . ':' . substr($fileList[0],28,2);
      $startHhmm2 = substr($fileList[1],26,2) . ':' . substr($fileList[1],28,2);
  }
  elsif ($dataSet =~ /AMSRE/)
  {
      $group = "AMSRE";
      $startHhmm1 = substr($fileList[0],23,2) . ':' . substr($fileList[0],25,2);
      $startHhmm2 = substr($fileList[1],23,2) . ':' . substr($fileList[1],25,2);
  }

 # print "the startHhmm1 is $startHhmm1, the startHhmm2 is $startHhmm2\n"; 
  ## $match = `imglist.k PLR/$dataSet.-10 | grep -e "$startHhmm1" -e "$startHhmm2"`;
  $match = `imglist.k $group/$dataSet.-10 | grep -e "$startHhmm1" -e "$startHhmm2"`;

 #print "the match is $match.\n\n";
  
  # if no match, the latest two geolocations have no corresponding rain rate data
  # than there is something wrong
  
  if (($startHhmm1 eq "") || ($startHhmm2 eq ""))
  {
      print "One of both of start HHMM is empty, the startHhmm1 is $startHhmm1, the startHhmm2 is $startHhmm2\n";
      print "The match is $match.\n\n";
      $errorExist = 1;
  }

  if ($match eq "")
  {
     print "Could not find the rain rate data for the latest two $dataSet geolocation files:\n@fileList\n";
     print "the imglist of the data set $dataSet is as following.\n";
     print `imglist.k $group/$dataSet.-10`;
     $errorExist = 1;
  } 
}


##--------------- Check the DIGIT.ERR log file for SFTP -----------------##

print "------ Check the DIGIT.ERR log file for SFTP ------\n";

$checkTime = 60;  # the frequency to set the program run in the cronjob

chdir("$dataDir") || die "Cannot change to directory $dataDir";

# check if the ERR log file is new and is not checked
$errLog = `find . -name "DIGIT.ERR" -mmin -$checkTime`;

if ($errLog ne "")
{
   # SFTP not succeed if either one of the following string is found in the err log file
   $sftpError = `grep -i -e "Connection closed"  -e "Connection reset" DIGIT.ERR`;
   if ($sftpError ne "")
   {
      print "SFTP is not successful, cat the ERR log file.\n";
      print `cat DIGIT.ERR`;
      $errorExist = 1;
      # append the stdout to the log file
   }
}

##--------------- Check the getBul.log log file for Errors or Warnings -----------------##

print "------ Check the getBul.log file for Errors or Warnings ------\n";

# will use technique I used in MIRS monitoring to only output the Error/Warnming msg when it first occur
# 09/08/08

## $errMsg = `grep -i -e "ERROR" -e "WARNING" $bulLogFile`;
# $errMsg = `grep -e "ERROR" -e "WARNING" $bulLogFile`;
$errMsg = `grep "ERROR" $bulLogFile`;
if ($errMsg ne "")
{
   print "There is Error or Warning in the $bulLogFile file, Please Check the log file.\n";
   print "Here is the error messages:$errMsg.\n";
   $errorExist = 1;
}



##--------------- Append the log file if there is error ---------------##
if ($errorExist == 1)
{
   `echo "THE DATE/TIME IS $startdate" | cat >> $logFile`;
   `echo "+++++++++++++++ STDERR Log +++++++++++++++" |cat >> $logFile`;
   `cat $stderrLog >> $logFile`;
   `echo "+++++++++++++++ STDOUT Log +++++++++++++++" |cat >> $logFile`;
   `cat $stdoutLog >> $logFile`;

   # open MAIL,"|mail -s 'Error found!' $userAddress < $stdoutLog";
   `mail -s 'Error found!' $userAddress < $stdoutLog`;
}

print "End of the program.\n";

