#!/usr/bin/perl -w

## =====================================================================
## Name:        cleanTRaPs.pl  
##                  
## Author:      Jianbin Yang/RSIS   07/16/2003
## Language:    Perl
## Purpose:     This program will go to the direcoties in FTP Server 
##              for each region to clean the digital and AREA TRaP
##		older than 48 hour.
##              This program will clean the TRaP tar file as well.
##
##              This program will run on FTP Server (GP16)   
##                  
## Modified:   08/04/2003
##
## Modified:   02/06/2004
##               Extend to 72 hours
##
## Modified:   03/10/2004
##               Add the code to clean the TRaP tar files olden than 15 days. 
##               
## Modified:   02/28/2008
##               Add to clean $year directory     
##
## Last Modified:   10/02/2009
##               Extended to 5 days for validating precipitation products
## =====================================================================


# Define the directories and files
$trapDataDir = '/data/ftp/pub/TRAP';          # the TRaP directory in FTP Server 
$archDir = "$trapDataDir/ARCH";

$deletedDays = 5.0;

# Define the directories to be cleaned in FTP server
@cleanDirs = qw (
                Atlantic   EastPacific    CentralPacific    NWPacific 
                SEPacific  SWPacific      NTerritory        NorthIndian
                SEIndian   SWIndian       Other
             );


##--------------------------------- Start --------------------------------------##

$startdate=`date`;
print "Start  $0 at $startdate \n";

chdir("$trapDataDir") || die "Cannot change to directory $trapDataDir: $!";
print  "TRAP Data directory is:";
system("pwd");


##-------- Clean the digital and AREA TRaP files for each regions --------##

@yearList = ();

$year = `date +%Y`;
chomp($year);
push(@yearList, $year);

$oldYear = `date -d "1 month ago" +%Y`;
chomp($oldYear);

if ($oldYear != $year)
{
   print "clean $oldYear";
   push(@yearList, $oldYear);
}

foreach $dir (@cleanDirs, @yearList)

#foreach $dir (@cleanDirs)
{
   if (-d $dir)
   {
      chdir("$dir") || die "Cannot change to directory $dir: $!";
      system("pwd");
      @fileDirList = `ls`;
   }
   else
   {
      next;
   }

   foreach $fileDir (@fileDirList)
   {
      chop($fileDir);
      
      if (-f $fileDir)
      {
         #if (-M $fileDir > 3.0)
         if (-M $fileDir > $deletedDays)
         {
            unlink("$fileDir") || die "cannot unlink $fileDir: $!";
            print "$fileDir is deleted.\n";
         }
      }
      elsif (-d $fileDir)
      {
         chdir("$fileDir") || die "Cannot change to directory $fileDir: $!";
         system("pwd");
         @fileList = `ls`;

         foreach $file (@fileList)
         {
            chop($file);

            if (-f $file)
            {
               if (-M $file > $deletedDays)
               {
                  unlink("$file") || die "cannot unlink $file: $!";
                  print "$file is deleted.\n";
               }
            }
         }  # End of the third Foreach loop

         chdir("..") || die "Cannot change to upper directory: $!";
 
      }     # End of the if/elsif 

   }        # End of the second Foreach loop 


   chdir("..") || die "Cannot change to upper directory: $!";

}           # End of the first Foreach loop   


##------------------ Clean the TRaP tar files ------------------##
 
chdir("$archDir") || die "Cannot change to directory $archDir: $!";
system("pwd");
@fileList = `ls`;

foreach $file (@fileList)
{
   chop($file);
   
   if ((-f $file) && ($file =~ /\.tar/))
   {
      if (-M $file > 15.0)
      {
         unlink("$file") || die "cannot unlink $file: $!";
         print "$file is deleted.\n";
      }
   }

}

print "End of clean old TRAP data\n";

