#!/usr/bin/perl -w
#
#
# $Id: cleanTRAP_ARCH2.pl,v 1.2 2011-03-08 20:12:20 cmuser Exp $
# $Log: not supported by cvs2svn $
# Revision 1.1.1.1  2010/04/27 15:13:11  cmuser
# Initial version
#
# Revision 1.1.1.1  2009/02/04 19:25:09  cmuser
# initial release
#
# Revision 1.1  2007/11/30 19:57:01  oper1
# First checkin.
#
#
#!/usr/bin/perl -w

## =====================================================================
## Name:        cleanTRAP_ARCH2.pl  
##                  
## Author:      Jianbin Yang/RSIS   01/17/2003
## Language:    Perl
## Purpose:     This program clean the TRaP data for the storm 
##		8 days after it was over .  It also send user alert 
##		for saving the data one and 6 day after the storm is over. 
##                        
##              Noted that this program is a alternated one to archive the tar.gz
##              file just created to the "archive" directory.  This is run when
##              there is problem for SAB analyst to save the tar files in time
##
#3		This ARCH2 program will make tar files for old storm no matter how old it is
##                  
## Modified:   07/31/2003
##
## Modified:   02/04/2004
##             (will) Extend the date from 1 to 2 
##
## Modified: 10/15/2006
##                Add the code to delete entry of the storm in the web server               
## 
## Modified: 02/15/2007
##
## Last Modified:  03/04/2011  
##                Email change from Jianbin to Renata
##
## =====================================================================

use Class::Struct;
use File::Path;           # for "rmtree"

####---------------------- Define  and delcare ------------------------####

##------------------------   Define structures   ------------------------##

# structure to store the information to call the program on web server
struct StormInfo =>
{
   stormName => '$',        
   year      => '$',        
   basinArea => '@',        
};

# Define the directories and files
$trapDataDir = '/data/TRAP/';          # the actual TRaP directory is '/data/TRAP/$year/$stormNameNNB'
$savedListPrefix = "trapSaved";
$tempOut = '/tmp/tarout';
$trapARCHDir = '/data/TRAP/archive';   # directory to temporarily save some TRaP tar files

$webServer1 = 'sabweb\@gp5';
$webHomeDir = "/home/sabweb/TRAP";
# $webHomeDir = "/home/oper/mcidas/pctrap-V3";

$basinCM = "$webHomeDir/updateBasinFile.pl";
$basinFile = "basinName.txt";		# must match the name in "area2digit.pl"

## initialize the arrays which alert the user to save 7 days or 2 days 
#  before they are deleted.  storms in alertList7 will be deleted 7 days
#  after the alert, and storms in alertList2 will be deleted 2 days after the alert.

@alertList7 = ();
@alertList2 = ();

@deleteStormList = ();  # array to store the struct StormInfo for the web server

$deleteDay = 1.0; 	# delete the entry which is older than this day
$deleteMode = 3;

# user email address for notification
# $userAddress = "jianbin.yang\@noaa.gov";

# email lists for SAB analysts who will save the data, add Michelle.Spampata on 10/13/06
# $userAddress = "'Michael.Turk\@noaa.gov' 'Sheldon.Kusselson\@noaa.gov' " . 
#               " 'Greg.Gallina\@noaa.gov' 'Michelle.Spampata\@noaa.gov' " . 
#               " 'jianbin.yang\@noaa.gov'";

$userAddress = "renata.barski\@noaa.gov";


####------------------------------------------------------------------------------####

####--------------------------------- Start --------------------------------------####


$startdate=`date`;
print "Start  $0 at $startdate \n";

chdir("$trapDataDir") || die "Cannot change to directory $trapDataDir: $!";
print  "TRAP Data directory is:";
system("pwd");

$in7days = `date --date '7 days' +%D`;
chop($in7days);
$in2days = `date --date '2 days' +%D`;
chop($in2days);

$year = `date +%Y`;
chop($year);
$oldYear = $year - 1;

if (-d $oldYear)
{
   purgeData($oldYear);
}

if (-d $year)
{
   purgeData($year);
}

print "the alert list are @alertList7\n @alertList2\n";

# If there is any storm to be alerted for saving, issue an email
if ((@alertList7 != 0) || (@alertList2 != 0))
{
   print "Send email to alert for saving\n";

   open MAIL,"|mail -s 'Warning:Storms to be saved before they are deleted!!' $userAddress";

   if (@alertList7 != 0)
   {
      print MAIL "1st Warning: Please save the TRaP data for following storms within 7 days,
             before they are deleted on $in7days:\n @alertList7 \n\n";
   }

   if (@alertList2 != 0)
   {
      print MAIL "2nd Warning: Please save the TRaP data for following storms ASAP,
             if you have not saved them yet! They will be deleted on $in2days:\n @alertList2 \n\n";
   }

   print MAIL "A tar.gz file for above each storm has been generated for the saving purpose. 
               They are under the 'year' directory of '/data/TRAP/' directory";


   close MAIL;
}
else
{
   print "***** No storm to be saved  *****.\n";
   print "No email sent out.\n";
}

if (@deleteStormList != 0)
{
   print "\nDelete the following storm entry on Web server.\n\n";
   foreach $stormInfo (@deleteStormList)
   { 
      $stName = $stormInfo->stormName;
      $yr     = $stormInfo->year;
      @basinList = @{$stormInfo-> basinArea};
      foreach $basin (@basinList)
      { 
         print "ssh $webServer1 $basinCM $yr  $stName $basin $deleteMode\n"; 
         #system("$basinCM $yr  $stName $basin $deleteMode")
         system("ssh $webServer1 $basinCM $yr  $stName $basin $deleteMode")
      }
   }
}


print "End of clean old TRAP data\n";

############  Subroutine "purgeData" ###############
########################################################

sub purgeData
{
   my($yyyy) = @_;

   chdir($yyyy) || die "Cannot change to directory $yyyy: $!";
   print  "Current TRAP Data directory is:";
   system("pwd");

   my(@stormList) = `ls`;
   my($wholeName, $tarName, $count);
   my($savedList) = $savedListPrefix . $yyyy;
   my(@lsInfoList) = ();

   foreach $storm (@stormList)
   {
      chop($storm);
      
      # if ($storm =~ /\.tar$/)
     # {
     #    next;
     # }
   
      # will skip the non-directory files under the year      
      if (-d $storm)
      {
          chdir($storm) || die "Cannot change to directory $storm: $!";
          system("pwd");
      }
      else 
      {
          next;
      }
      

      # get the last TRAP created in this directory, and check its modification time
      $lastTRAP = `ls -tr|tail -1`;

      chop($lastTRAP);
      
      ## check whether the directory is for a storm or not
      # if not a storm, skip it
        
      $count = 0;
      #**  if (!($lastTRAP =~ /$storm\.WT/))   # ugly
     # while (($lastTRAP !~ /$storm\.WT/) && ($count < 2))
     # changed the "WT" to "W", since some header is "WOAU?", 
     # will modify it if the header is not started with "W"
     # delete the "W",as for FKAU
     # while (($lastTRAP !~ /$storm\.W/) && ($count < 2))
      while (($lastTRAP !~ /$storm\./) && ($count < 2))
      {
         # $lastTRAP = `ls -tr |grep '$storm.W'|tail -1`;
         $lastTRAP = `ls -tr |grep '$storm.'|tail -1`;
         chop($lastTRAP);
         $count++;
         print "count 2: lastTRAP is $lastTRAP\n";
      }

      if ($count > 1)
      {
         print "the current directory $storm is not for a storm\n";
         chdir("..") || die "Cannot change to upper directory: $!";
         next;
      }

      print "the last TRAP for the storm is **$lastTRAP**\n";
 
      $tarName = $storm . '.' . "tar";
      $tarGzName = $tarName . '.' . "gz";
      $wholeName = $yyyy . '/' . $storm;

      # delete the whole storm data 9 days after the storm was over
      if (-M $lastTRAP > 9)
      {
         print "delete the whole storm directory $storm\n";
         #rmtree("../$storm") || die "cannot rmtree $storm: $!";
         if ( -s "../$tarGzName" )
         {
            print "delete the tar file $tarGzName\n";
            unlink("../$tarGzName") || die "cannot unlink $tarGzName: $!";
         }
         else
         {
             chdir("..") || die "Cannot change to upper directory: $!";
          #  system("tar cvfP '$tarName' '$storm' > $tempOut");
             system("tar cvf '$tarName' '$storm' > $tempOut");
             system("gzip -f '$tarName' ");

             print "save the tar file to ARCHIVE for storm older than 9 days.\n";
             system("cp -p '$tarGzName' '$trapARCHDir' ");

             chdir("$storm") || die "Cannot change to $storm directory: $!";
          }
          rmtree("../$storm") || die "cannot rmtree $storm: $!";
          

      }
      elsif ((-M $lastTRAP > 2.0) && (-M $lastTRAP <= 3.0))
      {
         print "$storm is over, will be purged in 7 days, please save it before it is removed\n";
         print "tar file: $tarName is created for it.\n";
         chdir("..") || die "Cannot change to upper directory: $!";         
       #  system("tar cvfP '$tarName' '$storm' > $tempOut");
         system("tar cvf '$tarName' '$storm' > $tempOut");
         system("gzip -f '$tarName' ");

         # make a copy to temporarily save the tar file for SAB to archive,
         # in case there is problem for SAB analysts to save on time, add 08/30/2004, 02/15/2007

         print "save the tar file to ARCHIVE.\n";
         system("cp -p '$tarGzName' '$trapARCHDir' ");

         chdir("$storm") || die "Cannot change to $storm directory: $!";
         push(@alertList7, $wholeName );   

         # get the ls information for every storm to be deleted
         # and later will append all into the saved list file

         $lsInfo = `ls -l ../$tarName'.gz'`;
         @lsInfos = split(/\s+/, $lsInfo);

         $lsInfo = $lsInfos[4] . " \t" . $lsInfos[5] . " " . $lsInfos[6] . " " .
                   $lsInfos[7] . "\t" . $tarName . ".gz";

         push(@lsInfoList, $lsInfo);
      }
      elsif ((-M $lastTRAP > 7.0) && (-M $lastTRAP <= 8.0))
      {
         print "$storm is over, will be purged in 2 days, please save it before it is removed\n";

         # if the tar.gz file is not there, assumed it had been saved by analyst. 
         # So won't issue 2nd warning 
         if ( -s "../$tarGzName" )
         {
            print "$tarGzName is still there, put $storm into the 2nd warning list.\n"; 
            push(@alertList2 , $wholeName );
         }
         else
         {
             chdir("..") || die "Cannot change to upper directory: $!";
          #  system("tar cvfP '$tarName' '$storm' > $tempOut");
             system("tar cvf '$tarName' '$storm' > $tempOut");
             system("gzip -f '$tarName' ");

             print "save the tar file to ARCHIVE for storm is 7-8 days old.\n";
             system("cp -p '$tarGzName' '$trapARCHDir' ");

             chdir("$storm") || die "Cannot change to $storm directory: $!";
          }

      }

      ##------------------  Call the program to delete the entry of  the storm on Web server ------------------##   
   
      if (-M $lastTRAP > $deleteDay)
      {
         if (!(-s $basinFile))
         {
            print "$basinFile is not exist for storm $storm, go the next one.\n";
            chdir("..") || die "Cannot change to upper directory: $!";
            next;
         }

         open(BASINFILE, "< $basinFile") || die "Can not open $basinFile for reading: $!";
         @basinList = <BASINFILE>;
         close(BASINFILE);
         # my @basinList = ();
         for ($i = 0; $i < $#basinList+1; $i++)
         {
            chomp($basinList[$i]);
         #   push(@basinList, $basinList1[$i]);
         }

         my $stormInfo = StormInfo->new();
         $stormInfo->stormName($storm);
         $stormInfo->year($yyyy);
         @{$stormInfo->basinArea} = @basinList;
         push (@deleteStormList, $stormInfo);
      }
          

      # go back to the year directory for next storm
      chdir("..") || die "Cannot change to upper directory: $!";
   }

   # append the ls information for the saved storms to the saved list
   if (@lsInfoList != 0)
   {
      # print the header of the saved list file if the file is being created
      if (-s $savedList)
      {
         $listHeader = "";
      }
      else
      {
         $listHeader = "size\t\t time\t\tname\n";
      }

      open (LISTFILE, ">> $savedList") || die " Can't seem to open $savedList:$!";
      print LISTFILE $listHeader;

      foreach $lsInfo (@lsInfoList)
      {
         print LISTFILE "$lsInfo\n";
      }

      close (LISTFILE);
   }

   # get the current exist storms in that year to see if it is empty in the year
   @stormList = `ls`;
   
   # go back to the "TRAP" directory
   chdir("..") || die "Cannot change to upper directory: $!";  # change to the upper year directory

   if ((@stormList == 0) && ($yyyy == $oldYear))
   {
      print "There is no storm in last year $yyyy, delete the directory $yyyy\n\n";
      rmdir($yyyy)|| die "Cannot rmdir $yyyy: $!";
   }
}
