#!/usr/bin/perl -w

## =====================================================================
## Name:        cleanTRAP_bkup.pl  
##                  
## Author:      Jianbin Yang/RSIS   01/17/2003
## Language:    Perl
## Purpose:     This program clean the TRaP data for the storm 
##		8 days after it was over .  It also send user alert 
##		for saving the data one and 6 day after the storm is over. 
##                         
##                  
## Modified:   07/31/2003
##
## Modified:   02/04/2004
##             (will) Extend the date from 1 to 2 
##
## Modified: 10/15/2006
##                Add the code to delete entry of the storm in the web server               
##
## Modified: 03/27/2007
##
## Modified: 08/27/2007
##               Count the number of TRaP generated for the storm and add to the email notification
## Modified: 10/30/2007
##               Add AMSRE TRaP
##
## Modified: 06/24/2008
##               Change the email notice to programmer only, as the other SAB user notice 
##               is sent out from Sabna4 where they actually do the saving
##
## Modified: 10/14/2008
##               Clean the inacitve storm entry for the active storm list
##
## Modified: 04/07/09
##               Updated the active storm list if the storm cross basins (for eTRaP!)
##
## Modified: 05/06/09
##               Add the call to delete the storm entry for the eTRaP web page (for eTRaP!)
##
## Modified: 08/18/09
##               Turn off the call to delete entry for TRaP page
## Modified: 09/25/09
##               Add the call to change the entry for eTRaP archive table 
##               when storm is ended or renamed.
##
## Modified: 03/04/2011
##               Change email from Jianbin to Renata
##
## Last modified: 11/23/2011
##               Added lines for SSMIS input
##
## =====================================================================

use Class::Struct;
use File::Path;           # for "rmtree"

####---------------------- Define  and delcare ------------------------####

##------------------------   Define structures   ------------------------##

# structure to store the information to call the program on web server
struct StormInfo =>
{
   stormName => '$',        
   year      => '$',        
   basinArea => '@',        
};

# structure to store the count of the TRaP generated for each storm
struct StormCount =>
{
   stormName => '$',
   year      => '$',
   allCount  => '$',
   amsuCount => '$',
   ssmiCount => '$',
   ssmisCount => '$',
   trmmCount => '$',
   amsreCount => '$',
};


# Define the directories and files

$HOME = $ENV{"HOME"};

$bkup = 'true';
$trapDataDir = '/data/Petrap/';          # the actual TRaP directory is '/data/TRAP/$year/$stormNameNNB'
$savedListPrefix = "trapSaved";
$tempOut = '$HOME/mcidas/tmp/tarout';

# $webServer1 = 'sabweb\@gp5.espc.nesdis.noaa.gov';
$webServer1 = 'sabweb\@10.144.3.87';
$webServer2 = 'sabweb\@gp50.ssd.nesdis.noaa.gov';
$webHomeDir = "/home/sabweb/TRAP";
# $webHomeDir = "/home/oper1/mcidas/pctrap-V3";

#$basinCM = "$webHomeDir/updateBasinFile.pl";
$basinFile = "basinName.txt";		# must match the name in "area2digit.pl"

$basinETRAPCM = "$webHomeDir/ETRAP/updateBasinFile_etrap.pl";
$archETRAPCM = "$webHomeDir/ETRAP/updateArchTbl_etrap.pl";

$srcDir = "$HOME/mcidas/pctrap-V3";
$stormFile = "$srcDir/activeStorms.txt";
$tempFile = "$srcDir/tempStList.txt";
$bkFile = "$srcDir/activeStorms.txt.oldBk";
$oldStFile = "$trapDataDir/oldStorm.txt";

## initialize the arrays which alert the user to save 7 days or 2 days 
#  before they are deleted.  storms in alertList7 will be deleted 7 days
#  after the alert, and storms in alertList2 will be deleted 2 days after the alert.

@alertList7 = ();
@alertList2 = ();

@deleteStormList = ();  # array to store the struct StormInfo for the web server

$deleteDay = 1.0; 	# delete the entry which is older than this day
$deleteMode = 3;

@stormCountList = ();   # array to store the struct StormCount

# user email address for notification
# $userAddress = "jianbin.yang\@noaa.gov";

## email lists for SAB analysts who will save the data, add Michelle.Spampata on 10/13/06
# $userAddress = "'Michael.Turk\@noaa.gov' " . 
#               " 'Greg.Gallina\@noaa.gov' 'Michelle.Spampata\@noaa.gov' " . 
#               " 'jianbin.yang\@noaa.gov'";

$userAddress = "robert.glassberg\@noaa.gov";


####------------------------------------------------------------------------------####

####--------------------------------- Start --------------------------------------####


$startdate=`date`;
print "Start  $0 at $startdate \n";

chdir("$trapDataDir") || die "Cannot change to directory $trapDataDir: $!";
print  "TRAP Data directory is:";
system("pwd");

$in7days = `date --date '7 days' +%D`;
chop($in7days);
$in2days = `date --date '2 days' +%D`;
chop($in2days);

$year = `date +%Y`;
chop($year);
$oldYear = $year - 1;

if (-d $oldYear)
{
   purgeData($oldYear);
}

if (-d $year)
{
   purgeData($year);
}

print "the alert list are @alertList7\n @alertList2\n";

# If there is any storm to be alerted for saving, issue an email
if (((@alertList7 != 0) || (@alertList2 != 0)) && ($bkup == 'false'))
{
   print "Send email to alert for saving\n";
   print " bkup $bkup\n";
   open MAIL,"|mail -s 'Warning:Storms to be saved on geoprod before they are deleted!!' $userAddress";

   if (@alertList7 != 0)
   {
      print MAIL "1st Warning: Please save the TRaP data for following storms within 7 days,
             before they are deleted on $in7days:\n @alertList7 \n\n";
   }

   if (@alertList2 != 0)
   {
      print MAIL "2nd Warning: Please save the TRaP data for following storms ASAP,
             if you have not saved them yet! They will be deleted on $in2days:\n @alertList2 \n\n";
   }

   print MAIL "A tar.gz file for above each storm has been generated for the saving purpose. 
               They are under the 'year' directory of '/data/TRAP/' directory";


   # print the TRaP count if there is any
   if (@stormCountList != 0)
   { 
      print MAIL "\n\nHere are the TRaP counts for the above storms.\n\n"; 
      foreach $stormCount (@stormCountList)
      {
         printf MAIL "Storm:%s/%s\n", $stormCount->year, $stormCount->stormName;
         printf MAIL "ALL  - %s\n", $stormCount->allCount;
         printf MAIL "AMSU - %s\n", $stormCount->amsuCount;
         printf MAIL "SSMI - %s\n", $stormCount->ssmiCount;
         printf MAIL "SSMIS - %s\n", $stormCount->ssmisCount;
         printf MAIL "TRMM - %s\n\n", $stormCount->trmmCount;
         printf MAIL "AMSRE - %s\n", $stormCount->amsreCount;
      }

   }

   close MAIL;
}
else
{
   print "***** No storm to be saved  *****.\n";
   print "No email sent out.\n";
}

if (@deleteStormList != 0)
{
   # all the inacitve storm entries
   @stLineList = ();

   print "\nDelete the following storm entry on Web server.\n\n";
   foreach $stormInfo (@deleteStormList)
   { 
      $stName = $stormInfo->stormName;
      $yr     = $stormInfo->year;
      @basinList = @{$stormInfo-> basinArea};

      # Turn off the following for update TraP page
      #foreach $basin (@basinList)
      #{ 
      #   print "ssh $webServer1 $basinCM $yr  $stName $basin $deleteMode\n"; 
      #   #system("$basinCM $yr  $stName $basin $deleteMode")
      #   system("ssh $webServer1 $basinCM $yr  $stName $basin $deleteMode");
      #}
      
      #------ Delete the inactive storm entry from the eTRaP Web page (for the last basin) ------#

      print "Delete the inactive storm entry from the eTRaP Web page!\n";
      #print "Run this:ssh $webServer1 $basinETRAPCM $yr  $stName $basinList[-1] $deleteMode\n";
      #system("ssh $webServer1 $basinETRAPCM $yr  $stName $basinList[-1] $deleteMode");

      #print "No connection to GP50 as of 05/08, so the following won't run.\n";
      #print "Run this:ssh $webServer2 $basinETRAPCM $yr  $stName $basinList[-1] $deleteMode\n";
      #system("ssh $webServer2 $basinETRAPCM $yr  $stName $basinList[-1] $deleteMode");

      #------ Update the storm archive page for storm end ------#

      #print "Run this:ssh $webServer1 $archETRAPCM $yr  $stName $basinList[-1]\n";
      #system("ssh $webServer1 $archETRAPCM $yr $stName $basinList[-1]");

#      print "Run this:ssh $webServer2 $archETRAPCM $yr  $stName $basinList[-1]\n";
      system("ssh $webServer2 $archETRAPCM $yr $stName $basinList[-1]");

      #------ Remove the inactive storm entry from the active storm list ------#
      #------ Only has one entry for each storm for the first basin, changed on 04/2009: should be last basin  ------#
     
      #$stLine = `grep "$stName.*$yr.*$basinList[0]" $stormFile`; 
      $stLine = `grep "$stName.*$yr.*$basinList[-1]" $stormFile`; 
      chomp($stLine);

      if ($stLine eq "")
      {
         print "WARNING!  There no storm entry for $yr-$stName in the file $stormFile.\n";
         next;
      }
      push(@stLineList, $stLine);      
   }

   if (@stLineList != 0)
   {
      # Rewrite active storm file to remove the inactive storm entries 
      @newDataList = (); 
      open(STORMFILE, "< $stormFile") || die "Can not open $stormFile for reading: $!";
      @dataList = <STORMFILE>;
      close(STORMFILE);

      for ($i = 0; $i < $#dataList+1; $i++)
      {
         chomp($dataList[$i]);
         $isExist = isElement(\@stLineList, $dataList[$i]);
         if ($isExist < 0)
         {
            push(@newDataList, $dataList[$i]);
         }
      }
   
      open(TEMPFILE, "> $tempFile") || die "Can not open $tempFile for writing: $!";
      flock(TEMPFILE, 2);  # and lock it

      for ($i = 0; $i < $#newDataList+1; $i++)
      {
         print TEMPFILE "$newDataList[$i]\n";
      }
      close(TEMPFILE);

      rename($stormFile, $bkFile)            or die "can't rename $stormFile to $bkFile: $!";
      rename($tempFile, $stormFile)          or die "can't rename $tempFile to $stormFile: $!";

      open(OLDFILE, ">> $oldStFile") || die "Can not open $oldStFile for appending:$!";
      foreach $stLine (@stLineList)
      {
         print OLDFILE "$stLine\n";
      }
      close(OLDFILE);

      # push the storm list file to GP36 for etrap to run, will turn off when Etrap operational
      # added on 10/15/2008
      print "Not run in backup:Call pushStList.sh to push the storm list file to GP36 for etrap to run.\n\n";
      #system ("$srcDir/pushStList.sh");

   }  # End of the IF for $stLineList
   else
   {
      print "WARNING! There is no storm entry for all inactive storms list in the deleteStormList.\n";
   }

}


print "End of clean old TRAP data\n";

############  Subroutine "purgeData" ###############
########################################################

sub purgeData
{
   my($yyyy) = @_;

   chdir($yyyy) || die "Cannot change to directory $yyyy: $!";
   print  "Current TRAP Data directory is:";
   system("pwd");

   my(@stormList) = `ls`;
   my($wholeName, $tarName, $count);
   my($savedList) = $savedListPrefix . $yyyy;
   my(@lsInfoList) = ();

   foreach $storm (@stormList)
   {
      chop($storm);
      
      # if ($storm =~ /\.tar$/)
     # {
     #    next;
     # }
   
      # will skip the non-directory files under the year      
      if (-d $storm)
      {
          chdir($storm) || die "Cannot change to directory $storm: $!";
          system("pwd");
      }
      else 
      {
          next;
      }
      

      # get the last TRAP created in this directory, and check its modification time
      $lastTRAP = `ls -tr|tail -1`;

      chop($lastTRAP);
      
      ## check whether the directory is for a storm or not
      # if not a storm, skip it
        
      $count = 0;
      #**  if (!($lastTRAP =~ /$storm\.WT/))   # ugly
     # while (($lastTRAP !~ /$storm\.WT/) && ($count < 2))
     # changed the "WT" to "W", since some header is "WOAU?", 
     # will modify it if the header is not started with "W"
     # delete the "W",as for FKAU
     # while (($lastTRAP !~ /$storm\.W/) && ($count < 2))
      while (($lastTRAP !~ /$storm\./) && ($count < 2))
      {
         # $lastTRAP = `ls -tr |grep '$storm.W'|tail -1`;
         $lastTRAP = `ls -tr |grep '$storm.'|tail -1`;
         print "count 2: lastTRAP is $lastTRAP\n";
         chop($lastTRAP);
         $count++;
      }

      if ($count > 1)
      {
         print "the current directory $storm is not for a storm\n";
         chdir("..") || die "Cannot change to upper directory: $!";
         next;
      }

      print "the last TRAP for the storm is **$lastTRAP**\n";
 
      $tarName = $storm . '.' . "tar";
      $tarGzName = $tarName . '.' . "gz";
      $wholeName = $yyyy . '/' . $storm;

      # delete the whole storm data 9 days after the storm was over
      if (-M $lastTRAP > 9)
      {
         print "delete the whole storm directory $storm\n";
         #rmtree("../$storm") || die "cannot rmtree $storm: $!";
         if ( -s "../$tarGzName" )
         {
            print "delete the tar file $tarGzName\n";
            #unlink("../$tarGzName") || die "cannot unlink $tarGzName: $!";
         }
      }
      elsif ((-M $lastTRAP > 2.0) && (-M $lastTRAP <= 3.0))
      {
         print "$storm is over, will be purged in 7 days, please save it before it is removed\n";
         print "tar file: $tarName is created for it.\n";
         chdir("..") || die "Cannot change to upper directory: $!";         
       #  system("tar cvfP '$tarName' '$storm' > $tempOut");
         system("tar cvf '$tarName' '$storm' > $tempOut");
         system("gzip -f '$tarName' ");
         chdir("$storm") || die "Cannot change to $storm directory: $!";
         push(@alertList7, $wholeName );   

         # get the ls information for every storm to be deleted
         # and later will append all into the saved list file

         $lsInfo = `ls -l ../$tarName'.gz'`;
         @lsInfos = split(/\s+/, $lsInfo);

         $lsInfo = $lsInfos[4] . " \t" . $lsInfos[5] . " " . $lsInfos[6] . " " .
                   $lsInfos[7] . "\t" . $tarName . ".gz";

         push(@lsInfoList, $lsInfo);


         ##--------- Count the number of the TRaP generated for the storm ---------##
          
         print "\nCount the storm $storm!!\n";
         my $stormCount = StormCount->new();
         $stormCount->stormName($storm);
         $stormCount->year($yyyy);
         $allCount = `$HOME/mcidas/pctrap-V3/SHOWTRAP ALL $yyyy$storm| grep $storm |grep -v HEHR|wc -l`;
         chomp($allCount);
         $amsuCount = `$HOME/mcidas/pctrap-V3/SHOWTRAP ALL $yyyy$storm AMSU| grep $storm |wc -l`;
         chomp($amsuCount);
         $ssmiCount = `$HOME/mcidas/pctrap-V3/SHOWTRAP ALL $yyyy$storm SSMI| grep $storm |wc -l`;
         chomp($ssmiCount);
         $ssmisCount = `$HOME/mcidas/pctrap-V3/SHOWTRAP ALL $yyyy$storm SSMIS| grep $storm |wc -l`;
         chomp($ssmisCount);
         $trmmCount = `$HOME/mcidas/pctrap-V3/SHOWTRAP ALL $yyyy$storm TRMM| grep $storm |wc -l`;
         chomp($trmmCount);
         $amsreCount = `$HOME/mcidas/pctrap-V3/SHOWTRAP ALL $yyyy$storm AMSRE| grep $storm |wc -l`;
         chomp($amsreCount);
         $stormCount->allCount($allCount);
         $stormCount->amsuCount($amsuCount);
         $stormCount->ssmiCount($ssmiCount);
         $stormCount->ssmisCount($ssmisCount);
         $stormCount->trmmCount($trmmCount);
         $stormCount->amsreCount($amsreCount);
  
         push(@stormCountList, $stormCount);
         
      }
      elsif ((-M $lastTRAP > 7.0) && (-M $lastTRAP <= 8.0))
      {
         print "$storm is over, will be purged in 2 days, please save it before it is removed\n";

         # if the tar.gz file is not there, assumed it had been saved by analyst. 
         # So won't issue 2nd warning 
         if ( -s "../$tarGzName" )
         {
            print "$tarGzName is still there, put $storm into the 2nd warning list.\n"; 
            push(@alertList2 , $wholeName );
         }
      }

      ##------------------  Call the program to delete the entry of  the storm on Web server ------------------##   
   
      if (-M $lastTRAP > $deleteDay)
      {
         if (!(-s $basinFile))
         {
            print "$basinFile is not exist for storm $storm, go the next one.\n";
            chdir("..") || die "Cannot change to upper directory: $!";
            next;
         }

         open(BASINFILE, "< $basinFile") || die "Can not open $basinFile for reading: $!";
         @basinList = <BASINFILE>;
         close(BASINFILE);
         # my @basinList = ();
         for ($i = 0; $i < $#basinList+1; $i++)
         {
            chomp($basinList[$i]);
         #   push(@basinList, $basinList1[$i]);
         }

         my $stormInfo = StormInfo->new();
         $stormInfo->stormName($storm);
         $stormInfo->year($yyyy);
         @{$stormInfo->basinArea} = @basinList;
         push (@deleteStormList, $stormInfo);
      }
          

      # go back to the year directory for next storm
      chdir("..") || die "Cannot change to upper directory: $!";
   }

   # append the ls information for the saved storms to the saved list
   if (@lsInfoList != 0)
   {
      # print the header of the saved list file if the file is being created
      if (-s $savedList)
      {
         $listHeader = "";
      }
      else
      {
         $listHeader = "size\t\t time\t\tname\n";
      }

      open (LISTFILE, ">> $savedList") || die " Can't seem to open $savedList:$!";
      print LISTFILE $listHeader;

      foreach $lsInfo (@lsInfoList)
      {
         print LISTFILE "$lsInfo\n";
      }

      close (LISTFILE);
   }

   # get the current exist storms in that year to see if it is empty in the year
   @stormList = `ls`;
   
   # go back to the "TRAP" directory
   chdir("..") || die "Cannot change to upper directory: $!";  # change to the upper year directory

   if ((@stormList == 0) && ($yyyy == $oldYear))
   {
      print "There is no storm in last year $yyyy, delete the directory $yyyy\n\n";
      rmdir($yyyy)|| die "Cannot rmdir $yyyy: $!";
   }
}

############  Subroutine "isElement" ###############
########################################################
## if it is one of the element of the array the index is return,
## otherwise return -1

sub isElement
{
   # my($element, @array) = @_;
   my($array, $element) = @_;
   my($foundIndex) = -1;

   # must specify $i as local variable, otherwise will conflict with $i in
   # the calling phase (main function)
   my($i);

   # can also write as @{$array}
   ### print " the array is @array\n";

   for ($i= 0; $i < @$array; $i++)
   {
      ### print "inside the isElement, the element is @$array[$i], i is $i\n";
      if (@$array[$i] eq $element)
      {
         $foundIndex = $i;
         last;
      }
   }

   return $foundIndex;
}

