#!/usr/bin/perl -w

## =====================================================================
## Name:        createTRaPDir.pl
##                  
## Author:      Jianbin Yang/QSS   08/28/2006
## Language:    Perl
## Purpose:     Create the directories structure on the SAB Web server for TRaP display           
##                  
## Usage:       createTRaPDir.pl <Name of the TRaP set> <basin>
##
##              Format of <Name of the TRaP set> is: 
##                     YYYYnameNNB.WTXXnn.FFFF.DDHHMM.SSSS.ddhhmm 
##                  or YYYYnameNNB.WTXXnn.FFFF.DDHHMM.SSSS.ddhhmm.nn
##                  or YYYYnameNNB.WTXXnn.FFFF.DDHHMM.SSSS.ddhhmm.nn.GIF
##
## NOTE:        This program will run on the Web server remotely
##
## Modified:   09/21/2006
##              Add the code to send the TRaP GIF file to the destination 
##		direcotry
##
## Modified:   10/05/2006
##              Change the agency directory name to be consistent with the ones on FTP server
##
## Modified:   10/17/2006
##              Call the program "updateBasinFile.pl" when it is the first storm for a basin
##
## Modified: 10/18/2006
##              Call the bash program to generate some TRaP web page
##
## Modified: 10/26/2006
##              Create the current html to include the agency html
##
## Modified: 03/01/2007
##		Handle the case when the name is LOW for Australian area, as it may exist before
##
## Modified: 03/07/2007
##              Force the FKAU TRaP to Appropriate agent, other than determine base on the station (Darwin)
##
## Modified: 04/04/2007
##		Month is added to the TRaP name for the rain rate time, so some modification need to be made here
##
## Modified: 06/25/2007
##		Handle the case when the storm becomes active again.  For example, 03B in NorthIndian 
##		after inactive on 06/21/07, get back on 06/25/07
##
## Modified: 07/03/2007
##		Add the New Delhi and New Zealand bulletin
##		Remove some code which copied unused text files for web page
##
## Modified: 10/29/2007
##		Add to send the AMSR-E TRaP to web page
##
## Last Modified: 12/26/2008
##              Add the Jakarta agent
##
## =====================================================================

# Define the directories and files

$webHomeDir = "/home/sabweb/TRAP";

$trapDataDir = "/data/web/mirror/www.ssd.noaa.gov/htdocs/PS/TROP/DATA/TRAP";
$newTrapList = "$trapDataDir/newTRaPList.log";

$basinCM = "$webHomeDir/updateBasinFile.pl";
$textDir = "$webHomeDir/txt";

$blankFile = "$webHomeDir/blank-template.txt";

# mode for the updateBasinFile.pl command line input
$createMode = 1;
$updateMode = 2;

# for the Sensor html text file
$sensorTemplateFile = "$webHomeDir/sensor-template.txt";

# define some constants list in the sensor template file
$tempSensor = "SENSOR";
$tempSetName = "PATH_SETNAME";
$tempTime = "RR_TIME";

# Files need to be copied to other directories
$currentGif = "current.gif";            # current gif file name
#! may not need $agencyTextFile = "current.htm";        # the fix name for the most recent agency text file
$nameFile = "name.txt";
# $dash = "-";
$dash = " -";

$amsuText = "amsu.txt";
$trmmText = "trmm.txt";
$ssmiText = "ssmi.txt";
$amsreText = "amsre.txt";

$amsuListText = "amsu-list.txt";
$trmmListText = "trmm-list.txt";
$ssmiListText = "ssmi-list.txt";
$amsreListText = "amsre-list.txt";

$emptyAmsuText = "$textDir/MT-AMSU.txt";
$emptyTrmmText = "$textDir/MT-TRMM.txt";
$emptySsmiText = "$textDir/MT-SSMI.txt";
$emptyAmsreText = "$textDir/MT-AMSRE.txt";

#! May not need the following three lines
$currentAmsuText = "current.amsu.txt";
$currentTrmmText = "current.trmm.txt";
$currentSsmiText = "current.ssmi.txt";
$currentAmsreText = "current.amsre.txt";

# the script to generate TRaP pages, change to "ops" on 11/05/07
#$genTRaPPageCM="$webHomeDir/gen-trap-pages.bash";
$genTRaPPageCM="$webHomeDir/gen-trap-pages-ops.bash";



##----------------------------- Start the Program -----------------------------##

$startdate=`date`;
chomp($startdate);

print "\n-------------------- Start  $0 at $startdate --------------------\n";

chdir("$trapDataDir") || die "Cannot change to directory $trapDataDir";
print  "TRaP Data directory is:";
system("pwd");

# remove the path from the command name

@comandFields = split(/\//, $0);
$commandName = $comandFields[-1];

# if no argument input, exit the program

# if ($ARGV[0] eq "")
if (@ARGV < 2)
{
   print "Usage: $commandName <Name of the TRaP set> <basin>\n";
   print "Format of <Name of the TRaP set> is: YYYYnameNNB.WTXXnn.FFFF.DDHHMM.SSSS.ddhhmm\n";
   die "Not enough arguments - exit script";
}

$trapSetName = $ARGV[0];
$basin = $ARGV[1];

print "trapSetName is $trapSetName, basin is $basin.\n";

# get the directories name of the specific TRaP set

@splitMsg = split(/\./, $trapSetName);
$splitMsgLen = @splitMsg;

# printf ("the number of splitMsg is %d.\n",($#splitMsg + 1));

if ($splitMsgLen < 6)
{
   print "Format of <Name of the TRaP set> is: 
          \rYYYYnameNNB.WTXXnn.FFFF.DDHHMM.SSSS.ddhhmm(.nn.GIF)\n\n";
   die "Input TRaP set name is not long enough or not correct - exit";
}
elsif ($splitMsgLen > 6)
{
   $trapSetName = $splitMsg[0];
   
   for ($i = 1; $i < 6; $i++)
   {
      $trapSetName .= '.';
      $trapSetName .= $splitMsg[$i];
   }
   print "The true TRaP set name is $trapSetName\n";
}

# system("echo $trapSetName >> $newTrapList");

$dirMsg = $splitMsg[0];
$year = substr($dirMsg, 0, 4);
$nameNnb = substr($dirMsg,4);
# $outputDir = "$trapDataDir/$year/$nameNnb";
$stormName = $nameNnb;
$bulID = $splitMsg[1];
$agency = $splitMsg[2];		# actually it is station ID here
$bulTime = $splitMsg[3];
$dataSource = $splitMsg[4];
# $rrTime = substr($splitMsg[5],0, 6);
$rrTime = substr($splitMsg[5],2, 6);


$basinFile = $basin . ".txt";

$basinFile = "$trapDataDir/$year/$basinFile";


if ($agency eq "KNHC")
{
   $agency = "TPC"; 
}
elsif ($agency eq "PHFO")
{
   # $agency = "CHPC";   # corrected it on 07/20/07
   $agency = "CPHC";
}

elsif ($agency eq "PGTW") 
{
   $agency = "JTWC"; 
}
elsif ($agency eq "RJTD")
{
   $agency = "Tokyo";
}
elsif ($agency eq "NFFN")
{
   $agency = "Fiji";
}
elsif ($agency eq "FMEE")
{
   $agency = "Reunion";
}
elsif ($agency eq "APRF")
{
   $agency = "Perth";
}
elsif ($agency eq "ABRF")
{
   $agency = "Brisbane";
}
elsif ($agency eq "ADRM")
{
   $agency = "Darwin";
   
   # on 03/07/07, fix the case for FKAU TRaP
   if (($basin =~ /SEIndian/) && ($bulID =~ /FKAU/))
   {
      $agency = "Perth";
   }
   elsif (($basin =~ /SWPacific/) && ($bulID =~ /FKAU/))
   {
      $agency = "Brisbane";
   }

}

elsif (($agency eq "DEMS") || ($agency eq "VIDP"))
{
   $agency = "NewDelhi";
}
elsif ($agency eq "NZKL")
{
   $agency = "Wellington";
}

elsif ($agency eq "WIIX")
{
   $agency = "Jakarta";
}



print "StormName is $stormName, year is $year, bulID is $bulID, agency is $agency, bul Time is $bulTime,
       \rdataSource is $dataSource, basin is $basin  \n";

print "dirMsg is $dirMsg, year is $year, nameNNB is $nameNnb.\n";


chdir("$trapDataDir") || die "Cannot change to directory $trapDataDir";
print  "TRaP Data directory is:";
system("pwd");


##------------ Create the yyyy/basin/stormName/agency/sensor directories if not exist -------------##

if (!(-d $year))
{
   print "Create the directory '$year' for the current year.\n";
   mkdir($year,0755) || die "cannot mkdir $year: $!";
}

chdir("$year") || die "Cannot change to directory $year";

if (!(-d $basin))
{
   $isBasinExist = 0;
   print "Create the directory '$basin'.\n";
   mkdir($basin,0755) || die "cannot mkdir $basin: $!";
   print "Call the program $basinCM to create the HTML text for basin $basin.\n";
   system ("$basinCM $year $stormName $basin $createMode");
   print "End of Calling the program $basinCM to create the basin HTML text.\n";
}
else
{
  $isBasinExist = 1;
}

chdir("$basin") || die "Cannot change to directory $basin";


if (!(-d $stormName))
{
   $isFirstTRaP = 1;
   print "Create the directory '$stormName' for the new storm.\n";
   mkdir($stormName,0755) || die "cannot mkdir $stormName: $!";
   if ($isBasinExist == 1)
   {
      print "Call the program $basinCM to update the basin HTML text for new storm $stormName.\n";

      # Noted that in the program "basinCM", it may change the mode to create mode 
      # if there is no storm entry in the basin file

      system ("$basinCM $year $stormName $basin $updateMode");
      print "End of Calling the program $basinCM to update the basin HTML text.\n";
   }
}
else
{
  $isFirstTRaP = 0;

  # handle the case with LOW in the name, mostly for system in Australian area

  # remove the if for LOW, and only consider it is First TRaP for LOW system.
  # 03B became active again after 3 days' quiet in NorthIndian on /06/25/07

  #if ($stormName =~ /\_LOW$/)
  #{
     $sizeBasin = -s $basinFile;
     $sizeBlank = -s $blankFile;
     print "sizeBasin is $sizeBasin, sizeBlank is $sizeBlank\n";
     # if ($sizeBasin <= $sizeBlank )
     if ( (-s $basinFile) == (-s $blankFile) )
     {
        print "Storm $stormName exist before, but the $basinFile is blank.\n";
        print "Call the program $basinCM to create the HTML text for basin $basin.\n";
        system ("$basinCM $year $stormName $basin $createMode");
        print "End of Calling the program $basinCM to create the basin HTML text.\n";

        if ($stormName =~ /\_LOW$/)
        {
           $isFirstTRaP = 1;
        }
     }
     else
     {
        # print "Storm $stormName exist before, the $basinFile is not blank.\n";
        $isNew = `grep -i $stormName $basinFile`; 
        
        if ($isNew eq "")
        {
           print "Storm $stormName exist before, but the storm is not in the $basinFile, so storm is new.\n";
           print "Call the program $basinCM to update the basin HTML text for new storm $stormName.\n";
           system ("$basinCM $year $stormName $basin $updateMode");
           print "End of Calling the program $basinCM to update the basin HTML text.\n"; 

           if ($stormName =~ /\_LOW$/)
           { 
              $isFirstTRaP = 1;
           }
        }
     }
   # }
}

chdir("$stormName") || die "Cannot change to directory $stormName";

if (!(-d $agency))
{ 
   $isAgencyNew = 1; 
   print "Create the directory '$agency'.\n";
   mkdir($agency,0755) || die "cannot mkdir $agency: $!";

   #-------------- Copy the empty amsu.txt, etc to the new agency directory --------------#
 
   print "Initialize the sensor and sensor-list files\n";
 
## will delete those #!

   system("cp $emptyAmsuText $agency/$amsuText");
   system("cp $emptyTrmmText $agency/$trmmText");
   system("cp $emptySsmiText $agency/$ssmiText");
   system("cp $emptyAmsreText $agency/$amsreText");

#   chmod(0755, "$agency/$amsuText");
#   chmod(0755, "$agency/$trmmText");
#   chmod(0755, "$agency/$ssmiText");

#!   system("cp $emptyAmsuText $amsuListText");
#!   system("cp $emptyTrmmText $trmmListText");
#!   system("cp $emptySsmiText $ssmiListText");


   # too check if there is another agency dir exist, use the following
   # Noted that it is in the storm dir now

   # $isOtherAgencyExist = `ls -R|grep "$year$stormName"`;
   # print "isOtherAgencyExist is $isOtherAgencyExist\n";

   # if ($isOtherAgencyExist ne "")
   # { do something for the agnency.htm}

}
else
{
  $isAgencyNew = 0;

  if ($isFirstTRaP == 1)
  {
     print "The Storm $stormName exist before, we consider the Agency is new, even though the dir exist.\n";
     $isAgencyNew = 1;

     #-------------- Copy the empty amsu.txt, etc to the new agency directory --------------#

     print "Initialize the sensor files\n";

     system("cp $emptyAmsuText $agency/$amsuText");
     system("cp $emptyTrmmText $agency/$trmmText");
     system("cp $emptySsmiText $agency/$ssmiText");
     system("cp $emptyAmsreText $agency/$amsreText");

#!     system("cp $emptyAmsuText $amsuListText");
#!     system("cp $emptyTrmmText $trmmListText");
#!     system("cp $emptySsmiText $ssmiListText");
  }
}


chdir("$agency") || die "Cannot change to directory $agency";

if (!(-d $dataSource))
{
   print "Create the directory '$dataSource'.\n";
   mkdir($dataSource,0755) || die "cannot mkdir $dataSource: $!";
}

chdir("$dataSource") || die "Cannot change to directory $dataSource";


print  "Current Data directory is:";
system("pwd");


##-------------- Send the TRaP data to destination directory --------------##

$destDir = `pwd`;
chomp($destDir);

print "destDir is $destDir\n";

chdir() || die "Cannot change to home directory";
print  "Current directory is:";
system("pwd");

print "Move the TRaP GIF file $trapSetName to $destDir\n";
system("mv $trapSetName* $destDir");
#system("cp -p $trapSetName* $destDir");

print "Update the log file $newTrapList\n";

system("echo $year/$basin/$stormName/$agency/$dataSource/$trapSetName >> $newTrapList");

##------------------- Create the basin Html Text file if there is none -------------------##

# $basinFile = $basin . ".txt";

# $basinFile = "$trapDataDir/$year/$basinFile";

if (!(-s $basinFile))
{
   print "$basinFile is not exist.  Create one.\n";
   system ("$basinCM $year $stormName $basin $createMode");
}

##------------------------------------------------------------------------##

##------------------- Call gen-trap-pages program here -------------------##

if ($isFirstTRaP == 1)
{
   print "Call the script $genTRaPPageCM to generate TRaP web pages.\n";
   # bash gen-trap-pages.bash 2006/Atlantic/HELENE/TPC/AMSU/2006HELENE.WTNT23.KNHC.240835.AMSU.240843
   system("bash $genTRaPPageCM $year/$basin/$stormName/$agency/$dataSource/$trapSetName");
}
print "isFirstTRaP is $isFirstTRaP\n";



##------------------------------------------------------------------------##

##------------------- Create the SENSOR Html Text file -------------------##
##------------------- And Copy some Text files and GIF -------------------##

#chdir("$trapDataDir/$year") || die "Cannot change to directory $trapDataDir/$year";
chdir("$trapDataDir/$year/$basin/$stormName/$agency") || die "Cannot change to directory $trapDataDir/$year/$basin/$stormName/$agency";
print  "Current Data directory is:";
system("pwd");

# will create amsu.txt here, 10/13/2006
# use tr/A-Z/a-z/;     
$tmSource = $dataSource;
$tmSource =~ tr/A-Z/a-z/;
$sensorFileName = $tmSource  . ".txt";

# sensor list file for archive , 10/17/2006
$sensorListFileName = $tmSource  . "-list" . ".txt";
# empty pattern in the list file
$emptyPattern = "Unavailable";


# Have to double "\" as one for Perl, another for "sed' in Shell
# $pathSetName = "$basin/$stormName/$agency/$dataSource/$trapSetName";
# $pathSetName = "$basin\\/$stormName\\/$agency\\/$dataSource\\/$trapSetName";
$pathSetName = "$year\\/$basin\\/$stormName\\/$agency\\/$dataSource\\/$trapSetName";

# change the rain rate time pattern
$dd = substr($rrTime, 0, 2);
$hhmm = substr($rrTime, 2, 4);
$rrTime1 = "$dd\\/$hhmm" . 'Z';

if ($dataSource eq "AMSU")
{
   $dataSource1 = "AMSU-B\\/MHS";
}
else
{
   $dataSource1 = $dataSource;
}
#system("sed -e 's/$tempSensor/$dataSource/' -e 's/$tempSetName/$pathSetName/g' -e 's/$tempTime/$rrTime1/g' $sensorTemplateFile > sensor_tm.txt");
system("sed -e 's/$tempSensor/$dataSource1/' -e 's/$tempSetName/$pathSetName/g' -e 's/$tempTime/$rrTime1/g' $sensorTemplateFile > sensor_tm.txt");

# $size=(-s "sensor_tm.txt"); 
# if ((-s "sensor_tm.txt") > 0)
if (-s "sensor_tm.txt")
{
   print "Create the sensor text file $sensorFileName and update the $sensorListFileName.\n";

   rename("sensor_tm.txt", $sensorFileName);
   #chmod(0755, $sensorFileName);
   chmod(0644, $sensorFileName);

   #------------------------ update the list file ------------------------#

   if (!(-s $sensorListFileName))
   {
      system("cp $sensorFileName $sensorListFileName");
   }
   else
   {
      $isEmpty = `grep -i $emptyPattern $sensorListFileName`;
      if ($isEmpty ne "")
      {
         system("cp $sensorFileName $sensorListFileName");
      }
      else
      {
         system("cat $sensorFileName $sensorListFileName > sensorList_tm.txt");
         rename("sensorList_tm.txt", $sensorListFileName);
      }
   }

   chmod(0644, $sensorListFileName);

   #--------- create the name text file ---------#

   print "Create the name file $nameFile\n";

   system("echo '$stormName$dash' > $trapDataDir/$nameFile");
   chmod(0644, $nameFile);
   
}
else
{
   print "WARNING!! Failed to get the new SENSOR text file, sensor_tm.txt is empty.  Exit the program $0!.\n";
   exit;
}


#------------------------ Copy the files ------------------------#

$agencyLegendText = "top-legend.txt"; 
$currentAgencyText = "current.agency.txt"; 
$currentLegendText = "current.top-legend.txt";  #! ??May not need

$agencyText = $agency . ".txt"; 
$AgencyAmsuText = $agency . '.' . $amsuText;
$AgencyTrmmText = $agency . '.' . $trmmText;
$AgencySsmiText = $agency . '.' . $ssmiText;
$AgencyAmsreText = $agency . '.' . $amsreText;

$AgencySensorList = $agency . "." . $sensorListFileName;
$AgencySensor = $agency . "." . $sensorFileName;


#--- Copy the Sensor text files ---#

print "Copy the sensor text files to the TRAP directory for most recent TRaP.\n";
print "***** The current directory is *****\n";
system("pwd");

system("cp $sensorListFileName $trapDataDir/$year/$basin/$stormName/$AgencySensorList");
## the following line may replace the lines:"cp $trmmText $trapDataDir/$year/$basin/$stormName/$AgencyTrmmText"
#system("cp $sensorFileName $trapDataDir/$year/$basin/$stormName/$AgencySensor");

system("cp $amsuText $trapDataDir/$amsuText");

system("cp $amsuText $trapDataDir/$AgencyAmsuText");  	#! ??May not need
system("cp $amsuText $trapDataDir/$currentAmsuText"); 	#! ??May not need
system("cp $amsuText $trapDataDir/$year/$currentAmsuText"); #! ??May not need

system("cp $amsuText $trapDataDir/$year/$basin/$stormName/$AgencyAmsuText");  #! ??May not need if use the $sensorFileName

system("cp $trmmText $trapDataDir/$trmmText");
#! system("cp $trmmText $trapDataDir/$AgencyTrmmText");
#! system("cp $trmmText $trapDataDir/$currentTrmmText");
#! system("cp $trmmText $trapDataDir/$year/$currentTrmmText");
system("cp $trmmText $trapDataDir/$year/$basin/$stormName/$AgencyTrmmText");  #! ??May not need if use the $sensorFileName

system("cp $ssmiText $trapDataDir/$ssmiText");
#! system("cp $ssmiText $trapDataDir/$AgencySsmiText");
#! system("cp $ssmiText $trapDataDir/$currentSsmiText");
#! system("cp $ssmiText $trapDataDir/$year/$currentSsmiText");
system("cp $ssmiText $trapDataDir/$year/$basin/$stormName/$AgencySsmiText");  #! ??May not need if use the $sensorFileName

system("cp $amsreText $trapDataDir/$amsreText");
system("cp $amsreText $trapDataDir/$year/$basin/$stormName/$AgencyAmsreText");  #! ??May not need if use the $sensorFileName


system("cp $textDir/$agencyText $trapDataDir/$currentAgencyText");
system("cp $textDir/$agencyLegendText $trapDataDir/$currentLegendText"); 	#! ??May not need



#--- Copy the current GIF file ---#

print "Copy the 24 hour TRaP to current GIF file : $currentGif.\n";

#system("cp $destDir/$trapSetName.24.hi.GIF $trapDataDir/$currentGif");
system("cp $destDir/$trapSetName.24.GIF $trapDataDir/$currentGif");  # change on 10/19
system("cp $destDir/$trapSetName.24.GIF ../$currentGif");

#--- Copy the agency text file ---#

#! $currentAgency = $agency . ".htm"; removed per Nancy email of 10/26
#! $currentListAgency = $agency . "-list.htm"; removed per Nancy email of 10/26

#! May not need the following if, as they are initialize on Nancy's program
if ($isAgencyNew == 1)
{
   print "Agency is new.  Initialize the agency text files:$agencyLegendText and $agencyText.\n";
   system("cp $textDir/$agencyText $trapDataDir/$year/$basin/$stormName/$agencyText");
   system("cp $textDir/$agencyLegendText $trapDataDir/$year/$basin/$stormName/$agencyLegendText");
}


print "\n-------------------- End of the program $0 --------------------\n\n";
