#!/bin/bash
# File :fetchNewPass.sh
# This shell script check whether there is new AMSU/TRMM/SSMI pass data file coming
# in.  If there is available and not processed and then store the name into the
# avaliable file
#
# Author: Jianbin Yang/RSIS   June 2002
#
# Modified: 09/13/2006
#                SSMI Geolocation file name change from SSGEO.EDRR to SSGEO.NPR.EDRR
#
# Modified: 06/04/2007
#		Restrict the search list to $numPass
#
# Modified: 09/07/2007
#               Add tne AMSRE Geolocation file
#
# Modified: 03/11/2008
#               Correct the sorting for the list of SSMI
#
# Last Modified: 11/29/2011
#               Added SSMIS source
#
################################################################################

echo "start $0 at `date`"

passBase="/data/Petrap"
passDataDir=$passBase/data/mcidas_data/polerPassData
#passDataDir="$dataDir/mcidas_data/polerPassData"

donelist=$passDataDir/passProcessed.log
localList=$passDataDir/localPasslist
newList=$passDataDir/newPassToProcess.log

amsub_file='amsub.D[0-9][0-9][0-9][0-9][0-9].S[0-9][0-9][0-9][0-9].E[0-9][0-9][0-9][0-9].??'
trmm_file='trmm.[0-9][0-9][0-9][0-9]-[0-9][0-9]-[0-9][0-9]T[0-9][0-9]-[0-9][0-9]-[0-9][0-9]Z'
#ssmi_file='SSGEO.EDRR.S[0-9].D[0-9][0-9][0-9][0-9][0-9].S[0-9][0-9][0-9][0-9].E*'
ssmi_file='SSGEO.NPR.EDRR.S[0-9].D[0-9][0-9][0-9][0-9][0-9].S[0-9][0-9][0-9][0-9].E*'
ssmis_file='GEO.EDEB.S[A-C].D[0-9][0-9][0-9][0-9][0-9].S[0-9][0-9][0-9][0-9].E*'

# amsre.Rain_V09_200709062148_A, amsre.Rain_V09_200709070156_D
amsre_file='amsre.Rain_???_[0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9]_?'

numPass=20

cd $passDataDir
pwd

# remove the previous new pass log file if it is exist
if [[ -f $newList ]]
then
   rm $newList
fi

# list all the AMSU pass files in pass data directory
#ls $amsub_file > /dev/null 2>&1
ls $amsub_file
success=$?
# if there are AMSU files, success will be 0
if (($success == 0 ))
then
   #echo "Get the local pass data list"  >> /home/oper3/rlg/fetch.log
   # ls $amsub_file > $localList
   #echo "amsu sort"  >> /home/oper3/rlg/fetch.log
   ls $amsub_file | sort -r   |head -$numPass | sort    > $localList
fi

# list all the TRMM pass files in pass data directory
#ls $trmm_file > /dev/null 2>&1
ls $trmm_file 
success=$?

# if there are TRMM files, success will be 0
if (($success == 0))
then
   #ls $trmm_file >> $localList
   #echo "trmm sort" >> /home/oper3/rlg/fetch.log
   ls $trmm_file | sort -r   |head -$numPass | sort    >> $localList
fi

# list all the SSMIS pass files in pass data directory
#ls $ssmis_file > /dev/null 2>&1
ls $ssmis_file
success=$?

# if there are SSMIS files, success will be 0
if (($success == 0))
then
   echo "success ssmis"
   #GEO.EDEB.SA.D11278.S2311.E0115
   # Separate fields with '.'. and Skip the first 3 fields, so that sort the time 
   #echo "ssmis sort $ssmis_file"  >> /home/oper3/rlg/fetch.log
   ls $ssmis_file | sort -r -t. -k 4 |head -$numPass | sort    >> $localList
fi

# list all the SSMI pass files in pass data directory
ls $ssmi_file > /dev/null 2>&1
success=$?

# if there are SSMI files, success will be 0
if (($success == 0))
then
   # ls $ssmi_file >> $localList
   # ls $ssmi_file | sort -r   |head -$numPass | sort    >> $localList

   #SSGEO.NPR.EDRR.S9.D08070.S2035.E2219.B4260304.NS
   # Separate fields with '.'. amd Skip the first 4 fields, so that sort the time 
   echo "ssmi sort $ssmi_file"
   ls $ssmi_file | sort -r -t. -k 2 |head -$numPass | sort    >> $localList
fi

# list all the AMSRE pass files in pass data directory
ls $amsre_file > /dev/null 2>&1
success=$?

# if there are AMSRE files, success will be 0
if (($success == 0))
then
   #echo "amsre sort"  >> /home/oper3/rlg/fetch.log
   ls $amsre_file | sort -r   |head -$numPass | sort    >> $localList
fi


# if the local file is exist and size is greater than zero
if [ -s $localList ]
then
   for file in `cat $localList`
   do
      echo "The pass data file is $file"
      ls -l $file
      fileToFetch=""

      # check if the file is in donelist or not
      if [[ -f $donelist ]]
      then
         exist=`grep $file $donelist`
         # if exist is empty then the file is not in files done list,
         # then need to encode
     #    if [[ $exist = "" ]]
         if [[ -z $exist ]]
         then
            echo "the file is not in $donelist"
            fileToFetch=$file
            echo $fileToFetch >> $newList

            # ----------------------------------------------
            # put the file into the donelist, since whenever the file is put into
            # the newList, the composite data or the pass data will be used in the
            # calling program (pctrap.pl).  These new pass datas all result in
            # being processed in both cases.
            # please note that: this might be changed in the coming enhancement
            # which checks whether the pass is within 6 hour of the FOS bulletin or not.
            # ----------------------------------------------
         #   echo $fileToFetch >> $donelist

         else
            echo "the file is in $donelist, and it was processed."
         fi
      else
         echo "$donelist is not exist, create it"
         fileToFetch=$file
         echo $fileToFetch > $newList
         echo $fileToFetch > $donelist
      fi

   done    # done with the for loop

   rm $localList

   # change the permission for the log file
   chmod 600 $donelist

else
   echo "no file in input directory"
fi         # end with the outer if for checking the numberOfFile in input dir


if ( [ -f $donelist ] && ((`cat $donelist |wc -l` > 1200)) )
then
   echo "cut the $donelist"
   cat $donelist |tail -1000 > tempdonelist
   mv tempdonelist $donelist
fi

echo "Done the fetching process"

exit

