#!/usr/bin/perl -w

## =====================================================================
## Name:        getBasinList.pl
##              (Adapted from area2digit-multi-fullyAuto.pl)
##                  
## Author:      Jianbin Yang/RSIS   04/03/2003
## Language:    Perl
## Purpose:     This program get the basin area and update the active storm list
##    
## Usage:       getBasinList.pl [<Position file>]  
##
## Please note that: 
##              The position track from the pctrap.pl is used Mcidas convention
##              That means east longitude is a negative number.  
##              But when the digital trap should use the international convention
##	        as the east longitude is a positive number.  So it is coded to 
##		convert back to international convention.
##		And noted that the second input argument "Actual time of Rain rate data"
##		is the time orbit passed over the storm.  It is input by SAB analyst currently.
##		It will be used in the file name of the digital TRaP.  If there is no input, 
##		The composite time of the image will be uesed as default.
##              
## Modified: 04/08/09
##         Updated the active storm list if the storm cross basins (for etrap)
##
## Modified: 08/03/2009
##         Adapted from area2digit-multi-fullyAuto.pl
##
## modified: 08/21/2009
##         Modified to update the basin list for all the pobabilities
## 
## Modified: 10/01/2009
##         Send the rain rate to FTP server
##
## Modified: 10/21/2009
##         Add the satellite ID to the file name when sending the rain rate to FTP server
##
## Modified: 02/17/2010
##         Fixed the longitude condition for WTNZ bulletin
##
## Last modified: 11/28/2011
##         Added SSMIS data source
##
## =====================================================================

#--------- Define the directories and files ---------#

$HOME = $ENV{"HOME"};

$srcDir = "$HOME/mcidas/pctrap-V3";
$dataDir = "/data/Petrap/data/mcidas_data";
$TRaPDataDir = '/data/Petrap';

# File to keep the position of the track of the storm and information to generate the digital TRaP.
# PLEASE NOTE: This file name must be CONSISTENT as the one in restoreAREA.pl, which was actually
# generated from pctrap.pl as the TRaP generated.
$positionFile = 'POS1.TXT';  	     	

$sftpfile1 = '/tmp/ftpSeverRmtPut1';
 
# The log file for this program
#$stdoutLog = 'GETBASIN.OUT';
#$stderrLog = 'GETBASIN.ERR';
#$logFile = '/data/TRAP/log/getBasin_test.log';

# file list the basin name actually is same as the subdirectory name 
$basinFile = "basinName.txt";

# the active storm list file
$stormFile = "$srcDir/activeStorms.txt"; 
#$stormFile = "$srcDir/activeStorms_bkup.txt"; 


##-------------------- Define some constants --------------------##

# For satellite ID hash table, key must match the satellite ID  in the POS1.TXT
%satelliteId = qw(
   NOAA-15 AMSUNK NOAA-16 AMSUNL NOAA-17 AMSUNM NOAA-18 AMSUNN NOAA-19 AMSUNP METOP-A AMSUM2 METOP-B AMSUM1
   DMSP-F13 SSMIS7 DMSP-F14 SSMIS8 DMSP-F15 SSMIS9
   DMSP-F16 SSMISA DMSP-F17 SSMISB
   TRMM TRMM
   AQUA AMSRE
);

#--------- basins and basin abbreviation ---------#

# Define the hash table for the basin (dir here) and the corresponding basin abbreviation (basin here)
%basinTbl = qw(
   Atlantic AL 
   EastPacific EP 
   CentralPacific CP
   NWPacific WP
   SWPacific SW
   SEPacific SE
   NorthIndian NI
   SWIndian WI
   SEIndian EI
   NTerritory NT 
);

#--------- basins ---------#
$Atlantic_basin 		= 'AL';
$EastPacific_basin 		= 'EP';
$CentralPacific_basin 		= 'CP';
$NorthwestPacific_basin 	= 'WP';
$SouthwestPacific_basin         = 'SW';
$SoutheastPacific_basin         = 'SE';
$NorthIndian_basin 		= 'NI';
$SouthwestIndian_basin		= 'WI';
$SoutheastIndian_basin		= 'EI';
$NorthernTerritory_basin	= 'NT';


#--------- anonymous ftp server and directories ---------#

# the domain name is not working since 11/25/08 to 11/26/08
$ftpServer1 = 'oper\@satepsanon.nesdis.noaa.gov';
$ftpHomeDir ='pub/TRAP';


#--------- Basins subdirectories ---------#

$Atlantic_dir 			= 'Atlantic';
$EastPacific_dir 		= 'EastPacific';
$CentralPacific_dir 		= 'CentralPacific';
$NorthwestPacific_dir 		= 'NWPacific';
$SouthwestPacific_dir 		= 'SWPacific';
$SoutheastPacific_dir 		= 'SEPacific';
$NorthIndian_dir 		= 'NorthIndian';
$SouthwestIndian_dir 		= 'SWIndian';
$SoutheastIndian_dir 		= 'SEIndian';
$NorthernTerritory_dir 		= 'NTerritory';
# $Other_dir			= 'Other';

#--------- the email notification list of different agent ---------#

# New define email list names are as following, must match the names in "TRaPEmailList.txt".  Added on 09/11/2006
# TPCList1, TPCList, AtlanticList, CPHCList, 
# PerthList, DarwinList, BrisbaneList, ReunionList, NewDelhiList
# NadiList, NWPacificList, and BCCList.

$TPCList 	= "TPCList";
$AtlanticList 	= "AtlanticList";
$CPHCList 	= "CPHCList";
$PerthList 	= "PerthList";
$DarwinList 	= "DarwinList";
$BrisbaneList 	= "BrisbaneList";
$ReunionList 	= "ReunionList";
$NewDelhiList 	= "NewDelhiList";
$NadiList 	= "NadiList";
$NWPacificList 	= "NWPacificList";
$ccList 	= "BCCList";
# $OtherList      = ""; 


##-------------------- initialize some variables --------------------##

# the following can be get from the "getBasin" function
$basin = "";
@emailList = ();		# emailList maybe empty even after the "getbasin" is called for JTWC, for SWPacifc? at 09/11/2006
$subDir = "";
$remoteTRAPDir = "";            # destination directory for the digital file in the FTP server
$basinArea = "";		# similar to "subdirectories", for TRaP List file 


####----------------------------------------------------------------------------####

####----------------------------  Start the program ----------------------------####

$startdate=`date`;
print "Start  $0 at $startdate \n";

chdir("$dataDir") || die "Cannot change to directory $dataDir";
print  "Data directory is:";
system("pwd");

# remove the path from the command name

@comandFields = split(/\//, $0);
$commandName = $comandFields[-1];

# if no argument input, exit the program

if (@ARGV < 1)
{
   print "Usage: $commandName [<Position file>] \n";
   print "Use default Posisiton file POS1.TXT.\n";
}
elsif (@ARGV == 1)
{
   $positionFile = $ARGV[0];
}


####-----------------------------------------------------------------####

####------ Get the forecast bulletin and position information ------####

## The coverage area are all the same between the 24 hour total and 6 hourly   ##
## so the part 1 is common for all the 6 hourly and 24 hour total digital TRaP ##

open (POSTXT, $positionFile) || die "can't open $positionFile: $!";
@posLines = <POSTXT>;
close (POSTXT) || die "can't close $positionFile: $!";

## redirect the print out to some files, to avoid the mcidas wondow error

#open STDOUT, "> $stdoutLog" || die "Can't create $stdoutLog:$!";
#open STDERR, "> $stderrLog" || die "Can't create $stderrLog:$!";


##------ Get the first line(header?) of the first part of digital TRaP ------##

@tempMsg = split(/\s+/, $posLines[0]);
$lat_0 = $tempMsg[-2];
$lon_0 = $tempMsg[-1];


# !!!!!!!!!!!!!!!
# !!!!!Noted on 08/13/2002, May need to get the basin from WTPA22 or WTPZ22 in getBulInfo.pl
# and write into POS.TXT, since if storm moved from E.Pac to C.pac, such as storm GUILLERMO,
# then the basin is still "EP", it is better to get the basin from bulletin WTPA22 other than determined by
# the WMO header itself. So plan to write the basin in the end of position file for WTPA /WTPZ bulletin
# in getbulInfo.pl, and give a specific name to the following tempMsg, so that in the getBasin function
# can get the basin in the end of @tempMsg
#!!!!!!!!!!!!!!!!

## Noted at 08/29
# will check if there is error msg when there is no basin in the POS file (the following line)

@tempMsg = split(/\s+/, $posLines[5]);
($stormName, $stNumber, $bulYjd, $wmoHd, $station, $bulTime, $curTime, $forecastHour, $basin) = @tempMsg;

# Change the year on 06/14/2007, use the rain rate year other than the forecast year, this consistent with the file name 
# $year = substr($bulYjd, 0, 4);
$year = substr($posLines[6], 0, 4);
## chomp($forecastHour);


print "the stormname is $stormName, bulYjd is $bulYjd, year is $year, wmoHd is $wmoHd, 
       bulTime is $bulTime, curtime is $curTime, stNumber is $stNumber, forecastHour is $forecastHour\n"; 


##----------------------------------------------------------------------------##

##------------- Get the basin, subdirecotry and email list to send -----------##

# Please Noted that the "$lon_0" is Micdas convention
# which is reverse to regular convention, so reverse it here
getBasin($wmoHd, $station, $lat_0, (-1) * $lon_0);

# Check if the basin is got succefully, emailList can be empty if it is from JTWC
if (($basin eq "") || ($subDir eq ""))
{
   print "ERROR!! Could not get the basin or subdirectory for the digital TRaP file in 
          \rprogram $commandName. Please contact the programmer.\n";
   die "-- exit script";
}

$remoteTRAPDir = "$ftpHomeDir/$subDir";

print "Basin just got, the basin is $basin\n";

##----------------------------------------------------------------------##

## PLEASE NOTED THAT:
## The longitude has to multiple by -1 to convert back from Mcidas convention


$fileNamePre = $posLines[6];
chomp($fileNamePre);

# get the satellite name
@tempMsg = split(/\./, $fileNamePre);

# The following line is incorrect as the rain rate time has 2 more digits for month, use -13
#$dataSource = substr($fileNamePre, -11, 4);
$dataSource = $tempMsg[4];

system("pwd");

#------------------------ Get the satellite ID ------------------------#

$satIDLine = $posLines[11];
@tempMsg = split(/\s+/, $satIDLine);
$satID = $tempMsg[0];
if ($satID eq "")
{
   print "WARNING: satID was empty in the POS file.\n";
   $satID = $tempMsg[1];
}
#$dataSetName = $satelliteId{$satID};
$fileNamePre2 = $fileNamePre;
#print "satIDLine is $satIDLine, satID is $satID, dataSource is $dataSource, dataSetName is $dataSetName\n";

if ($dataSource =~ /AMSU|SSMI|SSMIS/)
{
   $dataSetName = $satelliteId{$satID};
   $fileNamePre2 =~ s/$dataSource/$dataSetName/;
   print "The renamed file name is $fileNamePre2\n";
}

##------------------------------------------------------------------------------##

##------ Send (SFTP) out the Rain rate GIF to FTP server  -  Satepsanon  ------##

print " Create sftp script to send the TRaP files to $ftpServer1\n";

# the absolute remote ftp direcotry
$abs_remoteTRAPDir = "/data/Petrap/ftp/" . $remoteTRAPDir;

open  ( FTPOUT1,  ">$sftpfile1" )  ||  die " Can't open $sftpfile1 to write: $!";

   print  FTPOUT1   "# SFTP command file:  Put rain rate GIF files to FTP server\n";
   print  FTPOUT1   " sftp $ftpServer1 << EOF\n";
   print  FTPOUT1   "   cd  $abs_remoteTRAPDir\n";
   #print  FTPOUT1   "   put  $fileNamePre.raw.GIF\n";
   print  FTPOUT1   "   put  $TRaPDataDir/$year/$stormName/$fileNamePre.raw.GIF $fileNamePre2.raw.GIF\n";
   print  FTPOUT1   "   bye\n";
   print  FTPOUT1   "EOF\n";

close ( FTPOUT1 ) || die "Can't close $sftpfile: $!";
chmod  (0777,  $sftpfile1) ;

# Run the sftp file to get the files from NASA
$enddate=`date`;
print " Start  sftp rain rate GIF files to $ftpServer1 at $enddate \n";
open ( SFTPOUT,  " $sftpfile1  | " ) || die " Can't open $sftpfile1 to run: $!";

@sftpOutput = <SFTPOUT>;

close (SFTPOUT ) || die "Can't close $sftpfile1: $!";


##--------------------------------------------------------------------------------------------##

##------          Write the basin AREA name for the storm for the future use            ------##

# $basinFile

@splitMsg = split(/\./, $fileNamePre);
$dirMsg = $splitMsg[0];
$year = substr($dirMsg, 0, 4);
$nameNnb = substr($dirMsg,4);
$outputDir = "$TRaPDataDir/$year/$nameNnb";
$stormName = $nameNnb;

#--- case1: BasinFile not exist, storm is brand new ---#

$isActiveListUpdated = 0;

if (!(-s "$outputDir/$basinFile"))
{
   print "Create the Basin file $outputDir/$basinFile\n"; 

   # `touch $HEPassDataLog`;
   system("echo $basinArea > $outputDir/$basinFile");

   # It is a new storm as basin file created here, 
   # so update the active storm list file here

   #$stormStTime = `date +%Y%m%d/%H%M'Z'`;
   $stormStTime = `date +%Y%m%d%H%M'Z'`;
   chomp($stormStTime);
   
   #$stormEntry = $stormName . " " . $year . " " . $basinArea . "   " . $stormStTime;
   $stormEntry = $stormName . "+" . $year . "+" . $basinArea . "+" . $stormStTime;
   print "stormEntry $stormEntry will be added to the active storm list file: $stormFile\n";

   if (!(-s "$stormFile"))
   {
      system("echo $stormEntry > $stormFile");
   }
   else
   {
      system("echo $stormEntry >> $stormFile");   
   }

   $isActiveListUpdated = 1;
}

#--- case2: BasinFile exists, storm is not new ---#
else
{
   print "Basin file $outputDir/$basinFile exists.\n";

   $exist=`grep $basinArea $outputDir/$basinFile`;

   #--- CASE 2-1: BasinArea not in basinFile:cross basins.  Update basinFile ---#

   # Append to the Basin file and the active storm list 
   if ($exist eq "")
   {
      print "Storm $stormName cross the basin to $basinArea\n";
      print "Append to the Basin file $outputDir/$basinFile\n";
      system("echo $basinArea >> $outputDir/$basinFile");

      # Update the active storm list with new basin
      $curEntryPre = $stormName . "+" . $year;
      $curEntry=`grep $curEntryPre $stormFile`;
     
      #--- CASE 2-1-1: storm entry not in the active list:storm regeneration. create one for crossing basins entry ---#
      if ($curEntry eq "")
      {
         print "Storm entry for $stormName was not in the active list. Maybe the regeneration starts after 24 hours\n";
         @basinList = `cat $outputDir/$basinFile`;
         chomp (@basinList);
         $stormStTime = `date +%Y%m%d%H%M'Z'`;
         chomp($stormStTime);

         $stormEntry = $stormName . "+" . $year . "+" . $basinList[-2] . '-' . $basinList[-1] . "+" . $stormStTime;
         print "stormEntry $stormEntry will be added to the active storm list file: $stormFile\n";
         if (!(-s "$stormFile"))
         {
            system("echo $stormEntry > $stormFile");
         }
         else
         {
            system("echo $stormEntry >> $stormFile");
         }
      }

      #--- CASE 2-1-2: storm entry in the active list, modify the existing entry for crossing basins ---#
      else
      {
         chomp($curEntry);
         print "Existing Entry for $stormName is $curEntry\n";
         @splitMsg = split(/\+/, $curEntry);
         $basin1 = $splitMsg[2];
         print "Storm $stormName cross the basin from $basin1 to $basinArea.\n";
         $curEntryPre = $stormName . "+" . $year . "+" . $basin1;

         # The following new entry is for eTRaP to know it just cross basins, will be replaced with $basinArea in etrap process
         $newEntryPre = $stormName . "+" . $year . "+" . $basin1 . '-' . $basinArea;
         print "New stormEntry $newEntryPre will replace $curEntryPre to the active storm list file: $stormFile\n";
         system("sed 's/$curEntryPre/$newEntryPre/' $stormFile > basinCrTmp.txt");
         rename("basinCrTmp.txt", $stormFile)  or die "ERROR:can't rename basinCrTmp.txt to $stormFile: $!";
      }
      $isActiveListUpdated = 1;
   }
   
   #--- CASE 2-2: BasinArea in basinFile ---#

   # check if the storm regeneration after 24 hours and was removed from the active list
   # Case 1: just stay in the same basin
   # Case 2: or returns back to that basin: basin1->basin2->basin1
   # work up to here , reorganize the if/else for different cases 08/18
   else
   {
      $curEntryPre = $stormName . "+" . $year;
      $curEntry=`grep $curEntryPre $stormFile`;
      #$basinEntry = `grep $curEntryPre $stormFile | grep $basinArea`;

      #--- CASE 2-2-1: BasinArea in basinFile, storm not in the active list ---#
      if ($curEntry eq "")
      {
         print "Storm entry for $stormName was not in the active list.  Check if the regeneration start after 24 hours\n";
         @basinList = `cat $outputDir/$basinFile`;
         chomp (@basinList);
         $stormStTime = `date +%Y%m%d%H%M'Z'`;
         chomp($stormStTime);
        
         #--- CASE 2-2-1-1: Stay in Same basin, but storm regeneration after 24 hours ---#
         if ($basinArea eq $basinList[-1])
         {
            print "Stay in Same basin, but storm regeneration after 24 hours. Create the entry in active list\n";
            $stormEntry = $stormName . "+" . $year . "+" . $basinArea . "+" . $stormStTime;
         }
         #--- CASE 2-2-1-2: storm return back to basin, but storm regeneration after 24 hours ---#
         else
         {
            print "It is possible that the storm returns back to $basinArea.\n";
            print "Append to the Basin file $outputDir/$basinFile\n";
            system("echo $basinArea >> $outputDir/$basinFile");

            print "Crossing basins, but storm regeneration after 24 hours. Create the entry in active list\n";
            #$stormEntry = $stormName . "+" . $year . "+" . $basinList[-2] . '-' . $basinList[-1] . "+" . $stormStTime;
            $stormEntry = $stormName . "+" . $year . "+" . $basinList[-1] . '-' . $basinArea . "+" . $stormStTime;
         }

         print "stormEntry $stormEntry will be added to the active storm list file: $stormFile\n";
         if (!(-s "$stormFile"))
         {
            system("echo $stormEntry > $stormFile");
         }
         else
         {
            system("echo $stormEntry >> $stormFile");
         }

         $isActiveListUpdated = 1;
      }

      #--- CASE 2-2-2: BasinArea in basinFile, storm in the active list ---#
      else
      {
         #--- CASE 2-2-2-1 (most cases): BasinArea in basinFile and also in the active list  ---#

         if ( $curEntry =~ /$basinArea/)
         {
            print "basin in the active storm list\n";
            print "curEntry is $curEntry\n";
         }
         #--- CASE 2-2-2-2: BasinArea in basinFile, but not in the current entry of the active list  ---#
         else
         {
            print "It is possible that the storm returns back to $basinArea.\n";
            print "Append to the Basin file $outputDir/$basinFile\n";
            system("echo $basinArea >> $outputDir/$basinFile");

            chomp($curEntry);
            print "Existing Entry for $stormName is $curEntry\n";
            @splitMsg = split(/\+/, $curEntry);
            $basin1 = $splitMsg[2];
            print "Storm $stormName cross the basin from $basin1 back to $basinArea.\n";
            $curEntryPre = $stormName . "+" . $year . "+" . $basin1;

            # The following new entry is for eTRaP to know it just cross basins, will be replaced with $basinArea in etrap process
            $newEntryPre = $stormName . "+" . $year . "+" . $basin1 . '-' . $basinArea;
            print "New stormEntry $newEntryPre will replace $curEntryPre to the active storm list file: $stormFile\n";
            system("sed 's/$curEntryPre/$newEntryPre/' $stormFile > basinCrTmp.txt");
            rename("basinCrTmp.txt", $stormFile)  or die "ERROR:can't rename basinCrTmp.txt to $stormFile: $!";
    
            $isActiveListUpdated = 1;
         }
      }  # End of "if/else" for $curEntry
   }     # End of "if/else $exsit" for basinArea
}        # End of "if/else" for basinFile is new or not

# See if need to send the active storm list
if ($isActiveListUpdated == 1)
{
   print "Active storm list is updated.\n";
   # push the storm list file to Dev servers for etrap to run
   print "Call pushStList.sh to push the storm list file to Devs for etrap to run.\n\n";
   system ("$srcDir/pushStList.sh");
}

$enddate = `date`;
print "End of the program $0 at $enddate.\n\n";



####------------------------------------------------------------------------------------------------####


##%%%%%%%%%%%%%%%%%%%%%%%% Subroutines %%%%%%%%%%%%%%%%%%%%%%%%##


###############  Subroutine "getBasin" #################
########################################################

sub getBasin
{
   my($wmoHd, $station, $lat, $lon) = @_;
   my($i);
 
   print "Get the basin of the bulletin $wmoHd $station in $lat/$lon (regular longitude)\n"; 
  
   #----------- Atlantic -----------#

   if ($wmoHd =~ /^WTNT2/)
   {
      print "This will send to TPC\n";
      if ($basin eq "")
      {
         # $basin = $Atlantic_basin;
         $basin = $basinTbl{$Atlantic_dir};
      }

      # $emailList = $AtlanticList;
      # push(@emailList, $AtlanticList);
      @emailList = $AtlanticList;
      $subDir = $Atlantic_dir;
      $basinArea = $Atlantic_dir;
   }

   #----------- East Pacific -----------#

### Noted at 08/29/2003, will work more on the overlapped case Storm fall in around Central Pacific and East pacific
## for basin and email notification, if it is neccessary
# will check if there is error msg when there is no basin in the POS file 

   elsif($wmoHd =~ /^WTPZ2/)
   {
      print "This will send to TPC\n";
      if ($basin eq "")
      {
         # $basin = $EastPacific_basin;
         $basin = $basinTbl{$EastPacific_dir};
      }

      # $emailList = $TPCList;
      # push(@emailList, $TPCList);
      @emailList = $TPCList;
      $subDir = $EastPacific_dir;
      $basinArea = $EastPacific_dir;
   }

   #----------- Central Pacific -----------#

   elsif($wmoHd =~ /^WTPA2/)
   {
      print "This will send to CPHC\n";
      if ($basin eq "")
      {
         # $basin = $CentralPacific_basin;
         $basin = $basinTbl{$CentralPacific_dir};
      }
                                
      @emailList = $CPHCList;
      $subDir = $CentralPacific_dir;
      $basinArea = $CentralPacific_dir;
   }

   #----------- NW Pacific -----------#

   elsif($wmoHd =~ /^WTPN3/)
   {
      print "This will send to MWO Guam\n";
      # $basin = $NorthwestPacific_basin;
      $basin = $basinTbl{$NorthwestPacific_dir};
      # $emailList = $MWOGuamList;
      @emailList = $NWPacificList;
      $subDir = "$NorthwestPacific_dir/JTWC";
      $basinArea = $NorthwestPacific_dir;
   }

   # station ID of FKPQ30 is also RJTD.  06/28/2007 
   elsif(($wmoHd =~ /^WTJP2/) || ($wmoHd =~ /^FKPQ3/))		
   {
      print "This will send to Tokyo\n";
      # $basin = $NorthwestPacific_basin;
      $basin = $basinTbl{$NorthwestPacific_dir};
      # $emailList = $TokyoList;
      @emailList = $NWPacificList;
      $subDir = "$NorthwestPacific_dir/Tokyo";
      $basinArea = $NorthwestPacific_dir;
   }


   #----------- North Indian -----------#

#   elsif( (($wmoHd =~ /^WTIO3/) && ($station eq "PGTW"))
#          || ($wmoHd =~ /^WTIN/)
#          || ($wmoHd =~ /^FKIN/) )  
   elsif(($wmoHd =~ /^WTIO3/) && ($station eq "PGTW"))
   {
    #  if ($lat >= 0)
      print "This will send to RSMC New Delhi\n";
      # $basin = $NorthIndian_basin;
      $basin = $basinTbl{$NorthIndian_dir};
      @emailList = $NewDelhiList;

      # will let nancy know 06/28/07
      $subDir = "$NorthIndian_dir/JTWC";
      $basinArea = $NorthIndian_dir;
   }

   # FKIN20 VIDP, WTIN20 DEMS, stations are different, will check, and may need to modify the createTRaPDir for agency name
   elsif(($wmoHd =~ /^WTIN/) || ($wmoHd =~ /^FKIN/))
   {
    #  if ($lat >= 0)
      print "This will send to RSMC New Delhi\n";
      # $basin = $NorthIndian_basin;
      $basin = $basinTbl{$NorthIndian_dir};
      @emailList = $NewDelhiList;
      # ??
      $subDir = "$NorthIndian_dir/NewDelhi";
      $basinArea = $NorthIndian_dir;
   }



   # will send the notification to the neighborhood if the storm fall within 
   # 5/10 degree of the boundary

   ##----------- JTWC bulletins - WTXS -----------##

   # The TRaP from JTWC bulletin will alert no people, so the emailList is empty
   # except the WTIO31 and WTPN31, Maybe change in the future
   
   elsif(($wmoHd =~ /^WTXS3/) && ($station eq "PGTW"))
   {
      if ($lon < 90)
      {
         print "This will send to RSMC Reunion\n";
         # $basin = $SouthwestIndian_basin;
         $basin = $basinTbl{$SouthwestIndian_dir};
         $subDir = "$SouthwestIndian_dir/JTWC";
         $basinArea = $SouthwestIndian_dir;
         push(@emailList, $ReunionList);
         if ($lon >= 85)
         {
            push(@emailList, $PerthList);
         }                 
      }
      #elsif (($lon >= 90) && ($lon < 130))
      elsif (($lon >= 90) && ($lon < 125))
      {
         print "This will send to RSMC Perth\n";
         # $basin = $SoutheastIndian_basin;
         $basin = $basinTbl{$SoutheastIndian_dir};
         $subDir = "$SoutheastIndian_dir/JTWC";
         $basinArea = $SoutheastIndian_dir;
         push(@emailList, $PerthList);
         if ($lon < 95) 
         {
            push(@emailList, $ReunionList);
         }
         elsif ($lon >= 120)
         {
            push(@emailList, $DarwinList);
         }
      }
      #elsif (($lon >= 130) && ($lon < 142))
      elsif (($lon >= 125) && ($lon < 142))
      {
         print "This will send to RSMC Darwin\n";
         # $basin = $NorthernTerritory_basin;     
         $basin = $basinTbl{$NorthernTerritory_dir};     
         $subDir = "$NorthernTerritory_dir/JTWC";   
         $basinArea = $NorthernTerritory_dir;   
         @emailList = ($DarwinList, $PerthList, $BrisbaneList);
 #        if ($lon < 140)
  #       {
   #         $emailList = $emailList . " " . $PerthList;
    #     }
      }
   }   	# End of bulletin WTXS31-35 (PGTW)


   ##----------- Reunion bulletins -----------##

   # elsif(($wmoHd =~ /^WTIO2/) && ($station eq "FMEE") && ($lon < 90))   # WTIO30 ?
   # elsif(($wmoHd =~ /^WTIO/) && ($station eq "FMEE") && ($lon < 90))   # WTIO30 ?
   elsif( (($wmoHd =~ /^WTIO/) || ($wmoHd =~ /^FKIO/)) && ($station eq "FMEE") && ($lon < 90))   # WTIO30 ?
   {
      print "This will send to RSMC Reunion\n";
      # $basin = $SouthwestIndian_basin;
      $basin = $basinTbl{$SouthwestIndian_dir};
      @emailList = $ReunionList;
      $subDir = "$SouthwestIndian_dir/Reunion";
      $basinArea = $SouthwestIndian_dir;
      if  ($lon >= 85)
      {
         push(@emailList, $PerthList);
      }
   }

   ##----------- Australian bulletins -----------##   

   # elsif($wmoHd =~ /^WTAU0/) 
   elsif($wmoHd =~ /^AXAU/)
   {
      $tempWmo = $wmoHd;
      $lastDigit = chop($tempWmo);
  
      # For "WTAU05", 06, 07
    ##  if (($lastDigit >= 5) && ($station eq "APRF") && ($lon >= 80) && ($lon <= 140))
    ##  if (($lastDigit >= 5) && ($station eq "APRF") && ($lon >= 90) && ($lon < 130))
    #  if (($lastDigit >= 5) && ($station eq "APRF"))
      if ($station eq "APRF")
      {
         print "This will send to RSMC Perth\n";
         # $basin = $SoutheastIndian_basin;         # SE Indian Ocean
         $basin = $basinTbl{$SoutheastIndian_dir};         # SE Indian Ocean
         @emailList = $PerthList;
         $subDir = "$SoutheastIndian_dir/Perth";
         $basinArea = $SoutheastIndian_dir;
         if ($lon < 95)
         {
            push(@emailList, $ReunionList);
         }
         elsif ($lon >= 120)
         {
            push(@emailList, $DarwinList);
         }
      }
      # For "WTAU03", 04, 
    ##  elsif (($lastDigit <= 4) && ($station eq "ADRM") && ($lon >= 120) && ($lon <= 152))
     ## elsif (($lastDigit <= 4) && ($station eq "ADRM") && ($lon >= 130) && ($lon < 142))
      ## change on 03/17/04
     ## elsif (($lastDigit <= 4) && ($station eq "ADRM") && ($lon >= 125) && ($lon < 142))
     # elsif (($lastDigit <= 4) && ($station eq "ADRM"))
      elsif ($station eq "ADRM")
      {
         print "This will send to RSMC Darwin\n";
         # $basin = $NorthernTerritory_basin;         # Northern Territory
         $basin = $basinTbl{$NorthernTerritory_dir};         # Northern Territory

         # The TRaP in Northern Territory will send to other two agents:Perth and Brisbane
         @emailList = ($DarwinList, $PerthList, $BrisbaneList);
         $subDir = "$NorthernTerritory_dir/Darwin";
         $basinArea = $NorthernTerritory_dir;
      }
   }	 # End of the bulletin "AXAU"

   # elsif ((($wmoHd eq "WOAU01") || ($wmoHd eq "WOAU02")) && ($station eq "ABRF") 
   #       && ($lon > 132) && ($lon <= 170)) 
   #          && ($lon >= 142) && ($lon < 160))
   #elsif (($wmoHd =~ /^WOAU0/) && ($station eq "ABRF"))
   elsif ($station eq "ABRF")
   {  
      print "This will send to RSMC Brisbane\n";
      # $basin = $SouthwestPacific_basin;         # SW Pacific
      $basin = $basinTbl{$SouthwestPacific_dir};         # SW Pacific
      @emailList = $BrisbaneList;
      $subDir = "$SouthwestPacific_dir/Brisbane";
      $basinArea = $SouthwestPacific_dir;
      if ($lon < 152)
      {
         push(@emailList, $DarwinList);
      }
      elsif ($lon >= 155)
      {
        # if ($lat >= -15)
        # {

            push(@emailList, $NadiList);

        # }
        # elsif (($lat < -15) && ($lat >= -35))
        # {
        #    push(@emailList, $NadiList, $OtherList);
        # }
        # else
        # {
        #    push(@emailList, $OtherList);
        # }

      }
   }	 # End of the bulletin "AXAU"


   ##----------- ICAO Australian bulletin -----------##

   elsif($wmoHd =~ /^FKAU0/)
   {
      if (($lon >= 90) && ($lon < 125))
      {
         print "This will send to ICAO Perth\n";
         # $basin = $SoutheastIndian_basin;
         $basin = $basinTbl{$SoutheastIndian_dir};
         $subDir = "$SoutheastIndian_dir/Perth";
         $basinArea = $SoutheastIndian_dir;

         @emailList = $PerthList;
         if ($lon < 95)
         {
            push(@emailList, $ReunionList);
         }
         elsif ($lon >= 120)
         {
            push(@emailList, $DarwinList);
         }
      }
      elsif (($lon >= 125) && ($lon < 142))
      {
         print "This will send to ICAO Darwin\n";
         # $basin = $NorthernTerritory_basin;         # Northern Territory
         $basin = $basinTbl{$NorthernTerritory_dir};         # Northern Territory
         $subDir = "$NorthernTerritory_dir/Darwin";
         $basinArea = $NorthernTerritory_dir;

         # The TRaP in Northern Territory will send to other two agents:Perth and Brisbane
         @emailList = ($DarwinList, $PerthList, $BrisbaneList);
      }
      elsif (($lon >= 142) && ($lon <= 160))
      {
         print "This will send to ICAO Brisbane\n";
         # $basin = $SouthwestPacific_basin;         # SW Pacific
         $basin = $basinTbl{$SouthwestPacific_dir};         # SW Pacific
         $subDir = "$SouthwestPacific_dir/Brisbane";
         $basinArea = $SouthwestPacific_dir;

         @emailList = $BrisbaneList;
         if ($lon < 152)
         {
            push(@emailList, $DarwinList);
         }
         elsif ($lon >= 155)
         {
           # if ($lat >= -15)
           # {

               push(@emailList, $NadiList);

           # }
           # elsif (($lat < -15) && ($lat >= -35))
           # {
           #    push(@emailList, $NadiList, $OtherList);
           # }
           # else
           # {
           #    push(@emailList, $OtherList);
           # }
         }
      }	# End of elsif for Brisbane

   }	# End of if for FKAU


   ##----------- Jakarta bulletin -----------##

   elsif($wmoHd =~ /^WVID/)
   {
      #if (($lon >= 90) && ($lon < 125))
      #{
         print "This will send to Jakarta\n";
         $basin = $basinTbl{$SoutheastIndian_dir};
         #$subDir = "$SoutheastIndian_dir/Perth";
         $subDir = "$SoutheastIndian_dir/Jakarta";
         $basinArea = $SoutheastIndian_dir;

         @emailList = $PerthList;
         if ($lon < 95)
         {
            push(@emailList, $ReunionList);
         }
         elsif ($lon >= 120)
         {
            push(@emailList, $DarwinList);
         }
     # }
   } # End of elsif for Jakarta


   ##----------- Fiji bulletin -----------##

   # elsif(($wmoHd =~ /^WTPS0/) && ($station eq "NFFN") &&
   #elsif(($wmoHd =~ /^W.PS0/) && ($station eq "NFFN") &&
   #      (($lon >= 160) || ($lon < 0)) && ($lat >= -25))

   # modified on 04/06/07, for CLIFF, "WTPS01 NFFN in -26.00/-171.5"
   #elsif(($wmoHd =~ /^W.PS0/) && ($station eq "NFFN") &&
   elsif( (($wmoHd =~ /^W.PS0/) || ($wmoHd =~ /^FKPS/)) && ($station eq "NFFN") &&
         (($lon >= 160) || ($lon < 0)) )
   {
      print "This will send to RSMC Nadi\n";
      # $basin = $SoutheastPacific_basin;
      $basin = $basinTbl{$SoutheastPacific_dir};
      @emailList = $NadiList;
      # $subDir = "$SoutheastPacific_dir/Nadi";
      $subDir = "$SoutheastPacific_dir/Fiji";
      $basinArea = $SoutheastPacific_dir;
      if ($lon < 165)
      {
         push(@emailList, $BrisbaneList);
      }
   }


   ##----------- JTWC bulletins - WTPS3X -----------##

   elsif(($wmoHd =~ /^WTPS3/) && ($station eq "PGTW"))
   {
      #if (($lon >= 130) && ($lon < 142))
      if (($lon >= 125) && ($lon < 142))
      {
         print "This will send to RSMC Darwin\n";
         # $basin = $NorthernTerritory_basin;         # Northern Territory
         $basin = $basinTbl{$NorthernTerritory_dir};         # Northern Territory
         $subDir = "$NorthernTerritory_dir/JTWC";
         $basinArea = $NorthernTerritory_dir;
         @emailList = ($DarwinList, $PerthList, $BrisbaneList);
 #        if ($lon >= 132)
  #       {
   #         $emailList = $emailList . " " . $BrisbaneList;
    #     }
      }
      elsif (($lon >= 142) && ($lon < 160))
      {
         print "This may send to RSMC Brisbane\n";
         # $basin = $SouthwestPacific_basin;         # SW Pacific
         $basin = $basinTbl{$SouthwestPacific_dir};         # SW Pacific
         $subDir = "$SouthwestPacific_dir/JTWC";
         $basinArea = $SouthwestPacific_dir;
         @emailList = $BrisbaneList;
         if ($lon < 152)
         {
            push(@emailList, $DarwinList);
         }
         elsif ($lon >= 155)
         {
           # if ($lat >= -15)
           # {

               push(@emailList, $NadiList);

           # }
           # elsif (($lat < -15) && ($lat >= -35))
           # {
           #    push(@emailList, $NadiList, $OtherList);
           # }
           # else
           # {
           #    push(@emailList, $OtherList);
           # }
         }
      }
      #elsif ((($lon >= 160) || ($lon < 0)) && ($lat >= -25))
      elsif (($lon >= 160) || ($lon < 0))
      {
         print "This may send to RSMC Nadi\n";
         # $basin = $SoutheastPacific_basin;         # SE Pacific
         $basin = $basinTbl{$SoutheastPacific_dir};         # SE Pacific
         $subDir = "$SoutheastPacific_dir/JTWC";
         $basinArea = $SoutheastPacific_dir;
         @emailList = $NadiList;
         if ($lon < 165)
         {
            push(@emailList, $BrisbaneList);
         }
         ## if ($lat < -15)
         #if ($lat < -20)
         #{
         #   push(@emailList, $OtherList);
         #   print "\nPAY ATTENTION!! The storm is in south of -20 in Southeast Pacific (Near New Zealand Area).\n";
         #}

      }

#      elsif ((($lon >= 160) || ($lon < 0)) && ($lat < -25))
#      {
#         print "This may send to Other\n";
#         $basin = $Other_basin;         # SE Pacific
#         $subDir = "$SoutheastPacific_dir/JTWC";
#         $basinArea = $SoutheastPacific_dir;
#         @emailList = $OtherList;
#         if ($lon < 165)
#         {
#            push(@emailList, $BrisbaneList);
#         }
#         if ($lat > -35)
#         {
#            push(@emailList, $NadiList);
#         }
#      }

   }    # End of the bulletin "WTPS31-35 (PGTW)


   ##----------- New Zealand bulletins - WTNZ -----------##

   # WTNZ or WHNZ
   elsif(($wmoHd =~ /^W[T,H]NZ/) )
   {
      if (($lon < 160) && ($lon > 0 ))
      {
         print "This may send to RSMC Brisbane/Wellington\n";
         # $basin = $SouthwestPacific_basin;         # SW Pacific
         $basin = $basinTbl{$SouthwestPacific_dir};         # SW Pacific
         $subDir = "$SouthwestPacific_dir/Wellington";
         $basinArea = $SouthwestPacific_dir;
         @emailList = $BrisbaneList;
         if ($lon < 152)
         {
            push(@emailList, $DarwinList);
         }
         elsif ($lon >= 155)
         {
               push(@emailList, $NadiList);

         }
      }
      elsif (($lon >= 160) || ($lon < 0))
      {
         print "This may send to RSMC Nadi/Wellington\n";
         # $basin = $SoutheastPacific_basin;         # SE Pacific
         $basin = $basinTbl{$SoutheastPacific_dir};         # SE Pacific
         $subDir = "$SoutheastPacific_dir/Wellington";
         $basinArea = $SoutheastPacific_dir;
         @emailList = $NadiList;
         if ($lon < 165)
         {
            push(@emailList, $BrisbaneList);
         }

      }
   }    # End of the bulletin "WTNZ/WHNZ"

}	# End of the subroutine "getBasin"
