#!/usr/bin/perl -w
#
#
# $Id: getPosition.pl,v 1.1.1.1 2010-04-27 15:13:11 cmuser Exp $
# $Log: not supported by cvs2svn $
# Revision 1.1.1.1  2009/02/04 19:25:09  cmuser
# initial release
#
# Revision 1.1  2007/11/30 19:57:01  oper1
# First checkin.
#
#
#!/usr/bin/perl -w   

## =====================================================================
## Name:            getPosition.pl
##                  
## Author:          Jianbin Yang/RSIS   Dec 2002
## Language:        Perl
## Purpose:         
## Usage:  	    getPosition.pl [<strom name>] <WMO header> 
##                
## note : use hash table to store the storm struct information.
##        delete the print statement
## note again: the south latitude will be converted to a negative number
##     	       and the east longitude will be converted to a negative
##	       number consistent with the Mcidas system.
## 
## Last Modified:   02/28/2003
##
## =====================================================================

use Class::Struct;

####---------------------- Define structures and data ------------------------####

# create a definition for a "Storm" struct
struct Storm =>
{
   name      => '$',        # storm name for pctrp.k to call
   nameNNB   => '$',        # storm name with NNB if is available, this used for name convention
   wmoHd     => '$',        # the WMO header id 
   stationId => '$',        # the station id
   bulTime   => '$',        # the forecast bulletine time
   bulYJd    => '$',        # store the bulletine year and julian day
   iniTime   => '$',        # the initial time of the storm
   lat       => '$',        # the latitude of the center of the storm
   lon       => '$',        # the longitude of the center of the storm
};


# Define the directories and files
# $srcDir = '/home/oper1/mcidas/pctrap';
$dataDir = '/home/oper1/mcidas/data';
# $trapDataDir = "/data/TRAP";
$positionFile = "$dataDir/POSTRACK.TXT";


# Define the wmolists for the WMO header of the bulletin
@wmolist1 = qw(
   WTNT41 WTNT42 WTNT43 WTNT44 WTNT45
   WTPZ41 WTPZ42 WTPZ43 WTPZ44 WTPZ45
   WTPA41 WTPA42 WTPA43 WTPA44 WTPA45 
);

## must specify the WMO station (WSTN=PGTW) for the following WMO header
@wmolist2 = qw(
   WTPN31 WTPN32 WTPN33 WTPN34 WTPN35
   WTPS31 WTPS32 WTPS33 WTPS34 WTPS35
   WTXS31 WTXS32 WTXS33 WTXS34 WTXS35
   WTIO31 WTIO32 WTIO33 WTIO34 WTIO35
);

# @wmolist3 = qw(
#    WTIO31 WTIO32 WTIO33 WTIO34 WTIO35
# );

## bulletins from Reunion(IO21 or IO31?), station (WSTN=FMEE)

@wmolist3 = qw(
   WTIO20 WTIO21 WTIO22 WTIO23 WTIO24 WTIO25 WTIO30       # ??
);

## bulletins from Australia , station (WSTN=APRF)

@wmolist4 = qw(
   WTAU05 WTAU06               # ??
);

## bulletins from RSMC Tokyo, station (WSTN=RJTD)

@wmolist5 = qw(
   WTJP21 WTJP22 WTJP23 WTJP24 WTJP25
);

## bulletins from Fiji, station (WSTN=NFFN)

@wmolist6 = qw(
   WTPS01 WTPS02 WTPS03 WTPS04 WTPS05
);


## initialize some variable
$filename = "tempWxt.txt";
$arrayPrefix = "wmolist";            # the prefix of the above WMO header arrays
$functionPrefix = "getPos";          # the prefix of functions to get positions
$bulArrNumber = 6;                   # there are 6 kinds of bulletins
				     #	for the corresponding WMO headers
## initialize the array and hash table

# Define the array contained all the  bulletins 
@bulArray = (@wmolist1, @wmolist2, @wmolist3, @wmolist4, @wmolist5, @wmolist6);

$hourRange = 24;

####------------------------------- Start -------------------------------------####


# $path = $ENV{"PATH"};
# $ENV{"PATH"} = $path;

# $ENV{"MCPATH"} = "/home/oper1/mcidas/data:/home/mcidas/data";

print "\n--------------------- Start the get position Program ---------------------\n";

$startdate=`date`;
print "Start $0 at $startdate.\n"; 

if (@ARGV == 0)
{
   print "Usage: getPosition.pl [<strom name>] <WMO header>\n";
   die "No WMO header input - exit program";
}
elsif (@ARGV == 1)
{
   $wmohd = $ARGV[0];
}
elsif (@ARGV == 2)
{
   $stormName = $ARGV[0];
   $wmohd = $ARGV[1];
}
else
{
   print "Usage: getPosition.pl [<strom name>] <WMO header>\n";
   die "Too many arguments - exit program";
}

chdir("$dataDir") || die "Cannot change to directory $dataDir";
print  "Data directory:";
system("pwd");


# the alter way is using the following line, if the element is in the front of the array,  
# otherwise using the array reference to pass array to subroutine
# $isExist = isElement($wmohd, @wmolist3);

# $isExist = isElement(\@wmolist3, $wmohd);

$functionName = "";

for ($i = 1; $i < $bulArrNumber + 1; $i++)
{
   $arrayName = $arrayPrefix . $i;
   $isExist = isElement(\@{$arrayName}, $wmohd);
print "the array name is $arrayName, in for loop is Exist = $isExist\n";
  # if (($isExist > 0) || ($isExist == 0))
   if ($isExist >= 0)
   {
      $functionName = $functionPrefix . $i;
      last;
   }
}

print "out of the for loop is Exist = $isExist\n";
if ($isExist < 0)
{
   die "WMO header \"$wmohd\" is not in the lists - Exit program."
}

if ($functionName eq "")
{
   die "function error - Exit program.";
}

print "functionName is $functionName\n"; 

# call the specific function via the variable
# return 0 for success, -1 for failed to get possion 
$success = &$functionName();           
# &getPos4 ();

if ($success == 0)
{
   print "postion track is retrieved successfully. Write them into the POS file.\n";
   writeTrack();
}
else
{
   print "Warning: postion track is retrieved unsuccessfully!\n";
}

chdir("$dataDir") || die "Cannot change to dataDir $dataDir";


####--------------------------------------------------------------------------####

print "\nEnd of the PERL program.\n";

##%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  Subroutines %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%##

############  Subroutine "isElement" ###############
########################################################
## if it is one of the element of the array the index is return,
## otherwise return -1

sub isElement
{
   # my($element, @array) = @_;
   my ($array, $element) = @_;
   my ($foundIndex) = -1;
   # must specify $i as local variable, otherwise will conflict with $i in 
   # the calling phase (main function)
   my ($i);           

   # can also write as @{$array}
   ### print " the array is @array\n";

   for ($i= 0; $i < @$array; $i++)
   {
      ### print"inside the isElement, the element is @$array[$i], i is $i\n";
      if (@$array[$i] eq $element)
      {
         $foundIndex = $i;
         last;
      }
   }

   return $foundIndex;
}

############  Subroutine "getPos1" ###############
##################################################

sub getPos1
{
   print "This is in function getPos1.\n";
   system("wxtlist.k WMO=$wmohd DTIME=$hourRange DEV=T $filename");

#    $filename = "wxt/wtpa43";

   $posMsg = `grep  "\/" $filename`;
   print "the positon msg is ***\n$posMsg\n***\n";
   if ($posMsg eq "")
   {
      print "WARNING, there is no position listed in the bulletin.\n";
      return -1;
   }
   @splitMsg = split(/\n/, $posMsg);
   print "split 1 is $splitMsg[0]\n";
   print "split 2 is $splitMsg[1]\n";
   print "split 3 is $splitMsg[2]\n";

   foreach $posLine (@splitMsg)
   {
      # get the initial position
      if ($posLine =~ /^INITIAL\s+\d{2}\/\d{4}Z/)
      {
         print "this is the initial position.\n";
         @tempMsg = split(/\s+/, $posLine);
         $lat_0 = $tempMsg[2];
         $latPos = chop($lat_0);
         $lat_0 *= (($latPos eq "N" || $latPos eq "n") ? 1.0 : -1.0);
         $lon_0 = $tempMsg[3];
         $lonPos = chop($lon_0);

         # the conversion follows Mcidas definition (East-lon is negative)
         $lon_0 *= (($lonPos eq "W" || $lonPos eq "w") ? 1.0 : -1.0);
         print "lat is $lat_0, pos is $latPos, lon is $lon_0, pos is $lonPos\n";
      }

      # get the 12 hour forecast position,
      # most of the time the pattern is "12HR VT ", but some time "12HR VAT " 
      elsif ($posLine =~ /^12HR VA?T\s+\d{2}\/\d{4}Z/)
      {
        #  print "posLine is $posLine\n";
         @tempMsg = split(/\s+/, $posLine);
         $lat_12 = $tempMsg[3];
         $latPos =  chop($lat_12);
         $lon_12 = $tempMsg[4];
         $lonPos = chop($lon_12);
         $lat_12 *= (($latPos eq "N" || $latPos eq "n") ? 1.0 : -1.0);
         $lon_12 *= (($lonPos eq "W" || $lonPos eq "w") ? 1.0 : -1.0);
         print "lat is $lat_12, pos is $latPos, lon is $lon_12, pos is $lonPos\n";
      }  # end of "elseif" for checking the 12 forecast position

      # get the 24 hour forecast position
      elsif ($posLine =~ /^24HR VA?T\s+\d{2}\/\d{4}Z/)
      {
        #  print "posLine is $posLine\n";
         @tempMsg = split(/\s+/, $posLine);
         $lat_24 = $tempMsg[3];
         $latPos =  chop($lat_24);
         $lon_24 = $tempMsg[4];
         $lonPos = chop($lon_24);
         $lat_24 *= (($latPos eq "N" || $latPos eq "n") ? 1.0 : -1.0);
         $lon_24 *= (($lonPos eq "W" || $lonPos eq "w") ? 1.0 : -1.0);
         print "lat is $lat_24, pos is $latPos, lon is $lon_24, pos is $lonPos\n";
      }  # end of "elseif" for checking the 24 forecast position

   }     # end of "foreach"

  if ($lat_24 !~ /\d$/)
  {
     print "WARNING, 24 hour forecast is not the right one or not available!!\n";
     return -1;
  }

   get6_18_points();

   return 0;

}  	## End of subroutine getPos1

############  Subroutine "getPos2" ###############
##################################################

sub getPos2
{
   print "This is in function getPos2.\n";

   #****** Please Note ********
   # specify the station as PGTW here, since the "WTPN PHNC" 
   # is duplicated of "WTPN PGTW", and I believe the other
   # "WTPS31", "WTXS31" is from PGTW station, will confirm this!!!
   # "WTIO31" must be from PGTW station.
   #***************************
 
   system("wxtlist.k WMO=$wmohd WSTN=PGTW DTIME=$hourRange DEV=T $filename");

   $filename = "wxt/wtps31-dateline";
   # open and read the file to get the positions
   open(POS2, $filename) || die "Can not open $filename: $!"; 
   while (defined($line = <POS2>))
   {
      # chomp $line;
      # get the initial position
      if ($line =~ /^\s*WARNING POSITION:$/)
      {
         # read the next line
         $line = <POS2>;       
         if ($line =~ / --- NEAR/)
         {
           print "initial posLine is $line\n";
            @tempMsg = split(/\s+/, $line); 

            # use index -1 is better than 3, since whether there is space
            # in front of the line will result in different element "3" 
            $lat_0 = $tempMsg[-2];        
            chop($lat_0);     		# remove the correction digit
            $latPos = chop($lat_0);
            $lat_0 *= (($latPos eq "N" || $latPos eq "n") ? 1.0 : -1.0);
            $lon_0 = $tempMsg[-1];
            chop($lon_0);              	# remove the correction digit
            $lonPos = chop($lon_0);
            $lon_0 *= (($lonPos eq "W" || $lonPos eq "w") ? 1.0 : -1.0);
            print "lat is $lat_0, pos is $latPos, lon is $lon_0, pos is $lonPos\n";
         }
         else
         {
            print "WARNING, initial position is in the right format or not available!!\n";
            return -1;
         }
      }     # end of the "if" for initial position

      # get the 12 hour forecast
      elsif ($line =~ /^\s*12 HRS,\s+VALID AT:$/)
      {
         $line = <POS2>;
         if ($line =~ / --- /)
         {
           print "12 hour posLine is $line\n";
            @tempMsg = split(/\s+/, $line);
            $lat_12 = $tempMsg[-2];
            chop($lat_12);               # remove the correction digit
            $latPos =  chop($lat_12);
            $lon_12 = $tempMsg[-1];
            chop($lon_12);               # remove the correction digit
            $lonPos = chop($lon_12);
            $lat_12 *= (($latPos eq "N" || $latPos eq "n") ? 1.0 : -1.0);
            $lon_12 *= (($lonPos eq "W" || $lonPos eq "w") ? 1.0 : -1.0);
            print "lat is $lat_12, pos is $latPos, lon is $lon_12, pos is $lonPos\n";
         }
         else
         {
            print "WARNING, 12 hour forecast position is in the right format or not available!!\n";
            return -1;
         }
      }     # end of "elseif" for checking the 12 forecast position

      # get the 24 hour forecast position
      elsif ($line =~ /^\s*24 HRS,\s+VALID AT:$/)
      {
         $line = <POS2>;
         if ($line =~ / --- /)
         {
           print "24 hour posLine is $line\n";
            @tempMsg = split(/\s+/, $line);
            $lat_24 = $tempMsg[-2];
            chop($lat_24);               # remove the correction digit
            $latPos =  chop($lat_24);
            $lon_24 = $tempMsg[-1];
            chop($lon_24);               # remove the correction digit
            $lonPos = chop($lon_24);
            $lat_24 *= (($latPos eq "N" || $latPos eq "n") ? 1.0 : -1.0);
            $lon_24 *= (($lonPos eq "W" || $lonPos eq "w") ? 1.0 : -1.0);
            print "lat is $lat_24, pos is $latPos, lon is $lon_24, pos is $lonPos\n";
         }
      }     # end of "elseif" for checking the 24 forecast position
   
   }        # End of the while loop to read the forecast file

   close(POS2) || die "Can not close $filename: $!";

   if ($lat_24 !~ /\d$/)
   {
      print "WARNING, 24 hour forecast is not the right one or not available!!\n";
      return -1;
   }
   get6_18_points();  
   return 0;

}  	## End of subroutine getPos2

############  Subroutine "getPos3" ###############
##################################################

# sub getPos3
# {
#   print "This is in function getPos3.\n";
#   system("wxtlist.k WMO=$wmohd WSTN=PGTW DTIME=$hourRange DEV=T $filename");


#   return 0;

# }  ## End of subroutine getPos3

############  Subroutine "getPos3" ###############
############   Reunion bulletin    ###############
## WTIO20 - 25
## If the bulletin is not matched, or no 24 hour position
## This subroutine will return -1 for error.
##################################################

sub getPos3
{
   my ($i) = 0;
   print "This is in function getPos3.\n";
   system("wxtlist.k WMO=$wmohd WSTN=FMEE DTIME=$hourRange DEV=T $filename");   

     $filename = "wxt/wtio22-c";
   # $filename = "wxt/wtio22";   
   # $filename = "wxt/wtio20";
   # $filename = "wxt/wtio24";
   # $filename = "wxt/wtio24-2";
   # $filename = "wxt/wtps02";
   $posMsg = `grep -i "POSITION" $filename`;
   print "the positon msg is ***\n$posMsg\n***\n"; 
   if ($posMsg eq "")
   {
      print "WARNING, there is no position listed in the bulletin.\n";
      return -1;
   } 
   @splitMsg = split(/\n/, $posMsg);
   print "split 1 is $splitMsg[0]\n";
   print "split 2 is $splitMsg[1]\n";
   print "split 3 is $splitMsg[2]\n";

   foreach $posLine (@splitMsg)
   {

      # get the initial position
      if ($posLine =~ /^POSITION *:.*\//)
      # if (($posLine =~ /^POSITION/) && ($posLine =~ /[:\/]/))
      {
         print "this is the initial position.\n";
         @tempMsg = split(/\//, $posLine);
         if (@tempMsg != 2)
         {
            next;
         }
         @temp1 = split(/\s/, $tempMsg[0]);
         $lat_0 = $temp1[-1];     # the last element in "temp1"
         $latPos = chop($lat_0);
         $lat_0 *= (($latPos eq "N" || $latPos eq "n") ? 1.0 : -1.0);
         @temp2 = split(/\s/, $tempMsg[1]);
         $lon_0 = $temp2[0];
         $lonPos = chop($lon_0);

         # the conversion follows Mcidas definition (East-lon is negative)
         $lon_0 *= (($lonPos eq "W" || $lonPos eq "w") ? 1.0 : -1.0);
         print "lat is $lat_0, pos is $latPos, lon is $lon_0, pos is $lonPos\n";          
      }

      # get the forecast position, expecting forecast for every 12 hour
      elsif ($posLine =~ /^FORECASTED POSITION.*:.*\//)
      {
        #  print "posLine is $posLine\n";
         # i is a counter to determine if it is 12 or 24 hour forecast
         $i++;       
         print "this is $i position.\n";
         @tempMsg = split(/:/, $posLine);
         if (@tempMsg != 2)
         {
            next;
         }
         @temp1 = split(/[\s\/]/, $tempMsg[1]);
         # skip the first element if it is empty
         my ($j) = 0;
         if ($temp1[$j] eq "")
         {
            $j++;
         }
         my ($lat) = $temp1[$j];
         $latPos =  $temp1[$j + 1];
         my ($lon) = $temp1[$j + 2];
         $lonPos =  $temp1[$j + 3];
print "****lat is $lat, pos is $latPos, lon is $lon, pos is $lonPos\n";

         if ($latPos =~ /^[Ss]/)
         {
            $lat *= -1;
         }
         if ($lonPos =~ /^[Ee]/)
         {
            $lon *= -1;
         }

         print "lat is $lat, pos is $latPos, lon is $lon, pos is $lonPos\n";
         if ($i == 1)
         {
            $lat_12 = $lat;
            $lon_12 = $lon;
         } 
         elsif ($i == 2)
         {
            $lat_24 = $lat;
            $lon_24 = $lon;
         }
      }  # end of "elseif" for checking the 12/24 forecast position

   }      # end of "foreach"

   if ($i < 2)
   {
      print "WARNING, there is no 24 hour forecast!!\n";
      return -1;
   }

   get6_18_points();

   return 0;

}  	## End of subroutine getPos3

############  Subroutine "getPos4" ###############
############  Australia bulletin   ###############
## WTAU05 ?
##################################################

sub getPos4
{
   my ($i) = 0;
   print "This is in function getPos4.\n";
   system("wxtlist.k WMO=$wmohd DTIME=$hourRange DEV=T $filename");

     $filename = "wxt/wtau06";

   ## get the initial position 

   # $intPosMsg = `grep '\[' $filename`;
   $intPosMsg = `grep "TUDE" $filename`;
   print "the initial positon msg is ***\n$intPosMsg\n***\n";
   if ($intPosMsg eq "")
   {
      print "WARNING, there is no initial position listed in the bulletin.\n";
      return -1;
   }
   @splitMsg = split(/\n/, $intPosMsg);
   print "split 1 is $splitMsg[0]\n";
   print "split 2 is $splitMsg[1]\n";

   # this for loop get the initial position
   foreach $posLine (@splitMsg)
   {
      # get the initial latitude position
      if ($posLine =~ /^\s*LATITUDE .* \[.*\]$/)
      {
         print "this is the initial latitude position.\n";

         # Note that the position format is something like "[19.0S]"
         # So use the '[' and ']' to split the message
         @tempMsg = split(/[\[\]]/, $posLine);
     print "temp0 is $tempMsg[0]\n temp1 is $tempMsg[1]\n";
         $lat_0 = $tempMsg[1];     
         $latPos = chop($lat_0);
         $lat_0 *= (($latPos eq "N" || $latPos eq "n") ? 1.0 : -1.0);
      }

      # get the longitude position
      elsif ($posLine =~ /^\s*LONGITUDE .* \[.*\]$/)
      {
         print "this is the initial longitude position.\n";

         # Note that the position format is something like "[91.0E]"
         # So use the '[' and ']' to split the message
         @tempMsg = split(/[\[\]]/, $posLine);
     print "temp0 is $tempMsg[0]\n temp1 is $tempMsg[1]\n";
         $lon_0 = $tempMsg[1];
         $lonPos = chop($lon_0);

         # the conversion follows Mcidas definition (East-lon is negative)
         $lon_0 *= (($lonPos eq "W" || $lonPos eq "w") ? 1.0 : -1.0);
         print "lat is $lat_0, pos is $latPos, lon is $lon_0, pos is $lonPos\n";
      }  # end of "elseif" for longitude position
   }      # end of "foreach" for initial position

   ## get the 12/24 forecast position
   $posMsg = `grep -i "UTC " $filename`;
   print "the positon msg is ***\n$posMsg\n***\n";
   if ($posMsg eq "")
   {
      print "WARNING, there is no forecast position listed in the bulletin.\n";
      return -1;
   }
   @splitMsg = split(/\n/, $posMsg);
#   print "split 1 is $splitMsg[0]\n";
 #  print "split 2 is $splitMsg[1]\n";
  # print "split 3 is $splitMsg[2]\n";

   $i = 0;
   foreach $posLine (@splitMsg)
   {
      # get the 12/24 hour forecast position, expecting forecast for every 12 hour
      if ($posLine =~ /^AT \d{4}UTC .*HPA$/)
      {
         print "posLine is $posLine\n";
         # i is a counter to determine if it is 12 or 24 hour forecast
         $i++;
         print "this is $i position.\n";
         @tempMsg = split(/\s+/, $posLine);
         my ($lat) = $tempMsg[4];
         $latPos =  $tempMsg[5];
         my ($lon) = $tempMsg[6];
         $lonPos =  $tempMsg[7];
print "****lat is $lat, pos is $latPos, lon is $lon, pos is $lonPos\n";

         if ($latPos =~ /^[Ss]/)
         {
            $lat *= -1;
         }
         if ($lonPos =~ /^[Ee]/)
         {
            $lon *= -1;
         }

         print "lat is $lat, pos is $latPos, lon is $lon, pos is $lonPos\n";
         if ($i == 1)
         {
            $lat_12 = $lat;
            $lon_12 = $lon;
         }
         elsif ($i == 2)
         {
            $lat_24 = $lat;
            $lon_24 = $lon;
         }
      }  # end of "if" for checking the 12/24 forecast position
   }     # end of "foreach" for forecast position

   if ($i < 2)
   {
      print "WARNING, there is no 24 hour forecast!!\n";
      return -1;
   }

   get6_18_points();

   return 0;

}  	## End of subroutine getPos4

############  Subroutine "getPos5" ###############
############  RSMC Tokyo bulletin  ###############
## WTJP21 - 25
##################################################

sub getPos5
{
   print "This is in function getPos5.\n";
   system("wxtlist.k WMO=$wmohd DTIME=$hourRange DEV=T $filename");
   
   $filename = "wxt/wtjp21.txt";
   $posMsg = `grep -i "AT " $filename`;
   print "the positon msg is ***\n$posMsg\n***\n";
   if ($posMsg eq "")
   {
      print "WARNING, there is no position listed in the bulletin.\n";
      return -1;
   }
   @splitMsg = split(/\n/, $posMsg);
   print "split 1 is $splitMsg[0]\n";
   print "split 2 is $splitMsg[1]\n";
   print "split 3 is $splitMsg[2]\n";

   foreach $posLine (@splitMsg)
   {

      # get the initial position
      if ($posLine =~ /^AT .*MOVING/)
      # if (($posLine =~ /^POSITION/) && ($posLine =~ /[:\/]/))
      {
         print "this is the initial position.\n";
         @tempMsg = split(/\s/, $posLine);
         print "temp1 is $tempMsg[1], temp2 is $tempMsg[2]\n";
         $lat_0 = $tempMsg[1];     
         $latPos = chop($lat_0);
         $lat_0 *= (($latPos eq "N" || $latPos eq "n") ? 1.0 : -1.0);
         $lon_0 = $tempMsg[2];
         $lonPos = chop($lon_0);

         # the conversion follows Mcidas definition (East-lon is negative)
         $lon_0 *= (($lonPos eq "W" || $lonPos eq "w") ? 1.0 : -1.0);
         print "lat is $lat_0, pos is $latPos, lon is $lon_0, pos is $lonPos\n";
      }

      # get the forecast position, expecting forecast for every 24 hour
      # only need to get the first 24 hour forecast
      elsif ($posLine =~ /^FORECAST POSITION FOR/)
      {
          print "posLine is $posLine\n";
         @tempMsg = split(/\s/, $posLine);

         # there are 11 elements expected
         if (@tempMsg < 10)
         {
            next;
         }

         print "temp5 is $tempMsg[5], temp6 is $tempMsg[6]\n";
         $lat_24 = $tempMsg[5];
         $latPos = chop($lat_24);
         $lat_24 *= (($latPos eq "N" || $latPos eq "n") ? 1.0 : -1.0);
         $lon_24 = $tempMsg[6];
         $lonPos = chop($lon_24);

         # the conversion follows Mcidas definition (East-lon is negative)
         $lon_24 *= (($lonPos eq "W" || $lonPos eq "w") ? 1.0 : -1.0);
         print "lat is $lat_24, pos is $latPos, lon is $lon_24, pos is $lonPos\n";

         last;  # only get the first one, exit the foreach loop
      }  # end of "elseif" for checking the 12/24 forecast position

  }      # end of "foreach"

  if ($lat_24 !~ /\d$/) 
  {
     print "WARNING, 24 hour forecast is not the right one or not available!!\n";
     return -1;
  }

  get12_point();
  get6_18_points();

  return 0;

}  	## End of subroutine getPos5 

############  Subroutine "getPos6" ###############
############     Fiji bulletin     ###############
##################################################

sub getPos6
{
   my ($i) = 0;
   print "This is in function getPos6.\n";
   system("wxtlist.k WMO=$wmohd WSTN=NFFN DTIME=$hourRange DEV=T $filename");

   $filename = "wxt/wtps01-2";
   $posMsg = `grep -i " UTC" $filename`;
   print "the positon msg is ***\n$posMsg\n***\n";
   if ($posMsg eq "")
   {
      print "WARNING, there is no position listed in the bulletin.\n";
      return -1;
   }
   @splitMsg = split(/\n/, $posMsg);
   print "split 1 is $splitMsg[0]\n";
   print "split 2 is $splitMsg[1]\n";
   print "split 3 is $splitMsg[2]\n";

   foreach $posLine (@splitMsg)
   {
      # get the initial position
      if ($posLine =~ /^REPEAT POSITION .* UTC\.?$/)
      {
         print "this is the initial position.\n";
         @tempMsg = split(/\s+/, $posLine);
         $lat_0 = $tempMsg[2];     
         $latPos = chop($lat_0);
         $lat_0 *= (($latPos eq "N" || $latPos eq "n") ? 1.0 : -1.0);
         $lon_0 = $tempMsg[3];
         $lonPos = chop($lon_0);

         # the conversion follows Mcidas definition (East-lon is negative)
         $lon_0 *= (($lonPos eq "W" || $lonPos eq "w") ? 1.0 : -1.0);
         print "lat is $lat_0, pos is $latPos, lon is $lon_0, pos is $lonPos\n";
      }

      # get the 12 hour forecast position, expecting forecast for every 12 hour
      elsif ($posLine =~ /^FORECAST POSITION NEAR .* UTC$/)
      {
        #  print "posLine is $posLine\n";
         @tempMsg = split(/\s+/, $posLine);
         $lat_12 = $tempMsg[3];
         $latPos =  chop($lat_12);
         $lon_12 = $tempMsg[4];
         $lonPos = chop($lon_12);
         $lat_12 *= (($latPos eq "N" || $latPos eq "n") ? 1.0 : -1.0);
         $lon_12 *= (($lonPos eq "W" || $lonPos eq "w") ? 1.0 : -1.0);
         print "lat is $lat_12, pos is $latPos, lon is $lon_12, pos is $lonPos\n";
      }  # end of "elseif" for checking the 12 forecast position

      # get the 24 hour forecast position
      elsif ($posLine =~ /^\s+AND NEAR .* UTC\.?$/)
      {
        #  print "posLine is $posLine\n";
         @tempMsg = split(/\s+/, $posLine);
         $lat_24 = $tempMsg[3];
         $latPos =  chop($lat_24);
         $lon_24 = $tempMsg[4];
         $lonPos = chop($lon_24);
         $lat_24 *= (($latPos eq "N" || $latPos eq "n") ? 1.0 : -1.0);
         $lon_24 *= (($lonPos eq "W" || $lonPos eq "w") ? 1.0 : -1.0);
         print "lat is $lat_24, pos is $latPos, lon is $lon_24, pos is $lonPos\n";
      }  # end of "elseif" for checking the 24 forecast position

   }     # end of "foreach"

  if ($lat_24 !~ /\d$/)
  {
     print "WARNING, 24 hour forecast is not the right one or not available!!\n";
     return -1;
  }

   get6_18_points();

   return 0;

}  	## End of subroutine getPos6

####*********** End of all those "get position" subroutines! ***********#####

############  Subroutine "get12_point" ###############
##################################################
sub get12_point
{
   $lat_12 = ($lat_0 + $lat_24)/2;
   $lon_12 = ($lon_0 + $lon_24)/2;

 print "before adjust :
         lat-0 is $lat_0, lon-0 is $lon_0,
         lat-12 is $lat_12, lon-12 is $lon_12,
         lat-24 is $lat_24, lon-24 is $lon_24\n";

   # Check if it is cross dateline 180E/W
   if (($lon_0 / $lon_24) < 0 && (abs($lon_0) > 90))
   {
      if ($lon_12 <= 0)
      {
         $lon_12 += 180;
      }
      else
      {
         $lon_12 -= 180;
      }
   }

   print "after adjust:
         lat-0 is $lat_0, lon-0 is $lon_0,
         lat-12 is $lat_12, lon-12 is $lon_12,
         lat-24 is $lat_24, lon-24 is $lon_24\n";

}	## End of the subroutine "get12_point"


############  Subroutine "get6_18_points" ###############
##################################################
sub get6_18_points
{
   $lat_6 = ($lat_0 + $lat_12)/2;
   $lon_6 = ($lon_0 + $lon_12)/2;
   $lat_18 = ($lat_12 + $lat_24)/2;
   $lon_18 = ($lon_12 + $lon_24)/2;
  
 print "before adjust :
         lat-0 is $lat_0, lon-0 is $lon_0,
         lat-6 is $lat_6, lon-6 is $lon_6,
         lat-12 is $lat_12, lon-12 is $lon_12,
         lat-18 is $lat_18, lon-18 is $lon_18
         lat-24 is $lat_24, lon-24 is $lon_24\n";


 
   # Check if it is cross dateline 180E/W
   if (($lon_0 / $lon_12) < 0 && (abs($lon_0) > 90))
   {
      if ($lon_6 <= 0)
      {
         $lon_6 += 180;
      }
      else
      {
         $lon_6 -= 180;
      }   
   }

   if (($lon_12 / $lon_24) < 0 && (abs($lon_12) > 90))
   {
      if ($lon_18 <= 0)
      {
         $lon_18 += 180;
      }
      else
      {
         $lon_18 -= 180;
      }
   }

   print "after adjust:
         lat-0 is $lat_0, lon-0 is $lon_0,
         lat-6 is $lat_6, lon-6 is $lon_6,
         lat-12 is $lat_12, lon-12 is $lon_12,
         lat-18 is $lat_18, lon-18 is $lon_18
         lat-24 is $lat_24, lon-24 is $lon_24\n";

} 	## End of the subroutine "get6_18_points"


############  Subroutine "writeTrack" ###############
##################################################

sub writeTrack
{  
   my($i);
   open(POSTRACK, "> $positionFile") || die "Can not open $positionFile for writing: $!";
   for($i = 0; $i <= 24; $i += 6)
   {
      $latName = "lat_" . $i;
      $lonName = "lon_" . $i;
      printf POSTRACK "%2d%8.2f%8.2f\n", $i,${$latName}, ${$lonName};

      # all these names can work: ${$latName}, $$latName, and ${"lat_" . $i}
      #printf POSTRACK "%2d%8.2f%8.2f%8.1f\n", $i,${$latName}, $$lonName, ${"lat_" . $i} ;
   }  
   close(POSTRACK) || die "Can not close $positionFile: $!";

}	## End of subroutine "writeTrack"


############  Subroutine "getStormInfo1" ###############
########################################################

sub getStormInfo1
{
   my($filename) = @_;

   $tempMsg = `grep "DISCUSSION NUMBER" $filename`;

   if ($tempMsg ne "")
   {
      print "The header in list1 is $wmoid. \n";

      # start to get the storm name
      @splitMsg = split(/DISCUSSION/, $tempMsg);
      @nameMsg = split(/\s+/, $splitMsg[0]);
      $stormName = $nameMsg[-1];

      # the array to store all the storms name included the repeat one
      push(@stormNameList, $stormName);

      my $storm1 = Storm->new();
      $storm1->name($stormName);
      $storm1->nameNNB($stormName);
      $storm1->wmoHd($wmoid);

      # start to get the time of the bulletine
      $timeMsg = `grep $wmoid $filename`;
      @splitMsg = split(/\s+/, $timeMsg);
      $stationId = $splitMsg[1];
      $bulTime = $splitMsg[2];
      $bulYJd  = $splitMsg[3];
      $storm1->bulTime($bulTime);
      $storm1->bulYJd($bulYJd); 
      $storm1->stationId($stationId);   

      # get the initial time of the storm
      $positionMsg = `grep "INITIAL    " $filename`;
      if ($positionMsg eq "")
      {
         print "Position pattern not found, don't check this storm\n";
         return;
      }

      @splitMsg = split(/\s+/, $positionMsg);
      $iniTime = $splitMsg[1];
      $iniTime =~ s/\///g;          # delete the backslash between the day and hour

      $storm1->iniTime($iniTime);

      # start to get the position of the center of the storm
      $lat = $splitMsg[2];
      $lon = $splitMsg[3];
      $storm1->lat($lat);
      $storm1->lon($lon);


      printf "The storm name is %s, wmoHeader is %s, bulletin time is %s.\n", $storm1->name,
              $storm1->wmoHd, $storm1->bulTime;

      if (!exists($stormStructList{$stormName}))
      {
         $stormStructList{$stormName} = $storm1;
      } else
      {
         $later = compareTime($stormStructList{$stormName}->bulTime,
                              $storm1->bulTime);
         if ($later == 1)        
         {
            $stormStructList{$stormName} = $storm1;
         }
      }
  
   }   # end of outest if statement
}      # end of the subroutine 


############  Subroutine "getStormInfo2" ###############
########################################################

sub getStormInfo2
{
   my($filename) = @_;

   $tempMsg = `grep "WARNING NR" $filename`;   

   if ($tempMsg ne "")
   {
      print "The header in list2 is $wmoid. \n";

      # start to get the storm name
      @splitMsg = split(/WARNING NR/, $tempMsg);
      @nameMsg = split(/ /, $splitMsg[0]);
      # -1 is another subscript for the last element
      $stormName = $nameMsg[-1];
      $stormNameNNB = $stormName;

      if ($stormName =~ /\(.+\)/)
      {
         #   @f1 = split(/[\(\)]/, $&);
         @splitName = split(/[\(\)]/, $stormName);
         $stormName = $splitName[1];
         # -2 is another subscript for the second last element
         $stormNNB = $nameMsg[-2];
         $stormNameNNB = $stormName . $stormNNB;
      }
      # Storm name has been got

      # the array to store all the storms name included the repeat one
      push(@stormNameList, $stormName);

      my $storm1 = Storm->new();
      $storm1->name($stormName);
      $storm1->nameNNB($stormNameNNB);
      $storm1->wmoHd($wmoid);

      # start to get the time of the bulletine
      $timeMsg = `grep $wmoid $filename`;

      # ignore the WTPN PHNC bulletin
      if ($timeMsg =~ /^WTPN\d{2}\s+PHNC/)
     #  if (($wmoid =~ /WTPN/) && ($stationId eq "PHNC"))
      {
         print "The 'WTPN PHNC' bulletin is duplicated, ignore it. \n";
         return;
      }

      @splitMsg = split(/\s+/, $timeMsg);

      $stationId = $splitMsg[1];
      $bulTime = $splitMsg[2];
      $bulYJd = $splitMsg[3];
      $storm1->bulTime($bulTime);
      $storm1->bulYJd($bulYJd);
      $storm1->stationId($stationId);


      # start to get the initail time of the storm
      $timeMsg = `grep " --- NEAR " $filename`;    
      if ($timeMsg eq "")
      {
         print "Time and position pattern not found, don't check this storm\n";
         return;
      } 
    
      @splitMsg = split(/\s+/, $timeMsg);

      $i = 0;
     
      # skip the first element if it is empty
      if ($splitMsg[$i] eq "")
      {
         $i++;
      } 

      $iniTime = $splitMsg[$i];
      chop($iniTime);   # remove the last correction digit
      $storm1->iniTime($iniTime);

      # start to get the position of the center of the storm
      $lat = $splitMsg[$i+3];
      $lon = $splitMsg[$i+4];
      $latCorNum = chop($lat);         # remove the last correction digit    
      $lonCorNum = chop ($lon);


      # If there is no correction digit at the end of lat or lon, unchop it.
      if ($latCorNum =~ /[NnSs]/)
      {
         print "No correction digit at the lat end, append the chopped char again.\n";
         $lat .= $latCorNum;
      }

      if ($lonCorNum =~ /[EeWw]/)
      {
         print "No correction digit at the lon end, append the chopped char again.\n";
         $lon .= $lonCorNum;
      }

      $storm1->lat($lat);
      $storm1->lon($lon);

      printf "The storm name is %s, wmoHeader is %s, bulletin time is %s.\n", $storm1->name,
               $storm1->wmoHd, $storm1->bulTime;

      if (!exists($stormStructList{$stormName}))
      {
         $stormStructList{$stormName} = $storm1;
      } else
      {
         $later = compareTime($stormStructList{$stormName}->bulTime,
                              $storm1->bulTime);
         if ($later == 1)
         {
            $stormStructList{$stormName} = $storm1;
         }
      }

   }   # end of outest if statement
}      # end of the subroutine

