#!/usr/bin/perl -w
## =================================================================================
## Name:            getpPass.pl
##
## Language:        Perl
## Purpose:         Get AMSUB/TRMM/SSMI/SSMIS geolocation data files from the Polar Server
##
## Method:          a) Make list of files on local machine
##                  b) Create script to run sftp to do (c)
##                  c) Get directory listing of files-available on Poler Server
##                  d) Extract filenames from sftp session log of (c)
##                  e) Make list of files available on Poler Server but not on local
##                  f) Create sftp script to fetch the new files
##                  g) Fetch the newest available files
##
## Adapted from program: getp_cemscs  (written by R. Potash)
##
## Adaptor:         Jianbin Yang/RSIS           03-July-2002
##                  For Automated TRaP
##
## Modified:        09/02/2003   
##                  Change the filter time from 8 hours to 12 hours, since don't want to miss any
##		    new (unused) data for the old bulletin (maximum 12 hours old)
##                           
## Modified:   11/04/2003
##                  ls output from sftp change in Redhat 9, it was equivalent to "ls -l"
##                  so the code to get the remote file name has to be changed 
## Modified: 04/22/2005
##                  ls output from sftp change in Redhat Enterprise 4.  
##                  In Redhat 9 (or RH En 3), it lists one file per line.  In RH EN 4,
##                  it lists multiple files in one line, if the files are not long enough.
##
## Modified: 06/11/2006
##                SSMI data diretory changed. 
##
## Modified: 09/13/2006
##   		  SSMI geolocation file name change from SSGEO.EDRR to SSGEO.NPR.EDRR
##
## Modified: 09/17/2007
##                Get the AMSRE geolocation file
##
## Modified: 02/06/09
##         Changed to use $HOME without specifying the oper account
##
## Modified: 10/13/09
##         Turn off F-15
##
## Modified: 01/07/10
##         Turn off NOAA-17
##
## Last Modified: 11/29/11
##         Start handling SSMISA
##
##==================================================================================
#
##Define directories
#
$HOME = $ENV{'HOME'};
$DATAHOME = $ENV{'DATAHOME'};

#print "DATAHOME $DATAHOME\n";
$localDataDir  = "/data/Petrap/data/mcidas_data/polerPassData";
$passDataDir = "$dataDir/mcidas_data/polerPassData";

$remoteSite    = 'oper\@satepsdist4.nesdis.noaa.gov';
#$remoteSite    = 'gp10';
# $remoteSite    = 'gp19';
$remoteAmsuDataDir = '/gpfs1/ops/test/shared/Pmctaps/amsu';
$remoteTrmmDataDir = '/gpfs1/ops/test/shared/Pmctaps/trmm';
$remoteSsmisDataDir = '/gpfs1/ops/test/shared/Pmctaps/ssmis';
#$remoteTrmmDataDir = '/data/trmm/output7';
## $remoteSsmiDataDir = '/home/oper1/mcidas/data';
#$remoteSsmiDataDir = '/data/ssmi_raw_data';
#$remoteSsmisDataDir = '/data2/ssmis/interm_sngl';

#$remoteSite2    = 'eosb5';
#$remoteSite2    = 'eosp5';
$remoteSite2    = 'oper\@10.144.3.76';
$remoteAmsreDataDir = '/data5p/amsre/L2_RAIN/output';

$shelldate     = `date`;
$util_dir      = "/home/Plocal/bin";

#
#Creat AMSUB, TRMM and SSMI geolocation file name for sftp for current day
$amsub_file='amsub\.D';
$trmm_file='trmm\.';
# $ssmi_file='SSGEO\.EDRR\.S';
$ssmi_file='SSGEO\.NPR\.EDRR\.S';
$ssmis_file='GEO\.EDEB\.S';
$amsre_file='amsre\.Rain';		# amsre.Rain_V09_200709171544_D

$startdate=`date`;
print " Start  $0 at $startdate \n";

# Populate list of files wanted
  print  "Create a list of all the local files\n";

chdir("$localDataDir") || die "Cannot change to localDatadir $localDataDir";
 print  "Local data directory  $localDataDir \n";

###-----------------------------------------------------------------------------

# Create a list of all the local AMSU pass files with given prefixes
#
@localAmsuFileList=();
@list=` ls  -t $amsub_file* |head -50 ` ;
chomp(@list);

foreach $filename  (@list)
{
#   print " local amsu file: $filename\n ";
   push  ( @localAmsuFileList, $filename );
}
 
# Create a list of all the local TRMM pass files with given prefixes
#
@localTrmmFileList=();
@list=` ls  -t $trmm_file* |head -30 ` ;
chomp(@list);

foreach $filename  (@list)
{
#   print " local trmm file: $filename\n ";
   push  ( @localTrmmFileList, $filename );
}

# Create a list of all the local SSMI pass files with given prefixes
#
@localSsmiFileList=();
@list=` ls  -t $ssmi_file* |head -50 ` ;
chomp(@list);

foreach $filename  (@list)
{
 #  print " local ssmi file: $filename\n ";
   push  ( @localSsmiFileList, $filename );
}


# Create a list of all the local SSMIS pass files with given prefixes
#
@localSsmisFileList=();
@list=` ls  -t $ssmis_file* |head -50 ` ;
chomp(@list);

foreach $filename  (@list)
{
#   print " local ssmis file: $filename\n ";
   push  ( @localSsmisFileList, $filename );
}

# Create a list of all the local AMSRE pass files with given prefixes
#
@localAmsreFileList=();
@list=` ls  -t $amsre_file* |head -50 ` ;
chomp(@list);

foreach $filename  (@list)
{
 #  print " local amsre file: $filename\n ";
   push  ( @localAmsreFileList, $filename );
}


#-----------------------------------------------------------------------------
print "-------Generate sftp script for directory list\n";

# Generate sftp script for directory list ofRemote 
#
#$sftpfile1 = "$HOME/mcidas/tmp/polersftpRmtDir";
$sftpfile1 = "$HOME/mcidas/tmp/polersftpRmtDir";
$sftpfile2 = "$HOME/mcidas/tmp/polersftpRmtGet";
# print "SFTPFILE1 $sftpfile1\n";
if ( -s $sftpfile1 ) 
{  
#   unlink  $sftpfile1; 
   print " $sftpfile1 already exist, overwritten! \n";
}
if ( -e $sftpfile2 ) 
{ 
#   unlink  $sftpfile2;
   print " $sftpfile2 already exist, overwritten! \n";
}

open  ( LUOUT,  ">$sftpfile1" )  ||  die " Can't seem to open $sftpfile1: $!";

   print  LUOUT   "# SFTP command file: Get directory listing from remote server\n";
   print  LUOUT   "# File - Constructed $shelldate\n";
   print  LUOUT   "#sftp $remoteSite << EOF\n";
   print  LUOUT   "    cd $remoteAmsuDataDir \n";
   print  LUOUT   "    ls -l\n";
   print  LUOUT   "    cd $remoteTrmmDataDir \n";
   print  LUOUT   "    ls -l\n";
   print  LUOUT   "#    cd $remoteSsmiDataDir \n";
   print  LUOUT   "#    ls -l\n";
   print  LUOUT   "    cd $remoteSsmisDataDir \n";
   print  LUOUT   "     ls -l\n";
   print  LUOUT   "#    bye\n";
   print  LUOUT   "EOF\n";
#   print  LUOUT   "sftp $remoteSite2 << EOF2\n";
#   print  LUOUT   "    cd $remoteAmsreDataDir \n";
#   print  LUOUT   "    ls -l\n";
#   print  LUOUT   "    bye\n";
#   print  LUOUT   "EOF2\n";

close ( LUOUT );
chmod  (0777,  $sftpfile1) ;
print " Run the sftp file to get the directory listing from Plocal\n";
#-----------------------------------------------------------------------------
# Run the sftp file to get the directory listing from Remote

open ( SFTPOUT,  "$sftpfile1 |" );

@sftpOutput = <SFTPOUT>;

#unlink  $sftpfile1;

print " Finish  getting the directory listing from remoteMachine\n";

#-----------------------------------------------------------------------------
# Process the output list to find names of files available on NASA.
#   for all lines in sftp output log,             (Default loop arg)
#     Select lines with date string  ".Dnnnnn"  (n=digit)
#     Extract filename                           (from """"""""""")
#     Add name to listAvailable
# Note - match condition for selecting lines is admitedly kludgy...
# Only choose orbit within 12 hours
#-----------------------------------------------------------------------------

# Call utility script to get the time 12 hrs prior to current
$CurrentJday=`$util_dir/TKgetdayExe`;
$CurrentTime=`$util_dir/TKgettimeExe`;
chomp($CurrentJday);
chomp($CurrentTime);

# Get yyyyddd & hhmmss for cutting time rollback 12 hours
# ( Start Scan is less than 12 hrs old / end scan <~ 10.4 hrs old)
$max_age = -120000;
$TimeString=`$util_dir/TKinctimeExe -srcday $CurrentJday -srchms $CurrentTime -n_hms $max_age `;
  
@fields   =  split ( /\s+/,$TimeString );
print "\n TimeString $TimeString \n";
print "\n TimeString @fields\n";

$Cut_yyyyddd=$fields[0];
$Cut_hhmmss=$fields[1];
print " Cut_yyyyddd $Cut_yyyyddd \n";
print " Cut_hhmmss  $Cut_hhmmss \n";

@rmtFilesAvailable=();

foreach  (@sftpOutput)  
{
   # get the amsu pass files list
   if  (  /$amsub_file/ )
   {

      @fields   =  split ( /\s+/ );
      $filename = $fields[8];
      # $filename = $_;
#print "the amsub file is $filename \n";
      #if ($filename =~ /^$amsub_file\d{5}\.S\d{4}\.E\d{4}/) 
      if (($filename =~ /^$amsub_file\d{5}\.S\d{4}\.E\d{4}/) && ($filename !~ /\.NM$/))
      {
         print "the amsub file is **$&** \n";
       
         #---------------------------------------------------------
         # -rw-r--r--    1 oper1     ops         20234 Apr 30 20:32 
         # amsub.D02093.S0056.E0248.NL
         #---------------------------------------------------------
         #Get time from orbit name
         $orbit_yy=substr($filename,7,2);
         $orbit_ddd=substr($filename,9,3);
         $orbit_yyddd=substr($filename,7,5);
         $orbit_hh=substr($filename,14,2);
         $orbit_mm=substr($filename,16,2);

         chomp($orbit_yy);
         chomp($orbit_ddd);
         chomp($orbit_hh);
         chomp($orbit_mm);

         print "$filename - $orbit_yy  $orbit_ddd   $orbit_hh  $orbit_mm \n ";
     
         # Check if the file is within 12 hours
	 print "orbit_yyddd $orbit_yyddd\n";
#         $Orbit_yyyyddd = `$util_dir/TKydtocydExe $orbit_yyddd`;
	 $Orbit_yyyyddd = $orbit_yyddd + 2000000;
#	 $Orbit_yyyyddd = `$util_dir/TKdmytocydExe -day $orbit_dd -year $orbit_yyyy -month $orbit_MO`;
         $Orbit_hhmmss = $orbit_hh * 10000 + $orbit_mm * 100 ;
        print "Orbit_yyyyddd=$Orbit_yyyyddd \n Orbit_hhmmss =$Orbit_hhmmss\n ";

         if ( $Orbit_yyyyddd > $Cut_yyyyddd ||($Orbit_yyyyddd == $Cut_yyyyddd &&
              $Orbit_hhmmss >= $Cut_hhmmss ))
         {
            chomp( $filename );
            print "  rmt File is qualified to selecte : $filename \n";
            push  ( @rmtFilesAvailable, $filename ) ;
         } 
      }  
   }  # got the amsu pass file lists

   # get the trmm pass files list
   if  (  /$trmm_file/ )
   {
      @fields   =  split ( /\s+/ );
      $filename = $fields[8];
      # $filename = $_;

      if ($filename =~ /^$trmm_file\d{4}-\d{2}-\d{2}T\d{2}-\d{2}-\d{2}Z/)
      {
         print "the trmm file is $& \n";

         #---------------------------------------------------------
         # -rw-r--r--    1 oper1     ops         16938 May 13 15:42
         # trmm.2002-05-07T20-32-25Z
         #---------------------------------------------------------
         #Get time from orbit name
         $orbit_yyyy=substr($filename,5,4);
         $orbit_MO=substr($filename,10,2);
         $orbit_dd=substr($filename,13,2);    
         $orbit_hh=substr($filename,16,2);
         $orbit_mm=substr($filename,19,2);
         $orbit_ss=substr($filename,22,2);

         chomp($orbit_yyyy);
         chomp($orbit_MO);
         chomp($orbit_dd);
         chomp($orbit_hh);
         chomp($orbit_mm);
         chomp($orbit_ss);

        # print "$filename - $orbit_yyyy $orbit_MO $orbit_dd $orbit_hh $orbit_mm $orbit_ss \n ";


         $Orbit_yyyyddd = `$util_dir/TKdmytocydExe -day $orbit_dd -year $orbit_yyyy -month $orbit_MO`;

         $Orbit_hhmmss = $orbit_hh * 10000 + $orbit_mm * 100 + $orbit_ss;
         print "Orbit_yyyyddd=$Orbit_yyyyddd \n Orbit_hhmmss =$Orbit_hhmmss\n ";

         if ( $Orbit_yyyyddd > $Cut_yyyyddd ||($Orbit_yyyyddd == $Cut_yyyyddd &&
              $Orbit_hhmmss >= $Cut_hhmmss ))
         {
            chomp( $filename );
            print "  rmt File is qualified to selecte : $filename \n";
            push  ( @rmtFilesAvailable, $filename ) ;
         }
      }
   }  # got the trmm pass file lists


   # WILL get the ssmi pass files list in the following
   # ------
   # get the SSMI pass files list
   if  (  /$ssmi_file/ )
   {
      @fields   =  split ( /\s+/ );
      $filename = $fields[8];
      # $filename = $_;

      if (($filename =~ /^$ssmi_file\d\.D\d{5}\.S\d{4}\.E\d{4}/) && ($filename !~ /\.S9\./))
      {
         print "the ssmi file is $& \n";

         #---------------------------------------------------------
         # -rw-r--r--    1 oper1     ops         34145 Jun 27 19:51
         # SSGEO.EDRR.S7.D02178.S0504.E0712.B3745152.NS
         #
         # New SSMI file name since 9/12/2006 :
         # SSGEO.NPR.EDRR.S8.D06256.S0505.E0707.B487414 
         #---------------------------------------------------------
         #Get time from orbit name
#         $orbit_yy=substr($filename,15,2);
#         $orbit_ddd=substr($filename,17,3);
#         $orbit_yyddd=substr($filename,15,5);
#         $orbit_hh=substr($filename,22,2);
#         $orbit_mm=substr($filename,24,2);

         $orbit_yy=substr($filename,19,2);
         $orbit_ddd=substr($filename,21,3);
         $orbit_yyddd=substr($filename,19,5);
         $orbit_hh=substr($filename,26,2);
         $orbit_mm=substr($filename,28,2);

         chomp($orbit_yy);
         chomp($orbit_ddd);
         chomp($orbit_hh);
         chomp($orbit_mm);

        # print "$filename - $orbit_yy  $orbit_ddd   $orbit_hh  $orbit_mm \n ";

         # Check if the file is within 12 hours
#         $Orbit_yyyyddd = `$util_dir/TKydtocydExe $orbit_yyddd`;
	 $Orbit_yyyyddd = $orbit_yyddd + 2000000;
#        $Orbit_yyyyddd = `$util_dir/TKdmytocydExe -day $orbit_dd -year $orbit_yyyy -month $orbit_MO`;
         $Orbit_hhmmss = $orbit_hh * 10000 + $orbit_mm * 100 ;
        # print "Orbit_yyyyddd=$Orbit_yyyyddd \n Orbit_hhmmss =$Orbit_hhmmss\n ";

         if ( $Orbit_yyyyddd > $Cut_yyyyddd ||($Orbit_yyyyddd == $Cut_yyyyddd &&
              $Orbit_hhmmss >= $Cut_hhmmss ))
         {
            chomp( $filename );
            print "  rmt File is qualified to selecte : $filename \n";
            push  ( @rmtFilesAvailable, $filename ) ;
         }
      }
   }  # got the ssmi pass file lists

   # WILL get the ssmis pass files list in the following
   # ------
   # get the SSMIS pass files list
   if  (  /$ssmis_file/ )
   {
      @fields   =  split ( /\s+/ );
      $filename = $fields[8];
      # $filename = $_;
#      print "got into getpass SSMIS section \n";
       print "the ssmis file is **$filename** \n";
      if ($filename =~ /^$ssmis_file[ABC]\.D\d{5}\.S\d{4}\.E\d{4}/)
#      if ($filename =~ /^$ssmis_file//(A|B/)/)
#      if (($filename =~ /^$ssmis_file\A\.D\d{5}\.S\d{4}\.E\d{4}.B\d{7}.NS/) && ($filename !~ /\^5/))
      {
         print "The ssmis file is $& \n";

         #---------------------------------------------------------
         # SSMIS file name  :
         # GEO.EDEB.SB.D11271.S1003.E1149
         #---------------------------------------------------------
         #Get time from orbit name
#         $orbit_yy=substr($filename,15,2);
#         $orbit_ddd=substr($filename,17,3);
#         $orbit_yyddd=substr($filename,15,5);
#         $orbit_hh=substr($filename,22,2);
#         $orbit_mm=substr($filename,24,2);

         $orbit_yy=substr($filename,13,2);
         $orbit_ddd=substr($filename,15,3);
         $orbit_yyddd=substr($filename,13,5);
         $orbit_hh=substr($filename,20,2);
         $orbit_mm=substr($filename,22,2);

         chomp($orbit_yy);
         chomp($orbit_ddd);
         chomp($orbit_hh);
         chomp($orbit_mm);

         print "$filename - $orbit_yy  $orbit_ddd   $orbit_hh  $orbit_mm \n ";

         # Check if the file is within 12 hours
	 print "orbit_yyddd $orbit_yyddd\n";
#         $Orbit_yyyyddd = `$util_dir/TKydtocydExe $orbit_yyddd`;
	 $Orbit_yyyyddd = $orbit_yyddd + 2000000;
#        $Orbit_yyyyddd = `$util_dir/TKdmytocydExe -day $orbit_dd -year $orbit_yyyy -month $orbit_MO`;
         $Orbit_hhmmss = $orbit_hh * 10000 + $orbit_mm * 100 ;
         print "Orbit_yyyyddd=$Orbit_yyyyddd \n Orbit_hhmmss =$Orbit_hhmmss\n ";
	 print "Cut_yyyyddd=$Cut_yyyyddd \n ";
	 print "Cut_hhmmss=$Cut_hhmmss \n ";
         if ( $Orbit_yyyyddd > $Cut_yyyyddd ||($Orbit_yyyyddd == $Cut_yyyyddd &&
              $Orbit_hhmmss >= $Cut_hhmmss ))
         {
            chomp( $filename );
            print "  rmt File is qualified to selecte : $filename \n";
            push  ( @rmtFilesAvailable, $filename ) ;
         }
      }
   }  # got the ssmis pass file lists

   #----------- AMSRE geolocation file -----------#

   # get the amsre pass files list
   if  (  /$amsre_file/ )
   {

      @fields   =  split ( /\s+/ );
      $filename = $fields[8];
      # $filename = $_;
#print "the amsre file is $filename \n";
      if ($filename =~ /^$amsre_file.+\d{12}/) 
      {
         print "the amsre file is **$&** \n";
       
         #---------------------------------------------------------
         # -rw-r--r--    1 oper1     ops         16018 Sep 17 18:45 
         # amsre.Rain_V09_200709171454_A
         # amsre.Rain_V09_200709171544_D
         #---------------------------------------------------------
         #Get time from orbit name


         $orbit_yyyy=substr($filename,15,4);
         $orbit_MO=substr($filename,19,2);
         $orbit_dd=substr($filename,21,2);
         $orbit_hh=substr($filename,23,2);
         $orbit_mm=substr($filename,25,2);
         $orbit_ss=0;

         chomp($orbit_yyyy);
         chomp($orbit_MO);
         chomp($orbit_dd);
         chomp($orbit_hh);
         chomp($orbit_mm);

         print "$filename - $orbit_yyyy $orbit_MO $orbit_dd $orbit_hh $orbit_mm $orbit_ss \n ";

         $Orbit_yyyyddd = `$util_dir/TKdmytocydExe -day $orbit_dd -year $orbit_yyyy -month $orbit_MO`;

         $Orbit_hhmmss = $orbit_hh * 10000 + $orbit_mm * 100 + $orbit_ss;
         print "Orbit_yyyyddd=$Orbit_yyyyddd \n Orbit_hhmmss =$Orbit_hhmmss\n ";

         if ( $Orbit_yyyyddd > $Cut_yyyyddd ||($Orbit_yyyyddd == $Cut_yyyyddd &&
              $Orbit_hhmmss >= $Cut_hhmmss ))
         {
            chomp( $filename );
            print "  rmt File is qualified to selecte : $filename \n";
            push  ( @rmtFilesAvailable, $filename ) ;
         }

      }  
   }  # got the amsre pass file lists

}     # end of the foreach loop
 
#-----------------------------------------------------------------------------
# Generate list of files to be fetched:  
#    create assoc array - filename vs filecount; 
#    count is 1 if available on Poler Server,  0 if avalable locally.
#    files with net count >0  are what we want to get.

chomp(@rmtFilesAvailable);
chomp(@localAmsuFileList);
chomp(@localTrmmFileList);
chomp(@localSsmiFileList);
chomp(@localSsmisFileList);
chomp(@localAmsreFileList);

@amsufilesToFetch=();
@trmmfilesToFetch=();
@ssmifilesToFetch=();
@ssmisfilesToFetch=();
@amsrefilesToFetch=();

%count = ();

foreach  $filename  ( @rmtFilesAvailable )   { $count{$filename} = 1; }

foreach  $filename  ( @localAmsuFileList )   { $count{$filename} = 0; }

foreach  $filename  ( @localTrmmFileList )   { $count{$filename} = 0; }

foreach  $filename  ( @localSsmiFileList )   { $count{$filename} = 0; }

foreach  $filename  ( @localSsmisFileList )   { $count{$filename} = 0; }

foreach  $filename  ( @localAmsreFileList )   { $count{$filename} = 0; }

foreach  $filename  ( keys %count ) 
{ 
   if  ( $count{$filename} == 1 )
   { 
   #   print "\nSelected file : $filename\n ";
      if ($filename =~ /^$amsub_file/)
      { 
         push (@amsufilesToFetch, $filename);
      }
      elsif ($filename =~ /^$trmm_file/)
      {
         push (@trmmfilesToFetch, $filename);
      }
      elsif ($filename =~ /^$ssmi_file/)
      {
         push (@ssmifilesToFetch, $filename);
      }
      elsif ($filename =~ /^$ssmis_file/)
      {
         push (@ssmisfilesToFetch, $filename);
      }
      elsif ($filename =~ /^$amsre_file/)
      {
         push (@amsrefilesToFetch, $filename);
      }

   }
}

#-----------------------------------------------------------------------------
# Create another sftp script to fetch wanted files

if ( ( @amsufilesToFetch  == 0) && ( @trmmfilesToFetch  == 0) 
       && ( @ssmifilesToFetch  == 0)
       && ( @ssmisfilesToFetch  == 0)
       && ( @amsrefilesToFetch  == 0) ) 
{
   die "No files still wanted - exit script";
}

print " Create another sftp script to fetch wanted files\n";

open  ( LUOUT,  ">$sftpfile2" )  ||  die " Can't open $sftpfile2: $!";

   print  LUOUT   "# SFTP command file: Fetch files from remote server\n";
   print  LUOUT   "# File - Constructed $shelldate\n";
   print  LUOUT   "#sftp $remoteSite << EOF\n";
   print  LUOUT   "#   lcd  $localDataDir\n";

   print  LUOUT   "   cd  $remoteAmsuDataDir \n";   
   foreach  $filename  ( @amsufilesToFetch )
   {
     print  LUOUT "   cp  $filename $localDataDir\n";
   }

   print  LUOUT   "   cd  $remoteTrmmDataDir \n";
   foreach  $filename  ( @trmmfilesToFetch )
   {
     print  LUOUT "   cp  $filename $localDataDir\n";
   }

#   print  LUOUT   "   cd  $remoteSsmiDataDir \n";
#   foreach  $filename  ( @ssmifilesToFetch )
#   {
#     print  LUOUT "   cp  $filename $localDataDir\n";
#   }

   print  LUOUT   "   cd  $remoteSsmisDataDir \n";
   foreach  $filename  ( @ssmisfilesToFetch )
   {
     print  LUOUT "   cp  $filename $localDataDir\n";
   }

   print  LUOUT   "   bye\n";
   print  LUOUT   "EOF\n";

#   print  LUOUT   "sftp $remoteSite2 << EOF2\n";
#   print  LUOUT   "   lcd  $localDataDir\n";
#   print  LUOUT   "   cd  $remoteAmsreDataDir \n";
#   foreach  $filename  ( @amsrefilesToFetch )
#   {
#     print  LUOUT "   get  $filename\n";
#   }
#   print  LUOUT   "   bye\n";
#   print  LUOUT   "EOF2\n";


close ( LUOUT );
chmod  (0777,  $sftpfile2) ;

#-----------------------------------------------------------------------------
# Run the sftp file to get the files from NASA
$enddate=`date`;
print " Start  sftp wanted files at $enddate \n";
open ( SFTPOUT,  " $sftpfile2  | " ) || die " Can't open $sftpfile2 2nd time: $!";

@sftpOutput = <SFTPOUT>;

#unlink  $sftpfile2;

#-----------------------------------------------------------------------------
#                     (Snippet of directory listing from amsub)
