#!/usr/bin/perl -w

## =====================================================================
## Name:        notifyEmail.pl
##                  
## Author:      Jianbin Yang/QSS   09/12/2006
## Language:    Perl
## Purpose:     Read the new TRaP list and create email(s) to customers for 
##              TRaPs genreted in their area of interest.            
##                  
## Modified:   06/03/2008
##
## Last Modified:   03/23/2009
##                Changed to use $HOME without specifying the oper account
##                Changed the Operatilonal FTP server from GP16 to Satepsanon
##
## =====================================================================


#--------- Define the directories and files ---------#

$HOME = $ENV{"HOME"};

# $dataDir = '/home/oper1/mcidas/data';
$dataDir = "$HOME/mcidas/pctrap-V3";
$listFile = "$dataDir/TRaPList.txt";
$oldListFile = "$dataDir/TRaPList.txt.old";
$emailListFile = "$dataDir/TRaPEmailList.txt";
# $emailListFile = "$dataDir/TRaPEmailList-test.txt";

#--------- SAB Tropical Cyclone Homepage URL ---------#
$sabURL = 'http://www.ssd.noaa.gov/PS/TROP/trap.html';


#--------- anonymous ftp server and directories link ---------#
$ftpHomeDir ='pub/TRAP';
$ftpSatHomeDir ='TRAP';
# $ftpHomeDir ='pub/testTRAP';       # for testing purpose, will use the previous when become operational
$ftpURL = 'ftp://gp16.ssd.nesdis.noaa.gov/' . $ftpHomeDir;
$ftpURLSat = 'ftp://satepsanone.nesdis.noaa.gov/' . $ftpSatHomeDir;

$readmeDir = 'README';

#--------- the email notification list of different agent ---------#

# New define email list names are as following, must match the names in "TRaPEmailList.txt".  Added on 09/11/2006
# TPCList, AtlanticList, CPHCList, EastPacificList
# PerthList, DarwinList, BrisbaneList, ReunionList, NewDelhiList
# NadiList, NWPacificList, and BCCList.

# $TPCList        = "TPCList";
# $AtlanticList   = "AtlanticList";
# $CPHCList       = "CPHCList";
# $PerthList      = "PerthList";
# $DarwinList     = "DarwinList";
# $BrisbaneList   = "BrisbaneList";
# $ReunionList    = "ReunionList";
# $NewDelhiList   = "NewDelhiList";
# $NadiList       = "NadiList";
# $NWPacificList  = "NWPacificList";
# $ccList         = "BCCList";

# initialize the email lists

@TPCList        = ();
@AtlanticList   = ();
@CPHCList       = ();
@EastPacificList = ();
@PerthList      = ();
@DarwinList     = ();
@BrisbaneList   = ();
@ReunionList    = ();
@NewDelhiList   = ();
@NadiList       = ();
@NWPacificList  = ();
@BCCList        = ();


@allEmailList = ();

$notifyPeriod = 2;	# in hours, must be matched with the crontab job setting

$startdate=`date`;
print " Start  $0 at $startdate \n";

chdir("$dataDir") || die "Cannot change to directory $dataDir";
print  "Data directory is:";
system("pwd");

# remove the path from the command name


if (!(-s $listFile))
{
   print "The new TRaP list $listFile is not exist. -- Exit script.\n";
   exit;
}

##--------------------------------------------------------------------##

##--------------------- Read the email list file ---------------------## 

print "\n##--------------------- Read the email list file ---------------------##\n\n";

open(EMAILLIST, "< $emailListFile") || die "Can not open $emailListFile for reading: $!";
@dataList = <EMAILLIST>;
close(EMAILLIST);

for ($i = 0; $i < $#dataList+1; $i++)
{
   chomp($dataList[$i]);
#   print "the line in the FOR is $dataList[$i].\n";
   if ($dataList[$i] =~ /^\w+\:/)
   {
      @splitList = split(/\:/, $dataList[$i]);
      $listName = $splitList[0];
      $i++;
      while (($dataList[$i] !~ /^END$/) && ($i < $#dataList+1))
      {
         chomp($dataList[$i]);

         # Skip the comment
         if ($dataList[$i] =~ /^\#/)
         {
            $i++;
            next;
         }
         elsif ($dataList[$i] =~ /^\+\+\w+/)
         {
            # remove the "++"
            $subList = substr($dataList[$i], 2);
            push (@{$listName}, @{$subList});
         }
         elsif ($dataList[$i] =~ /\@/)
         {
            push (@{$listName}, $dataList[$i]);
         }
         $i++;
      }
      print "listName is $listName, email list is @{$listName}\n";
   }

}	# End of FOR to read the email list


print "TPCList is **@TPCList**\n, AtlanticList is **@AtlanticList**\n, EastPacificList is **@EastPacificList**\n";
print "NWPacificList is **@NWPacificList**, BrisbaneList is **@BrisbaneList**\n";

# get the Bcc list string
$bccList = "";
if (@BCCList != 0)
{
   for ($j = 0; $j < $#BCCList+1; $j++)
   {
      $bccList = $bccList . " -b " . $BCCList[$j];
   }
   print "bccList is $bccList.\n";
}


##--------------------------------------------------------------------##

##--------------------- Read the TRaP list file  ---------------------##

print "\n##--------------------- Read the TRaP list file  ---------------------##\n\n";

open(TRAPLIST, "< $listFile") || die "Can not open $listFile for reading: $!";
@dataList = <TRAPLIST>;
close(TRAPLIST);

print "total line number of the list file is $#dataList\n";

$year = "";
$year2 = "";

for ($i = 0; $i < $#dataList+1; $i++)
{
   @TRaPNameList = ();
   @allMailList  = ();

   chomp($dataList[$i]);
#   print "the line in the FOR is $dataList[$i].\n";
   if ($dataList[$i] =~ /^\w+\:/)
   {
      @splitList = split(/\:/, $dataList[$i]);
      $basinName = $splitList[0];
      $i++;

      # this is for the TRaP name
      while (($dataList[$i] !~ /ENDNAME-$/) && ($i < $#dataList+1))
      {
         chomp($dataList[$i]);
         # look for pattern like "2006IOKE.WTJP21.RJTD.061800.SSMI.061949"
         if ($dataList[$i] =~ /^\d{4}.*\..*\..*\..*\..*\./) 
         {   
            if ($year eq "")
            {
               $year = substr ($dataList[$i], 0, 4);
            }
            elsif ($year != substr ($dataList[$i], 0, 4) && ($year2 eq ""))
            {
               $year2 = substr ($dataList[$i], 0, 4);
            }
               
            push (@TRaPNameList, $dataList[$i]);
#            print "the line in while/if is:$dataList[$i]\n";
         }
         $i++;
      }
#      print "the line in if is $dataList[$i]\n";
      print "year is $year, year2 is $year2.\n";
 
      # this is for the email list
      while (($dataList[$i] !~ /ENDEMAIL-$/) && ($i < $#dataList+1))
      {
         chomp($dataList[$i]);
#        print "the line in while/if is:$dataList[$i]\n";
         
         if (($dataList[$i] !~ /\-/) && ($dataList[$i] ne "" ))
         {
            push (@allMailList, @{$dataList[$i]});
#           print "the line in email while/if is:$dataList[$i]\n";
         }
   
        $i++;
      }


      #--------------------------------------------------------------------#

      #--------------------- Send EMAIL to customers  ---------------------#
 
      if ((@TRaPNameList != 0) && (@allMailList != 0))
      {
         print "\nbasinName is $basinName, TRaPNameList is **@TRaPNameList**\n, allMailList is **@allMailList**\n";
         print "Send email to customer for basin $basinName.\n";

 
         $emailListStr = "";

         # link the email lists to a long string, NOTE don't use $i here!!
         for ($j = 0; $j < $#allMailList+1; $j++)
         {          
            $emailListStr = $emailListStr . " " . $allMailList[$j];          
         }
         print "emailListStr is $emailListStr.\n";

         $finalURL = "$ftpURL/$basinName";
         $finalURLSat = "$ftpURLSat/$basinName";


         print "Send email to $emailListStr to report the new digital TRaP.\n";
         #open  MAIL,"|mail -s 'New TRaP Created' $ccList $emailList $commonList";
         open  MAIL,"|mail -s 'New TRaP Created' $bccList $emailListStr";

# $emailFile = "emailFile-" . $basinName;
# open(MAIL, "> $emailFile") || die "Can not open $emailFile for writing: $!";

#         print MAIL "This email is sent is $bccList $emailListStr\n";
    
#         print MAIL "------ THIS IS A TEST ------!\n\n";
         print MAIL "Dear TRaP Customer,\n\n";

         print MAIL "TRaPs can be viewed as GIF images as soon as they are created on the SSD web page at $sabURL\n\n";
         print MAIL "New TRaPs have been generated in the last $notifyPeriod hours in your area of interest. ";
         print MAIL "These TRaPs fall in the $basinName basin, and will be available for 72 hours on our FTP server.\n";
#         print MAIL "This TRaP was generated for $stormName using $orbitTime2 $dataSource ";
#         print MAIL "rain rate data and the $bulTime2 $station bulletin, and will be available for 72 hours.\n";
         print MAIL "\nTo access them please go to\n";
         print MAIL "$finalURLSat and find TRaP set:\n\n";

         foreach $trapSet (@TRaPNameList)
         {
            print MAIL "$trapSet\n";
         }

         $finalURL1 = "$ftpURLSat/$year";
         print MAIL "\nFor the latest 24 hour totals in ASCII format go to:\n";
         print MAIL "$finalURL1 ";
        
         if ($year2 ne "")
         {
            $finalURL2 = "$ftpURLSat/$year2";
            print MAIL " or $finalURL2";
         }
         print MAIL "\n\n";

         print MAIL "\nFor information on how to decipher the text file please see the ";
         print MAIL "document located in directory \"$readmeDir\"\n";

         #   print MAIL "A gif of this TRaP can be found at:\n$sabURL\n";

         print MAIL "\nPlease note that you can find the same files under $finalURL as well\n";

         print MAIL "\nIf you have any questions or wish to have your name removed from our notification system, ";
         print MAIL "please contact Michael.Turk\@noaa.gov.";
         close MAIL;

      }
      else
      {
         print "\nNo TRaP generated in basin $basinName.\n";
#        print "\nbasinName is $basinName, TRaPNameList is **@TRaPNameList**\n, allMailList is **@allMailList**\n";
      }


   }	# End of "if" in the current basin

   #  chomp($dataList[$i]);
#   print "the line in if is $dataList[$i]\n";
      
}  	# End of for to read the TRaP list


rename($listFile, $oldListFile)          or die "can't rename $listFile to $oldListFile: $!";

print "End of the process.\n";



