#!/usr/bin/perl -w   

## ===================================================================================
## Name:            pctrap.pl (Version 3.0)    pctrap-auto.pl (fully Automate)
##                  
## Author:          Jianbin Yang/RSIS   June 2002
## Language:        Perl
## Purpose:         
##                  
## note : use hash table to store the storm struct information.
##        delete the print statement
##
## Modified:   04/30/2003
##        to include the Reunion, Astrulia, Tokyo and Fiji bulletins
##
## Modified:   07/18/2003
##
##        Change the AMSU criteria distance from 700km to 900 km at 15Z July 9, 2003
##
## Modified:   08/21/2003
##        change the email address from "noaa.gov" to "ssdmail2.wwb.noaa.gov"
##
## Modified:   11/20/2003
##
## Modified: 01/14/2004
##        change PLB to PLR      
##
## Modified: 01/20/2004
##        add the pass time into the command line of pctrp program, 
##        so that the image time of TRaP change to that time.      
##
## Modified: 01/23/2004
##        Add the code to check if a TRaP with the same file name was generated before,
##        if so add the '-2' in the end of the TRaP set name
## 
## Modified: 02/04/2004
##        Change the TPC/CPHC bulletin from WMO header WT**4* to WT**2*
##
## Modified: 03/05/2004
##        Add the bulletin header WTAU07 for Perth Center in Australia
##
## Modified: 04/16/2004
##        Add the ICAO bulletin (FKAU) for Australia
##
## Modified: 05/20/2004
##        Delete the entry of the geolocation file from the pass log file
##        if the rain rate data is not avaliable at that time.
##
## Modified: 07/08/2005
##        Add the Hydro Estimator rain rate. 
##        upper left(lat/lon) :  61/-164
##        lower right(lat/lon):  13/-62
##  
##        Modified on 07/21/2006, to check if the H-E RR is within the 6 hours period of forecast
##  
## Modified: 02/08/2006
##        Add the WTIO30 bulletin for Reunion
##
## Modified: 07/14/2006
##        Use NOAA-18 MHS rain rate data for TRaP
##
## Modified: 07/28/2006
##        Output the Forecast time to the end of the POS.TXT for the frame label
##        of 24 hr TRaP sent to SAB homepage
##
## Modified: Aug  11 13:02 UTC 2006 on GP36
##           TRMM and SSMi distance criteria
##
## Modified: 08/23/2006  at 19Z on GP22, and at Aug 15 19:42:03 UTC 2006 on GP36
##        Change the interval time from 6 hours to 3 hours
##
## Modified: 09/25/2006
##        Add the code for Fully automate TRaP
##
## Modified: 10/05/2006
##        Use the H-E data with closest time to forecast time
##
## Modified: 03/20/2007
##     	  Stop sending the TRaP for the "tropical low", "disturbance" and "ex-" cyclone to the auto page 
##
## Modified: 04/04/2007
##        Add the month of the rain rate time to the TRaP name
##
## Modified: 04/18/2007
##        Calculate and record the image display center to the Position file,
##        Stop sending the TRaP for the Extratropical or Subtropical system to auto page
##
## Modified: 05/18/2007
##	  Add to use the Metop MHS rain rate data
##
## Modified: 06/26/2007
##	  Add the bulletin for New Delhi and New Zealand 
##        Add FKPS01 for Fiji and FKIO20 for Reunion    07/05/2007
##	  Handle the TD as the unamed system in FKPQ    07/05/2007
##
## Modified: 09/18/2007
##	  Add to use the AMSRE rain rate data 
##        Change the order of calling the saveTRAPGIF.pl after the POS1.TXT was saved.  09/20/07
##
## Notes: 09/20/07, will turn off the call to area2digit, and compare the backup version
## and run the program tomorrow morning
##
## Modified: 11/05/2007    21Z
##        Turn on the AMSRE TRaP for public:SSD Web, FTP Server, etc...
##
## Modified: 01/30/2008   
##        Send all the TRaP data to SAB Data server
##
## Modified: 01/31/2008   16Z
##        Not use the RSMC Tokyo (WTJP) and Fiji (WT/HPS0) bulletins as replaced by ICAO ones
##
## Modified: 04/01/2008   17Z
##        Replaced the WTAU bulletins with AXAU
##
## Modified: 09/30/08
##         Added FKPQ32-34
##
## Modified: 10/02/08
##         Added the Satellite ID to the GIF file and text file,
##         also include the orbit file name in the text file
##
## Last Modified: 02/06/09
##         Changed to use $HOME without specifying the oper account
##
## Modified: 12/19/08
##         Added Jakarta bulletin
##
## Modified: 02/24/09
##         Kill the current process if the previous process is still running
##
## Modified: 08/12/09 (for eTRaP replacement)
##         (1). Turn off the distribution to SAB Data server
##         (2).
##         For operational version:
##           Turn off the digital TRaP creation and other distributions by calling other program
##         For back up version: 
##           Call the other program (backup) to update the basin and storm list when storm is new.
##
## Modified: 11/12/2009
##         Check the positions to eliminate the duplicate forecast
##         (1). for the correction bulletin with the same WMO header and number
##         (2). For the incorrect issued bulletin with same WMO header and different number (WTPN31 vs WTPN33)
##
## Modified: 03/04/2011
##         email update from Jianbin to Renata
##
## Modified: 07/04/2011
##         email update from Renata to C. Davenport and R. Glassberg
##
## Last modified: 11/21/2011
##         add SSMIS source
##
## ===================================================================================


use Class::Struct;
use Date::Calc qw(Add_Delta_DHMS);

####---------------------- Define  and delcare ------------------------####

##------------------------   Define structures   ------------------------##

# create a definition for a "Storm" struct
# PLEASE NOTED that the following longitudes are from the POS file got from getBulInfo.pl
# And they use Mcidas convention which longitude is reverse to the regular one.

struct Storm =>
{
   name      => '$',        # storm name for pctrp.k to call
   nameNNB   => '$',        # storm name with NNB if is available, this used for name convention
   stNumber  => '$',        # number of the storm for that basin in that year 
   wmoHd     => '$',        # the WMO header id 
   stationId => '$',        # the station id
   bulTime   => '$',        # the forecast bulletine time
   bulYJd    => '$',        # store the bulletine year and julian day (forecast day)
   bulMmdd   => '$',        # the month and day of the bulletin (forecast date)  
   iniTime   => '$',        # the initial time of the storm
   forecastHour => '$',     # TPC and CPHC bulletin will -3 from the hh in iniTime, other is same
   lat_0     => '$',        # the initial latitude of the center of the storm
   lon_0     => '$',        # the initial longitude of the center of the storm
   lat_6     => '$',        # the latitude of 6 hour forecast
   lon_6     => '$',        # the longitude of 6 hour forecast
   lat_12    => '$',        # the latitude of 12 hour forecast
   lon_12    => '$',        # the longitude of 12 hour forecast
   lat_18    => '$',        # the latitude of 18 hour forecast
   lon_18    => '$',        # the longitude of 18 hour forecast
   lat_24    => '$',        # the latitude of 24 hour forecast
   lon_24    => '$',        # the longitude of 24 hour forecast
   isValid   => '$',        # the storm structure is valid or not
};


# create a definition for a "PassData" struct
struct PassData =>
{
 #   sourceName  => '$',      # data source name of the pass, will be "AMSU", "TRMM" or "SSMI"
   maxDist   => '$',        # the criteria for checking the pass is covered the storm or not
   passList  => '@',        # the new available pass list for specific data source
};

# create a definition for a "TrapRecord" struct
# This is used to hold the information of every TRaP generated, will be the element
# in the "TrapReport" array

struct TrapRecord =>
{
   wmoHd     	=> '$',	    # The WMO header of the bulletin to use to generate the TRaP
   stormName 	=> '$',     # The storm name of the TRaP
   sourceList  	=> '@',     # Array of the data sources used to generate the TRaP for the storm,
                            # The element would be "AMSU", "TRMM" or "SSMI".
};


##----------------------- Define the directories and files -----------------------##

$HOME = $ENV{"HOME"};
$DATAHOME = $ENV{"DATAHOME"};
#$HOMEW = "/home/Petrap/work_area/GHE";
# $util_dir = '/home/oper1/TKutil';
print "HOMEW $HOMEW\n";
$util_dir = "/home/Plocal/bin";
$srcDir = "$HOME/mcidas/pctrap-V3";
$binDir = "$HOME/mcidas/bin";
$dataDir = "$DATAHOME/data";
$trapDataDir = "/data/Petrap";
#$passDataDir = "$dataDir/polerPassData";
$passDataDir = "$HOME/mcidas/data/polerPassData";

$HEPassDataLog = "$passDataDir/HEPassProcessed.log";
$passDataLog = "$passDataDir/passProcessed.log";
$newPassLog = "$passDataDir/newPassToProcess.log";
$trapSetLog = "/data/Petrap/trapSet.log";
$getBulLog = "$trapDataDir/log/getBul.log";

$wxtFile = "$dataDir/wxt.doc";		# This file hold the bulletin result when searching the active storm
$bulFile = "$dataDir/POS.DOC";		# This file hold the bulletin content for specific storm

# Please note that this name must be consistent with the one in getBulData.pl
# it is generated from that program
$bulDataFile = "$dataDir/bulInfo.txt";  # This file hold information archived from bulletin for specific storm 
   
$posFile = "POS.TXT"; 		        # the file is in the mcidas/data directory, and hold the position track
$fullPosFile = "POS1.TXT";	        # the file is in the mcidas/data directory, and hold the position track with TRaP set name

#$trapList = "$dataDir/TRaPList.txt";    # list for the custormer email notification

## define commands ##

$fetchPassCM = "$srcDir/fetchNewPass.sh";     # Command/Shell Script to fetch the new pass
# $pctrpCM = "$srcDir/pctrp-mcenv2.sh";          # Command/Shell Script to call PCTRP
$pctrpCM = "$srcDir/pctrp-mcenv3.sh";          # Command/Shell Script to call PCTRP
# $pctrpCM = "$srcDir/pctrp-mcenv3-new.sh";          # Command/Shell Script to call PCTRP
$svGifCM = "$srcDir/saveGIF-mcenv.sh";          # Command/Shell Script to save GIFs
#$area2digitCM = "$srcDir/area2digit-multi-fullyAuto.pl";  # Command/Shell Script to generate digital TRaPs

# Command/Shell Script to get the basin and update the storm list when the storm is new.
$area2digitCM = "$srcDir/getBasinList.pl"; 

#--------- SAB Data Server ---------#

$sabDataServer = 'sab\@sabna4.ssd.nesdis.noaa.gov';
$sabDataHomeDir = "/home/sab/TRAP";
$sabDataTrapDir = "/data/TRAP";
#$sabDataRmtCM = "$sabDataHomeDir/createSABDir.pl";

$sftpfile = '/tmp/sabSeverRmtPut';

## Define the AMSUB/TRMM/SSMI geolocation data file names ##

$amsubId = 'amsub\.D';
$trmmId = 'trmm\.';
# $ssmiId = 'SSGEO\.EDRR\.S';
$ssmiId = 'SSGEO\.NPR\.EDRR\.S';
$ssmisId = 'GEO\.EDEB\.S';

$amsreId = 'amsre\.Rain';
$amsreAccend = '\_A';
$amsreAcceLat = -10;		# the cutting latitude for AMSRe Accending pass

# HE US CONUS file name 
$heusId = 'HECOMB';
# HE Global file name 
$heusId_G = 'GHE';

##---------------- Declare some constant variables and arrays ----------------##

# the constants is maximum distant as a good pass, 
# and it is the distant between the orbit and the storm center
# $AMSU_DIST = 700;    # unit is kms
# $AMSU_DIST = 900;      # modified at Wed Jul  9 14:57:31 UTC 2003
# $TRMM_DIST = 600;
# $SSMI_DIST = 650; 
$AMSU_DIST = 900;      # modified at Aug  11 13:02 UTC 2006
# $TRMM_DIST = 300;
$TRMM_DIST = 325;      # modified at Dec 15 15:57:08 UTC 2006
# $SSMI_DIST = 500; 
$SSMI_DIST = 550;      # modified at Aug 15 14:28:13 UTC 2006
$SSMIS_DIST = 700;      
$AMSRE_DIST = 525;

$HE_DIST = 0;	       # don't need to use this for H-E data, so assign it as 0	

# the full scan period of the orbits
$AMSU_SCAN_PERIOD = 2.67;        # in seconds
$TRMM_SCAN_PERIOD = 1.89;
$SSMI_SCAN_PERIOD = 3.81;        # will double chech it, it is different fomr webie side
$SSMIS_SCAN_PERIOD = 3.81;       
$AMSRE_SCAN_PERIOD = 1.5;        # in seconds

# the interval to use single orbit compare to the forecast time of the bulletin
# $interval = 60000;     		 # 6 hours, will be used in the TK Utility function
$interval = 30000;     		 # change to 3 hours, will be used in the TK Utility function, at Aug 15 19:42:03 UTC 2006
$interval_GHE = 41000;     		 # change to 3 hours, will be used in the TK Utility function, at Aug 15 19:42:03 UTC 2006
$insertCovLog = 0;

#-------- H-E data corners --------#

# HECOMB-1HR for the US CONUS
$HEUS_UL_LAT = 61;		# Upper Left corner latitude of the HECOMB US CONUS Image
$HEUS_UL_LON = -164; 		# Upper Left corner longitude
$HEUS_LR_LAT = 13;		# Lower Right corner latitude
$HEUS_LR_LON = -62;		# Lower Right corner longitude

$HEUS_B_LR_LAT = 10;		# Lower Right corner latitude - bigger coverage
$HEUS_B_LR_LON = -60.5;		# Lower Right corner longitude - bigger coverage

$HE_DIS_DEG = 3;		# the interval distance of the storm center to the four corners in degree (3)

# Define the wmolists for the WMO header of the bulletin
# change the WMO header from WT**4* to WT**2* (from discussion to advisory bulletin)
@wmolist1 = qw(
   WTNT21 WTNT22 WTNT23 WTNT24 WTNT25
   WTPZ21 WTPZ22 WTPZ23 WTPZ24 WTPZ25
   WTPA21 WTPA22 WTPA23 WTPA24 WTPA25 
);

## must specify the WMO station (WSTN=PGTW) for the following WMO header
@wmolist2 = qw(
   WTPN31 WTPN32 WTPN33 WTPN34 WTPN35
   WTPS31 WTPS32 WTPS33 WTPS34 WTPS35
   WTXS31 WTXS32 WTXS33 WTXS34 WTXS35
   WTIO31 WTIO32 WTIO33 WTIO34 WTIO35
);

## @wmolist3 to @wmolist6 are RSMC bulletins

## bulletins from Reunion(IO21 or IO31?), station (WSTN=FMEE)
# remove WTIO30 from the list, noted at 10/02/2003

@wmolist3 = qw(
   WTIO20 WTIO21 WTIO22 WTIO23 WTIO24 WTIO25        
);

## bulletins from Australia , station (WSTN=APRF)
# Will confirm the WTAU01-02, 04, and find more bulletin header

##### Replaced by AXAU #####
#@wmolist4 = qw(
#  WTAU01 WTAU02 WTAU03 WTAU04 WTAU05 WTAU06 WTAU07   
#  WOAU01 WOAU02           
#);
##### Replaced by AXAU #####

## Jakarta bulletin, station (WIIX). May need to add more headers later. 02/23/09
@wmolist4 = qw(
WVID10
);


## bulletins from RSMC Tokyo, station (WSTN=RJTD)
## Turn off on 01/31/2008

#@wmolist5 = qw(
#   WTJP21 WTJP22 WTJP23 WTJP24 WTJP25
#);

## bulletins from Fiji, station (WSTN=NFFN)
## Turn off on 01/31/2008

#@wmolist6 = qw(
#   WTPS01 WTPS02 WTPS03 WTPS04 WTPS05
#   WHPS01 WHPS02 WHPS03 WHPS04 WHPS05
#);

## ICAO bulletin, will add FKIO or maybe FKPS, FSPQ if it is necessary

@wmolist7 = qw(
   FKAU01 FKAU02 FKAU03 FKAU04 FKAU05 FKAU06
   FKPS01 FKPS02
   FKPQ30 FKPQ31 FKPQ32 FKPQ33 FKPQ34
   FKIN20
   FKIO20
);

## bulletins from New Delhi

@wmolist8 = qw(
   WTIN20
 );

## bulletins from New Zealand

@wmolist9 = qw(
   WTNZ41 WHNZ41
 );

@wmolist10 = qw(
   AXAU01 AXAU02 AXAU03 AXAU04 AXAU05 AXAU21
 );


# replace "noaa.gov" with "ssdmail2.wwb.noaa.gov", so that the email won't be affect
# by the problem in Silver Spring mail server, it will go into our mail server directory.
# Please aware that if our mail server change, these address need to change.
# Noted at Aug 20th, 2003

$userAddress = "'SABSupervisor\@noaa.gov' 'clay.davenport\@noaa.gov' 'robert.glassberg\@noaa.gov'";


##---------------- initialize the array and hash table ----------------##

@stormNameList = ();

# key is WMO header, and the value is the Storm structure
# The hash table of Storm structure which hold all the active storm information
# and their bulletins information.  Same storm from different bulletins
# will possibly included in the array

%stormStructList = ();

# key is the storm name, and value is the array of the wmo header for the bulletin
# This hash table is used to keep track of multiple bulletins for the same storm.
# If ICAO bulletin FKAU and RSMC bulletin WTAU (WOAU) are available, use ICAO

%sameStormList = ();

# key is the data source name, and the value is the PassData structure
%passDataStructList = ();
%newPassDataStructList = ();
%curPassDataStructList = ();
# Add on 07/19/2005
%newHEPassDataStructList = ();
%curHEPassDataStructList = ();

# Add on 10/10/2006 for the New HE TRaP generation
@allnewHEPassList = ();

# This array is used for reporting the new TRaPs, it is array of "TrapRecord" structure
@TrapReport = ();          

# This array contains a list of the name of all the TRaP sets.  It is used for notifying
# the user.
@TrapSetList = (); 

# Define the data source array
## @sourceArray = qw(AMSU TRMM SSMI);

# AMSRE Single orbit RR dataset name
$amsreDataSet = "AMSRESINRR";

# will turn on the AMSUNM when NOAA-17 get back, turn on at 1422Z 12/08/2003
@PolarSourceArray = qw(AMSUNK AMSUNL AMSUNM AMSUNN AMSUM2 AMSUM1 AMSUNP TRMM SSMIS7 SSMIS8 SSMIS9 SSMISA SSMISB SSMISC);
@AmsreSourceArray = ($amsreDataSet);
@GoesSourceArray = qw(HECOMB-1HR);
@GoesSourceArray_G = qw(GHE-1HR);

# @sourceArray = qw(AMSUNK AMSUNL TRMM SSMIS7 SSMIS8 SSMIS9);

# Define the hash table for the data set name and the corresponding satellite name
%satelliteId1 = qw(
   AMSUNK NOAA-15 AMSUNL NOAA-16 AMSUNM NOAA-17 AMSUNN NOAA-18 AMSUNP NOAA-19 AMSUM2 METOP-A AMSUM1 METOP-B
   TRMM TRMM 
   SSMIS7 DMSP-F13 SSMIS8 DMSP-F14 SSMIS9 DMSP-F15 SSMISA DMSP-F16 SSMISB DMSP-F17 SSMISC DMSP-F18 HECOMB-1HR GOES-10-12
);

%satelliteId = (%satelliteId1, $amsreDataSet, "AQUA");

$hourRange = 24;
$checkICAOhr = 6;      	# hour to check if ICAO bulletin is too old
$checkRSMChr = 4;      	# hour to correspondedly check RSMC after ICAO bulletin too old check

######------------------------------------------------------------------------------######

######--------------------------- Start the program --------------------------------######

$path = $ENV{"PATH"};
$ENV{"PATH"} = $path;

$ENV{"MCPATH"} = "$HOME/mcidas/data:/home/mcidas/data:$DATAHOME/data";

print "\n--------------------- Start the Automated TRaP Program ---------------------\n";

$startdate=`date`;
chomp($startdate);
$jdate=`date +%j`;
chomp($jdate);
print "Start $0 at $startdate $jdate\n"; 

$starthhmm=`date +%H:%M`;
chomp($starthhmm);
$starthhmm = $starthhmm . 'Z';

chdir("$dataDir") || die "Cannot change to directory $dataDir";
print  "Data directory:";
system("pwd");

@comandFields = split(/\//, $0);
$commandName = $comandFields[-1];

$acct=`whoami`;
chomp($acct);

$tmpPsfile = "/tmp/ps-ef_TRaP.out";

system("ps -ef|grep $acct > $tmpPsfile"); 
#$numOfProcess = `ps -ef|grep $acct|grep $commandName|grep "sh -c"|wc -l`;
$numOfProcess = `grep $commandName $tmpPsfile |grep "bin\/sh -c"|wc -l`;
print "Total number of the process for $commandName is $numOfProcess\n";


if ($numOfProcess > 1)
{
   $oldProcess = `grep $commandName $tmpPsfile |grep "sh -c"`;
   print "WARNING:Old main process is still running as following:\n$oldProcess\n";
   print "EXIT the current process for $commandName at $startdate\n";
   exit;
}


####--------------------------------------------------------------------------####

##------------------------ Search all the FOS bulletins ------------------------##

print "%%%%%%%%%%%%% start search wmolist 1 %%%%%%%%%%%%%\n";

foreach $wmoid (@wmolist1)
{
   system("wxtlist.k WMO=$wmoid DTIME=$hourRange DEV=T $wxtFile");
 
   # if the file is bigger than 33 bytes, then it is not "No matches found"

#   if ($wmoid eq "WTNT41")
#   {
#     print "the wmo header is $wmoid\n";
#     `cp -p wxt/other/tpc/wtnt41.txt.bk wxt/other/tpc/wtnt41.txt`;
#     $wxtFile = "wxt/other/tpc/wtnt41.txt"; 
#   }

   if ((-s $wxtFile) > 33)
   {  
      system("$HOME/mcidas/pctrap-V3/getBulData4.pl $wmoid $wxtFile $bulDataFile >> $getBulLog"); 
      $size = -s $bulDataFile;
      print "the size of the bulDataFile for $wmoid is $size\n";
      
      # will read the bulDataFile and store it into the array "stormStructList"
      storeBulInfo($bulDataFile);
   }

# $wxtFile = "wxt.doc";
 
}

print "%%%%%%%%%%%%% start search wmolist 2 %%%%%%%%%%%%%\n";

foreach $wmoid (@wmolist2)
{
   #system("wxtlist.k WMO=$wmoid WSTN=PGTW DTIME=$hourRange DEV=T $wxtFile");

   if ($wmoid =~ /WTPN/)
   {
      system("wxtlist.k WMO=$wmoid WSTN=PGTW MATCH='NORTHWESTPAC' DTIME=$hourRange DEV=T $wxtFile");
   }
   else
   {
      system("wxtlist.k WMO=$wmoid WSTN=PGTW DTIME=$hourRange DEV=T $wxtFile");
   }

   # if the file is bigger than 33 bytes, then it is not "No matches found"

#    if ($wmoid eq "WTPN31")
#    {
#     `cp -p wxt/other/wtpn31.bk wxt/other/wtpn31`;
#     $wxtFile = "wxt/other/wtpn31";
#    }

   if ((-s $wxtFile) > 33)
   {
      system("$HOME/mcidas/pctrap-V3/getBulData4.pl $wmoid $wxtFile $bulDataFile >> $getBulLog");
      $size = -s $bulDataFile;
      print "the size of the bulDataFile for $wmoid is $size\n";

      # will read the bulDataFile and store it into the array "stormStructList"
      storeBulInfo($bulDataFile);
   }

# $wxtFile = "wxt.doc";

}

print "%%%%%%%%%%%%% start search wmolist 3 %%%%%%%%%%%%%\n";

foreach $wmoid (@wmolist3)
{
   # system("wxtlist.k WMO=$wmoid WSTN=FMEE DTIME=$hourRange DEV=T $wxtFile");
   
   # add the MATCH to avoid using the bulletin without line feed , noted at 10/02/2003
   # may switch back if the messy bulletin is fixed
 
   # On 02/04/2004, found that sometimes it is "PAN PAN", but not "SECURITE", so modify it
   #system("wxtlist.k WMO=$wmoid WSTN=FMEE MATCH='SECURITE ' DTIME=$hourRange DEV=T $wxtFile");

   # Avoid to get the bulletin only with PART ONE, 11/16
   #system("wxtlist.k WMO=$wmoid WSTN=FMEE MATCH='(KT) ' DTIME=$hourRange DEV=T $wxtFile");
   system("wxtlist.k WMO=$wmoid WSTN=FMEE MATCH='(KT) ' 'OTHER INFORMATION' DTIME=$hourRange DEV=T $wxtFile");

   # if the file is bigger than 33 bytes, then it is not "No matches found"

   if ((-s $wxtFile) > 33)
   {
      system("$HOME/mcidas/pctrap-V3/getBulData4.pl $wmoid $wxtFile $bulDataFile >> $getBulLog");
      $size = -s $bulDataFile;
      print "the size of the bulDataFile for $wmoid is $size\n";

      # will read the bulDataFile and store it into the array "stormStructList"
      storeBulInfo($bulDataFile);
   }
}   

print "%%%%%%%%%%%%% start search wmolist 4 %%%%%%%%%%%%%\n";

foreach $wmoid (@wmolist4)
{
   # On 01/18/2005, add "MATCH" to avoid the lower case forecast
   # On 01/25/2005, WOAU01 don't have 'HPA'
 print "Run the cmd:wxtlist.k WMO=$wmoid DTIME=$hourRange DEV=T $wxtFile\n";
   system("wxtlist.k WMO=$wmoid DTIME=$hourRange DEV=T $wxtFile");
   # system("wxtlist.k WMO=$wmoid MATCH='HPA' DTIME=$hourRange DEV=T $wxtFile");

   # if the file is bigger than 33 bytes, then it is not "No matches found"

   if ((-s $wxtFile) > 33)
   {
      system("$HOME/mcidas/pctrap-V3/getBulData4.pl $wmoid $wxtFile $bulDataFile >> $getBulLog");
      $size = -s $bulDataFile;
      print "the size of the bulDataFile for $wmoid is $size\n";

      # will read the bulDataFile and store it into the array "stormStructList"
      storeBulInfo($bulDataFile);
   }
}

## Turn off on 01/31/2008
#print "%%%%%%%%%%%%% start search wmolist 5 %%%%%%%%%%%%%\n";

#foreach $wmoid (@wmolist5)
#{
#   system("wxtlist.k WMO=$wmoid DTIME=$hourRange DEV=T $wxtFile");
#
#   # if the file is bigger than 33 bytes, then it is not "No matches found"
#
#   if ((-s $wxtFile) > 33)
#   {
#      system("$HOME/mcidas/pctrap-V3/getBulData4.pl $wmoid $wxtFile $bulDataFile >> $getBulLog");
#      $size = -s $bulDataFile;
#      print "the size of the bulDataFile for $wmoid is $size\n";
#
#      # will read the bulDataFile and store it into the array "stormStructList"
#      storeBulInfo($bulDataFile);
#   }
#}

#print "%%%%%%%%%%%%% start search wmolist 6 %%%%%%%%%%%%%\n";

#foreach $wmoid (@wmolist6)
#{
#   system("wxtlist.k WMO=$wmoid WSTN=NFFN DTIME=$hourRange DEV=T $wxtFile");
#
#   # if the file is bigger than 33 bytes, then it is not "No matches found"
#
#   if ((-s $wxtFile) > 33)
#   {
#      system("$HOME/mcidas/pctrap-V3/getBulData4.pl $wmoid $wxtFile $bulDataFile >> $getBulLog");
#      $size = -s $bulDataFile;
#      print "the size of the bulDataFile for $wmoid is $size\n";
#
#      # will read the bulDataFile and store it into the array "stormStructList"
#      storeBulInfo($bulDataFile);
#   }
#}

print "%%%%%%%%%%%%% start search wmolist 7 %%%%%%%%%%%%%\n";

foreach $wmoid (@wmolist7)
{
   system("wxtlist.k WMO=$wmoid DTIME=$hourRange DEV=T $wxtFile");

   $size = -s $wxtFile;
   print "the size of the wxtFile for $wmoid is $size\n";

   # if the file is bigger than 33 bytes, then it is not "No matches found"

   if ((-s $wxtFile) > 33)
   {
       my $stormslist = `wxtlist.k WMO=$wmoid NUM=10|grep TC:|tr -d ' ' |sort|uniq|cut -d: -f2`;
      # noted on 04/22/04, will change the program to "getBulData4.pl"
       foreach $storm ($stormslist) {
	   system("wxtlist.k WMO=$wmoid MAT=$storm DTI=$hourRange DEV=T $wxtFile");
	   system("$HOME/mcidas/pctrap-V3/getBulData4.pl $wmoid $wxtFile $bulDataFile >> $getBulLog");
	   $size = -s $bulDataFile;
	   print "the size of the bulDataFile for $wmoid is $size\n";

      # will read the bulDataFile and store it into the array "stormStructList"
	   storeBulInfo($bulDataFile);
       }
   }
}

print "%%%%%%%%%%%%% start search wmolist 8 %%%%%%%%%%%%%\n";

foreach $wmoid (@wmolist8)
{
   system("wxtlist.k WMO=$wmoid MATCH='TROPICAL CYCLONE WARNING' DTIME=$hourRange DEV=T $wxtFile");

   # if the file is bigger than 33 bytes, then it is not "No matches found"

   if ((-s $wxtFile) > 33)
   {
      # noted on 04/22/04, will change the program to "getBulData4.pl"
      system("$HOME/mcidas/pctrap-V3/getBulData4.pl $wmoid $wxtFile $bulDataFile >> $getBulLog");
      $size = -s $bulDataFile;
      print "the size of the bulDataFile for $wmoid is $size\n";

      # will read the bulDataFile and store it into the array "stormStructList"
      storeBulInfo($bulDataFile);
   }
}

print "%%%%%%%%%%%%% start search wmolist 9 %%%%%%%%%%%%%\n";

foreach $wmoid (@wmolist9)
{
   system("wxtlist.k WMO=$wmoid  DTIME=$hourRange DEV=T $wxtFile");

   # if the file is bigger than 33 bytes, then it is not "No matches found"

   if ((-s $wxtFile) > 33)
   {
      system("$HOME/mcidas/pctrap-V3/getBulData4.pl $wmoid $wxtFile $bulDataFile >> $getBulLog");
      $size = -s $bulDataFile;
      print "the size of the bulDataFile for $wmoid is $size\n";

      # will read the bulDataFile and store it into the array "stormStructList"
      storeBulInfo($bulDataFile);
   }
}



print "%%%%%%%%%%%%% start search wmolist 10 %%%%%%%%%%%%%\n";

foreach $wmoid (@wmolist10)
{
   system("wxtlist.k WMO=$wmoid  DTIME=$hourRange DEV=T $wxtFile");

   # if the file is bigger than 33 bytes, then it is not "No matches found"

   if ((-s $wxtFile) > 33)
   {
      system("$HOME/mcidas/pctrap-V3/getBulData4.pl $wmoid $wxtFile $bulDataFile >> $getBulLog");
      $size = -s $bulDataFile;

      # will read the bulDataFile and store it into the array "stormStructList"
      storeBulInfo($bulDataFile);
   }
}



## Done with search all the bulletins

# Only uncomment when generating artificial bulletins

print "BulInfo file $bulDataFile\n";

# storeBulInfo($bulDataFile);

print " the storm name list is  @stormNameList \n";

##---------------  Will Exit the Program if NO Storm Activated ----------------##


# if change the stormNameList from hash to array, then use the next commented 'if' 

if (@stormNameList == 0)  
{
   print "NO Storm Activated, EXIT the Program now.\n";
   exit;
}


## compare the wmo headers for the same storm, if they are ICAO and RSMC bulletin for
## Australia area, then delete the RSMC one 
## will do the same for Reunion in the future

foreach $stNm ( sort keys %sameStormList)
{
   print "storm $stNm from bulletins: @{$sameStormList{$stNm}}\n";
   if (@{$sameStormList{$stNm}} > 1)
   {
      $num = @{$sameStormList{$stNm}};
      print "there is $num of bulletins for this storm.\n";

      #---------- Check for the incorrect issued bulletin and remove the dupplicated one ----------#
      # This is for the case "2009PARMA.WTPN33.PGTW.030300 and the duplicated WTPN31.PGTW.030300"

      @stWmoList = @{$sameStormList{$stNm}};
      # remove the sixth character, and only compare the first five characters
      chop (@stWmoList);

      # seen is a hash for an array which includes the index of the same WMO headers (first five characters)
      %seen = ();
      $i=0;
      foreach $item (@stWmoList)
      {
         print "item is $item, i is $i\n";
         push(@{$seen{$item}}, $i);
         if ( @{$seen{$item}} > 1)
         {
            print "Duplicated WMO header with different ID found, it is $dupWmo, the index is @{$seen{$item}}\n";
            $dupWMO1 = ${$sameStormList{$stNm}}[${$seen{$item}}[0]];
            $dupWMO2 = ${$sameStormList{$stNm}}[${$seen{$item}}[1]];
            print "dupWMO1=$dupWMO1, dupWMO2=$dupWMO2\n\n";

            # Check for the positions for these two headers
            print "Call sub getPosFile to check for the positions for these two headers\n";
            getPosFile("/tmp/trapPos1.txt", \$stormStructList{$dupWMO1});
            getPosFile("/tmp/trapPos2.txt", \$stormStructList{$dupWMO2});
            $posString1 = `cat /tmp/trapPos1.txt |head -5`;
            $posString2 = `cat /tmp/trapPos2.txt |head -5`;
            if ($posString1 eq $posString2)
            {
               print "Please NOTES:Positions are the same,";
               print " it is duplicated forecast for the same sation and header with different WMO ID, delete the incorrect one!\n";

               # locate the one which is not issued previously, default is WMO2
               $deletedWMO = $dupWMO2;
               $wmo2TRAP = `$srcDir/SHOWTRAP 1 $stNm $dupWMO2 |grep $stNm`;
               if ($wmo2TRAP ne "")
               {
                  print "There was TRaP created from $dupWMO2 before, deleted the $dupWMO1\n";
                  $deletedWMO = $dupWMO1;
               }
               else
               {
                  $wmo1TRAP = `$srcDir/SHOWTRAP 1 $stNm $dupWMO1 |grep $stNm`;
                  if ($wmo1TRAP eq "")
                  {
                     print "There was no TRaP created from both $dupWMO1 abd $dupWMO2 before, deleted the default one:$dupWMO2\n";
                  }
               }

               delete $stormStructList{$deletedWMO};
            }
            # will delete this part, it is for testing
            else
            {
               print "The Positions are different between two WMO IDs.\n";
               print "For $dupWMO1:\n$posString1\n";
               print "For $dupWMO2::\n$posString2\n";
               print "Compared done.\n";
            } 
         }
         $i++;
      }


      #---------- Check for the Australia ICAO and RMSC bulletin to remove the duplicated one ----------#

      # delete the storm from WTAU/WOAU bulletin if FKAU one is also available

     # $t1 = ${$sameStormList{$stNm}}[0];
     # $t2 = ${$sameStormList{$stNm}}[1];

      $rsmcWmo= "";
      $icaoWmo = "";
      foreach $wmo ( @{$sameStormList{$stNm}} )
      {
         #if ($wmo =~ /WTAU/)
         # {
         # $wmo = WOAU01;
         # }
         # print "the WMO is $wmo.\n\n";
     #    if ($wmo =~ /WTAU|WOAU/)
         if ($wmo =~ /AXAU/)
         {
            $rsmcWmo = $wmo;
         }
         elsif ($wmo =~ /FKAU/)
         {
            $icaoWmo = $wmo;
         }
      }

      if (($rsmcWmo ne "") && ($icaoWmo ne ""))
      {
         print "There are two bulletins $rsmcWmo and $icaoWmo for the storm $stNm around Australia.\n";

         $checkTimeStr = `date --d "$checkICAOhr hours ago" +%Y%j%H%M`;
	 $checkRSMCTimeStr = `date --d "$checkRSMChr hours ago" +%Y%j%H%M`;
	 chomp ($checkTimeStr);
         $bulDdhhmm = $stormStructList{$icaoWmo}->bulTime;
         #$bulHhmm  = substr($bulDdhhmm, 2);
         $ICAObulTimeStr = $stormStructList{$icaoWmo}->bulYJd . substr($stormStructList{$icaoWmo}->bulTime, 2);
         $RSMCbulTimeStr = $stormStructList{$rsmcWmo}->bulYJd . substr($stormStructList{$rsmcWmo}->bulTime, 2);
         print "+++++Check ICAO time String is $checkTimeStr, ICAO Bulletin time string is $ICAObulTimeStr.\n\n"; 
         print "+++++RSMC Bulletin time string is $RSMCbulTimeStr.\n\n"; 
         if ($checkTimeStr < $ICAObulTimeStr)
         {
            print "ICAO bulletin is not older than $checkICAOhr hours, 
                   \rDelete the $rsmcWmo one from the storm structure list.\n\n";
            delete $stormStructList{$rsmcWmo};
         }
         elsif ( $checkRSMCTimeStr > $RSMCbulTimeStr)
         {
            print "RSMC bulletin is also older than $checkICAOhr hours,
                   \rDelete the $rsmcWmo one from the storm structure list.\n\n";
            delete $stormStructList{$rsmcWmo};
         }

      }


      $rsmcWmo= "";
      $icaoWmo = "";
      foreach $wmo ( @{$sameStormList{$stNm}} )
      {
        if ($wmo =~ /WTNZ/)
         {
            $rsmcWmo = $wmo;
         }
         elsif ($wmo =~ /FKPS/)
         {
            $icaoWmo = $wmo;
         }
      }

      if (($rsmcWmo ne "") && ($icaoWmo ne ""))
      {
         print "There are two bulletins $rsmcWmo and $icaoWmo for the storm $stNm around Australia.\n";

         $checkTimeStr = `date --d "$checkICAOhr hours ago" +%Y%j%H%M`;
        $checkRSMCTimeStr = `date --d "$checkRSMChr hours ago" +%Y%j%H%M`;
         chomp ($checkTimeStr);
         $bulDdhhmm = $stormStructList{$icaoWmo}->bulTime;
         #$bulHhmm  = substr($bulDdhhmm, 2);
         $ICAObulTimeStr = $stormStructList{$icaoWmo}->bulYJd . substr($stormStructList{$icaoWmo}->bulTime, 2);
         $RSMCbulTimeStr = $stormStructList{$rsmcWmo}->bulYJd . substr($stormStructList{$rsmcWmo}->bulTime, 2);
         print "+++++Check ICAO time String is $checkTimeStr, ICAO Bulletin time string is $ICAObulTimeStr.\n\n"; 
         print "+++++RSMC Bulletin time string is $RSMCbulTimeStr.\n\n"; 
         if ($checkTimeStr < $ICAObulTimeStr)
         {
            print "ICAO bulletin is not older than $checkICAOhr hours, 
                   \rDelete the $rsmcWmo one from the storm structure list.\n\n";
            delete $stormStructList{$rsmcWmo};
         }
         elsif ( $checkRSMCTimeStr > $RSMCbulTimeStr)
         {
            print "RSMC bulletin is also older than $checkRSMChr hours,
                   \rDelete the $rsmcWmo one from the storm structure list.\n\n";
            delete $stormStructList{$rsmcWmo};
         }

      }

   }

}


####--------------------------------------------------------------------------####

##---------- Get the IMGLIST of AMSUB/TRMM/SSMI singel orbit dataset -----------##
##----------                           And H-E                        ----------##
##----- these lists will be used to get the pos number for specific storm ------##

# Noted at 10/24/03, work up to here, will use the following imglist files around line 745, to get the 
# pos number of the dataset, then will modify the saveDirFile subroutine to use the orbit pass time,
# then modify the getPos subroutine to add the pass time, then modify the pctrpauto,pgm. 

foreach $name (@PolarSourceArray)
{
   system ("imglist.k PLR/$name.-10 DEV=T $name.imgl");
}

#foreach $name (@AmsreSourceArray)
#{
   #system ("imglist.k AMSRE2/$name.-10 DEV=T $name.imgl");
#   system ("imglist.k AMSRE/$name.-20 DEV=T $name.imgl");
#}

foreach $name (@GoesSourceArray)
{
   #system ("imglist.k DPD/$name.-10 DEV=T $name.imgl");
   system ("imglist.k DPD/$name.-40 DEV=T $name.imgl");
}

foreach $name_G (@GoesSourceArray_G)
{
   #system ("imglist.k DPD/$name.-10 DEV=T $name.imgl");
   system ("imglist.k DPD/$name_G.-40 DEV=T $name_G.imgl");
#   print "in loop name_G $name_G.imgl\n";
}

####--------------------------------------------------------------------------####


##---------------- Get the new AMSUB/TRMM/SSMI/SSMIS pass data lists -----------------##
##---------------- these lists will be used if bulletin is old -----------------##

# if there is no new pass available for three data source, the subroutine return 0
print "Get the new available AMSU/TRMM/SSMI/SSMIS pass data lists.\n";
`$fetchPassCM`;    #bring this out from the subroutine to show work in log
$isNewPassExist = getNewPassLists();

print " isNewPassExist is $isNewPassExist.\n";


if ( -s $newPassLog )
{

   # ----------------------------------------------
   # put the file into the donelist, since whenever the file is put into
   # the newList, the composite data or the pass data will be used in the
   # calling program (pctrap.pl).  These new pass datas all result in
   # being processed in both cases.
   # please note that: this might be changed in the coming enhancement
   # which checks whether the pass is within 6 hour of the FOS bulletin or not.
   # ----------------------------------------------

   print "Append the newPassLog file to the pass processed DataLog file,
          \rthen remove the new pass log file.\n";
   `cat $newPassLog >> $passDataLog`;
   `cp -p $newPassLog newPasslog-sv`;
   unlink $newPassLog;
}


##--- Get the current AMSUB/TRMM/SSMI/SSMIS pass data lists not older than 10 hours (modify time) ---##
##---------------- these lists will be used if bulletin is new -----------------## 

print "Get the available AMSU/TRMM/SSMI/AMSRE/SSMIS pass data lists.\n";
getCurrentPassLists();



# ##-------------------   Get the H-E list, put this inside the while loop as it depends on the forecast time   -------------------##
# ##---------------- these lists will be used no matter bulletin is new or old -----------------##

# print "Get the available H-E pass data lists.\n";
# $isNewHEPassExist = getHEPassLists();


# if ($isNewHEPassExist == 1)
# {
#   $isNewPassExist = 1;
# }

print "before chdir\n";
print  "Data directory:";
system("pwd");

chdir("$dataDir") || die "Cannot change to dataDir $dataDir";

print "after chdir\n";
print  "Data directory:";
system("pwd");


####--------------------------------------------------------------------------####

##---------------- Kick off the TRaPs which meets the criteria -----------------##

print "%%%%%%%%%%%%% the storm struct %%%%%%%%%%%%%%%%%\n";

dbmopen(%lastBulAcs, "bulAcsDB", 0644) || die "can't dbmopen bulAcsDB: $!";

$numberOfTrap = 0;

while (($wmoHd, $st) = each(%stormStructList))
{
   printf "**********\nThe storm name is %s, the nameNNB is %s,
          \rthe initial time is %s, the lat is %s, the lon is %s.\n",
          $st->name, $st->nameNNB, $st->iniTime, $st->lat_0, $st->lon_0;

   # get the following structure members's value, so that they can be used in the system call
   $stname = $st->name;
   $wmoheader = $st->wmoHd;
   $station = $st->stationId;

   # bulInfo store the WMO header, id and time of the bulletine
   # and it is unique for every bulletine
   $bulInfo = $st->wmoHd . " " . $st->stationId . " " .
              $st->bulTime . " " . $st->bulYJd;
   print "The bulletin information is ***$bulInfo***.\n";

   #------------- Check positions for duplicated forecast -------------#

   # This is for the same WMo header, but not for different header, can be compare the first five characters of the header
   #  Will do that later. may put in where the the ICAO for FKAU and RMSC. 11/03
   # First test if the passing $st struct as argument to subroutine "getPosFile" work, if so, pass the $st{$wom1}, $st{$wmo2} 
   # to get the position files for comparison in the ICAO section (sameStormList) 

   $preBulInfo = $lastBulAcs{$st->wmoHd};

   if (($preBulInfo ne "") && ($preBulInfo ne $bulInfo))
   {
       print "The new bulletin information is ***$bulInfo***.\n";
      print "The previous bulletin information is ***$preBulInfo***.\n";
 
      # check the bulTime difference
      @splitTmMsg = split(/\s+/, $preBulInfo);
      $preBulYJd = $splitTmMsg[-1];
      $preBulTime = $splitTmMsg[-2];
      $preSationId = $splitTmMsg[-3];

      if ($preSationId ne $station)
      {
         print "WARNING:WMO header is the same. but stationID is different!\n";
      }
      if ($preBulYJd == $st->bulYJd)
      {
         # Can not get the accurate difference when it is over two days, such as: 162100 and 170300
         # but hopefully all the correction is issued within 3 hours

         $diffBulTime = abs($st->bulTime - $preBulTime);
         print "The difference of the previous bulTime and the current one is $diffBulTime\n";
         if ($diffBulTime < 500 )
         {
            print "The difference is less than 5 hours, Compare the positions!\n";
            # Locate the previous position file
            $curYear = `date +%Y`;
            chomp($curYear);
            $yesterdayYear = `date --d yesterday +%Y`;
            chomp($yesterdayYear);
            $preStDir = "$trapDataDir/$curYear/$stname";
            print "previous storm Dir is $preStDir\n";

            if (!(-d $preStDir))
            {
               print "Check for yesterday's year\n";
               $preStDir = "$trapDataDir/$yesterdayYear/$stname";
            }
            if (-d $preStDir)
            {
               $prePosFile = `ls -tr $preStDir/*$stname*$wmoheader*$preBulTime*txt.Z|tail -1`;
               if ($prePosFile eq "")
               {
                  print "WARNING:Can not locate the previous position file!\n";
               }
               else
               {
                  chomp($prePosFile);   
                  $prePosString = `zcat $prePosFile |head -5`;
                  # may change the subroutine to getPosFile
                  #getPosFile("/tmp/trapPos.txt");
                  getPosFile("/tmp/trapPos.txt", \$st);
                  $curPosString = `cat /tmp/trapPos.txt |head -5`;

                  if ($prePosString eq $curPosString)
                  {
                     print "Please NOTES:Positions are the same, it is duplicated forecast!\n";   
                     $lastBulAcs{$st->wmoHd} = $bulInfo;
                  }
                  # will delete this part, it is for testing
                  else
                  {
                     print "The Positions are different between previous bulletin and current.\n";
                     print "previou:\n$prePosString\n";
                     print "Current:\n$curPosString\n";
                     print "Compared done.\n";
                  }
               }
            }
            else
            {
               print "WARNING:directory $preStDir is not exist!\n";
            }
                         
         }
      }      

   }
   

   # get the postion track file from the storm structure for the pctrp to read
   # Modified at 04/30/03, To generate the TPC digital TRaP, some bulletin and storm
   # information will also append into the end of the position track, for the program to 
   # to archive for digital TRaP, this won't affect the generation of AREA TRaP 
   # getPosFile($posFile);       # turned off on 10/22/2003, will DELETE this part

   ##------------------------------------------------------------
   # create the directory name and part of the file name for TRaP
   $year=`date +%Y`;
   chomp($year);
   $fileNamePre = $year . $st->nameNNB . "." . $st->wmoHd . "." . $st->stationId
                   . "." . $st->bulTime . ".";

   ##------------------------------------------------------------------------------
   # create the string which will be used to change the MEMO of the TRaP AREA files 
   print "nameNNB $st->nameNNB\n";
   @memoName = split(/_/, $st->nameNNB);

   print "memoName @memoName\n";
   print "memoName[0] $memoName[0]\n";
   
   $memoPre = $memoName[0] . "." .$st->wmoHd . $st->stationId . $st->bulTime;
   print "memoPre $memoPre\n";

   # get the bulletin again for saving it into the TRaP set as a record
   system("wxtlist.k WMO=$wmoheader WSTN=$station DTIME=$hourRange DEV=T $bulFile");

# if ($wmoheader eq "WTNT41")
# {
 #  $bulFile = "wxt/other/tpc/wtnt41.txt";
# }

   # force the array empty for every storm
   @dataSourceUsed = ();


   ##--------------------------------------------------------------------------------------------------##
   ##-- Get the H-E list, put this inside the while loop as it depends on the forecast time of storm --##
##   ##---------------- these lists will be used no matter bulletin is new or old -----------------##

   print "Get the available H-E pass data lists.\n";
   $isNewHEPassExist = getHEPassLists($st->bulYJd, $st->forecastHour);
   print "isNewHEPassExist $isNewHEPassExist\n";

   ##-------------------------------------------------------------------------##
   ##-- If the active storm have a new bulletin, check if current passes cover it --##
   ## Use the current pass list ##

   if (($lastBulAcs{$st->wmoHd} eq "") || ($lastBulAcs{$st->wmoHd} ne $bulInfo))
   {
      $lastBulAcs{$st->wmoHd} = $bulInfo;
      print "New bulletin comes in, Checke current passes for storm $stname.\n\n";
   
      # %passDataStructList = %curPassDataStructList;
      %passDataStructList = (%curPassDataStructList, %curHEPassDataStructList);
  
      # @AmsubPassList = @curAmsubPassList;
      # @TrmmPassList = @curTrmmPassList;
      # @SsmiPassList = @curSsmiPassList;
   }

   ##--------------------------------------------------------------------------##
   ##-- If the storm did not have a new bulletin, check if new pass cover it --##
   ## Use the new pass list ##

   else
   { 
      if (($isNewPassExist == 0) && ($isNewHEPassExist == 0))
      {
         print "There is no new pass available now, go to next storm if there is any.\n";
         next;  # skip the remaining 
      }
      print "No new bulletin, Check new passes for storm $stname.\n\n";

      # %passDataStructList = %newPassDataStructList;
      %passDataStructList = (%newPassDataStructList, %newHEPassDataStructList);

## turn off the following new list at 15Z 12/08/2006

#printf "\n##--------------------------------------------------------------------------##\n";
#printf "print the HE new list:\n"; 
#while (($sourceName, $passData5) = each(%newHEPassDataStructList))
#{
#   printf "the source name is %s, the max distance is %d\n", $sourceName, $passData5->maxDist;
#
#   foreach $filename (@{$passData5->passList})
#   {
#      printf "HE filename is $filename\n";
#   }
#
#}

#printf "\n##--------------------------------------------------------------------------##\n";
#printf "print the Polar new list:\n";
#while (($sourceName, $passData6) = each(%newPassDataStructList))
#{
#   printf "the source name is %s, the max distance is %d\n", $sourceName, $passData6->maxDist;
#
#   foreach $filename (@{$passData6->passList})
#   {
#      printf "Polar filename is $filename\n";
#   }
#
#}

printf "\n##-----------------------------End printing New List---------------------------------------------##\n";


     # @AmsubPassList = @newAmsubPassList;
     # @TrmmPassList = @newTrmmPassList;
     # @SsmiPassList = @newSsmiPassList;
   }


   ##-------------  check the passes of AMSUB/TRMM/SSMI/AMSRE -------------##
   ##-------------  from the hash table: %passDataStructList  -------------##
   print "sourcenames? before while through structure %passDataStructList\n";
   while (($sourceName, $passData) = each(%passDataStructList))
   {
      printf "the source name is %s, the max distance is %d, the center storm lat is %s, 
              \rthe lon is %s \n", $sourceName, $passData->maxDist, $st->lat_0, $st->lon_0; 

      # if the latutide is out of 35N - 35S, don't use TRMM data
      if ($sourceName eq "TRMM")
      {
         # $dataSetName = "TRMM";
         $tempLat = $st->lat_0;

         # Please noted that there is no position in the end of lat_0 any more in Version 2.0
         # chop($tempLat);   # chop the 'N' or 'S' at the end
 
         # if the latutide is out of 35N - 35S, then skip checking TRMM pass data
         if ($tempLat > 35)
         {
            print "The latutide of the storm center is OUT of 35N - 35S,
                   \rSkip checking TRMM pass data.\n";
            next;
         }
      }
#      print "Loop list @{$passData->passList} \n";

      foreach $filename (@{$passData->passList})
      {
         # call the subroutine to determine/evaluate whether the pass covers the storm
    
         # Please Noted that the "$st->lon_0" is Micdas convention 
         # which is reverse to regular convention, so reverse it here  

         # for H-E data
	 print "Covered or not covered, the pass filename is $filename\n";
         if ($filename =~ /^GHE/)
         {
            # $isCover = evaluateHEPassCover($filename,  $st->lat_0, (-1)*($st->lon_0) );
            $isCover = evaluateHEPassCover($filename,  $st->lat_0, (-1)*($st->lon_0),
                                           $st->bulYJd, $st->iniTime, $st->forecastHour);
	    print "the HE is Cover is $isCover\n";
         }
         # For Polar data
         else
         {
            $isCover = evaluatePassCover($filename, $passData->maxDist, $st->lat_0, (-1)*($st->lon_0), 
                                      $st->bulYJd, $st->iniTime, $st->forecastHour);
	    print "the Polar isCover is $isCover\n";
	    $insertCovLog = 1;
         }     

         if ($isCover == 1)
         {
            print "There is new $sourceName pass cover storm $stname.\n";
            print "The pass filename is $filename\n";
            print "The index of scan line when pass is over the storm is $shortestInd\n";

            # get the single orbit dataset name for this pass 
            # (TRMM, AMSUNK, AMSUNL, AMSUNM, AMSUNN, AMSUM2, AMSUM1, SSMIS7, SSMIS8, SSMIS9, SSMISA, SSMISB, SSMISC) 
            if ($sourceName eq "TRMM")
            {  
               $dataSetName = "TRMM";
            }
            elsif ($sourceName eq "AMSU")
            {
               $dataSetName = "AMSU" . substr($filename, -2, 2);
	       print "filename xxx$filename xxx\n";
	       print "extended datasetname $dataSetName\n";
            }
            elsif ($sourceName eq "SSMI") 
            {
               # $dataSetName = "SSMI" . substr($filename, 11, 2);
               $dataSetName = "SSMI" . substr($filename, 15, 2);
            }

            elsif ($sourceName eq "SSMIS") 
            {
               # $dataSetName = "SSMI" . substr($filename, 11, 2);
               $dataSetName = "SSMIS" . substr($filename, 10, 1);
	       print "filename xxx$filename xxx\n";
	       print "extended datasetname $dataSetName\n";
            }

            elsif ($sourceName eq "AMSRE")
            {
               $dataSetName = $amsreDataSet;
            }

 
           # For HE data, source name is same as the dataset name
           elsif ($sourceName =~ /^GHE/)
           {
              $dataSetName = $sourceName;
	      $posNumber = $posNumber_G
           }


   # get the postion track file from the storm structure for the pctrp to read
   # Modified at 04/30/03, To generate the TPC digital TRaP, some bulletin and storm
   # information will also append into the end of the position track, for the program to 
   # to archive for digital TRaP, this won't affect the generation of AREA TRaP 
   # And write the pass time into this file (10/23/2003)
   
            getPosFile($posFile, \$st);


            #-------- get the pos number from the IMGLIST for Polar data --------# 

	    system("pwd");
	    print "DataSetName  $dataSetName.imgl\n";
 	    print "filename at check $filename\n";

	    if ($filename !~ /^GHE/)
            {
               $imgTime = $orbitSt_yyddd . "  " . $orbitSt_hh . ':' . $orbitSt_mm; 
               $tempMsg = `grep '$imgTime' $dataSetName.imgl`;
               @splitMsg = split(/\s+/, $tempMsg);
               $posNumber = $splitMsg[1];

               if ($posNumber eq "")
               {
                  print "Warning: Could not find the corresponding pos number in the Imglist, check next pass. ";

                  print "Append the Imglist to the log file $dataSetName.imgl.\n";
                  print `cat $dataSetName.imgl`;

                  # delete the entry of this gelocation file name from the pass log file
                  # so that it will be processed next time

                  print "Delete the entry of this gelocation file name from the pass log file.\n";
                  system("sed 's/$filename//' $passDataLog > temp.txt");
                  system("cp -p $passDataLog $passDataLog.old");
                  system("mv temp.txt $passDataLog");

                  print "Check next pass.\n\n";
                  next;
               }
            }


            # get the pass yyyddd and hhmmss before pass to the pctrpCM
            $passedStorm_yyyddd = $passedStorm_yyyyddd - 1900000;
            $passedStorm_hhmmss = $passedStorm_hh . $passedStorm_mm . $passedStorm_ss;
	    print "\n##the passedStorm_yyyddd is $passedStorm_yyyddd, passedStorm_hhmmss is $passedStorm_hhmmss ##\n";
	    print "posNumber $posNumber\n";
            system ("$pctrpCM $stname $dataSetName $posNumber $posFile $passedStorm_yyyddd $passedStorm_hhmmss");
#	    system("cp -p pcout.txt pcout${stname}_${passedStorm_yyyddd}_${passedStorm_hhmmss}.txt");
           # system ("$pctrpCM $stname $dataSetName $posNumber $posFile");
           #  system ("$pctrpCM $stname $sourceName '$st->wmoHd'");
	    print "sourcename before savedirfile call $sourceName\n";
	    print "dataSetName before savedirfile call xx $dataSetName xx\n";
#	    print "satellite ID before savedirfile call $satelliteId1\n";
	    print "satellite ID with Dataset before savedirfile call $satelliteId1{${dataSetName}}\n";

            #$setName = saveDirFile($year, $st->nameNNB, $fileNamePre, $sourceName, $memoPre, $bulFile);

            # write the orbit file name and the satellite ID to the text file in the subroutine

            $setName = saveDirFile($year, $st->nameNNB, $fileNamePre, $sourceName, $memoPre, $bulFile, $filename, $satelliteId1{${dataSetName}},$dataSetName);

            if ($setName ne "")
            {
               push(@TrapSetList, $setName );
               # change the value of the array from dataSet name to satellite id.  11/25/2003
               # push(@dataSourceUsed, $dataSetName);
               push(@dataSourceUsed, $satelliteId1{$dataSetName});
               $numberOfTrap++;
               print "TraP is created - 2.\n";

               #-------------------------------------------------------------------------#
          
               #-------- generate digital TRaP and write the TRaP and email list --------#

               # Only for the NON-HE TRaP
               # Those "low", "disturbance", "ex-" and some wrong name with "." are not sent

               print "\nThis is running as back up, so it won't send convert to digital TRaP or send to SAB Home page.\n\n";

               # add "!~ /^\d{3,}$/" on 12/12/2007 to avoid the reunion bulletin for all digits
               # "PHENOMENON:        ZONE OF DISTURBED WEATHER 4 (0420072008) 1000 HPA"

               #if ($sourceName !~ /^HE/)
               if ( ($sourceName !~ /^HE/) 
		    && ($stname !~ /^LOW$/) && ($stname !~ /\_LOW$/)
                    && ($stname !~ /^DISTURB/)
                    && ($stname !~ /^\d{3,}$/)
                    && ($stname !~ /^EX[\-,\_]/i)
                    && ($stname !~ /^EXTRA[\-,\_]/i)
                    && ($stname !~ /^SUB[\-,\_]/i)
                    && ($stname !~ /\./) 
                    && ($stname !~ /^TD/i) ) # For FKPQ/ICAO bulletin, unamed system call TD, on 07052007
               {
                  $dt = `date`;
                  print "\n##----------- Start to call program $area2digitCM at $dt for the run of $starthhmm-----------##\n";
                  system ("$area2digitCM $fullPosFile");
                  $dt = `date`;
                  print "\n##----------- End to call program $area2digitCM at $dt for the run of $starthhmm-----------##\n\n";
                  #sleep(2);
               }
               else
               {
                  print "\nWARNING!!!: This TRaP '$setName' is not converted to digital TRaP or sent to Auto TRaP pages\n";
                  print "Know that 'HE', 'LOW', 'DISTURBABCe' and 'EX-' TRaP are not sent to Auto page.\n\n";
               }

            }
            else
            {
               print "Warning:no set name return, TRaP might not generate.\n";
            }

## The following is for the AMSRE rain rate in the future, 07/26/05
            # if ($sourceName eq "AMSRE")
            # {
            #    last;  # exit the foreach loop, as we just need one TRaP from the AMSRE composite
            # }  

            # delete the GIF saved in the data directory
            system ("rm $dataDir/TRAP??.GIF");

         }  # End of the if "isCover"
 
      }     # End of the foreach loop
   }        # End of the while loop for pass lists


   if (@dataSourceUsed != 0)
   {
      # Extract the unique elements from "dataSourceUsed" list
      %seen = ();
      @uniq = ();
      foreach $item (@dataSourceUsed)
      {
         push(@uniq, $item) unless $seen{$item}++;
      }
   
      my $trapData1 = TrapRecord->new();
      $trapData1->wmoHd($wmoheader);
      $trapData1->stormName($stname);
      ## @{$trapData1->sourceList} = @dataSourceUsed;
      @{$trapData1->sourceList} = @uniq;
##      $trapData1->sourceList($stname);
##      print "trapData1 $trapData1->stormName($stname)\n"; 
##      print "trapData1 $trapData1->sourceList($stname)\n"; 
      push(@TrapReport, $trapData1)
   } 

}    
##-------------- End of the Big WHILE loop for all active storms --------------##


dbmclose(%lastBulAcs) || die "can't dbmclose bulAcsDB: $!";

rename("do-sv", "distout");
@do_stats = stat( "distout" );
my($dosec, $domin, $dohour, $doday ) = gmtime ( $do_stats[9] );
#system("cp -p distout distout_${doday}_${dohour}_${domin}");

if ($insertCovLog > 0)
{
    print " \n";
    print "DISTOUT inserted at this point\n";
    print " \n";
    system("cat distout_${doday}_${dohour}_${domin} >> /data/Petrap/log/autoTRAP.log");
    print " \n";
    print "DISTOUT insert complete\n";
    print " \n";
}

# delete those imglist files
foreach $name (@PolarSourceArray, @AmsreSourceArray)
{
   unlink("$name.imgl");
}

foreach $name (@GoesSourceArray)
{
   unlink("$name.imgl");
}
foreach $name_G (@GoesSourceArray_G)
{
   unlink("$name_G.imgl");
}


##---------------------------- Append the HE pass log ----------------------------##

#  10/06/2006 the following will move to outside of the subroutine, after the while loop? so that all the pass will add to the log file later

#print "AllNewHE @allnewHEPassList\n";

if (@allnewHEPassList != 0)
{
   # Extract the unique elements from "dataSourceUsed" list
   %seen = ();
   @uniq = ();
   foreach $item (@allnewHEPassList)
   {
#       print "Item in list $item\n";
      push(@uniq, $item) unless $seen{$item}++;
   }

   print "Append the new H-E list to the H-E log $HEPassDataLog\n\n";

   open (HELOGFILE, ">>$HEPassDataLog") || die " Can't seem to open $HEPassDataLog:$!";

   foreach $HEfile (@uniq)
   {
      print HELOGFILE "$HEfile\n";
   }

   close (HELOGFILE);

}


####---------------------------------------------------------------------------####

##--------- Send email to notify user if there is any new TRaP created ----------##

# send email to notify user there are new TRaP/Storm
# if there is no element in array newTrap, the index of last element is -1

if ($numberOfTrap > 0)            
{
   print "Send email to report the new TRaPs.\n";

   open MAIL,"|mail -s 'Warning--New TRaP created on geotest' $userAddress";

#   print MAIL "------------ This is just a test for Single orbit TraP ------------\n\n"; 

   print MAIL "Warning: There are $numberOfTrap sets of new TRaP just created on geotest.\n";
   print MAIL "The following are the new TraP:\n";
  
  
   # Modified at 04/26/03, change the following TrapReport from hash to array,
   #  change the stormStrucList from array to hash, key as WMO header
   # so that it is easy to get the coresponding lat and lon. 

### will replace the WMO header with the issuing agent 07/19/03
 
  # while (($stname, $dataSources) = each(%TrapReport))
   foreach $trapRec (@TrapReport)
   {
      $stname = $trapRec->stormName;
      $wmoHd = $trapRec->wmoHd;
      $lat = $stormStructList{$wmoHd}->lat_0;
      $lon = $stormStructList{$wmoHd}->lon_0;
      $stationId = $stormStructList{$wmoHd}->stationId;
      print "trapRec $trapRec\n";
      print MAIL "TRaPs for storm $stname from @{ $trapRec->sourceList } are created, bulletin issuing agency is $stationId.\n";
      print "Warning: TRaPs of $stname from @{$trapRec->sourceList} are created, center is $lat, $lon.\n";
   }   

   print MAIL "\nYou can cut and paste the following comands into the McIDAS window one at a time,";
   print MAIL " before you view and send the TRaP images to website.\n";
   print MAIL "\n*************** Commands ******************\n";
   print "*************** Commands ******************\n";
   foreach $trapSet (@TrapSetList)
   {
      print MAIL "OS \"restoreAREA.pl $trapSet\n";
      print "OS \"restoreAREA.pl $trapSet\n";
   }

   print MAIL "\n%%%%%%%%%%%%%%% Another way  %%%%%%%%%%%%%%%%\n";
   print MAIL "(If the remote session of GP22 is not in the same machine received email)\n";
   print MAIL "\nYou can use 'SHOWTRAP' command to list the recent/ALL TRaP with/without storm name specified.\n";
   print MAIL "Usage: SHOWTRAP [<number to list>] [<pattern1>] [<pattern2>] [<pattern3>]\n";
   print MAIL "       <number to list> -- (optional) the most recent number of TRaP sets to list\n";
   print MAIL "                           default is 10, ALL for list all.\n";
   print MAIL "       <pattern> -- (optional) the pattern to match the TRaP set name (can be storm name)\n";
   print MAIL "In McIDAS window of GP22, type one of the follwing commands.\n";
   print MAIL "Command Format: OS \"SHOWTRAP\n";
   print MAIL "                OS \"SHOWTRAP \$number/ALL\n";
   print MAIL "                OS \"SHOWTRAP \$number \$pattern\n";
   print MAIL "                OS \"SHOWTRAP ALL \$pattern\n";


   close MAIL;

   open (LOGFILE, ">>$trapSetLog") || die " Can't seem to open $trapSetLog:$!";
#   open (LISTFILE, ">>$trapList") || die " Can't seem to open $trapList:$!";

   foreach $trapSet (@TrapSetList)
   {
      print LOGFILE "OS \"restoreAREA.pl $trapSet\n";
#      print LISTFILE  $trapSet\n";
   }

   close (LOGFILE);  
#   close (LISTFILE);
}
else 
{
   print "***** No new TRaP is created  *****.\n";
   print "No email sent out.\n";
}

$dt = `date`;
chomp($dt);
print "\nEnd of the PERL program at $dt for the run of $starthhmm.\n";



##########%%%%%%%%%%%%%%%%%%%%%%%% SUBROUTINES %%%%%%%%%%%%%%%%%%%%%%%%##########


############  Subroutine "storeBulInfo" ###############
########################################################

sub storeBulInfo
{
   my($bulDataFile) = @_;
   my($i, $j);

   if (!(-s $bulDataFile))
   {
      print "The bulletine information file $bulDataFile is not existed.\n";
      return;
   }   
   elsif ((`cat $bulDataFile|wc -l`) < 18)
   {
      my($num) = `cat $bulDataFile|wc -l`;
      print "The bulletine information file $bulDataFile is not completed,
              and the number of lines in the file is $num.\n";
      return;
   }

   open(BULINFO, "$bulDataFile") || die "Can not open $bulDataFile for reading: $!";
   print "read the Bulinfo file into the storm structure\n";
   @infoLines = <BULINFO>;
   close(BULINFO) || die "Can not close $bulDataFile: $!";

   # remove the bulletin data file after it is read.  
   # There will be one come out for another bulletin if there is any
   unlink $bulDataFile;

   # remove all the newline in the end of the elments
   for($i = 0; $i < 18; $i++)
   {
      chomp($infoLines[$i]);
   }

   my $storm1 = Storm->new();

   $j = 0;
   $storm1->name($infoLines[$j++]);
   $storm1->nameNNB($infoLines[$j++]);
   $storm1->stNumber($infoLines[$j++]);
   $storm1->wmoHd($infoLines[$j++]);
   $storm1->stationId($infoLines[$j++]);
   $storm1->bulTime($infoLines[$j++]);
   $storm1->bulYJd($infoLines[$j++]);
   $storm1->bulMmdd($infoLines[$j++]);
   $storm1->iniTime($infoLines[$j++]);
   $storm1->forecastHour($infoLines[$j++]);
   
   # the next line maybe is basin if the bulletin is from TPC/CPHC,
   # otherwise is the empty line
#   if ($infoLines[$j++] =~ /^[A-Z]{2}/)
#   {
#      $basin = $infoLines[$j];
#   }
#   else
#   {
#      $basin = "";
#   }

   $basin = $infoLines[$j++];
   print "^^^^!!! basin is **$basin**\n";   
   for($i = 0; $i <= 24; $i += 6)
   {
      $latName = "lat_" . $i;
      $lonName = "lon_" . $i;
      $storm1->$latName($infoLines[$j++]);
      $storm1->$lonName($infoLines[$j++]);
   }

   printf "############This is in storeBulInfo: the storm name is %s, wmoHd is  %s, lat_0 is %f, lon_0 is %s\n",
        $storm1->name, $storm1->wmoHd, $storm1->lat_0, $storm1->lon_0;

   push(@stormNameList, $storm1->name);
   push(@{$sameStormList{$storm1->name}},$storm1->wmoHd);
   $stormStructList{$storm1->wmoHd} = $storm1;
  ## push(@stormStructList, $storm1);

}   ## End of the Subroutine "storeBulInfo"




##############  Subroutine "getPosFile" ################
# get the postion track file from the storm structure for the pctrp to read
#
# Modified at 04/30/03, To generate the TPC digital TRaP, some bulletin and storm
# information will also append into the end of the position track, for the program to
# to archive for digital TRaP, this won't affect the generation of AREA TRaP
########################################################

sub getPosFile
{
   my($posFile,$stStruct) = @_;
   my($i);

   open(POSTRACK, "> $posFile") || die "Can not open $posFile for writing: $!";
   print "write the postion track file from the storm structure\n";

   for($i = 0; $i <= 24; $i += 6)
   {
      $latName = "lat_" . $i;
      $lonName = "lon_" . $i;
      printf POSTRACK "%2d%8.2f%8.2f\n", $i, $$stStruct->$latName, $$stStruct->$lonName;
   }

   #printf POSTRACK "%s %s %s %s %s %s %s %s\n", $st->name, $st->stNumber, $st->bulYJd,
   #                 $st->wmoHd, $st->stationId, $st->bulTime, $st->bulMmdd, $st->forecastHour;
   printf POSTRACK "%s %s %s %s %s %s %s %s\n", $$stStruct->name, $$stStruct->stNumber, 
                    $$stStruct->bulYJd, $$stStruct->wmoHd, $$stStruct->stationId, $$stStruct->bulTime,
                    $$stStruct->bulMmdd, $$stStruct->forecastHour;

   close(POSTRACK) || die "Can not close $posFile: $!";

}  ## End of the Subroutine "getPosFile" 



############## Subroutine "compareTime" ################
########################################################

sub compareTime
{
   my($time1, $time2) = @_;
   ($day1, $hour1, $min1) = unpack("A2 A2 A2", $time1);
   ($day2, $hour2, $min2) = unpack("A2 A2 A2", $time2);
   local($later) = 0;
 
   if ($day2 == $day1)
   {
      print "the day is same.\n";
      if ($hour2 == $hour1)
      {
         print "the hour is same.\n";
         if ($min2 > $min1)
         {
             print "use min2.\n";
             $later = 1;
         }
      } elsif ($hour2 > $hour1)
      {
         print "use hour2.\n";
         $later = 1;
      }
   }
   # if day2 is the end of last month, and day1 is the begin of this month
   # then $day2-$day1 is greater than 20, and day2 is not the later
   # use 20, since the wxtlist default DTIME is 96 hour (4 days)
   # now the time range is set to 24 hour
   elsif (($day2 > $day1) && (($day2 - $day1) < 20))
   {
      print "use day2  -- 1.\n";
      $later = 1;
   }
   # if day1 is the end of last month, and day2 is the begin of this month
   # then $day1-$day2 is greater than 20, day2 is the later, then use day2
   elsif (($day1 > $day2) && (($day1 - $day2) > 20))
   {
      print "use day2  -- 2.\n";
      $later = 1;
   }
   
   return $later;

}  ## End of the Subroutine "compareTime"


############## Subroutine "getNewPassLists" ################
############################################################

sub getNewPassLists 
{
   my @newAmsubPassList = ();
   my @newTrmmPassList = ();
   my @newSsmiPassList = ();
   my @newSsmisPassList = ();
   my @newAmsrePassList = ();

   #`$fetchPassCM`;
   chdir("$passDataDir") || die "Cannot change to pass data Dir $passDataDir: $!";
   print  "Pass Data directory:";
   system("pwd");

   if ( -s $newPassLog )
   {
      print "Read the new pass data log file. \n";

      open  ( NEWLIST,  "$newPassLog" ) || die " Can not open $newPassLog: $!";

      while ( defined( $filename = <NEWLIST> ))
      {
         chomp($filename);
         
         if ($filename =~ /^$amsubId/)
         {
            push(@newAmsubPassList, $filename);
         }
         elsif ($filename =~ /^$trmmId/) 
         {
            push(@newTrmmPassList, $filename);
         }
         elsif ($filename =~ /^$ssmiId/)
         {
            push(@newSsmiPassList, $filename);
	 }
         elsif ($filename =~ /^$ssmisId/)
         {
            push(@newSsmisPassList, $filename);
         }
         elsif ($filename =~ /^$amsreId/)
         { 
            push(@newAmsrePassList, $filename);
         }
      }
    
      $isNewPassExist = 1;
   }
   else
   {
      print "There is no new pass available right now.\n";
      $isNewPassExist = 0;
   }

   ##------------ Create a hash table of structure "PassData" --------------##

   # create AMSU pass struct
   my $passData1 = PassData->new();
   $passData1->maxDist($AMSU_DIST);
   @{$passData1->passList} = @newAmsubPassList;
   $newPassDataStructList{"AMSU"} = $passData1;   

   # create TRMM pass struct
   my $passData2 = PassData->new();
   $passData2->maxDist($TRMM_DIST);
   @{$passData2->passList} = @newTrmmPassList;
   $newPassDataStructList{"TRMM"} = $passData2;

   # create SSMI pass struct
   my $passData3 = PassData->new();
   $passData3->maxDist($SSMI_DIST);
   @{$passData3->passList} = @newSsmiPassList;
   $newPassDataStructList{"SSMI"} = $passData3;

   # create SSMIS pass struct
   my $passData4 = PassData->new();
   $passData4->maxDist($SSMIS_DIST);
   @{$passData4->passList} = @newSsmisPassList;
   $newPassDataStructList{"SSMIS"} = $passData4;

   # create AMSRE pass struct
   my $passData5 = PassData->new();
   $passData5->maxDist($AMSRE_DIST);
   @{$passData5->passList} = @newAmsrePassList;
   $newPassDataStructList{"AMSRE"} = $passData5;

   return $isNewPassExist;

}  ## End of the Subroutine "getNewPassLists"
              

############## Subroutine "getCurrentPassLists" ################
############################################################

sub getCurrentPassLists
{
   my @curAmsubPassList = ();
   my @curTrmmPassList = ();
   my @curSsmiPassList = ();
   my @curSsmisPassList = ();
   my @curAmsrePassList = ();

   chdir("$passDataDir") || die "Cannot change to pass data Dir $passDataDir: $!";
   print  "Pass Data directory:";
   system("pwd");

   # Only get those orbit data less than 10 hours old in the system time
   # which include all those orbit end time within 10 hour (600 mins)
   # Please noted that the TPC/CPHC bulletin always start with the -3 hours forecast
   # and sometimes -4 compared to the system time.  Therefore, use 10 hour
 
   # my(@amsuList) = `find . -name "$amsubId*" -mmin -600 -exec ls -t {} \;`;   # not working
   my(@amsuList) = `find . -name "$amsubId*" -mmin -600 -exec ls -t {} ';'`;

   foreach $filename (@amsuList)
   {
      chomp($filename);
      # remove the "./" in front of the file name
      substr($filename, 0, 2) = "";
      push(@curAmsubPassList, $filename);
   }


   my(@trmmList) = `find . -name "$trmmId*" -mmin -600 -exec ls -t {} ';'`;

   foreach $filename (@trmmList)
   {
      chomp($filename);
      substr($filename, 0, 2) = "";
      push(@curTrmmPassList, $filename);
   }

   my(@ssmiList) = `find . -name "$ssmiId*" -mmin -600 -exec ls -t {} ';'`;

   foreach $filename (@ssmiList)
   {
      chomp($filename);
      substr($filename, 0, 2) = "";
      push(@curSsmiPassList, $filename);
   }

   my(@ssmisList) = `find . -name "$ssmisId*" -mmin -600 -exec ls -t {} ';'`;

   foreach $filename (@ssmisList)
   {
      chomp($filename);
      substr($filename, 0, 2) = "";
      push(@curSsmisPassList, $filename);
   }

   my(@amsreList) = `find . -name "$amsreId*" -mmin -600 -exec ls -t {} ';'`;

   foreach $filename (@amsreList)
   {
      chomp($filename);
      # remove the "./" in front of the file name
      substr($filename, 0, 2) = "";
      push(@curAmsrePassList, $filename);
   }

   ##------------ Create a hash table of structure "PassData" --------------##
   # create AMSU pass struct
   my $passData1 = PassData->new();
   $passData1->maxDist($AMSU_DIST);
   @{$passData1->passList} = @curAmsubPassList;
   $curPassDataStructList{"AMSU"} = $passData1;   

   # create TRMM pass struct
   my $passData2 = PassData->new();
   $passData2->maxDist($TRMM_DIST);
   @{$passData2->passList} = @curTrmmPassList;
   $curPassDataStructList{"TRMM"} = $passData2;

   # create SSMI pass struct
   my $passData3 = PassData->new();
   $passData3->maxDist($SSMI_DIST);
   @{$passData3->passList} = @curSsmiPassList;
   $curPassDataStructList{"SSMI"} = $passData3;

   # create SSMIS pass struct
   my $passData4 = PassData->new();
   $passData4->maxDist($SSMIS_DIST);
   @{$passData4->passList} = @curSsmisPassList;
   $curPassDataStructList{"SSMIS"} = $passData4;

   # create AMSRE pass struct
   my $passData5 = PassData->new();
   $passData5->maxDist($AMSRE_DIST);
   @{$passData5->passList} = @curAmsrePassList;
   $curPassDataStructList{"AMSRE"} = $passData5;



 #  ##------ Call utility script to get the time 9 hrs prior to current ------##
 #  my($CurrentYJday)=`$util_dir/TKgetdayExe`;
 #  my($CurrentTime)=`$util_dir/TKgettimeExe`;
 #  chomp($CurrentYJday);
 #  chomp($CurrentTime);

 #  # Get yyyyddd & hhmmss for cutting time rollback 9 hours
 #  # ( Start Scan is less than 8 hrs old / end scan <~ 7.4 hrs old)
 #  my($max_age) = -90000;
 #  my($TimeString) = `$util_dir/TKinctimeExe -srcday $CurrentYJday -srchms $CurrentTime -n_hms $max_age `;

 #  my(@fields) = split ( / +/,$TimeString );
 # print "\n TimeString $TimeString \n";
 # print "\n TimeString @fields\n";

 #  my($Cut_yyyyddd) = $fields[0];
 #  my($Cut_hhmmss) = $fields[1];
 # print " Cut_yyyyddd $Cut_yyyyddd \n";
 # print " Cut_hhmmss  $Cut_hhmmss \n";


 #  my(@list) = `ls`;

 #  ##------ Only choose orbit within ------##

 #  foreach $filename (@list)
 #  {
 #     chomp($filename);

 #     # AMSUB Orbit
 #     if ($filename =~ /^$amsubId/)
 #     {
 #        #Get time from orbit name
 #        $orbit_yyddd = substr($filename,7,5);
 #        $orbit_hhmm = substr($filename,14,4);
 
 #        # print "$filename - $orbit_yyddd  $orbit_hhmm \n ";
 
 #         # Check if the file is within 9 hours
 #        $Orbit_yyyyddd = `$util_dir/TKydtocydExe $orbit_yyddd`;
 #        $Orbit_hhmmss = $orbit_hhmm * 100 ;
 #       # print "Orbit_yyyyddd=$Orbit_yyyyddd \n Orbit_hhmmss =$Orbit_hhmmss\n ";

 #        if (( $Orbit_yyyyddd > $Cut_yyyyddd ||($Orbit_yyyyddd == $Cut_yyyyddd &&
 #             $Orbit_hhmmss >= $Cut_hhmmss ))
 #        {
 #           push(@curAmsubPassList, $filename);
 #        }
 #     }
 #     elsif ($filename =~ /^$trmmId/)
 #     {
 #        push(@curTrmmPassList, $filename);
 #     }
 #     elsif ($filename =~ /^$ssmiId/)
 #     {
 #        push(@curSsmiPassList, $filename);
 #     }
 #  }

}  ## End of Subroutine "getCurrentPassLists" 


############## Subroutine "getHEPassLists" ################
############################################################

sub getHEPassLists
{
   my($bulYjd, $forecastHour) = @_;

   # my $tmpHElist = "/tmp/HElist.txt";
   my ($pos, $yyddd, $hhmm, $HEnameLine, $isNewExist);
 #  my ($newHEarrayPrefix) = "newHEPassList";
 #  my ($curHEarrayPrefix) = "curHEPassList";
   my @HESelected = ();
   my @newHEPassList = ();
   my @curHEPassList = ();
#   my @allnewHEPassList = ();        # this line will be moved out 10/06/2006
   my @newSsmiPassList = ();
#   my @newSsmisPassList = ();

   my $foreYyddd = substr($bulYjd, 2); 
   my $foreHhmm = $forecastHour . "00";
   my $foreYydddhhmm = $foreYyddd . $foreHhmm;

# print "inside getHEPassLists: yydddhhmm is $yydddhhmm.\n";

   # these variables are for checking the HE which time is the closest to the forecast time
   # may be older, later or same time as the forecast time
   my $closeOlderHETime;   	# will get the closest old HE -  maximum older time
   my $closeOlderHEPos;
   my $closeLaterHETime;	# will get the closest new HE -  minumum later time
   my $closeLaterHEPos;
   my $sameHETime;
   my $sameHEPos;

   $isNewExist = 0;

   if (!(-s $HEPassDataLog))
   {
      `touch $HEPassDataLog`;
   }


   chdir("$dataDir") || die "Cannot change to data Dir $dataDir: $!";
   print  "Data directory:";
   system("pwd");


   foreach $name_G (@GoesSourceArray_G)
   {
        
      # empty the list from the previous used
      @HESelected = ();
      @newHEPassList = ();
      @curHEPassList = ();

      #$closeOldHETime = 0;      # will get the closest old HE -  maximum older time
      #$closeOldHEPos = "";
      $closeLaterHETime_G = `date -d "1 hour" +%y%j%H%M`;        # will get the closest new HE -  minumum later time
      chomp($closeLaterHETime_G);
      $closeLaterHEPos_G = "";
      $sameHETime_G = "";
      $sameHEPos_G = "";

      # result will be sth like "   4  G-10 IMG      19 JUL 05200  17:00:00    40  110 1 "
      # `grep "0:00" $name_G.imgl | head -1 >> $tmpHElist`;
      # $imglLine = `grep "0:00" $name.imgl | head -1`;
      print "imgfile name $name_G.imgl\n";
      $HEimgList_G = `grep "COMPOSITE" $name_G.imgl`;
      #print "HEimglist $HEimgList_G\n";

      @splitTmp = split(/\n/, $HEimgList_G);

      # find out the closest time HE image, same or older or later time
      foreach $imglLine ( @splitTmp)
      {
         @splitMsg = split(/\s+/, $imglLine); 
         $pos = $splitMsg[1];
         $yyddd = $splitMsg[5];
         $hhmm = $splitMsg[6];
         $hhmm =~ s/\://g;          # delete the : between the day and hour  
         $hhmm = substr($hhmm, 0, 4);
         $yydddhhmm = $yyddd . $hhmm;

#	 print "HE time $yydddhhmm Bull time $foreYydddhhmm\n";
#	 print "closeLaterHETime_G $closeLaterHETime_G\n";
        if ($yydddhhmm == $foreYydddhhmm + 600)
         {
            $sameHETime_G = $yydddhhmm;
            $sameHEPos_G = $pos;
#	    print "sameHETime_G sameHEPos_G $sameHETime_G $sameHEPos_G\n"; 
            last;
         }
         
         if ($yydddhhmm == $foreYydddhhmm + 400)
         {
            $sameHETime_G = $yydddhhmm;
            $sameHEPos_G = $pos;
#	    print "sameHETime_G sameHEPos_G $sameHETime_G $sameHEPos_G\n"; 
            last;
         }
         
         if ($yydddhhmm == $foreYydddhhmm + 200)
         {
            $sameHETime_G = $yydddhhmm;
            $sameHEPos_G = $pos;
#	    print "sameHETime_G sameHEPos_G $sameHETime_G $sameHEPos_G\n"; 
            last;
         }
         
         if ($yydddhhmm == $foreYydddhhmm)
         {
            $sameHETime_G = $yydddhhmm;
            $sameHEPos_G = $pos;
#	    print "sameHETime_G sameHEPos_G $sameHETime_G $sameHEPos_G\n"; 
            last;
         }
      }

#GHE sectiom

# 10/06/2006 will change the following to check three of the above images

#      print "before select sameHEPos_G closeLaterHEPos_G\n";
#      print "$sameHEPos_G $closeLaterHEPos_G\n"; 
      # Select the closest time H-E images to use
      if ($sameHEPos_G ne "")
      {
         $yyddd = substr($sameHETime_G, 0, 5);
         $hhmm = substr($sameHETime_G, -4);
         $HEnameLine = $name_G . '.' . $yyddd . '.' . $hhmm; 
         print "HEnameLine is $HEnameLine\n";
	 print "sameHEPos_G $sameHEPos_G\n";
         push(@HESelected, $HEnameLine);
	 $posNumber_G = $sameHEPos_G
      }
      # there is no HE newer than the forecast time
      elsif ($closeLaterHEPos_G ne "")
      {
         $yyddd = substr($closeOlderHETime_G, 0, 5);
         $hhmm = substr($closeOlderHETime_G, -4);
         $HEnameLine = $name_G. '.' . $yyddd . '.' . $hhmm;
         print "later HEnameLine is $HEnameLine\n";
	 print "closeOlderHEPos_G $closeOlderHEPos_G\n";
         push(@HESelected, $HEnameLine);
	 $posNumber_G = $closeLaterHEPos_G
      }
      else
      {
         # $diffOld = abs($foreYydddhhmm - $closeOlderHETime_G);
         # $diffNew = abs($foreYydddhhmm - $closeLaterHETime_G);
         if (substr($foreYydddhhmm, -2) < substr($closeOlderHETime_G, -2))
         {
            $diffOld = substr($foreYydddhhmm, -2) + 60 - substr($closeOlderHETime_G, -2);
         }
         else
         {
            $diffOld = substr($foreYydddhhmm, -2) - substr($closeOlderHETime_G, -2);
         }

         if (substr($closeLaterHETime_G, -2) < substr($foreYydddhhmm, -2))
         {
            $diffNew = substr($closeLaterHETime_G, -2) + 60 - substr($foreYydddhhmm, -2);
         }
         else
         {
            $diffNew = substr($closeLaterHETime_G, -2) - substr($foreYydddhhmm, -2);
         }
print "Care minute only: diffOld is $diffOld, diffNew is $diffNew\n";

         $yyddd = substr($closeOlderHETime_G, 0, 5);
         $hhmm = substr($closeOlderHETime_G, -4);
#         $HEnameLineOld = $name . '.' . $yyddd . '.' . $hhmm . '.' . $closeOlderHEPos_G;
         $HEnameLineOld = $name . '.' . $yyddd . '.' . $hhmm;
         $yyddd = substr($closeLaterHETime_G, 0, 5);
         $hhmm = substr($closeLaterHETime_G, -4);
#         $HEnameLineNew = $name . '.' . $yyddd . '.' . $hhmm . '.' . $closeLaterHEPos_G;
         $HEnameLineNew = $name . '.' . $yyddd . '.' . $hhmm;

         if ($diffOld == $diffNew)
         {
            @HESelected = ($HEnameLineOld, $HEnameLineNew);
	    print "closeOlderHEPos_G $closeLaterHEPos_G\n";
         }
         elsif ($diffOld < $diffNew)
         {
            push(@HESelected, $HEnameLineOld);
	    print "closeOlderHEPos_G $closeOlderHEPos_G\n";
         }
         else 
         {
            push(@HESelected, $HEnameLineNew);
	    print "closeOlderHEPos_G $closeLaterHEPos_G\n";
         }
         print "interp HEnameLine is $HEnameLineOld, $HEnameLineNew\n";
      }

    #  $HEnameLine = $name . '.' . $yyddd . '.' . $hhmm . '.' . $pos;
    #  print "HEnameLine is $HEnameLine\n";
    #  push(@curHEPassList, $HEnameLine);

      push(@curHEPassList, @HESelected);

      foreach $HEnameLine (@HESelected)
      {
         $exist=`grep $HEnameLine $HEPassDataLog`;

         if ($exist eq "")
         {      
            push(@newHEPassList, $HEnameLine);
            push(@allnewHEPassList, $HEnameLine);
            $isNewExist = 1;
         }
      }  


      # create H-E pass struct
      my $passData1 = PassData->new();
      $passData1->maxDist($HE_DIST);
      @{$passData1->passList} = @newHEPassList;
      $newHEPassDataStructList{$name_G} = $passData1;

      my $passData2 = PassData->new();
      $passData2->maxDist($HE_DIST);
      @{$passData2->passList} = @curHEPassList;
      $curHEPassDataStructList{$name_G} = $passData2;
#      print "isNewExist $isNewExist\n";

   }  # End of the For loop for different H-E data     
#   print "newHEPassList @newHEPassList\n";
#   print "curHEPassList @curHEPassList\n";

   # unlink("$tmpHElist");

   return $isNewExist;

}  ## End of the Subroutine "getHEPassLists"



############## Subroutine "evaluatePassCover" ################
# Please noted that this function is modified in Version 2.0
# Since there is no position (N S E W) in the input $lat and $lon
# which are read from the POS TXT file 
##############################################################

sub evaluatePassCover
{
   my($filename1,$maxdist1, $lat, $lon, $bulYjd, $iniTime, $forecastHour) = @_; 
   my($centerLat, $centerLon);
  #  my($latPosition) = chop($lat);
  #  my($lonPosition) = chop($lon);

  # $lat *= (($latPosition eq "N" || $latPosition eq "n") ? 1.0 : -1.0);
  # $lon *= (($lonPosition eq "E" || $lonPosition eq "e") ? 1.0 : -1.0);

   # Check if it is an AMSRE Accending pass and it is at the south of 10S
   if ($filename1 =~ /^$amsreId.+\d{12}$amsreAccend$/)
   {
      print "This is an an AMSRE Accending pass, file name is $filename1.\n";
      if ($lat < $amsreAcceLat)
      {
         print "The storm locates at the south of 10S, skip this AMSRE pass.\n";
         return 0;
      }
   }


#   system("dist $passDataDir/$filename1 $maxdist1 $lat $lon > do1");
   print "Before dist2\n";
   print "file $binDir/dist2 $passDataDir/$filename1 distance lat lon $maxdist1 $lat $lon\n"; 
   print "In evalPassCov before dist2 call binDir $binDir\n";
   system("$binDir/dist2 $passDataDir/$filename1 $maxdist1 $lat $lon > do1");
   $tempMsg = `grep "The final value of isCover=" do1`;
   @splitMsg = split(/isCover=/, $tempMsg);
   $isCover=$splitMsg[-1];
   chomp($isCover);

   $tempMsg = `grep "The index of the shortest distance=" do1`;
   @splitMsg = split(/distance=/, $tempMsg);
   $shortestInd=$splitMsg[-1];
   chomp($shortestInd);

   $tempMsg = `grep "Found the shortest distance" do1`;
   @splitMsg = split(/it is/, $tempMsg);
   my($shortestDist)=$splitMsg[-1];
   chomp($shortestDist);

   $tempMsg = `grep "The closest center lat" do1`;
   $_ = $tempMsg;
   #($centerLat, $centerLon) = /(\-?\d+\.\d+)\s?\D+[a-zA-Z]+\s?(\-?\d+\.\d+)/;
   ($centerLat, $centerLon) = /(\-?\d+\.\d+)[\s\,a-zA-Z]+(\-?\d+\.\d+)/;

# turn on the following line at 16:15 Z 09/26 to check the SSMi pass
 #  print "\nIn sub 'evaluatePassCover', the pass is $filename1, shortest Dist is $shortestDist km\n";

   `cat do1 >> do-sv`;
   unlink do1;

   if ($isCover == 0)
   {
      return $isCover;
   }

   ##---------------- isCover = 1 (currently), the pass cover the storm ---------------##
   ##------- will check if the pass within the 6 hours interval of the forecast -------##


# wonder if still need the iniTime here, after get Mike's response, may delete it from the subroutine call
# NOTED at 09/10/2003

   # get the time string of the 6 hours before the forecast
   my($bulHhmmss) = $forecastHour . "0000";

   #----------- oldest time string -----------#

   my($TimeString1) = `$util_dir/TKinctimeExe -srcday $bulYjd -srchms $bulHhmmss -n_hms -$interval `;
   chomp($TimeString1);
   my(@fields)   =  split ( /\s+/,$TimeString1 );
   my($Cut_yyyyddd) = $fields[0];
   my($Cut_hhmmss) = $fields[1];
   
   # zero-padded value to $Cut_hhmmss 
   $Cut_hhmmss = sprintf("%06d", $Cut_hhmmss);

   $TimeString1 = $Cut_yyyyddd . $Cut_hhmmss;

   #----------- latest time string -----------#

   my($TimeString2) = `$util_dir/TKinctimeExe -srcday $bulYjd -srchms $bulHhmmss -n_hms $interval `;
   @fields = split ( /\s+/,$TimeString2 );
   $Cut_yyyyddd = $fields[0];
   $Cut_hhmmss = $fields[1];

   # zero-padded value to $Cut_hhmmss 
   $Cut_hhmmss = sprintf("%06d", $Cut_hhmmss);

   $TimeString2 = $Cut_yyyyddd . $Cut_hhmmss;

print "\nIn sub 'evaluatePassCover', the bulHhmmss is $bulHhmmss, shortest Dist is $shortestDist, pass center lat is $centerLat and lon is $centerLon.\n\n";

   #----------- get the pass time over the storm -----------# 

   ## get the start time of the AMSU, TRMM or SSMI orbit
   ## then get the actual time the orbit over the storm

   # filename for AMSU/TRMM/SSMI geolocation files are as following:
   # amsub.D03213.S1218.E1412.NM
   # trmm.2003-08-01T15-52-26Z
   # SSGEO.EDRR.S9.D03213.S1646.E1826.B1878283.NS
   # change the SSMI file name in September, 2006
   # SSGEO.NPR.EDRR.S8.D06277.S1536.E1720.B4904445.NS  
   # SSMIS file name
   # GEO.EDEB.SA.D11270.S0755.E1001
   print "Troubleshoot: filename1 used is $filename1\n";
   
   if ($filename1 =~ /^$amsubId/)
   {   
      $orbitSt_yyddd = substr($filename1,7,5);
      $orbitSt_hh = substr($filename1,14,2);
      $orbitSt_mm = substr($filename1,16,2);
      $orbitSt_dd = "";
      # the time period (in seconds) of the orbit start time to the time the pass over the storm
      $passedPeriod = $shortestInd * $AMSU_SCAN_PERIOD;
   }
   elsif ($filename1 =~ /^$trmmId/)
   {
      $orbitSt_yyyy = substr($filename1,5,4);
      $orbitSt_MM = substr($filename1,10,2);
      $orbitSt_dd = substr($filename1,13,2);
      $orbitSt_hh = substr($filename1,16,2);
      $orbitSt_mm = substr($filename1,19,2);
    #  $orbitSt_hhmm = $orbitSt_hh . $orbitSt_mm;
      $passedPeriod = $shortestInd * $TRMM_SCAN_PERIOD;

      # this yyddd is used to get/grep the pos number from imglist in main program.
      $orbitSt_yyddd = `$util_dir/TKdmytocydExe -day $orbitSt_dd -year $orbitSt_yyyy -month $orbitSt_MM`;  
      # chomp($orbitSt_yyddd);
      # substr($orbitSt_yyddd, 0, 2) = "";
      $orbitSt_yyddd = substr($orbitSt_yyddd, 2, 5);
   }
   elsif ($filename1 =~ /^$ssmiId/)
   {
      # $orbitSt_yyddd = substr($filename1,15,5);
      # $orbitSt_hh = substr($filename1,22,2);
      # $orbitSt_mm = substr($filename1,24,2);
      $orbitSt_yyddd = substr($filename1,19,5);
      $orbitSt_hh = substr($filename1,26,2);
      $orbitSt_mm = substr($filename1,28,2);
      $orbitSt_dd = "";
      $passedPeriod = $shortestInd * $SSMI_SCAN_PERIOD;
   }
   elsif ($filename1 =~ /^$ssmisId/)
   {
      # $orbitSt_yyddd = substr($filename1,19,5);
      # $orbitSt_hh = substr($filename1,22,2);
      # $orbitSt_mm = substr($filename1,24,2);
      $orbitSt_yyddd = substr($filename1,13,5);
      $orbitSt_hh = substr($filename1,20,2);
      $orbitSt_mm = substr($filename1,22,2);
      $orbitSt_dd = "";
      $passedPeriod = $shortestInd * $SSMIS_SCAN_PERIOD;
   }
   elsif ($filename1 =~ /^$amsreId/)
   {
      print "Troubleshoot: filename used in amsre block is $filename\n";
      $orbitSt_yyyy = substr($filename,15,4);
      $orbitSt_MM = substr($filename,19,2);
      $orbitSt_dd = substr($filename,21,2);
      $orbitSt_hh = substr($filename,23,2);
      $orbitSt_mm = substr($filename,25,2);

      $passedPeriod = $shortestInd * $AMSRE_SCAN_PERIOD;

      # this yyddd is used to get/grep the pos number from imglist in main program.
      $orbitSt_yyddd = `$util_dir/TKdmytocydExe -day $orbitSt_dd -year $orbitSt_yyyy -month $orbitSt_MM`;
      $orbitSt_yyddd = substr($orbitSt_yyddd, 2, 5);
   }

   if ($orbitSt_dd eq "")
   {
      $dmy = `$util_dir/TKcydtodmyExe -cyd $orbitSt_yyddd`;
      chomp($dmy);
      @splitMsg = split(/\s+/, $dmy);
      $orbitSt_dd = $splitMsg[0];
      $orbitSt_MM = $splitMsg[1];
      $orbitSt_yyyy = $splitMsg[2];
      if ($orbitSt_dd < 10)
      {
         $orbitSt_dd = '0' . $orbitSt_dd;
      }
      if ($orbitSt_MM < 10)
      {
         $orbitSt_MM = '0' . $orbitSt_MM;
      }
   }
   ## start time of the orbit is got up to here

   # use some perl function to calculate the pass time over the storm
 
   #   $orbit_ddhhmm = $orbit_dd . $orbit_hhmm;
     
   ($passedStorm_yyyy, $passedStorm_MM, $passedStorm_dd, $passedStorm_hh, $passedStorm_mm, $passedStorm_ss) =
     Add_Delta_DHMS($orbitSt_yyyy, $orbitSt_MM, $orbitSt_dd, $orbitSt_hh, $orbitSt_mm, 0, 0, 0, 0, $passedPeriod);
   
  print "\nthe filename is $filename1, the shortestInd is $shortestInd, passedPeriod is $passedPeriod,
           the pass time is: yyyy- $passedStorm_yyyy, MM- $passedStorm_MM, dd- $passedStorm_dd,
           hh- $passedStorm_hh, min- $passedStorm_mm, ss- $passedStorm_ss.\n\n"; 

#   if ($passedStorm_MM < 10)
#   {
#      $passedStorm_MM = '0' . $passedStorm_MM;
#   }

#   if ($passedStorm_dd < 10)
#   {
#      $passedStorm_dd = '0' . $passedStorm_dd;
#   }
   if ($passedStorm_hh < 10)
   {
      $passedStorm_hh = '0' . $passedStorm_hh;
   }
   if ($passedStorm_mm < 10)
   {
      $passedStorm_mm = '0' . $passedStorm_mm;
   }
   if ($passedStorm_ss < 10)
   {
      $passedStorm_ss = '0' . $passedStorm_ss;
   }

   ## evlauate if the orbit within the 6 hour interval of the forecast ##
   ## noted at 09/08/2003, will compare the orbit time and the forcast mark time 
      
   $passedStorm_yyyyddd = `$util_dir/TKdmytocydExe -day $passedStorm_dd -year $passedStorm_yyyy -month $passedStorm_MM`;
   # chomp($passedStorm_yyyyddd);
   $passedStorm_yyyyddd = substr($passedStorm_yyyyddd, 0, 7);
   $passedStorm_timeString =  $passedStorm_yyyyddd . $passedStorm_hh . $passedStorm_mm . $passedStorm_ss;

   print "the forecastHour is $forecastHour, TimeString1 is $TimeString1, TimeString2 is $TimeString2\n";      
   print "the passesStorm time string is $passedStorm_timeString, compare the time strings.\n";      
 
   if (($passedStorm_timeString < $TimeString1) || ($passedStorm_timeString > $TimeString2)) 
   {
      print "the pass time Out of range\n"; 
      $isCover = 0;
   }
 
   return $isCover;

}  ## End of Subroutine "evaluatePassCover"



############## Subroutine "evaluateHEPassCover" ################
# Please noted that this function is modified in Version 2.0
# Since there is no position (N S E W) in the input $lat and $lon
# which are read from the POS TXT file
##############################################################

sub evaluateHEPassCover
{
   #my($filename1,$lat, $lon) = @_;
   my($filename1,$lat, $lon, $bulYjd, $iniTime, $forecastHour) = @_;
   my($isCover, $heus_lr_lat, $heus_lr_lon);
   my($numOfQuarter, $indexOf12);


   ##------ get the image time from the name line ------##  
 
   ## get the start time of the AMSU, TRMM or SSMI orbit
   ## then get the actual time the orbit over the storm

   # filename for H-E file name line are as following:
   # HECOMB-1HR.05203.1400.87

   @splitMsg = split(/\./, $filename1);
   $orbitSt_yyddd = $splitMsg[1];
   my($orbitSt_hhmm) = $splitMsg[2];
   $orbitSt_hh = substr($orbitSt_hhmm,0,2);
   $orbitSt_mm = substr($orbitSt_hhmm,2,2);
   $orbitSt_dd = "";


# work up to here, 07/22/2005, will use the define corners to check:
# lat: (13 + 3) < $lat < (61 - 3)
# lon:  (-164 + 3) < $lon < (-62 -3)

   if ($filename1 =~ /^$heusId/)
   {
      # check the H-E data is big coverage or small coverage, 
      # usually from 00Z, for each image (every 15 minute) 5 big, then 7 small
      
      $numOfQuarter = $orbitSt_hh * 4 + $orbitSt_mm / 15;
      $indexOf12 = $numOfQuarter % 12;
  
      # big coverage
      if (($indexOf12 >= 0) && ($indexOf12 <= 4))
      {
         $heus_lr_lat = $HEUS_B_LR_LAT;
         $heus_lr_lon = $HEUS_B_LR_LON;
         print "this image has larger coverage\n";
      }
      elsif (($indexOf12 >= 5) && ($indexOf12 <= 11))
      {
         $heus_lr_lat = $HEUS_LR_LAT;
         $heus_lr_lon = $HEUS_LR_LON;
         print "this image has smaller coverage\n";
      }

      #if (   ($lat > ($HEUS_LR_LAT + $HE_DIS_DEG)) && ($lat < ($HEUS_UL_LAT - $HE_DIS_DEG))
      #    && ($lon > ($HEUS_UL_LON + $HE_DIS_DEG)) && ($lon < ($HEUS_LR_LON - $HE_DIS_DEG)) 
      #   )
      if (   ($lat > ($heus_lr_lat + $HE_DIS_DEG)) && ($lat < ($HEUS_UL_LAT - $HE_DIS_DEG))
          && ($lon > ($HEUS_UL_LON + $HE_DIS_DEG)) && ($lon < ($heus_lr_lon - $HE_DIS_DEG)) 
         ) 
      {
         $isCover = 1;
      }
      else
      {
         $isCover = 0;
         return $isCover;
      }
   }
   elsif ($filename1 =~ /^$heusId_G/)
   {
       $isCover = 1;
   }
   else
   {
      print "WARNING: Undefined H-E file ID\n";
      $isCover = 0;
      return $isCover;
   }
   

   ##---------------- isCover = 1 (currently), the H-E data cover the storm ---------------##
   ##------- will check if the H-E data within the 6 hours interval of the forecast -------##

   # get the time string of the 6 hours before the forecast
   my($bulHhmmss) = $forecastHour . "0000";

   #----------- oldest time string -----------#

   my($TimeString1) = `$util_dir/TKinctimeExe -srcday $bulYjd -srchms $bulHhmmss -n_hms -$interval_GHE `;
   chomp($TimeString1);
   my(@fields)   =  split ( /\s+/,$TimeString1 );
   my($Cut_yyyyddd) = $fields[0];
   my($Cut_hhmmss) = $fields[1];

   # zero-padded value to $Cut_hhmmss
   $Cut_hhmmss = sprintf("%06d", $Cut_hhmmss);

   $TimeString1 = $Cut_yyyyddd . $Cut_hhmmss;

   #----------- latest time string -----------#

   my($TimeString2) = `$util_dir/TKinctimeExe -srcday $bulYjd -srchms $bulHhmmss -n_hms $interval_GHE `;
   @fields = split ( /\s+/,$TimeString2 );
   $Cut_yyyyddd = $fields[0];
   $Cut_hhmmss = $fields[1];

   # zero-padded value to $Cut_hhmmss
   $Cut_hhmmss = sprintf("%06d", $Cut_hhmmss);

   $TimeString2 = $Cut_yyyyddd . $Cut_hhmmss;

print "In sub 'evaluateiHEPassCover', the bulHhmmss is $bulHhmmss.\n";



   ##---------------- get the time pass over the storm from the H-E file name line ---------------##


   # For H-E data, the pass period is assumed as 0 

   $passedPeriod = 0;

   # $passedPeriod = $shortestInd * $TRMM_SCAN_PERIOD;

   #$posNumber = $splitMsg[3];

   if ($orbitSt_dd eq "")
   {
      $dmy = `$util_dir/TKcydtodmyExe -cyd $orbitSt_yyddd`;
      chomp($dmy);
      @splitMsg = split(/\s+/, $dmy);
      $orbitSt_dd = $splitMsg[0];
      $orbitSt_MM = $splitMsg[1];
      $orbitSt_yyyy = $splitMsg[2];
      if ($orbitSt_dd < 10)
      {
         $orbitSt_dd = '0' . $orbitSt_dd;
      }
      if ($orbitSt_MM < 10)
      {
         $orbitSt_MM = '0' . $orbitSt_MM;
      }
   }
   ## start time of the orbit is got up to here

   # use some perl function to calculate the pass time over the storm

   #   $orbit_ddhhmm = $orbit_dd . $orbit_hhmm;

   ($passedStorm_yyyy, $passedStorm_MM, $passedStorm_dd, $passedStorm_hh, $passedStorm_mm, $passedStorm_ss) =
     Add_Delta_DHMS($orbitSt_yyyy, $orbitSt_MM, $orbitSt_dd, $orbitSt_hh, $orbitSt_mm, 0, 0, 0, 0, $passedPeriod);

  print "\nthe filename is $filename1, passedPeriod is $passedPeriod,
           the pass time is: yyyy- $passedStorm_yyyy, MM- $passedStorm_MM, dd- $passedStorm_dd,
           hh- $passedStorm_hh, min- $passedStorm_mm, ss- $passedStorm_ss.\n\n";


#   if ($passedStorm_MM < 10)
#   {
#      $passedStorm_MM = '0' . $passedStorm_MM;
#   }

#   if ($passedStorm_dd < 10)
#   {
#      $passedStorm_dd = '0' . $passedStorm_dd;
#   }
   if ($passedStorm_hh < 10)
   {
      $passedStorm_hh = '0' . $passedStorm_hh;
   }
   if ($passedStorm_mm < 10)
   {
      $passedStorm_mm = '0' . $passedStorm_mm;
   }
   if ($passedStorm_ss < 10)
   {
      $passedStorm_ss = '0' . $passedStorm_ss;
   }


   ##----- evlauate if the orbit within the 6 hour interval of the forecast -----##

   $passedStorm_yyyyddd = `$util_dir/TKdmytocydExe -day $passedStorm_dd -year $passedStorm_yyyy -month $passedStorm_MM`;
   $passedStorm_yyyyddd = substr($passedStorm_yyyyddd, 0, 7);

   $passedStorm_timeString =  $passedStorm_yyyyddd . $passedStorm_hh . $passedStorm_mm . $passedStorm_ss;

   print "the forecastHour is $forecastHour, TimeString1 is $TimeString1, TimeString2 is $TimeString2\n";
   print "the passesStorm time string is $passedStorm_timeString, compare the time strings.\n";

   if (($passedStorm_timeString < $TimeString1) || ($passedStorm_timeString > $TimeString2))
   {
      print "The H-E data time is Out of range.\n";
      $isCover = 0;
   }

   return $isCover;

}  ## End of Subroutine "evaluateHEPassCover"



############## Subroutine "saveDirFile" ################
##############################################################

sub saveDirFile
{
   my($yyyy, $stNameNNB, $fileNamePrefix, $sourceName, $memoPrefix, $bulDoc, $orbitFile, $satId, $dataSetName ) = @_;

   # the following array is for convert the digital Month to letter Month
   my @months = qw(NONE JAN FEB MAR APR MAY JUN JUL AUG SEP OCT NOV DEC);

   ##------ Get the ddhhmm - the day, hour and minute of the composite data ------##
   #!!!! DELETE this part might need to delete in the futrue, since we use the pass time 
   #!!!! got from evaluatePass subroutine  
 
   $timeMsg = `lwu.k LIST AREA0630 3 4`;
   @splitMsg = split(/\s+/, $timeMsg);
   my($yyyddd) = $splitMsg[2];
   my($hhmmss) = $splitMsg[3];
  # $ddd = $yyyddd % 1000;
   my($yyddd)  = substr($yyyddd,1,5);
#   $hhmm = substr($hhmmss,0,4);
   @PolarSourceArray = qw(AMSUNK AMSUNL AMSUNM AMSUNN AMSUM2 AMSUM1 AMSUNP TRMM SSMIS7 SSMIS8 SSMIS9 SSMISA SSMISB SSMISC);
# For satellite ID hash table, key must match the satellite ID  in the POS1.TXT
   %satelliteId = qw(
		     NOAA-15 AMSUNK NOAA-16 AMSUNL NOAA-17 AMSUNM NOAA-18 AMSUNN NOAA-19 AMSUNP METOP-A AMSUM2 METOP-B AMSUM1
		     DMSP-F13 SSMIS7 DMSP-F14 SSMIS8 DMSP-F15 SSMIS9
		     DMSP-F16 SSMISA DMSP-F17 SSMISB DMSP-F18 SSMISC
		     TRMM TRMM
		     AQUA AMSRE
		     );
#   $positionFile = "$HOME/mcidas/data/POS1.TXT";

#   open (POSTXT, $positionFile) || die "can't open $positionFile: $!";
#   @posLines = <POSTXT>;
#   close (POSTXT) || die "can't close $positionFile: $!";

#------------------------ Get the satellite ID ------------------------#

   print "outside logic satID is $satId, sourceName is $sourceName\n";

#   $dataSetName = $satelliteId{$satId};
   print "dateSetName in trap is $dataSetName\n";
##------------------------------------------------------------------------------##

   if ($hhmmss < 100)
   {
      $hhmm = "0000";
   }
   else
   {
      # remove the ss (last two digits) from $hhmm
      $hhmm = $hhmmss;
      chop($hhmm);
      chop($hhmm);

      # zero-padded value to $Cut_hhmmss
      $hhmm = sprintf("%04d", $hhmm);
   }
   
   my($dd) = `datelist.k $yyddd FORM=\(DD\)`;
   chomp($dd);
   my($ddhhmm) = $dd . $hhmm;

   ##!!!! DELETE may delete above section

   # use the pass time in the end of TRaP set name 
   # my($passDdhhmm) = $passedStorm_dd . $passedStorm_hh . $passedStorm_mm;
   my($passMMDdhhmm) = $passedStorm_MM . $passedStorm_dd . $passedStorm_hh . $passedStorm_mm;

   # generate the filename combined of the prefix, source name and ddhhmm pass over the storm  
   # $fileName = $fileNamePrefix . $sourceName . "." . $ddhhmm;

   # For H-E TRaP
   if ($sourceName =~ /^GHE/)
   {
      if ($sourceName =~ /GHE-1HR/)
      {
         $sourceName2 = "GHEHR";
	 $dataSetName = "GHEHR";
      }
      # For the average 3 HR H-E, not sure the datset name yet
      elsif ($sourceName =~ /HECOMB-3HR/)  
      {
         $sourceName2 = "HE3HR";
      }
 
      $fileName = $fileNamePrefix . $dataSetName . "." . $passMMDdhhmm;
   }
   # For Polar TRaP
   else
   {
      $fileName = $fileNamePrefix . $dataSetName . "." . $passMMDdhhmm;
   }

   # check if the year is same, 03/30/07 
   if ($yyyy != $passedStorm_yyyy)
   {
      print "The current year $yyyy is not the same as the rain rate pass year $passedStorm_yyyy, take the second one.\n";
      $yyyy = $passedStorm_yyyy;
      substr($fileName, 0, 4) = $yyyy;
   }

   ## check if a TRaP with the same file name was generated before
   ## check from the trapSet log file or the TrapSet list

   $matchFile = `grep $fileName $trapSetLog`;
   if ($matchFile ne "")
   {
      print "WARNING!!!! the same file name is exist in trapSet log file, it is $matchFile\n";
      print "add the suffix -2 in the end of the TRaP set name.\n";
      $fileName = $fileName . '-2';
   }
   else
   {
      foreach $matchFile (@TrapSetList)
      {
         if ($fileName eq $matchFile)
         {
            print "WARNING!!!! the same file name is exist in TrapSet list file, it is $matchFile\n";
            print "add the suffix -2 in the end of the TRaP set name.\n";
            $fileName = $fileName . '-2';
            last;
         }
      }
   }


   ##------------ Create the yyyy/stormNameNNB directory if not exist -------------##
   chdir("$trapDataDir") || die "Cannot change to directory $trapDataDir";

   if (!(-d $yyyy))
   {
      print "Create the directory '$yyyy' for the current year.\n";
      mkdir($yyyy,0755) || die "cannot mkdir $yyyy: $!";
   }

   chdir("$yyyy") || die "Cannot change to directory $yyyy";

   if (!(-d $stNameNNB))
   {
      print "Create the directory '$stNameNNB' for the new storm.\n";
      mkdir($stNameNNB,0755) || die "cannot mkdir $stNameNNB: $!";
   }
   else
   {
      print "directory $stNameNNB exist.\n";
   }


   chdir("$dataDir") || die "Cannot change to directory $dataDir";
   print  "Current Data directory is:";
   system("pwd");


   ##------ Save the forecast bulletin and position/storm information file ------##

   # append the filename of the TRaP set in the end of the position file for digital TRaP naming
   system("echo $fileName > nametemp.txt");
  
   # append the pass time stamp in the end of the text file for SAB analyst 
   # to label the TRaP sent to homepage

   # passedStorm_MM will change from number to letter (Jan, Feb...Dec)   
   $frameLabelStr = "$passedStorm_hh$passedStorm_mm UTC $passedStorm_dd $months[$passedStorm_MM] $passedStorm_yyyy";
   system("echo $frameLabelStr >> nametemp.txt");


   # Add  the forecast time into the position file for the 24 hr TRaP label , on 07/28/2006
   
   my($bulYJd) = $st->bulYJd;
   my($forecast_hh)=  $st->forecastHour;

   $forecastDmy = `$util_dir/TKcydtodmyExe -cyd $bulYJd`;
   chomp($forecastDmy);
   @splitMsg = split(/\s+/, $forecastDmy);
   $forecast_dd = $splitMsg[0];
   $forecast_MM = $splitMsg[1];
   $forecast_yyyy = $splitMsg[2];
   if ($forecast_dd < 10)
   {
      $forecast_dd = '0' . $forecast_dd;
   }
   if ($forecast_MM < 10)
   {
      $forecast_MM = '0' . $forecast_MM;
   }

   $forecast_mm = "00";

   $frameLabelStr = "$forecast_hh$forecast_mm UTC $forecast_dd $months[$forecast_MM] $forecast_yyyy";
   system("echo 'Forecast date and time' >> nametemp.txt");
   system("echo $frameLabelStr >> nametemp.txt");


   ##------------------------------------------------------------------------------------------------------------##

   ##----------- Add the new image center location (average of initial and 24 hour forecast positions -----------##

   # Add on 04/16/07

   my($lat0) = $st->lat_0;
   my($lon0) = $st->lon_0;
   my($lat24) = $st->lat_24;
   my($lon24) = $st->lon_24;

   my($latMid) = ($lat0 + $lat24)/2;
   my($lonMid) = ($lon0 + $lon24)/2;


   # Check if it is cross dateline 180E/W
   if (($lon0 / $lon24) < 0 && (abs($lon0) > 90))
   {
      if ($lonMid <= 0)
      {
         $lonMid += 180;
      }
      else
      {
         $lonMid -= 180;
      }
   }

   print "Image display center:$latMid, $lonMid\n\n";

   #my ($centerStr) = "Image display center: " . $latMid . "    " . $lonMid;
   my ($centerStr) = "Image display center: " . sprintf("%8.2f%8.2f", $latMid, $lonMid);
   system("echo $centerStr >> nametemp.txt");


   # Add the rain rate orbit file name and the satellite ID

   system("echo $satId $orbitFile >> nametemp.txt");

   # system("echo '$passedStorm_dd $passedStorm_hh $passedStorm_mm' >> nametemp.txt");
   system("cat $posFile nametemp.txt > temp1.txt"); 

   # save the original bulletin and position file 

   $outputDir = "$trapDataDir/$yyyy/$stNameNNB";

   system("cp $bulDoc $outputDir/$fileName.bul");
#   system("cp $posFile $outputDir/$fileName.txt");
   system("cp -p temp1.txt $fullPosFile");
   system("mv temp1.txt $outputDir/$fileName.txt");


   ##-- Save the AREA and GIF files to specific directory and naming conventions --##

   # save the TRaP Gif files from AREA630-637
   print "Call $svGifCM to save the TRaP Gif files from AREA630-637!!!\n\n";
   system("$svGifCM $fileName");
   print "Done with calling $svGifCM to save the TRaP Gif files from AREA630-637!!!\n\n";

#   $outputDir = "$trapDataDir/$yyyy/$stNameNNB";
   system("pwd");
   print "Save the TRaPs GIF file named $fileName \nin diretory $outputDir\n";
   # save the GIF files, file name must same as the ones in saveTRAPGIF.pl
   system("cp TRAPrr.GIF $outputDir/$fileName.raw.GIF");
#   system("mv TR631.GIF $outputDir/$fileName.GIF");
#   system("mv TR632.GIF $outputDir/$fileName.24.GIF")
   system("cp TRAP00.GIF $outputDir/$fileName.00.GIF");
   system("cp TRAP06.GIF $outputDir/$fileName.06.GIF");
   system("cp TRAP12.GIF $outputDir/$fileName.12.GIF");
   system("cp TRAP18.GIF $outputDir/$fileName.18.GIF");
   system("cp TRAP24.GIF $outputDir/$fileName.24.hi.GIF");

##   rename("TR631.GIF", "$outputDir/$fileName.GIF");
#   system("mv TR630.GIF $outputDir/$fileName.raw.GIF");
#   system("mv TR631.GIF $outputDir/$fileName.GIF");
#   system("mv TR632.GIF $outputDir/$fileName.24.GIF");
#   system("mv TR633.GIF $outputDir/$fileName.00.GIF");
#   system("mv TR634.GIF $outputDir/$fileName.06.GIF");
#   system("mv TR635.GIF $outputDir/$fileName.12.GIF");
#   system("mv TR636.GIF $outputDir/$fileName.18.GIF");
#   system("mv TR637.GIF $outputDir/$fileName.24.hi.GIF");

   $success=$?;

   # if the mv command not work, it might be because pctrpauto.k did not work correctly
   # and no GIF files generate
   if ($success != 0)
   {
      print "!!Warning: GIF files not generated, pctrpauto.k might not work correctly.\n";
      print "Skip the remaining steps for saving the AREA file, return empty string.\n";
      return "";
   }


   ##---------------------- Modify the MEMO of the AREA files ------------------------## 
   #-- the MEMO include the storm name,bulletin information and the type of the TRaP--## 
   print "change the MEMO to include the storm name,bulletin information.\n";

   @split_memo = split(/\./, $memoPrefix);
   print "split_memo1 $split_memo[0]\n";
   @trunc_stnm = split(/~/, $split_memo[0]);
   print "trunc $trunc_stnm[0]\n";
   $reconstr_memo = $trunc_stnm[0].$split_memo[1];
   print "rec_mem $reconstr_memo\n";
   system("imgcha.k LA.630 MEMO=$reconstr_memo'-rw' DEV=T imctemp.txt1");
   system("imgcha.k LA.631 MEMO=$reconstr_memo'-r' DEV=T imctemp.txt1");
   system("imgcha.k LA.632 MEMO=$reconstr_memo'-4' DEV=T imctemp.txt");
   system("imgcha.k LA.633 MEMO=$reconstr_memo'-0' DEV=T imctemp.txt");   
   system("imgcha.k LA.634 MEMO=$reconstr_memo'-6' DEV=T imctemp.txt");
   system("imgcha.k LA.635 MEMO=$reconstr_memo'-2' DEV=T imctemp.txt");
   system("imgcha.k LA.636 MEMO=$reconstr_memo'-8' DEV=T imctemp.txt");
   system("imgcha.k LA.637 MEMO=$reconstr_memo'-4h' DEV=T imctemp.txt");
#   system("imgcha.k LA.631 MEMO=$reconstr_memo'rr' DEV=T imctemp.txt1");


   print "Save the TRaPs AREA file named $fileName \nin diretory $outputDir\n";

   # save the AREA files
   system("cp AREA0630 $outputDir/$fileName.raw");
   system("cp AREA0631 $outputDir/$fileName");
   system("cp AREA0632 $outputDir/$fileName.24");
   system("cp AREA0633 $outputDir/$fileName.00");
   system("cp AREA0634 $outputDir/$fileName.06");
   system("cp AREA0635 $outputDir/$fileName.12");
   system("cp AREA0636 $outputDir/$fileName.18");
   system("cp AREA0637 $outputDir/$fileName.24.hi");


   print "Compress the data of TRaP set $fileName \n";

   # compress the AREA files and text files
   system("compress $outputDir/$fileName*");

   ##------------------------------------------------------------------##

   ##------ Send (SFTP) out the TRaP to SAB Data server - Sabna4 ------##

   # Turn off 0n 08/12/2009 (will change the date)
   #print " Create sftp script to send the TRaP files to $sabDataServer\n";

   #system ("ssh $sabDataServer $sabDataRmtCM  $yyyy $stNameNNB $fileName");

   # the absolute remote ftp direcotry
   $abs_remoteTRAPDir = "$sabDataTrapDir/$yyyy/$stNameNNB";

   open  ( FTPOUT,  ">$sftpfile" )  ||  die " Can't open $sftpfile to write: $!";

   print  FTPOUT   "# SFTP command file:  Put digit files to FTP server\n";
   print  FTPOUT   " sftp $sabDataServer << EOF\n";
   print  FTPOUT   "   cd  $abs_remoteTRAPDir\n";
   print  FTPOUT   "   mput  $outputDir/$fileName*\n";
   print  FTPOUT   "   bye\n";
   print  FTPOUT   "EOF\n";

   close ( FTPOUT ) || die "Can't close $sftpfile: $!";
   chmod  (0777,  $sftpfile) ;

   # Run the sftp file to get the files from NASA
   #$enddate=`date`;
   #print " Start  sftp digit files to $sabDataServer at $enddate \n";
   #open ( SFTPOUT,  " $sftpfile  | " ) || die " Can't open $sftpfile to run: $!";

   #@sftpOutput = <SFTPOUT>;

   ##--------------------------------------------------------------------------------------------##

   return $fileName;
}
