#!/usr/bin/perl -w
#
#
# $Id: pctrpHE.pl,v 1.1.1.1 2010-04-27 15:13:11 cmuser Exp $
# $Log: not supported by cvs2svn $
# Revision 1.1.1.1  2009/02/04 19:25:09  cmuser
# initial release
#
# Revision 1.1  2007/11/30 19:57:01  oper1
# First checkin.
#
#
#!/usr/bin/perl -w   

## =====================================================================
## Name:            pctrpHE.pl
##                  
## Author:          Jianbin Yang/RSIS   August 2005
## Language:        Perl
## Purpose:         
## # Usage: pctrpbest.pl <Storm Name> <Data Source> <bulletin header> <HE type>    
## Usage: pctrpHE.pl <Storm Name> <bulletin header> <HE type>    
## note : use hash table to store the storm struct information.
##        delete the print statement
## Last Modified:   05/08/2005
##
## =====================================================================


use Class::Struct;

####---------------------- Define structures and data ------------------------####

# create a definition for a "Storm" struct
struct Storm =>
{
   name      => '$',        # storm name for pctrp.k to call
   nameNNB   => '$',        # storm name with NNB if is available, this used for name convention
   wmoHd     => '$',        # the WMO header id 
   stationId => '$',        # the station id
   bulTime   => '$',        # the forecast bulletine time
   bulYJd    => '$',        # store the bulletine year and julian day
   iniTime   => '$',        # the initial time of the storm
   lat       => '$',        # the latitude of the center of the storm
   lon       => '$',        # the longitude of the center of the storm
};



# Define the directories and files
$srcDir = '/home/oper1/mcidas/pctrap-V3';
$dataDir = '/home/oper1/mcidas/data';
$trapDataDir = "/data/TRAP";
# $trapSetLog = "$dataDir/trapSet.log"; 
$pctrpCM = "$srcDir/pctrpHE-mcenv.sh";          # Command/Shell Script to call PCTRP


##--------------------------------- Start --------------------------------------##

$path = $ENV{"PATH"};
$ENV{"PATH"} = $path;

$ENV{"MCPATH"} = "/home/oper1/mcidas/data:/home/mcidas/data";

print "\n--------------------- Start the Automated TRaP Program ---------------------\n";

$startdate=`date`;
print "Start $0 at $startdate.\n"; 

chdir("$dataDir") || die "Cannot change to directory $dataDir";
print  "Data directory:";
system("pwd");

####--------------------------------------------------------------------------####

if ($#ARGV != 2)
{
   print "Usage: $0 <Storm Name> <Data Source> <bulletin header> <HE type>\n";
   die "No name input - exit script";
}

$stormName = $ARGV[0];
# $sourceName = $ARGV[1];
# $imageTime = $ARGV[2];
$wmoHd = $ARGV[1];
$heType = $ARGV[2];

$sourceName = "AUTO";

# only get the wmo header(WTxxnn), not included the bulletin time
$header = substr($wmoHd, 0, 6); 

# $fileNamePre = $stormName . "." . $wmoHd . "." . $sourceName . '-' . $heType;
$fileNamePre = $stormName . "." . $wmoHd . "."  . $heType;

print "Command line:$pctrpCM X $sourceName $header $stormName\n";

# system ("$pctrpCM X $sourceName $header $stormName");
system ("$pctrpCM X $sourceName $stormName");
saveDirFile($stormName, $fileNamePre, $sourceName);

print "\nEnd of the HE TRaP program.\n";


############## Subroutine "saveDirFile" ################
##############################################################

sub saveDirFile
{
   my($stormName, $fileNamePrefix, $sourceName) = @_;

   ##------ Get the ddhhmm - the day, hour and minute of the composite data ------##
 
   # $timeMsg = `lwu.k LIST AREA0650 3 4`;
   $timeMsg = `imglist.k LA.650 |grep 650`;
   


print "timeMsg is $timeMsg\n";

   @splitMsg = split(/\s+/, $timeMsg);
   # $yyyddd = $splitMsg[2];
   $yyddd = $splitMsg[6];
   $hhmmss = $splitMsg[7];

   $hhmmss =~ s/\://g;   # delete the :
#   $yyddd  = substr($yyyddd,1,5);
   $hhmm = substr($hhmmss, 0, 4);

print "yyddd is $yyddd, hhmm is $hhmm\n";

   $dd = `datelist.k $yyddd FORM=(DD)`;
   chomp($dd);
   $ddhhmm = $dd . $hhmm;

   # generate the filename combined of the prefix, source name and ddhhmm 
   $fileName = $fileNamePrefix . "." . $ddhhmm;

   if (!(-d $stormName))
   {
      print "Create the directory '$stormName' for the new storm.\n";
      mkdir($stormName,0755) || die "cannot mkdir $stormName: $!";
   }
   else
   {
      print "directory $stormName exist.\n";
   }


   chdir("$dataDir") || die "Cannot change to directory $dataDir";
   print  "Current Data directory is:";
   system("pwd");

   ##-- Save the AREA and GIF files to specific directory and naming conventions --##

   $outputDir = "$dataDir/$stormName";
  ## $outputDir = "$dataDir/BEST_TRAP/$stormName";

   print "Save the TRaPs GIF file named $fileName \nin diretory $outputDir\n";
   # save the GIF files
#   rename("TR651.GIF", "$outputDir/$fileName.GIF");
   system("mv TR651.GIF $outputDir/$fileName.GIF");
   system("mv TR652.GIF $outputDir/$fileName.24.GIF");
   system("mv TR653.GIF $outputDir/$fileName.00.GIF");
   system("mv TR654.GIF $outputDir/$fileName.06.GIF");
   system("mv TR655.GIF $outputDir/$fileName.12.GIF");
   system("mv TR656.GIF $outputDir/$fileName.18.GIF");
   system("mv TR657.GIF $outputDir/$fileName.24.hi.GIF");

   $success=$?;

   # if the mv command not work, it might be because pctrpauto.k did not work correctly
   # and no GIF files generate
   if ($success != 0)
   {
      print "!!Warning: GIF files not generated, pctrpauto.k might not work correctly.\n";
      print "Skip the remaining steps for saving the AREA file, return empty string.\n";
      return "";
   }


   print "Save the TRaPs AREA file named $fileName \nin diretory $outputDir\n";

   # save the AREA files
   system("cp AREA0650 $outputDir/$fileName.raw");
   system("cp AREA0651 $outputDir/$fileName");
   system("cp AREA0652 $outputDir/$fileName.24");
   system("cp AREA0653 $outputDir/$fileName.00");
   system("cp AREA0654 $outputDir/$fileName.06");
   system("cp AREA0655 $outputDir/$fileName.12");
   system("cp AREA0656 $outputDir/$fileName.18");
   system("cp AREA0657 $outputDir/$fileName.24.hi");

   # save the original bulletin and position text output 
  # system("cp POS.DOC $outputDir/$fileName.bul");
   system("cp POSBEST.TXT $outputDir/$fileName.txt");

   print "Compress the data of TRaP set $fileName \n";

   # compress the AREA files and text files
   system("compress $outputDir/$fileName*");

   # Move the storm directory to "BEST_TRAP"
   if (!(-d "HE_TRAP"))
   {
      print "Create the directory 'HE_TRAP' for the HE TRaP.\n";
      mkdir("HE_TRAP",0755) || die "cannot mkdir HE_TRAP: $!";
   }

   if (-d "HE_TRAP/$stormName")
   {
      system("mv $outputDir/* HE_TRAP/$stormName/");
   }
   else
   {
      system("mv $outputDir HE_TRAP");
   }
}

