#!/usr/bin/perl -w
#
#
# $Id: pctrpbest.pl,v 1.1.1.1 2010-04-27 15:13:11 cmuser Exp $
# $Log: not supported by cvs2svn $
# Revision 1.1.1.1  2009/02/04 19:25:09  cmuser
# initial release
#
# Revision 1.1  2007/11/30 19:57:01  oper1
# First checkin.
#
#
#!/usr/bin/perl -w   

## =====================================================================
## Name:            pctrpbest.pl
##                  
## Author:          Jianbin Yang/RSIS   June 2002
## Language:        Perl
## Purpose:         
## Usage: pctrpbest.pl <Storm Name> <Data Source> <bulletin header>     
## note : use hash table to store the storm struct information.
##        delete the print statement
## Last Modified:   02/6/2002
##
## =====================================================================


use Class::Struct;

####---------------------- Define structures and data ------------------------####

# create a definition for a "Storm" struct
struct Storm =>
{
   name      => '$',        # storm name for pctrp.k to call
   nameNNB   => '$',        # storm name with NNB if is available, this used for name convention
   wmoHd     => '$',        # the WMO header id 
   stationId => '$',        # the station id
   bulTime   => '$',        # the forecast bulletine time
   bulYJd    => '$',        # store the bulletine year and julian day
   iniTime   => '$',        # the initial time of the storm
   lat       => '$',        # the latitude of the center of the storm
   lon       => '$',        # the longitude of the center of the storm
};



# Define the directories and files
$srcDir = '/home/oper/mcidas/pctrap';
$dataDir = '/home/oper/mcidas/data';
$trapDataDir = "/data/TRAP";
$trapSetLog = "$dataDir/trapSet.log"; 
$pctrpCM = "$srcDir/pctrpbest-mcenv.sh";          # Command/Shell Script to call PCTRP


##--------------------------------- Start --------------------------------------##

$path = $ENV{"PATH"};
$ENV{"PATH"} = $path;

$ENV{"MCPATH"} = "/home/oper/mcidas/data:/home/mcidas/data";

print "\n--------------------- Start the Automated TRaP Program ---------------------\n";

$startdate=`date`;
print "Start $0 at $startdate.\n"; 

chdir("$dataDir") || die "Cannot change to directory $dataDir";
print  "Data directory:";
system("pwd");

####--------------------------------------------------------------------------####

if ($#ARGV != 2)
{
   print "Usage: $0 <Storm Name> <Data Source> <bulletin header>\n";
   die "No name input - exit script";
}

$stormName = $ARGV[0];
$sourceName = $ARGV[1];
# $imageTime = $ARGV[2];
$wmoHd = $ARGV[2];

# only get the wmo header(WTxxnn), not included the bulletin time
$header = substr($wmoHd, 0, 6); 

$fileNamePre = $stormName . "-best" .  "." . $wmoHd . "." . $sourceName;

print "Command line:$pctrpCM X $sourceName $header $stormName\n";

system ("$pctrpCM X $sourceName $header $stormName");
saveDirFile($stormName, $fileNamePre, $sourceName);

print "\nEnd of the best TRaP program.\n";


############## Subroutine "saveDirFile" ################
##############################################################

sub saveDirFile
{
   my($stormName, $fileNamePrefix, $sourceName) = @_;

   ##------ Get the ddhhmm - the day, hour and minute of the composite data ------##
 
   $timeMsg = `lwu.k LIST AREA0650 3 4`;
   @splitMsg = split(/\s+/, $timeMsg);
   $yyyddd = $splitMsg[2];
   $hhmmss = $splitMsg[3];
  # $ddd = $yyyddd % 1000;
   $yyddd  = substr($yyyddd,1,5);
#   $hhmm = substr($hhmmss,0,4);

   if ($hhmmss < 100)
   {
      $hhmm = "0000";
   }
   else
   {
      # remove the ss (last two digits) from $hhmm
      $hhmm = $hhmmss;
      chop($hhmm);
      chop($hhmm);
      if ($hhmm < 10)
      {
         $hhmm = "000" . $hhmm;
      }
      elsif ($hhmm < 100)
      {
         $hhmm = "00" . $hhmm;
      }
      elsif ($hhmm < 1000)
      {
         $hhmm = '0' . $hhmm;
      }
   }
   

   $dd = `datelist.k $yyddd FORM=(DD)`;
   chomp($dd);
   $ddhhmm = $dd . $hhmm;

   # generate the filename combined of the prefix, source name and ddhhmm 
   $fileName = $fileNamePrefix . "." . $ddhhmm;

   if (!(-d $stormName))
   {
      print "Create the directory '$stormName' for the new storm.\n";
      mkdir($stormName,0755) || die "cannot mkdir $stormName: $!";
   }
   else
   {
      print "directory $stormName exist.\n";
   }


   chdir("$dataDir") || die "Cannot change to directory $dataDir";
   print  "Current Data directory is:";
   system("pwd");

   ##-- Save the AREA and GIF files to specific directory and naming conventions --##

   $outputDir = "$dataDir/$stormName";
  ## $outputDir = "$dataDir/BEST_TRAP/$stormName";

   print "Save the TRaPs GIF file named $fileName \nin diretory $outputDir\n";
   # save the GIF files
#   rename("TR651.GIF", "$outputDir/$fileName.GIF");
   system("mv TR651.GIF $outputDir/$fileName.GIF");
   system("mv TR652.GIF $outputDir/$fileName.24.GIF");
   system("mv TR653.GIF $outputDir/$fileName.00.GIF");
   system("mv TR654.GIF $outputDir/$fileName.06.GIF");
   system("mv TR655.GIF $outputDir/$fileName.12.GIF");
   system("mv TR656.GIF $outputDir/$fileName.18.GIF");
   system("mv TR657.GIF $outputDir/$fileName.24.hi.GIF");

   $success=$?;

   # if the mv command not work, it might be because pctrpauto.k did not work correctly
   # and no GIF files generate
   if ($success != 0)
   {
      print "!!Warning: GIF files not generated, pctrpauto.k might not work correctly.\n";
      print "Skip the remaining steps for saving the AREA file, return empty string.\n";
      return "";
   }


   print "Save the TRaPs AREA file named $fileName \nin diretory $outputDir\n";

   # save the AREA files
   system("cp AREA0650 $outputDir/$fileName.raw");
   system("cp AREA0651 $outputDir/$fileName");
   system("cp AREA0652 $outputDir/$fileName.24");
   system("cp AREA0653 $outputDir/$fileName.00");
   system("cp AREA0654 $outputDir/$fileName.06");
   system("cp AREA0655 $outputDir/$fileName.12");
   system("cp AREA0656 $outputDir/$fileName.18");
   system("cp AREA0657 $outputDir/$fileName.24.hi");

   # save the original bulletin and position text output 
  # system("cp POS.DOC $outputDir/$fileName.bul");
   system("cp POSBEST.TXT $outputDir/$fileName.txt");

   print "Compress the data of TRaP set $fileName \n";

   # compress the AREA files and text files
   system("compress $outputDir/$fileName*");

   # Move the storm directory to "BEST_TRAP"
   if (!(-d "BEST_TRAP1"))
   {
      print "Create the directory 'BEST_TRAP' for the Best TRaP.\n";
      mkdir("BEST_TRAP1",0755) || die "cannot mkdir BEST_TRAP: $!";
   }

   system("mv $outputDir BEST_TRAP1");
}

