#!/usr/bin/perl -w
#
#
# $Id: restoreAREA-arch.pl,v 1.1.1.1 2010-04-27 15:13:11 cmuser Exp $
# $Log: not supported by cvs2svn $
# Revision 1.2  2009/06/19 14:52:30  cmuser
# modified four scripts for CCR00196 (Prepare TRaP for NOAA-N'
#
# Revision 1.1.1.1  2009/02/04 19:25:09  cmuser
# initial release
#
# Revision 1.1  2007/11/30 19:57:01  oper1
# First checkin.
#
#
#!/usr/bin/perl -w

## =====================================================================
## Name:        restoreAREA-arch.pl
##                  
## Author:      Jianbin Yang/RSIS   07/10/2002
## Language:    Perl
## Purpose:     Restore/copy one set of TRaP AREA files in specific directory 
##              into AREA0640-0647 in mcidas/data directory           
##              Notes: This is for resending the pass TRaPs            
##       
## Usage:       restoreAREA.pl <Name of the TRaP set> 
##              Format of <Name of the TRaP set> is: 
##                     YYYYnameNNB.WTXXnn.FFFF.DDHHMM.SSSS.ddhhmm 
##                  or YYYYnameNNB.WTXXnn.FFFF.DDHHMM.SSSS.ddhhmm.nn
##                  or YYYYnameNNB.WTXXnn.FFFF.DDHHMM.SSSS.ddhhmm.nn.GIF
##
## Modified:   09/25/2002
##
## Modified:   02/26/2004
##              Save frames display as the GIF files, Noted that the frame number
##              MUST BE CONSISTENT with the AREAL batch programs.
##              This GFI files will be used in the area2digit.pl
##
## Last Modified:   11/26/2008
##              Resending the pass TRaPs
## =====================================================================

# Define the directories and files

$HOME = $ENV{"HOME"};

$dataDir = "$HOME/mcidas/data";
$trapDataDir = '/data/TRAP';          # the actual directory is '/data/TRAP/$year/$stormNameNNB'
$positionFile = 'POS2.TXT';           # File to keep the position of the track of the storm
# $totalGifFile = 'TRAP24hh.GIF';       # This GIF file may be used in the area2digital.pl program 
				      # to send to FTP server along with the digital TRaP file
 				      # Added on 07/25/2003

# $gifPrefix = 'TRAP';
$gifPrefix = 'TRAP_ARCH';
$startdate=`date`;
print "Start  $0 at $startdate \n";

chdir("$dataDir") || die "Cannot change to directory $dataDir";
print  "Data directory is:";
system("pwd");

# remove the path from the command name

@comandFields = split(/\//, $0);
$commandName = $comandFields[-1];

# if no argument input, exit the program

# if ($ARGV[0] eq "")
if ($#ARGV == -1)
{
   print "Usage: $commandName <Name of the TRaP set>\n";
   print "Format of <Name of the TRaP set> is: YYYYnameNNB.WTXXnn.FFFF.DDHHMM.SSSS.ddhhmm\n";
   die "No name input - exit script";
}

$trapSetName = $ARGV[0];
print "trapSetName is $trapSetName.\n";

# get the directories name of the specific TRaP set

@splitMsg = split(/\./, $trapSetName);
$splitMsgLen = @splitMsg;

# printf ("the number of splitMsg is %d.\n",($#splitMsg + 1));

if ($splitMsgLen < 6)
{
   print "Format of <Name of the TRaP set> is: 
          \rYYYYnameNNB.WTXXnn.FFFF.DDHHMM.SSSS.ddhhmm(.nn.GIF)\n\n";
   die "Input TRaP set name is not long enough or not correct - exit";
}
elsif ($splitMsgLen > 6)
{
   $trapSetName = $splitMsg[0];
   
   for ($i = 1; $i < 6; $i++)
   {
      $trapSetName .= '.';
      $trapSetName .= $splitMsg[$i];
   }
   print "The true TRaP set name is $trapSetName\n";
}


$dirMsg = $splitMsg[0];
$year = substr($dirMsg, 0, 4);
# $nameNnb = substr($dirMsg,4, (length($dirMsg) - 4));
$nameNnb = substr($dirMsg,4);
$outputDir = "$trapDataDir/$year/$nameNnb";

print "dirMsg is $dirMsg, year is $year, nameNNB is $nameNnb.\n";
# printf ("the length of dirMsg is %d.\n",length($dirMsg));

if (!(-d $outputDir))
{
   die "Directory:$outputDir is not exist -- exit script";
}

# uncompress the TRaP data before restored
print "Uncompress the TRaP AREA files and text file before restored.\n";
system("uncompress $outputDir/$trapSetName*");

# restore the TRaP AREA files to AREA0640-0647
print "Restore the specific TRaP AREA files to AREA0640-0647.\n";

system("cp $outputDir/$trapSetName.raw AREA0640");
system("cp $outputDir/$trapSetName AREA0641");
system("cp $outputDir/$trapSetName.24 AREA0642");
system("cp $outputDir/$trapSetName.00 AREA0643");
system("cp $outputDir/$trapSetName.06 AREA0644");
system("cp $outputDir/$trapSetName.12 AREA0645");
system("cp $outputDir/$trapSetName.18 AREA0646");
system("cp $outputDir/$trapSetName.24.hi AREA0647");

$success=$?;
if ($success != 0)
{
   print "!!Warning: Restoring AREA files not success.\n";
   die "TRaP Files not found, please input the correct TRaP name again - exit script.";
}

# restore the bulletin and position text output to dataDir
print "Restore the correlating bulletin and position text output.\n";
system("cp $outputDir/$trapSetName.bul POS2.DOC");
system("cp $outputDir/$trapSetName.txt $positionFile");
#system("cp $outputDir/$trapSetName.24.hi.GIF $totalGifFile");

print "Restoring success.\n";

# compress the TRaP data after restored
print "Compress the TRaP AREA files and text file after restored.\n";
system("compress $outputDir/$trapSetName*");

print "End of the restore process.\n";

# isolate the variable names from the command line: storm name, 
# bulletin id, bulletin time, and satellite

$stormName = $nameNnb;
$bulID = $splitMsg[1];
$agency = $splitMsg[2];
$bulTime = $splitMsg[3];
$dataSource = $splitMsg[4];

$bulHhmm = substr ($bulTime, 2, 4);

# get the latitude and longitude of the TRaP from POS2.TXT 
print "Get the 12 hour point position.\n";

open (POSTXT, $positionFile) || die "can't open $positionFile: $!";

while ( defined ($tempMsg = <POSTXT>))
{
   if ($tempMsg =~ /^12/)
   {
      print "get the right msg:$tempMsg \n";
      last;
   }
}

close (POSTXT) || die "can't open $positionFile: $!";

# @splitMsg2 = split(/\s{2,}/, $tempMsg);
@splitMsg2 = split(/\s+/, $tempMsg);

$lat = $splitMsg2[1];
$lon = $splitMsg2[2];

if ($agency eq "KNHC") {
$agency = 'TPC'; }

if ($agency eq "PGTW") {
$agency = 'JTWC'; }

print "StormName is $stormName, bulID is $bulID, agency is $agency, bul Time is $bulHhmm,
       \rdataSource is $dataSource, lat is $lat, lon is $lon \n";

print "Start to display the 24 hour TRaP and rain rate image...\n";

open STDOUT, ">restore.out" || die "Can't create restore.out:$!";
open STDERR, ">restore.err" || die "Can't create restore.err:$!";


# Copy the GIF files 
$gifRR = $gifPrefix . "rr";

system("cp $outputDir/$trapSetName.raw.GIF $gifRR.GIF");
system("cp $outputDir/$trapSetName.24.hi.GIF $gifPrefix'24'.GIF");
system("cp $outputDir/$trapSetName.00.GIF $gifPrefix'00'.GIF");
system("cp $outputDir/$trapSetName.06.GIF $gifPrefix'06'.GIF");
system("cp $outputDir/$trapSetName.12.GIF $gifPrefix'12'.GIF");
system("cp $outputDir/$trapSetName.18.GIF $gifPrefix'18'.GIF");


if ($dataSource eq "SSMI")
{
   print "Call AREAL-TRAP for SSMI\n";
   #system("batch.k $lat $lon $stormName $agency $bulHhmm AREAL-TRAP CONTINUE=YES");
}
elsif ($dataSource eq "AMSU") 
{
   print "Call AREAL-TRAPA for AMSU\n";
   #system("batch.k $lat $lon $stormName $agency $bulHhmm AREAL-TRAPA CONTINUE=YES");
}
elsif ($dataSource eq "TRMM")
{
   print "Call AREAL-TRAPT for TRMM\n";
   #system("batch.k $lat $lon $stormName $agency $bulHhmm AREAL-TRAPT CONTINUE=YES");
}
elsif ($dataSource eq "AMSRE") 
{
   print "Call AREAL-TRAPAS for AMSRE\n";
   #system("batch.k $lat $lon $stormName $agency $bulHhmm AREAL-TRAPAS CONTINUE=YES");
}
elsif ($dataSource =~ /^HE/)
{
   print "Call AREAL-TRAPHE for TRMM\n";
   #system("batch.k $lat $lon $stormName $agency $bulHhmm AREAL-TRAPHE CONTINUE=YES");
}

# save the frames display as GIF files, frame number MUST BE CONSISTENT with 
# the above AREAL-TRAP etc. program

# fixed this on 05/10/2007
# system("frmsave.k 1 $gifPrefix'rr'.GIF FORM=GIF");

#$gifRR = $gifPrefix . "rr";
#system("frmsave.k 1 $gifRR.GIF FORM=GIF");
#system("frmsave.k 2 $gifPrefix'24'.GIF FORM=GIF");
#system("frmsave.k 3 $gifPrefix'00'.GIF FORM=GIF");
#system("frmsave.k 4 $gifPrefix'06'.GIF FORM=GIF");
#system("frmsave.k 5 $gifPrefix'12'.GIF FORM=GIF");
#system("frmsave.k 6 $gifPrefix'18'.GIF FORM=GIF");


print "End of the program.\n";
