#!/usr/bin/perl -w
#
#
# $Id: saveDigTRaP.pl,v 1.1.1.1 2010-04-27 15:13:11 cmuser Exp $
# $Log: not supported by cvs2svn $
# Revision 1.1.1.1  2009/02/04 19:25:09  cmuser
# initial release
#
# Revision 1.1  2007/11/30 19:57:01  oper1
# First checkin.
#
#
#!/usr/bin/perl -w

## =====================================================================
## Name:        saveDigTRaP.pl 
##                  
## Author:      Jianbin Yang/Perot System   03/29/2007
## Language:    Perl
## Purpose:     This program make the tar file for the digital TRaP, Area file and GIF files
##		in some time periods (specified in the input argument), 
##		usually 36 hours before landfall.  
##         	This program will may run on the SAB external hard drive where the files are saved. 
##
## Notes:	This program is for the current TRaP name with Month in the rain rate time
##
## Usage:	saveDigTRaP.pl <storm name> <country> [<start time 1> <end time 1> [<start time 2> <end time 2> [<start time 3> <end time 3>]]]
##              Note that:  the time format can be [yyyy]MMddhh['Z'], Month, date and hour are required, the others are optional
##			    time 2 and 3... input is optional                        
## 
## Modified:   03/29/2007
##
## Modified: 04/03/2007
##
## Modified: 07/09/2007
##                Rain rate time includes month, so modify the program for this
##		  Add country name input
##
## Last Modified: 09/05/2007
##                Take care of the ls for "Argument list too long",
##		  Check the size of the tar file
##
## =====================================================================


# Define the directories and files
$trapDataDir = '/data/TRAP/digitTRAP';          # the actual TRaP directory is '/data/TRAP/$year/$stormNameNNB'
$saveDataDir = '/data/TRAP/digitTRAP/save';

$tempOut = '/tmp/tarout';

# Constants and array

@months = qw(NONE Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);

# Lower limit of the tar file size to check if the tar file is empty, usually is 10240
$tarSizeLimit = 20000;

# initialize the arrays  

# @tarList = ();


##--------------------------------- Start --------------------------------------##

$startdate=`date`;
print "Start  $0 at $startdate \n";

if (@ARGV < 2)
{
   print "Usage: saveDigTRaP.pl <storm name> <country/area> [<start time 1> <end time 1> [<start time 2> <end time 2> [...]]] \n\n";
   print "Noted that: time periods input are optional.  If there is no time period input, it will save the whole storm.\n";
   print "            the time format can be [yyyy]MMddhh['Z'], Month, date and hour are required, the others are optional.\n";
   print "            time period must be input as a pair of start time and end time\n\n";
   die "No storm name input - exit program";
}
else
{
   if ($ARGV[1] =~ /\d{6,}/)
   {
      print "Second argument expects country/area input, it looks like a time input.\n";
      print "Usage: saveDigTRaP.pl <storm name> <country/area> [<start time 1> <end time 1> [<start time 2> <end time 2> [...]]] \n\n";
      print "Noted that: time periods input are optional.  If there is no time period input, it will save the whole storm.\n";
      print "            the time format can be [yyyy]MMddhh['Z'], Month, date and hour are required, the others are optional.\n";
      print "            time period must be input as a pair of start time and end time\n\n";
      die "No country/area name input - exit program";
   }

   $stormName = $ARGV[0];
   $country = $ARGV[1];
   @timeList = @ARGV;

   # remove the first two values from the array
   @tmpList = reverse(@timeList);
   $stormName1 = pop(@tmpList);
   $country1 = pop(@tmpList);

   @timeList = reverse(@tmpList);
}

print "stormName1 is $stormName1, country1 is $country1, timeList is @timeList\n";

$numOfTimePeriod = @timeList;

if (($numOfTimePeriod % 2) != 0)
{
   print "The total number of time you enter is $numOfTimePeriod.\n";
   print "Please enter the time periods as a pair of start time and end time. Exit program.\n";
   #die "Please enter the time periods as a pair of start time and end time. Exit program\n";
}

# create the storm directory temporaly for the tar file, and delete later

chdir("$trapDataDir") || die "Cannot change to directory $trapDataDir: $!";
print  "TRAP Data directory is:";
system("pwd");


$curYear = `date +%Y`;
chomp($curYear);
$lastYear = $curYear - 1;

$curTime = `date +%m%d%H`;
chomp($curTime);

$tarNamePre = $stormName . '-' . $country . "-DIGIT";
$tarNameSuf = ".tar";

$tarName2 = $stormName . "-DIGIT" . $curTime . "Z.tar";


##------------------------------------------------------------------------------------------##

##------------ for the case without any time period input from the command line ------------##

if ($numOfTimePeriod == 0)
{
   print "Will save all the digital TRaP for the whole life of the storm $stormName.\n"; 
   if (-f "$saveDataDir/$tarName")
   {
      print "$saveDataDir/$tarName exists, create the tar file with other name $tarName2\n";
      $tarName = $tarName2;
   }

   if (-d "$trapDataDir/$curYear/$stormName/")
   {
      $year = $curYear;
   }
   elsif (-d "$trapDataDir/$lastYear/$stormName/") 
   {
      $year = $lastYear;
   }
   else
   {
      die "Error!!  There is no this storm $stormName exists in the system,  Please check again.\n";
   }

   # create the storm directory temporaly for the tar file, and delete later
   chdir("$saveDataDir") || die "Cannot change to directory $saveDataDir: $!";

   if (!(-d $year))
   {
      print "Create the directory '$year' for the current year.\n";
      mkdir($year,0755) || die "cannot mkdir $year: $!";
   }

   if (!(-d $stormName))
   {
      print "Create the directory '$stormName' temporary for saving .\n";
      mkdir($stormName,0755) || die "cannot mkdir $stormName: $!";
   }

   chdir("$trapDataDir") || die "Cannot change to directory $trapDataDir: $!";
   print  "TRAP Data directory is:";
   system("pwd");

   $tarName = $tarNamePre . $tarNameSuf;
 
   chdir ("$year") || die "Cannot change to directory $year: $!";
   system("pwd");
   system("tar cvf '$saveDataDir/$year/$tarName' $stormName> $tempOut");

   system ("rm -r $saveDataDir/$year/$stormName");

   if ((-s "$saveDataDir/$year/$tarName") < $tarSizeLimit)
   {
      $size = -s "$saveDataDir/$year/$tarName";
      print "\nWARNING!!! The tar file is too small and maybe empty!  The size of the tar file is $size\n\n";
   }

   print "\nEnd of saving the TRAP data, the location and name of the tar file is $saveDataDir//$year/$tarName\n";

   exit;
}

##---------------------------------------------------------------------------------##

##------------ for those with time period(s) input in the command line ------------##

@saveList = ();

for ($i = 0; $i < $#timeList+1; $i++)
{
  $stTime = $timeList[$i];
  $endTime = $timeList[++$i];
  print "stTime is $stTime, endTime is $endTime\n";

  # time format is [yyyy]MMddhh['Z']


  #------------ Get the start date and time ------------#

  if ($stTime =~ /^\d{10}Z?$/i)
  {
     $stYear = substr($stTime, 0, 4);
     $stMon  = substr($stTime, 4, 2);
     $stDd   = substr($stTime, 6, 2);
     $stHh   = substr($stTime, 8, 2);
  }
  elsif ($stTime =~ /^\d{6}Z?$/i)
  {
     $stMon  = substr($stTime, 0, 2);
     $stDd   = substr($stTime, 2, 2);
     $stHh   = substr($stTime, 4, 2);

     # get the year
     # if the storm falls in both years, check more
     if ((-d "$trapDataDir/$curYear/$stormName/") && (-d "$trapDataDir/$lastYear/$stormName/"))
     {
        if (($stMon == 0) || ($stMon > 12))
        {
           die "Error!! Please check the input start time format -  month is out of range.\n";
        }
        $mon = $months[$stMon]; 
        
        $curList=`ls -l $trapDataDir/$curYear/$stormName/$curYear$stormName*|grep $mon`;
        $lastList=`ls -l $trapDataDir/$lastYear/$stormName/$lastYear$stormName*|grep $mon`; 
  
        if ($curList ne "")
        {
           $stYear = $curYear;
        }
        elsif ($lastList ne "")
        {
           $stYear = $lastYear;
        }
        else
        {
           die "Error!! No data match.  Please check the input start time.\n";
        }
 
     }
     elsif (-d "$trapDataDir/$curYear/$stormName/")
     {
        $stYear = $curYear;
     }
     elsif (-d "$trapDataDir/$lastYear/$stormName/")
     {
        $stYear = $lastYear;
     }
     else
     {
        die "Error!! No data found for storm $stormName in year $curYear or $lastYear, please spedify the year or check again.\n";
     }

  }
  else
  {
     die "Error!! Please check the input start time format.  Should be [yyyy]MMddhh['Z'].\n";
  }


  # Check the time input start format
  
  if (($stMon == 0) || ($stMon > 12))
  {
     die "Error!! Please check the input start time format -  month is out of range.\n";
  }

  if (($stDd == 0) || ($stDd > 31) || ($stHh > 24))
  {
     die "Error!! Please check the input start time format -  date or hour is out of range.\n";
  }
  
  # $stDdhh = $stDd . $stHh;
  $stMmDdhh = $stMon . $stDd . $stHh;

print "stYear is $stYear. stMmDdhh is $stMmDdhh\n\n";

  #------------ Get the End date and time ------------#  
  
  if ($endTime =~ /^\d{10}Z?$/i)
  {
     $endYear = substr($endTime, 0, 4);
     $endMon  = substr($endTime, 4, 2);
     $endDd   = substr($endTime, 6, 2);
     $endHh   = substr($endTime, 8, 2);
  }

  elsif ($endTime =~ /^\d{6}Z?$/i)
  {
     $endMon  = substr($endTime, 0, 2);
     $endDd   = substr($endTime, 2, 2);
     $endHh   = substr($endTime, 4, 2);

     # get the year
     # if the storm falls in both years, check more
     if ((-d "$trapDataDir/$curYear/$stormName/") && (-d "$trapDataDir/$lastYear/$stormName/"))
     {
        if (($endMon == 0) || ($endMon > 12))
        {
           die "Error!! Please check the input end time format -  month is out of range.\n";
        }
        $mon = $months[$endMon];

        $curList=`ls -l $trapDataDir/$curYear/$stormName/$curYear$stormName*|grep $mon`;
        $lastList=`ls -l $trapDataDir/$lastYear/$stormName/$lastYear$stormName*|grep $mon`;

        if ($curList ne "")
        {
           $endYear = $curYear;
        }
        elsif ($lastList ne "")
        {
           $endYear = $lastYear;
        }
        else
        {
           die "Error!! No data match.  Please check the input end time.\n";
        }

     }
     elsif (-d "$trapDataDir/$curYear/$stormName/")
     {
        $endYear = $curYear;
     }
     elsif (-d "$trapDataDir/$lastYear/$stormName/")
     {
        $endYear = $lastYear;
     }
     else
     {
        die "Error!! No data found for storm $stormName in year $curYear or $lastYear, please spedify the year or check again.\n";
     }

  }
  else
  {
     die "Error!! Please check the input end time format.  Should be [yyyy]MMddhh['Z'].\n";
  }


  # Check the time input end format

  if (($endMon == 0) || ($endMon > 12))
  {
     die "Error!! Please check the input end time format -  month is out of range.\n";
  }

  if (($endDd == 0) || ($endDd > 31) || ($endHh > 24))
  {
     die "Error!! Please check the input end time format -  date or hour is out of range.\n";
  }

  #$endDdhh = $endDd . $endHh;
  $endMmDdhh = $endMon . $endDd . $endHh;

print "endYear is $endYear. endMmDdhh is $endMmDdhh\n\n";

  #---------------------------------------------------------------------------#

  #------------ Get the list of files to save if the year is same ------------#

  # only do that for the first start year
  if ($i == 1)
  {
     # create the storm directory temporaly for the tar file, and delete later
     chdir("$saveDataDir") || die "Cannot change to directory $saveDataDir: $!";

     if (!(-d $stYear))
     {
        print "Create the directory '$stYear' for the current year.\n";
        mkdir($stYear,0755) || die "cannot mkdir $stYear: $!";
     }

     chdir("$stYear") || die "Cannot change to directory $stYear";

     if (!(-d $stormName))
     {
        print "Create the directory '$stormName' temporary for saving .\n";
        mkdir($stormName,0755) || die "cannot mkdir $stormName: $!";
     }

     $stYear1 = $stYear;
     $stMmDdhh1 = $stMmDdhh;
print "stYear1 is $stYear1, stMmDdhh1 is $stMmDdhh1\n";
 }
#  chdir("$trapDataDir") || die "Cannot change to directory $trapDataDir: $!";
#  print  "TRAP Data directory is:";
#  system("pwd");



  if ($stYear == $endYear)
  {
     if (($endMon - $stMon) > 2 )
     {
        die "Error!! Time period should not be longer than 2 months, Please check the time again.\n";
     }

     chdir ("$trapDataDir/$stYear/$stormName") || die "Cannot change to directory $stYear/$stormName: $!";

     # @list=`ls  -rt $stYear$stormName* ` ;
     @list=`ls  -rt |grep "$stYear$stormName" ` ;
     chomp(@list);

     foreach $filename (@list)
     {
        @splitMsg = split(/\./, $filename);
        $splitMsgLen = @splitMsg;

        if ($splitMsgLen < 6)
        {
           print "The file $filename is not a TRaP file, skip.\n";
           next;
        }

        # $rrDdhh = substr($splitMsg[5],0, 4);
        $rrMmDdhh = substr($splitMsg[5],0, 6);

        if ($stMmDdhh > $endMmDdhh)
        {
           die "Error!! Start time is later than the end time, Please check date and hour again.\n";
        }

        if (($rrMmDdhh >= $stMmDdhh) && ($rrMmDdhh <= $endMmDdhh))
        {
           rename("$filename", "$saveDataDir/$stYear1/$stormName/$filename");
           push(@saveList, $filename);
        }
       
     } 	# End of the Foreach for the file lists if the year is same

  }	

  #------------ Get the list of files to save if the year is not the same ------------#

  elsif ($stYear < $endYear)
  {
     if (($stMon < $endMon) || (($endYear - $stYear) > 1) )
     {
        die "Error!! Time period should not be longer than a year, Please check the time again.\n";  
     }
     elsif (($stMon < 11) || ($endMon > 2))
     {
        die "Error!! Time period should not be longer than 2 months, please check the time again.\n";
     }

     chdir ("$trapDataDir/$stYear/$stormName") || die "Cannot change to directory $stYear/$stormName: $!";

     #@list=`ls  -rt $stYear$stormName* ` ;
     @list=`ls  -rt |grep "$stYear$stormName" ` ;
     chomp(@list);

     foreach $filename (@list)
     {
        @splitMsg = split(/\./, $filename);
        $splitMsgLen = @splitMsg;

        if ($splitMsgLen < 6)
        {
           print "The file $filename is not a TRaP file, skip.\n";
           next;
        }

        # $rrDdhh = substr($splitMsg[5],0, 4);
        $rrMmDdhh = substr($splitMsg[5],0, 6);

        if ($rrMmDdhh >= $stMmDdhh) 
        {
           rename("$filename", "$saveDataDir/$stYear1/$stormName/$filename");
           push(@saveList, $filename);
        }

     }	# End of the Foreach loop for start year

     # for end year

     chdir ("$trapDataDir/$endYear/$stormName") || die "Cannot change to directory $endYear/$stormName: $!";

     #@list=`ls  -rt $endYear$stormName* ` ;
     @list=`ls  -rt|grep "$endYear$stormName" ` ;
     chomp(@list);

     foreach $filename (@list)
     {
        @splitMsg = split(/\./, $filename);
        $splitMsgLen = @splitMsg;

        if ($splitMsgLen < 6)
        {
           print "The file $filename is not a TRaP file, skip.\n";
           next;
        }

        $rrMmDdhh = substr($splitMsg[5],0, 6);

        if ($rrMmDdhh <= $endMmDdhh)
        {
           rename("$filename", "$saveDataDir/$stYear1/$stormName/$filename");
           push(@saveList, $filename);
        }

     }  # End of the Foreach loop for end year


  }	# End of the else/if for different year

}	# End of the outer FOR loop for the  time periods

# print "saveList is @saveList\n";

##-------------------------- Get the tar file --------------------------##

chdir ("$saveDataDir/$stYear1") || die "Cannot change to directory $saveDataDir/$stYear1: $!";
system("pwd");
#if (-f "$saveDataDir/$stYear1/$tarName")
#{
#   print "$saveDataDir/$tarName exists, create the tar file with other name $tarName2\n";
#   $tarName = $tarName2;
#}

$tarName = $tarNamePre . $stMmDdhh1 . 'Z' . $tarNameSuf;

system("tar cvf '$tarName' $stormName> $tempOut");

##------------ Move the TRaP files back to the original dir ------------##

chdir ("$stormName") || die "Cannot change to directory $stormName: $!";

@list=`ls  -rt *$stormName* ` ;
chomp(@list);

foreach $filename (@list)
{
   @splitMsg = split(/\./, $filename);
   $splitMsgLen = @splitMsg;

   if ($splitMsgLen < 6)
   {
      print "The file $filename is not a TRaP file, skip.\n";
      next;
   }

   $yyyy = substr($splitMsg[0], 0, 4);
#   print "rename to $trapDataDir/$yyyy/$stormName/$filename.\n";
   rename("$filename", "$trapDataDir/$yyyy/$stormName/$filename");
}

$lsOut = `ls`;

if ($lsOut ne "")
{
   print "WARNING!! Some TRaP files maybe not be moved back to their original dir.\n";
}
else
{
   chdir ("$saveDataDir") || die "Cannot change to directory $saveDataDir: $!";
   print "Delete the storm dir under $saveDataDir/$stYear1.\n";
   system ("rm -r $saveDataDir/$stYear1/$stormName");
}


print  "Current Data directory is:";
system("pwd");

if ((-s "$saveDataDir/$stYear1/$tarName") < $tarSizeLimit)
{
   $size = -s "$saveDataDir/$stYear1/$tarName";
   print "\nWARNING!!! The tar file is too small and maybe empty!  The size of the tar file is $size\n\n";
}

print "\nEnd of saving the TRAP data, the location and name of the tar file is $saveDataDir/$stYear1/$tarName\n";

