#!/usr/bin/perl -w

## =====================================================================
## Name:        saveTRAPGIF.pl 
##              (Adapt from restoreAREA.pl)
##                  
## Author:      Jianbin Yang/RSIS   07/10/2002
## Language:    Perl
## Purpose:     Call the batch files to display TRaP AREA files and save in GIF Format
##              the program is called within the Main pctrap program and use AREA0630-0637            
##                  
## Usage:       saveTRaPGIF.pll <Name of the TRaP set> 
##              Format of <Name of the TRaP set> is: 
##                     YYYYnameNNB.WTXXnn.FFFF.DDHHMM.SSSS.ddhhmm 
##                  or YYYYnameNNB.WTXXnn.FFFF.DDHHMM.SSSS.ddhhmm.nn
##                  or YYYYnameNNB.WTXXnn.FFFF.DDHHMM.SSSS.ddhhmm.nn.GIF
##
## Modified:   09/25/2002
##
## Modified:   02/26/2004
##              Save frames display as the GIF files, Noted that the frame number
##              MUST BE CONSISTENT with the AREAL batch programs.
##              This GIF files will be used in the area2digit.pl
##
## Modified:  04/17/2007
##		Center image on average of initial and 24 hour forecast positions
##		Other than the 12 hour forecast position
##
## Modified:  09/21/2007
##		Add Amsre data
##
## Modified: 10/06/2008
##              Add the Satellite ID to the GIF files, correct the sensor info for MHS
## 
## Last Modified: 11/29/2011
##              Added SSMIS source
## 
## =====================================================================

# Define the directories and files

$HOME = $ENV{"HOME"};
$DATAHOME = $ENV{"DATAHOME"};
$dataDir = "$DATAHOME/data/mcidas_data";
#$trapDataDir = '/data/TRAP';          # the actual directory is '/data/TRAP/$year/$stormNameNNB'

# Correct the following at 09/19/07 19Z, POS1.TXT has display center
# $positionFile = 'POS.TXT';           # File to keep the position of the track of the storm
$positionFile = 'POS1.TXT';           # File to keep the position of the track of the storm
# $totalGifFile = 'TRAP24hh.GIF';       # This GIF file may be used in the area2digital.pl program 
				      # to send to FTP server along with the digital TRaP file
 				      # Added on 07/25/2003

# Sensor to Satellite ID table
%sensorTbl = qw(
   NOAA-15 AMSU-B NOAA-16 AMSU-B NOAA-17 AMSU-B NOAA-18 MHS NOAA-19 MHS METOP-A MHS METOP-B MHS
   TRMM TMI
   DMSP-F13 SSMI DMSP-F14 SSMI DMSP-F15 SSMI
   DMSP-F16 SSMIS DMSP-F17 SSMIS DMSP-F18 SSMIS
   AQUA AMSRE
   GOES-10-12 GHE-1HR
);


$gifPrefix = 'TRAP';
$startdate=`date`;

$centerStr = "Image display center";  # must consistant with the one in pctrap-auto.pl

print "Start  $0 at $startdate \n";

chdir("$dataDir") || die "Cannot change to directory $dataDir";
print  "Data directory is:";
system("pwd");

# remove the path from the command name

@comandFields = split(/\//, $0);
$commandName = $comandFields[-1];

# if no argument input, exit the program

# if ($ARGV[0] eq "")
if ($#ARGV == -1)
{
   print "Usage: $commandName <Name of the TRaP set>\n";
   print "Format of <Name of the TRaP set> is: YYYYnameNNB.WTXXnn.FFFF.DDHHMM.SSSS.ddhhmm\n";
   die "No name input - exit script";
}

$trapSetName = $ARGV[0];

open STDOUT, ">/data/Petrap/log/saveGif.out" || die "Can't create saveGif.out:$!";
open STDERR, ">/data/Petrap/log/saveGif.err" || die "Can't create saveGif.err:$!";


print "trapSetName is $trapSetName.\n";
print "HOME in STG $HOME \n";

# get the directories name of the specific TRaP set

@splitMsg = split(/\./, $trapSetName);
$splitMsgLen = @splitMsg;

# printf ("the number of splitMsg is %d.\n",($#splitMsg + 1));

if ($splitMsgLen < 6)
{
   print "Format of <Name of the TRaP set> is: 
          \rYYYYnameNNB.WTXXnn.FFFF.DDHHMM.SSSS.ddhhmm(.nn.GIF)\n\n";
   die "Input TRaP set name is not long enough or not correct - exit";
}
elsif ($splitMsgLen > 6)
{
   $trapSetName = $splitMsg[0];
   
   for ($i = 1; $i < 6; $i++)
   {
      $trapSetName .= '.';
      $trapSetName .= $splitMsg[$i];
   }
   print "The true TRaP set name is $trapSetName\n";
}


$dirMsg = $splitMsg[0];
$year = substr($dirMsg, 0, 4);
# $nameNnb = substr($dirMsg,4, (length($dirMsg) - 4));
$nameNnb = substr($dirMsg,4);
# $outputDir = "$trapDataDir/$year/$nameNnb";

print "dirMsg is $dirMsg, year is $year, nameNNB is $nameNnb.\n";
# printf ("the length of dirMsg is %d.\n",length($dirMsg));


# restore the TRaP AREA files to AREA0640-0647

#system("cp $outputDir/$trapSetName.raw AREA0640");
#system("cp $outputDir/$trapSetName AREA0641");
#system("cp $outputDir/$trapSetName.24 AREA0642");
#system("cp $outputDir/$trapSetName.00 AREA0643");
#system("cp $outputDir/$trapSetName.06 AREA0644");
#system("cp $outputDir/$trapSetName.12 AREA0645");
#system("cp $outputDir/$trapSetName.18 AREA0646");
#system("cp $outputDir/$trapSetName.24.hi AREA0647");


# isolate the variable names from the command line: storm name, 
# bulletin id, bulletin time, and satellite

$stormName = $nameNnb;
$bulID = $splitMsg[1];
$agency = $splitMsg[2];
$bulTime = $splitMsg[3];
$dataSource = $splitMsg[4];

$bulHhmm = substr ($bulTime, 2, 4);

# get the latitude and longitude of the TRaP from POS2.TXT 
print "Get the 12 hour point position.\n";

open (POSTXT, $positionFile) || die "can't open $positionFile: $!";

while ( defined ($tempMsg = <POSTXT>))
{
   if ($tempMsg =~ /^12/)
   {
      print "get the right msg:$tempMsg \n";
      last;
   }
}

close (POSTXT) || die "can't open $positionFile: $!";

# @splitMsg2 = split(/\s{2,}/, $tempMsg);
@splitMsg2 = split(/\s+/, $tempMsg);

$lat12 = $splitMsg2[1];
$lon12 = $splitMsg2[2];

##------------- Get the average of initial and 24 hour forecast positions -------------##

$tempMsg = `grep '$centerStr' $positionFile`;
@splitMsg2 = split(/\s+/, $tempMsg);

$lat = $splitMsg2[-2];
$lon = $splitMsg2[-1];

print "lat12 is $lat12, lon12 is $lon12, the $centerStr is $lat, $lon\n";

if ($agency eq "KNHC") {
$agency = 'TPC'; }

if ($agency eq "PGTW") {
$agency = 'JTWC'; }

print "StormName is $stormName, bulID is $bulID, agency is $agency, bul Time is $bulHhmm,
       \rdataSource is $dataSource, lat is $lat, lon is $lon \n";


##------------- Get the Satellite ID and correct the Sensor -------------##

$satID = ` sed -n '12p' $positionFile | awk '{print \$1}'`;
chomp($satID);

$sensor = $sensorTbl{$satID};
print "Satellite ID is $satID, sensor is $sensor.\n"; 

 
print "Start to display the 24 hour TRaP and rain rate image...\n";
print "current directory before AREAL batch  ";
system("pwd");
print "datasource before block $dataSource \n";

system("cp -v AREA06* $HOME/mcidas/data/");

if ($dataSource eq "SSMI")
{
   print "Call AREAL-AUTOTRAP for SSMI\n";
   #system("batch.k $lat $lon $stormName $agency $bulHhmm AREAL-TRAP1 CONTINUE=YES");
   system("batch.k $lat $lon $stormName $agency $bulHhmm $satID $sensor ${dataDir}/AREAL-AUTOTRAP CONTINUE=YES");
}
elsif ($dataSource eq "SSMISC") 
{
   print "Call AREAL-AUTOTRAP for SSMIS\n";
   #system("batch.k $lat $lon $stormName $agency $bulHhmm AREAL-TRAP1 CONTINUE=YES");
   system("batch.k $lat $lon $stormName $agency $bulHhmm $satID $sensor ${dataDir}/AREAL-AUTOTRAP CONTINUE=YES");
}
elsif ($dataSource eq "SSMISB") 
{
   print "Call AREAL-AUTOTRAP for SSMIS\n";
   #system("batch.k $lat $lon $stormName $agency $bulHhmm AREAL-TRAP1 CONTINUE=YES");
   system("batch.k $lat $lon $stormName $agency $bulHhmm $satID $sensor ${dataDir}/AREAL-AUTOTRAP CONTINUE=YES");
}
elsif ($dataSource eq "SSMISA") 
{
   print "Call AREAL-AUTOTRAP for SSMIS\n";
   #system("batch.k $lat $lon $stormName $agency $bulHhmm AREAL-TRAP1 CONTINUE=YES");
   system("batch.k $lat $lon $stormName $agency $bulHhmm $satID $sensor ${dataDir}/AREAL-AUTOTRAP CONTINUE=YES");
}
elsif ($dataSource eq "AMSUNN") 
{
   print "Call AREAL-AUTOTRAPA for AMSU\n";
   system("batch.k $lat $lon $stormName $agency $bulHhmm $satID $sensor ${dataDir}/AREAL-AUTOTRAPA CONTINUE=YES");
}
elsif ($dataSource eq "AMSUNP") 
{
   print "Call AREAL-AUTOTRAPA for AMSU\n";
   system("batch.k $lat $lon $stormName $agency $bulHhmm $satID $sensor ${dataDir}/AREAL-AUTOTRAPA CONTINUE=YES");
}
elsif ($dataSource eq "AMSUM2") 
{
   print "Call AREAL-AUTOTRAPA for AMSU\n";
   system("batch.k $lat $lon $stormName $agency $bulHhmm $satID $sensor ${dataDir}/AREAL-AUTOTRAPA CONTINUE=YES");
}
elsif ($dataSource eq "AMSUM1") 
{
   print "Call AREAL-AUTOTRAPA for AMSU\n";
   system("batch.k $lat $lon $stormName $agency $bulHhmm $satID $sensor ${dataDir}/AREAL-AUTOTRAPA CONTINUE=YES");
}
elsif ($dataSource eq "TRMM")
{
   print "Call AREAL-AUTOTRAPT for TRMM\n";
   system("batch.k $lat $lon $stormName $agency $bulHhmm $satID $sensor ${dataDir}/AREAL-AUTOTRAPT CONTINUE=YES");
}
elsif ($dataSource eq "AMSRE")
{
   print "Call AREAL-AUTOTRAPAS for AMSRE\n";
   system("batch.k $lat $lon $stormName $agency $bulHhmm $satID $sensor ${dataDir}/AREAL-AUTOTRAPAS CONTINUE=YES");
}
elsif ($dataSource =~ /^GHE/)
{
   print "Call AREAL-TRAPGHE1 for H-E\n";
   system("batch.k $lat $lon $stormName $agency $bulHhmm ${dataDir}/AREAL-TRAPGHE1 CONTINUE=YES");
}

# save the frames display as GIF files, frame number MUST BE CONSISTENT with 
# the above AREAL-TRAP etc. program

print "Save the GIF files\n";

$gifRR = $gifPrefix . "rr";
system("frmsave.k 1 '$DATAHOME/data/'$gifRR.GIF FORM=GIF");
system("frmsave.k 2  '$DATAHOME/data/'$gifPrefix'24'.GIF FORM=GIF");
system("frmsave.k 3  '$DATAHOME/data/'$gifPrefix'00'.GIF FORM=GIF");
system("frmsave.k 4  '$DATAHOME/data/'$gifPrefix'06'.GIF FORM=GIF");
system("frmsave.k 5  '$DATAHOME/data/'$gifPrefix'12'.GIF FORM=GIF");
system("frmsave.k 6  '$DATAHOME/data/'$gifPrefix'18'.GIF FORM=GIF");


print "End of the program.\n";
