#!/usr/bin/perl -w
#
#
# $Id: updateBasinFile.pl,v 1.1.1.1 2010-04-27 15:13:11 cmuser Exp $
# $Log: not supported by cvs2svn $
# Revision 1.1.1.1  2009/02/04 19:25:09  cmuser
# initial release
#
# Revision 1.1  2007/11/30 19:57:01  oper1
# First checkin.
#
#
#!/usr/bin/perl -w

## =====================================================================
## Name:        updateBasinFile.pl
##                  
## Author:      Jianbin Yang/QSS   10/11/2006
## Language:    Perl
## Purpose:     The program creates the basin text file in the Web server, 
##              or update or delete the entry for a storm within the basin text file
## 
## Usage:       updateBasinFile.pl <Year> <Storm Name> <basin Area> <Update Mode>
##             
##              "Update Mode" values: 
##               1 - Create New basin Text file
##               2 - Add the entry for the storm in the basin file
##               3 - Delete the entry for an inactive storm
##                   or delete the file if there is no entry 
##
## PLEASE NOTE: This script will run on the web server remotely             
##                  
## Modified:   10/16/2006
##                  Check the basin file and switch Mode if neccessary
##
## Last Modified: 03/19/2007
##		Change the storm page name to include the basin abbreviation
##
## =====================================================================

# Define the directories and files

$webHomeDir = "/home/sabweb/TRAP";

$trapDataDir = "/data/web/mirror/www.ssd.noaa.gov/htdocs/PS/TROP/DATA/TRAP";

$templateFile = "$webHomeDir/basin-template.txt";
$blankFile = "$webHomeDir/blank-template.txt";
$newFile = "$trapDataDir/newBasin.txt";

# define some constants list in the basin template file
$tempYear = "YEAR";
$tempStorm = "STORM_NAME";
$tempStormLow = "STORMNAME_LOW";

#--------- basins and basin abbreviation ---------#

# Define the hash table for the basin (dir here) and the corresponding basin abbreviation (basin here)
# This hash table should be the same as the one in area2digit.pl

%basinTbl = qw(
   Atlantic AL
   EastPacific EP
   CentralPacific CP
   NWPacific WP
   SWPacific SW
   SEPacific SE
   NorthIndian NI
   SWIndian WI
   SEIndian EI
   NTerritory NT
);


##----------------------------- Start the Program -----------------------------##

$startdate=`date`;
print "\nStart  $0 at $startdate \n\n";

# remove the path from the command name

@comandFields = split(/\//, $0);
$commandName = $comandFields[-1];

# if no argument input, exit the program

if (@ARGV < 4)
{
   print "Usage: $commandName <Year> <Storm Name> <basin Area> <Mode>\n";
   die "Not enough input arguments - exit script";
}
else
{
   $year = $ARGV[0];
   $stormName = $ARGV[1];
   $basinArea = $ARGV[2];
   $mode =  $ARGV[3];
}

$basinFile = $basinArea . ".txt";
$oldFile = $basinArea . ".txt.old";

$stormNameLow = $stormName;
$stormNameLow =~ tr/A-Z/a-z/;
$stormNameLow = substr($stormName, 0,1) . substr($stormNameLow, 1);

# Add the sasin abbreviation to the storm name
# $stormNameLow = $stormNameLow . '_' . $basinTbl{$basinArea};
$stormNameLow = $stormNameLow . '-' . $basinTbl{$basinArea};

print "Year is $year, StormName is $stormName, basinArea is $basinArea, update mode is $mode, basinFile is $basinFile.\n";

chdir("$trapDataDir/$year") || die "Cannot change to directory $trapDataDir/$year";
print  "Work directory is:";
system("pwd");

# check the basin file for update mode - 2, if line number is 1, means blank, then switch to create mode -1

if ($mode == 2)
{
   $lineNum = `wc -l $basinFile | awk '{print \$1}'`;
   chomp($lineNum);
   print "line Number is $lineNum\n";
   $line = `cat $basinFile`;
   chomp($line);
   print "line is **$line**\n";
   if (($lineNum == 1) && ($line eq "<br>"))
   {
   
      print "Mode was $mode, switch to create mode - 1\n";
      $mode = 1;
   }
}


##-------------------------------------------------------------------------------------------------##

##------------------------ Start to Create/Update the basin HTML text file ------------------------##


if ($mode == 1)
{
   if (-s $basinFile)
   {
      print "WARNING!! $basinFile will be overwritten!\n";
      rename($basinFile, $oldFile)          or die "can't rename $basinFile to $oldFile: $!";
   }

   #system("cp -p $templateFile $basinFile");
   #system("sed -e 's/$tempYear/$year/' -e 's/$tempStormLow/$stormNameLow/' -e 's/$tempStorm/$stormName/' $basinFile > basin_tm.txt");
   system("sed -e 's/\\/$tempYear/\\/$year/' -e 's/$tempStormLow/$stormNameLow/' -e 's/$tempStorm/$stormName/' $templateFile > $basinFile");
   
   #system("mv basin_tm.txt $basinFile");
}
elsif ($mode == 2)
{
   $stormLine = `grep $tempStorm $templateFile`;
   chomp($stormLine);
   $stormLine =~ s/\/$tempYear/\/$year/;
   $stormLine =~ s/$tempStormLow/$stormNameLow/;
   $stormLine =~ s/$tempStorm/$stormName/g;
   print "The added stormLine is $stormLine\n";
 
   open(BASINFILE, "< $basinFile") || die "Can not open $basinFile for reading: $!";
   @dataList = <BASINFILE>;
   close(BASINFILE);

   open(NEWFILE, "> $newFile") || die "Can not open $newFile for writing: $!";
   flock(NEWFILE, 2);         # and lock it
 
   $i = 0;
     
   while (($dataList[$i] !~ /^\<\/blockquote\>$/) && ($i < $#dataList+1))
   {
      chomp($dataList[$i]);
      print NEWFILE "$dataList[$i]\n";
      $i++;
   }
  
   print NEWFILE "<br>\n";
   print NEWFILE "$stormLine\n";
   chomp($dataList[$i]);  
   print NEWFILE "$dataList[$i]\n";

   flock(NEWFILE, 8);         # and unlock it
   close(NEWFILE) || die "Can not close $newFile: $!";

   rename($basinFile, $oldFile)          or die "can't rename $basinFile to $oldFile: $!";
   rename($newFile, $basinFile)          or die "can't rename $newFile to $basinFile: $!";
}
elsif ($mode == 3)
{
   $stormLine = `grep $tempStorm $templateFile`;
   chomp($stormLine);
   $stormLine =~ s/\/$tempYear/\/$year/;
   $stormLine =~ s/$tempStormLow/$stormNameLow/;
   $stormLine =~ s/$tempStorm/$stormName/g;
   print "The deleted stormLine is $stormLine\n";

   $stormList = `grep -i "\<a href=" $basinFile`;

   if ($stormList eq "")
   {
      die "ERROR: There is no any storm entry in the $basinFile, exit program $commandName for Mode 3\n";
   }
   elsif ($stormList !~ /$stormLine/)
   {
      print "The stormLine is $stormLine\n";
      die "ERROR: There is no entry for storm $stormName in the $basinFile, exit program $commandName for Mode 3\n";
   }

   @splitMsg = split(/\n/, $stormList);
   $lineNum = @splitMsg;
   if ($lineNum == 1)
   {
      system("cp $blankFile $basinFile");
      # Not sure if SAB want to do the following, 10/13/2006
      # system("rm -r $basinArea");
   }
   elsif ($lineNum > 1)
   {
      @newDataList = ();

      open(BASINFILE, "< $basinFile") || die "Can not open $basinFile for reading: $!";
      @dataList = <BASINFILE>;
      close(BASINFILE);

      for ($i = 0; $i < $#dataList+1; $i++)
      {
         chomp($dataList[$i]);
         if ($dataList[$i] !~ /$stormLine/)
         {
            push(@newDataList, $dataList[$i]);
         }
         else
         {
            # remove the <br> line before or after the storm line
            if ($dataList[$i+1] =~ /^<br>/)
            {
              $i++;
            }
            else
            {
               $lastEle = pop(@newDataList); 
               print "the last element to pop is $lastEle\n";
            }
         }
      }
    
      open(NEWFILE, "> $newFile") || die "Can not open $newFile for writing: $!";
      flock(NEWFILE, 2);  # and lock it

      for ($i = 0; $i < $#newDataList+1; $i++)
      {
         print NEWFILE "$newDataList[$i]\n";
      }
      close(NEWFILE);

      rename($basinFile, $oldFile)          or die "can't rename $basinFile to $oldFile: $!";
      rename($newFile, $basinFile)          or die "can't rename $newFile to $basinFile: $!";

      # Not sure if SAB want to do the following, 10/13/2006
      # system("rm -r $basinArea/$stormName");

   }  	# End of the else if for line numbe > 1
   else
   {
      print "line Number is $lineNum.\n";
      die "ERROR: There is no any storm entry in the $basinFile, exit program $commandName for Mode 3\n";
   }
}

#system("chmod +x $basinFile");
chmod(0755, $basinFile);

$enddate=`date`;
print "\nEnd of the program $0 at $enddate \n\n";

