/*********************************************************************** 
File name: dist2.c (dist2-2006.c) 
Author:    Jianbin Yang/RSIS        04/26/2002

Purpose:
        This program read the binary geolocations file of the pass and
        get all the lat/lon of the center of the pass, and calculate
        great-circle distance between each of them and the input point
        (the center of the storm). Then compare the distance with the
        input Maximum cover distance to determine the pass cover the
        sotrm or not.

Usage:  dist <pass file name> <coverDistance> <lat> <lon>
  
Note:
        Calculations assume a spherical Earth with radius 6367 km.  
        (I think this should cause no more than 0.2% error.)

Modifiedi at:  11/22/2002
        SSMI geolocation file will be added the pass time as third 
	argument in every line.  The modification is to handle this
        change, so that the third or fourth argument in every line
        is ignore by the program. Only the first two convert to
        lat and lon.

Modified : 03/22/2005
        Filter out more pass by checking the distance to the ends (2)
        of the pass. must be >= 200 km for inside of the orbit, and eliminate
        all if outside of the orbit

modified :04/07/2005
modified :12/17/2005
modified: 08/21/2006
Modified: 08/17/2007
               print the center location of the pass which is closest to the input point
     
Modified: 09/07/2007
         	Add the AMSRE data

Modified: 02/19/2008
                For running in the IBM server.  Swap 2/4 bytes for AMSRE

Modified: 03/10/2008
                Fixed the numScan for SSMI file

Modified: 11/29/2011
                Refined logging by adding header and trailer

Last Modified: 11/29/2011
                Added SSMIS sources

***********************************************************************/


#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <memory.h>
#include <ctype.h>
#include <math.h>

/* approximate radius of Earth in kilometers.  True radius varies from
   6357km (polar) to 6378km (equatorial).  
*/
#define EARTH_RADIUS  6368 
#define PI (atan2(1, 1) * 4)       /* define an accurate value for PI */

#define SSMI_PATTERN "SSGEO.NPR.EDRR"
#define SSMIS_PATTERN "GEO.EDEB"
#define TRMM_PATTERN "trmm."
#define AMSRE_PATTERN "amsre.Rain"
#define ACCE_PATTERN "_A"		/* Accending pattern for AMSRE pass */
#define MIRS_PATTERN "geo.ndemirs" 
#define AMSR2_PATTERN "geo.amsr2" 
#define GMI_PATTERN "geo.gpmmi"

// 09/07/2007, work up to here to add the AMSRE, will check if it is an ascending '_A', and if the lat <-10
// if all fit, return isCover=0, and skip the checking. Noted that this checking is placed in the Perl program. 09/26 

#define MAX_LEN 256
// #define MIN_END_DIS 200		   * the minimum distance (km) between the storm center of the end of pass *	
#define MIN_END_DIS 150		   /* the minimum distance (km) between the storm center of the end of pass */	

/* The distance bewteen two points: TRMM ~ 13, SSMI ~ 24, AMSU ~17 km, so 20 points should be greater than 200 km */
/* AMSRE is about 10-12 km, on 09/18/07 */
#define CLOSE_END_INDEX 20

/* Function prototypes */
float degreeToRadian(float degree);
float radianToDegree(float radian);
float greatCircleDistance(float lat1, float lon1, float lat2, float lon2);

int isInOrOutOrbit(int isTRMM, float firstLat, float firstLon, float secondLat, float secondLon, 
                   float centerLat, float centerLon);

void fbyte2(void *buffer, int num);
void fbyte4(void *buffer, int num);


int main (int argc, char *argv[])
{
   FILE  *dataFP;
   float *midLat;
   float *midLon;
   float lat;
   float lon;
   float lat1;
   float lon1;
   float lat2;
   float lon2;
   float distance;
   float shortestDis;               /* the shortest distance to the storm center within the orbit */

   float beginDis;         	    /* the distance from the beginning of the orbit to the storm center */ 
   float endDis;		    /* the distance from the end of the orbit to the storm center */
   float shortBeginDis;             /* the distance from the beginning of the orbit to the shortest index */ 
   float shortEndDis;		    /* the distance from the end of the orbit to the shortest index */

/*
   float firstLat;		    * the latitude of the edge point near the storm center *
   float firstLon;
   float secondLat;		    * the latitude of the second edge point neat the sotrm center *
   float secondLon;
*/
   int   sedIndex = 1;		    // * the second point of the SSMI orbit is same as the first one at the beginning, 
                                     //  so will use the third one actually, will check with SSMI programmer for reason * 


   float coverDistance;
   int   isCover = 0;
   short numScan;                   /* Number of scans for the pass */
   short *numPtr;
   char b[2];

   int   i;
   int   shortestInd;               /* the index of the scani line with the shortest distance to the storm */
   char  tempStr[MAX_LEN];           /* string to hold one input line */
   // char nswap;
   //   union {char ns0, ns1; 
   //     short unumScan;} U;
   //   numScan = -5243;
   //   U.unumScan = numScan;
   //   printf("numScan before %d\n", numScan);
   //   printf("U.ns0 %d\n", U.ns0);
   //   printf("U.ns1 %d\n", U.ns1);

   //   nswap = U.ns0;
   //   U.ns0 = U.ns1;
   //   U.ns1 = nswap;
   //   numScan = U.unumScan;
      printf("numScan entry %i\n", numScan);

   if (argc != 5)
   {
      printf("%s:four arguments required\n", argv[0]);
      printf("Command line format:\n");
      printf("%s <file name> <coverDistance> <lat> <lon>\n", argv[0]);
      exit(EXIT_FAILURE);
   } 
     argc = argc - 1;

   coverDistance = (float)atof(argv[2]);
   printf("Num of args is %i", argc);
   printf(" \n");
   printf("Start of Dist Calc for Bulletin/Pass pair\n");
   printf(" \n");
   printf("original lat is %s, lon is %s.\n", argv[3], argv[4]);

   lat = (float) atof(argv[3]);
   lon = (float) atof(argv[4]);

   printf("original-float lat is %f, lon is %f.\n", lat, lon);

   lat2 = degreeToRadian(lat);
   lon2 = degreeToRadian(lon);

   printf("program:%s, file name:%s, coverDistance:%f, lat:%f, lon:%f\n",
        argv[0], argv[1], coverDistance, lat2, lon2);

   /******************************* 
    Please note that SSMI geolocation file is text format,
    while AMSU and TRMM file are binary format
   *******************************/

   /* if the pattern of of the input file occurr in the file name, 
      it will return the string after that other than NULL.
      Then we handle this SSMI file differrently than AMSU or TRMM
   */
   printf("Before SSMI pattern %i.\n",numScan);

   if (strstr(argv[1], SSMI_PATTERN) != NULL) 
   {
      sedIndex = 2;    /* use the third point as the second one */

      if ((dataFP = fopen(argv[1], "r")) == NULL)
      {
         fprintf(stderr, "Cannot open file '%s' ... Exiting program\n", argv[1]);
         exit(EXIT_FAILURE);
      } else {
         printf("The file is open successfully 1.\n");
      }

      /* get the first line in the file and store to tempStr */
      fgets(tempStr, MAX_LEN, dataFP);
      printf ("Value of tempStr:%s\n", tempStr);
      
      /* get the number of scan from tempStr */
      sscanf(tempStr, "%hd", &numScan);
      //fscanf(dataFP,"%d", &numScan);
      
      /*
      if( tempStr != NULL){
	printf ("Value of tempStr:%s\n", tempStr);
      }else{
	printf ("tempStr is NULL\n");
      }
      */

      printf ("The number of scan is %ig.\n", numScan);    
      
      midLat = (float*) malloc (numScan * sizeof(float));
      midLon = (float*) malloc (numScan * sizeof(float));

      i = 0;  

      /* read the lat array and lon array of SSMI pass data*/
      /* get every line in the file and then get the first two number of the line
         as lat and lon, will ignore the remaining numbers in that line */ 
         
      while ( (fgets(tempStr, MAX_LEN, dataFP)!= NULL) && (i < numScan))
      {
         sscanf(tempStr,"%f%f ", &lat1, &lon1);
         midLat[i] = lat1;
         midLon[i] = lon1;
         printf ("the Sat lat is %f, lon is %f\n", lat1, lon1); 
         i++;
      }

   } else {
 
/*
      printf("Very beginning non-SSM %d.\n",numScan);
      
      if (strstr(argv[1], TRMM_PATTERN) != NULL)
      {
         isTRMM =1;
         printf("It is a TRMM orbit, isTRMM=%d.\n", isTRMM);
      }
*/
   
      if ((dataFP = fopen(argv[1], "rb")) == NULL)
      {
         fprintf(stderr, "Cannot open file '%s' ... Exiting program\n", argv[1]);
         exit(EXIT_FAILURE);
      } else {
         printf("the file is open successfully.2\n");
      }
      printf("Before numPtr non-SSM %i.\n",numScan);
 
      numPtr = &numScan;

      printf("After numPtr non-SSM %i.\n",numScan);

      if (fread(numPtr, 2, 1, dataFP) != 1)
      {
         printf ("Unable to read the data file.\n");
         exit(EXIT_FAILURE);
      } else {
         printf("After fread non-SSM %i.\n",numScan);
	 
         printf("the num scan is got successfully.\n");

	 // Byte swap 2-byte integer numScan

         if (strstr(argv[1], AMSRE_PATTERN) != NULL)
         {
	    fbyte2(numPtr, 1);
         }
	 // fbyte2(numPtr, 1);


         printf("the number of scan is %d \n", numScan);
      }

      /* allocate memory block to hold the one dimention array of 
         latitudes/longitude which record the center FOV geolocation
      */

      midLat = (float*) malloc (numScan * sizeof(float));
      midLon = (float*) malloc (numScan * sizeof(float));

      printf ("Value of numScan is %d.\n", numScan);  

      if (strstr(argv[1], SSMIS_PATTERN) != NULL) 
	{
	  if (fread(midLon, numScan * sizeof(float), 1, dataFP) != 1)
	    printf("read lon error.\n");   

	  if (fread(midLat, numScan * sizeof(float), 1, dataFP) != 1)
	    printf("read lat error.\n");
	} 
      else if (strstr(argv[1], GMI_PATTERN) != NULL) 
	{
           for (i = 0; i < numScan; i++ )
           {
	      if (fread(&midLat[i], sizeof(float), 1, dataFP) != 1)
	      printf("read lon error.\n");   

	      if (fread(&midLon[i], sizeof(float), 1, dataFP) != 1)
	       printf("read lat error.\n");
           } 
	} 
      else
	{
	  if (fread(midLat, numScan * sizeof(float), 1, dataFP) != 1)
	    printf("read lat error.\n");

	  if (fread(midLon, numScan * sizeof(float), 1, dataFP) != 1)
	    printf("read lon error.\n");   
	}

      // Byte swap 4-byte floating point midLat and midLon values
      if (strstr(argv[1], AMSRE_PATTERN) != NULL)
      {
         fbyte4(midLat, numScan);
         fbyte4(midLon, numScan); 
      }
      // fbyte4(midLat, numScan);
      // fbyte4(midLon, numScan); 
   }  

   fclose(dataFP);

   /*test statments 12/2015 */
   if ( (strstr(argv[1], MIRS_PATTERN) != NULL) || (strstr(argv[1], AMSR2_PATTERN) != NULL) )
   {
      printf("Got here 5");
      for (i = 0; i < numScan; i++ )
      {
         midLat[i] = midLat[i]*100;
         midLon[i] = midLon[i]*100;
	 /*  printf ("MIRS AMSR the lat is %f, lon is %f\n", lat1, lon1); */ 
      } 
   }
   /*   printf ("strstr %s\n",  strstr(argv[1]) */  
   /* printf ("MIRS AMSR the lat is %f, lon is %f\n", lat1, lon1) */

   /* initialize the shortest distance and the index to the first point */
   printf("lat1 lon1 %f %f\n",lat1,lon1);
   printf("midLat[0] midLon[0] %f %f\n",midLat[0],midLon[0]);
   printf("dr_midLat %f\n",degreeToRadian(midLat[0]));
   printf("dr_midLon %f\n",degreeToRadian(midLon[0]));
   printf("lat2 lon2 %f %f\n",lat2,lon2);

   shortestDis = greatCircleDistance(degreeToRadian(midLat[0]),
                                     degreeToRadian(midLon[0]),
                                     lat2,lon2);
   printf("shortestDis %f\n",shortestDis);
   shortestInd = 0;
 
   beginDis = shortestDis;

   /* the for loop compare the distance and the cover distance
      to find out whether the new pass cover the storm */

   //printf("Value of numScan before for loop on l248 is: %d.\n", numScan);

   for (i = 0; i < numScan; i++ )
   {
      /* calulate the distance between the center of the storm and
         one center scan of the pass */
     printf("i midLat midLon %i %f %f\n",i,midLat[i],midLon[i]);

      distance = greatCircleDistance(degreeToRadian(midLat[i]),
                                     degreeToRadian(midLon[i]),
                                     lat2,lon2);

      /* get the shortest distance */
      if (shortestDis > distance)
      {
         shortestDis = distance;
         shortestInd = i;
      }

      /* if it is true, the pass cover the storm */
/*      if (distance <= coverDistance)
      {
         printf("Found the shorter distance, it is %f.\n", distance);
         printf("The index is %d.\n", i);
         isCover = 1;            * the new pass covers the storm *
         break;
      }     
*/

   }  /* End of the for loop */
  
   endDis = distance;

   // check if the shortest index is close to two ends of the orbit
   shortBeginDis = greatCircleDistance(degreeToRadian(midLat[0]),
                                       degreeToRadian(midLon[0]),
                                       degreeToRadian(midLat[shortestInd]),
                                       degreeToRadian(midLon[shortestInd])
                                      );
   shortEndDis = greatCircleDistance(degreeToRadian(midLat[numScan - 1]),
                                       degreeToRadian(midLon[numScan - 1]),
                                       degreeToRadian(midLat[shortestInd]),
                                       degreeToRadian(midLon[shortestInd])
                                      );


   printf("\nThe beginning distance is %.3f, begin lat is %.3f, lon is %.3f, second lat is %.3f, second lon is %.3f,"
         " and the end distance is %.3f, end lat is %.3f, and lon is %.3f; last second lat is %.3f, last second lon is %.3f\n ", 
           beginDis, midLat[0], midLon[0], midLat[sedIndex], midLon[sedIndex], 
           endDis, midLat[numScan - 1], midLon[numScan - 1], midLat[numScan - 2], midLon[numScan - 2]);

   printf("\nThe middle lat is %.3f, lon is %.3f, the second middle lat is %.3f, lon is %.3f\n\n",
             midLat[(int)numScan/2], midLon[(int)numScan/2],  midLat[(int)numScan/2+1], midLon[(int)numScan/2+1]); 

   printf ("\nshortBeginDis is  %.3f, shortEndDis is %.3f.\n", shortBeginDis, shortEndDis);

   if ((shortestDis <= coverDistance)&& (shortBeginDis >= MIN_END_DIS) && (shortEndDis >= MIN_END_DIS))
   {
      printf("The pass is over the storm.  Found the shorter distance, it is %f.\n", shortestDis);
      isCover = 1;            /* the new pass covers the storm */
   }

 
   printf("Found the shortest distance, it is %f km.\n", shortestDis);
   printf("The index of the shortest distance= %d.\n", shortestInd);
   printf("The closest center lat is %f, lon is %f\n", midLat[shortestInd], midLon[shortestInd]);
   printf("the number of scan is %d. \n", numScan);

   printf("the Great circle distance to last scan is %f km\n", distance);
 

   /* Don't modify the following output line if not necessary, 
      since it will be GREPed by the pctrap.pl program.  If have to do so,
      please modify the correlating part in program pctrap.pl.
   */
   printf("The final value of isCover=%d\n", isCover);
 
   return 0;
}


/********************   function definition   ************************/

/******** Function:degreeToRadian ********/
/* convert the degrees to radians */
float degreeToRadian(float degree)
{
   return (degree * PI / 180.0);
}

/******** Function:radianToDegree ********/
/* convert the radians to degrees */
float radianToDegree(float radian)
{
   return (radian * 180.0 / PI); 
} 

/******** Function:greatCircleDistance ********/
/* given coordinates of two places in radians, compute distance in meters */
float greatCircleDistance (float  lat1, float lon1, float lat2, float lon2)
{
   float dlon;
   float dlat;
   float a;
   float d;
  
   dlon = lon2 - lon1;
   dlat = lat2 - lat1;   

   a = (float)pow(sin(dlat / 2),2) 
       + cos(lat1) * cos(lat2) * (float)pow(sin(dlon / 2), 2);
   d = 2 * atan2(sqrt(a), sqrt(1 - a));

   /*  This is a simpler formula, but it's subject to rounding errors
       for small distances. 
       d = &acos(sin(lat1) * sin(lat2)
                   + cos(lat1) * cos(lat2) * cos(lon1-lon2));
   */

     return (EARTH_RADIUS * d); 
}


/******** Function:isInOrOutOrbit ********/

int isInOrOutOrbit(int isTRMM, float firstLat, float firstLon, float secondLat, float secondLon,
                   float centerLat, float centerLon)
{
   int outsideOrb = 0;
   float diffCenter;   		/* the discrepancy of the center lat/lon to the first end piont lat/lon */
   float diffSecond;		/* the discrepancy of the second end point lat/lon to the first end piont lat/lon */
   if (isTRMM == 1)
   {
      // use longitude as the discrepancy 
      diffCenter = centerLon - firstLon;
      diffSecond = secondLon - firstLon;

      // this can use more complicated method to tell if is outside the orbit more accurate,
      // such as check the slope/degree of the orbit, if it is close to 45 degree, only consider that quadrant,
      // the center fell in the other three quadrant is considerred as outside the orbit        
      if ((diffCenter/diffSecond) < 0)
      {
         outsideOrb = 1;
      }
         
   }
   else
   {
      // use latitude as the discrepancy
      diffCenter = centerLat - firstLat;
      diffSecond = secondLat - firstLat;

      if ((diffCenter/diffSecond) < 0)
      {
         outsideOrb = 1;
      }
      

   }

   return outsideOrb;
}

// 2-byte swapping function
void fbyte2(void *buffer, int num)
{
  char *cbuf = (char *)buffer;
  int i, n;

  n = num;
  for (i = 0; i < n; i++)
  {
    char b;

    b = cbuf[0];
    cbuf[0] = cbuf[1];
    cbuf[1] = b;
    cbuf += 2;
  }
}

// 4-byte swapping function
void fbyte4(void *buffer, int num)
{
  char *cbuf = (char *)buffer;
  int i, n;

  n = num;
  for (i = 0; i < n; i++){
    char b;

    b = cbuf[0];
    cbuf[0] = cbuf[3];
    cbuf[3] = b;
    b = cbuf[1];
    cbuf[1] = cbuf[2];
    cbuf[2] = b;
    cbuf += 4;
  }
}






